SystemOrganization addCategory: #'System-Support'!

Notification subclass: #FontSubstitutionDuringLoading
	instanceVariableNames: 'familyName pixelSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!FontSubstitutionDuringLoading commentStamp: '<historical>' prior: 0!
signaled by font loading code when reading a DiskProxy that calls for a missing font.!

!FontSubstitutionDuringLoading class methodsFor: 'instance creation' stamp: 'nk 11/8/2004 15:07'!
forFamilyName: aName pixelSize: aSize
	^(self new)
		familyName: aName;
		pixelSize: aSize;
		yourself.! !

!FontSubstitutionDuringLoading methodsFor: 'accessing' stamp: 'AlainPlantec 11/29/2009 21:33'!
defaultAction
	familyName ifNil: [ familyName := 'NoName' ].
	pixelSize ifNil: [ pixelSize := 12 ].

	^((familyName beginsWith: 'Comic')
		ifTrue: [ TextStyle named: (StandardFonts buttonFont familyName) ]
		ifFalse: [ TextStyle default ]) fontOfSize: pixelSize.! !

!FontSubstitutionDuringLoading methodsFor: 'accessing' stamp: 'nk 11/8/2004 15:01'!
familyName
	"Answer the value of familyName"

	^ familyName! !

!FontSubstitutionDuringLoading methodsFor: 'accessing' stamp: 'nk 11/8/2004 15:01'!
familyName: anObject
	"Set the value of familyName"

	familyName := anObject! !

!FontSubstitutionDuringLoading methodsFor: 'accessing' stamp: 'nk 11/8/2004 15:01'!
pixelSize
	"Answer the value of pixelSize"

	^ pixelSize! !

!FontSubstitutionDuringLoading methodsFor: 'accessing' stamp: 'nk 11/8/2004 15:01'!
pixelSize: anObject
	"Set the value of pixelSize"

	pixelSize := anObject! !

!FontSubstitutionDuringLoading methodsFor: 'accessing' stamp: 'nk 11/8/2004 16:55'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(;
		nextPutAll: familyName;
		nextPut: $-;
		print: pixelSize;
		nextPut: $).! !

Model subclass: #Project
	instanceVariableNames: 'world changeSet transcript parentProject previousProject displayDepth viewSize thumbnail nextProject guards projectParameters isolatedHead inForce version urlList environment lastDirectory lastSavedAtSeconds projectPreferenceFlagDictionary resourceManager'
	classVariableNames: 'AllProjects CurrentProject GoalFreePercent GoalNotMoreThan UIProcess'
	poolDictionaries: ''
	category: 'System-Support'!

!Project commentStamp: 'tk 12/2/2004 12:38' prior: 0!
A Project stores the state of a complete Squeak desktop, including
the windows, and the currently active changeSet.  A project knows who
its parent project is.  When you change projects, whether by entering
or exiting, the screen state of the project being exited is saved in
that project.

A project is retained by its view in the parent world.  It is
effectively named by the name of its changeSet, which can be changed
either by renaming in a changeSorter, or by editing the label of its
view from the parent project.

As the site of major context switch, Projects are the locus of
swapping between the old MVC and the new Morphic worlds.  The
distinction is based on whether the variable 'world' contains a
WorldMorph or a ControlManager.

Saving and Loading
Projects may be stored on the disk in external format.  (Project
named: 'xxx') exportSegment, or choose 'store project on file...'.
Projects may be loaded from a server and stored back.  Storing on a
server never overwrites;  it always makes a new version.  A project
remembers the url of where it lives in urlList.  The list is length
one, for now.  The url may point to a local disk instead of a server.
All projects that the user looks at, are cached in the Squeaklet
folder.  Sorted by server.  The cache holds the most recent version
only.

When a project is loaded into Squeak, its objects are converted to
the current version.  There are three levels of conversion.  First,
each object is converted from raw bits to an object in its old
format.  Then it is sent some or all of these messages:
	comeFullyUpOnReload: smartRefStream  		Used to
re-discover an object that already exists in this image, such as a
resource, global variable, Character, or Symbol.  (sent to objects in
outPointers)
	convertToCurrentVersion: varDict refStream: smartRefStrm
		fill in fields that have been added to a class since
the object was stored.  Used to set the extra inst var to a default
value.  Or, return a new object of a different class.  (sent to
objects that changed instance variables)
	fixUponLoad: aProject refStream: smartRefStrm
	change the object due to conventions that have changed on the
project level.  (sent to all objects in the incoming project)

Here is the calling sequence for storing out a Project:
Project saveAs
Project storeOnServer
Project storeOnServerWithProgressInfo
Project storeOnServerInnards
Project exportSegmentFileName:directory:
Project exportSegmentWithChangeSet:fileName:directory:
ImageSegment writeForExportWithSources:inDirectory:changeSet:
---------
Isolation (not used any more)
When you accept a method, the entire system feels the change, except
projects that are "isolated".  In an isolated project, all new global
variables (including new classes) arestored in the project-local
environment, and all changes to preexisting classes are revoked when
you leave the project.  When you enter another project, that
project's changes are invoked.  Invocation and revocation are handled
efficiently by swapping pointers.  To make a project be isolated,
choose 'isolate changes of this project' from the 'changes...'
section of the screen menu.  You can use an isolated project for
making dangerous change to a system, and you can get out if it
crashes.  A foreign application can have the separate environment it
wants.  Also, you can freeze part of the system for a demo that you
don't want to disturb.  An isolated project shares methods with all
subprojects inside it, unless they are isolated themselves.   Each
isolated project is the head of a tree of projects with which it
shares all methods.

You may 'assert' all changes ever made in the current project to take
effect above this project.  This amounts to exporting all the globals
in the current environment, and zapping the revocation lists to that
the current state of the world will remain in force upon exit from
this project.

[Later: A project may be 'frozen'.  Asserts do not apply to it after
that.  (Great for demos.)  You should be informed when an assert was
blocked in a frozen project.]

Class definitions are layered by the isolation mechanism.  You are
only allowed to change the shape of a class in projects that lie
within its isolation scope.  All versions of the methods are
recompiled, in all projects.  If you remove an inst var that is in
use in an isolated project, it will become an Undeclared global.  It
is best not to remove an inst var when it is being used in another
isolated project. [If we recompile them all, why can't we diagnose
the problem before allowing the change??]

Senders and Implementors do not see versions of a method in isolated
projects.  [again, we might want to make this possible at a cost].
When you ask for versions of a method, you will not get the history
in other isolated projects.

Moving methods and classes between changeSets, and merging changeSets
has no effect on which methods are in force.  But, when you look at a
changeSet from a different isolated project, the methods will contain
code that is not in force.  A changeSet is just a list of method
names, and does not keep separate copies of any code.

When finer grained assertion is needed, use the method (aProject
assertClass: aClass from: thisProject warn: warnConflicts).

How isolated changes work: The first time a class changes, store its
MethodDictionary object.  Keep parallel arrays of associations to
Classes and MethodDictionaries.  Traverse these and install them when
you enter an "ioslated project".  When you leave, store this
project's own MethodDictionaries there.
	To do an assert, we must discover which methods changed here,
and which changed only in the project we are asserting into.  There
is one copy of the 'virgin' method dictionaries in the system.  It is
always being temporarily stored by the currently inForce isolated
project.

isolatedHead - true for the top project, and for each isolated
project.  false or nil for any subproject that shares all methods
with its parent project.

inForce -  true if my methods are installed now.  false if I am
dormant. [is this equivalent to self == Project Current?]

classArray - list of associations to classes

methodDictArray - the method dictionaries of those classes before we
started changing methods.  They hang onto the original
compiledMethods.  (If this project is dormant, it contains the method
dictionaries of those classes as they will be here, in this project).

orgArray - the class organizations of the classes in classArray.

UsingIsolation (class variable) - No longer used.

When you want to save a project in export format from within that
very project, it gets tricky.  We set two flags in parentProject,
exit to it, and let parentProject write the project.
ProjectViewMorph in parentProject checks in its step method, does the
store, clears the flags, and reenters the subProject.

!

!Project class methodsFor: 'utilities' stamp: 'sw 9/12/2001 23:05'!
chooseNaturalLanguage
	"Have the current project choose a new natural language"

	self current chooseNaturalLanguage! !

!Project class methodsFor: 'constants' stamp: ''!
current
	"Answer the project that is currently being used."

	^CurrentProject! !

!Project class methodsFor: 'utilities' stamp: 'dtl 4/3/2005 14:02'!
interruptName: labelString
	"Create a Notifier on the active scheduling process with the given label."

	^ self interruptName: labelString preemptedProcess: nil
! !

!Project class methodsFor: 'utilities' stamp: 'nice 4/19/2009 16:25'!
interruptName: labelString preemptedProcess: theInterruptedProcess
	"Create a Notifier on the active scheduling process with the given label."
	| preemptedProcess projectProcess |
	ActiveHand ifNotNil:[ActiveHand interrupted].
	ActiveWorld := World. "reinstall active globals"
	ActiveHand := World primaryHand.
	ActiveHand interrupted. "make sure this one's interrupted too"
	ActiveEvent := nil.

	projectProcess := self uiProcess.	"we still need the accessor for a while"
	preemptedProcess := theInterruptedProcess ifNil: [Processor preemptedProcess].
	"Only debug preempted process if its priority is >= projectProcess' priority"
	preemptedProcess priority < projectProcess priority ifTrue:[
		projectProcess suspend.
		preemptedProcess := projectProcess.
	] ifFalse:[
		preemptedProcess suspend offList.
	].
	Debugger openInterrupt: labelString onProcess: preemptedProcess
! !

!Project class methodsFor: 'squeaklet on server' stamp: 'mir 8/8/2001 17:57'!
loaderUrl
	"Return a url that will allow to launch a project in a browser by composing a url like
	<loaderURL>?<projectURL>"

	^AbstractLauncher extractParameters at: 'LOADER_URL' ifAbsent: [nil].! !

!Project class methodsFor: 'initialization' stamp: 'tak 8/3/2005 18:35'!
localeChanged
	self current localeChanged! !

!Project class methodsFor: 'utilities' stamp: 'alain.plantec 6/1/2008 19:55'!
maybeForkInterrupt

	Preferences cmdDotEnabled ifFalse: [^self].
	[self interruptName: 'User Interrupt'] fork
! !

!Project class methodsFor: 'squeaklet on server' stamp: 'ar 10/11/2000 15:42'!
parseProjectFileName: aString
	"It was formerly possible to have periods in projct names and this messed up some parsing methods. Try to handle that more gracefully and allow for a change in scheme at a later time.
	ar 10/11/2000: Switch to a different version encoding scheme. The new scheme is
		baseName.NNN.ext
	where NNN is at least three digits wide and encodes the version in a human readable form.
	Examples:
		Project parseProjectFileName: 'My Project.007.pr'.
		Project parseProjectFileName: 'My.First.Project.042.prj'.
		Project parseProjectFileName: 'My Project.123456.p r o j e c t'.
	The 'dot' is determined on FileDirectory>>dot to compensate for platforms wishing to use something different from a period. Also allows parsing the former encoding of file using Base64 encoded versions of the form
	Project parseProjectFileName: 'aa.bb.cc|AQ.ss'
	"
	| suffix baseName version versionAndSuffix index tokens |
	"answer an array with: 
		1 = basic project name
		2 = version string
		3 = suffix (pr)"

	"First check for the old style versions"
	index := aString findLast:[:ch| ch = $|].
	index = 0 ifFalse:["Old style version"
		baseName := aString copyFrom: 1 to: index-1.
		versionAndSuffix := aString copyFrom: index+1 to: aString size.
		(versionAndSuffix occurrencesOf: $.) = 0 ifTrue: [^ #('no suffix')].
		version := versionAndSuffix copyUpTo: $..
		suffix := versionAndSuffix copyFrom: version size+1 to: versionAndSuffix size.
		"Decode Base64 encoded version"
		version isEmpty
			ifTrue:[version := 0]
			ifFalse:[version := Base64MimeConverter decodeInteger: version unescapePercents].
		^{baseName. version. suffix}].
	"New style versions"
	tokens := aString findTokens: FileDirectory dot.
	tokens size < 2 "Not even a single dot"
		ifTrue:[^{aString. 0. ''}].
	tokens size < 3 ifTrue:["Only one dot"
		self flag: #arNote. "We could allow project file names of the form 'project.001' (e.g., no project extension) or '.001.pr' (without a base name) but I don't think its a good idea."
		^{tokens first. 0. tokens last}].
	suffix := tokens last.
	version := tokens at: tokens size - 1.
	(version anySatisfy:[:ch| ch isDigit not]) ifTrue:[
		"Non-digit version??? I don't think so..."
		baseName := aString copyFrom: 1 to: aString size - suffix size - 1.
		^{baseName. 0. suffix}].
	baseName := aString copyFrom: 1 to: aString size - suffix size - version size - 2.
	version := version asInteger.
	^{baseName. version. suffix}! !

!Project class methodsFor: 'squeaklet on server' stamp: 'ar 2/27/2001 13:43'!
projectExtension
	^'pr'! !

!Project class methodsFor: 'utilities' stamp: 'RAA 6/3/2000 18:28'!
resumeProcess: aProcess
	"Adopt aProcess as the project process -- probably because of proceeding from a debugger"

	UIProcess := aProcess.
	UIProcess resume! !

!Project class methodsFor: 'utilities' stamp: 'RAA 6/3/2000 18:49'!
spawnNewProcess

	UIProcess := [
		[World doOneCycle.  Processor yield.  false] whileFalse: [].
	] newProcess priority: Processor userSchedulingPriority.
	UIProcess resume! !

!Project class methodsFor: 'utilities' stamp: 'RAA 6/3/2000 18:49'!
spawnNewProcessAndTerminateOld: terminate

	self spawnNewProcess.
	terminate
		ifTrue: [Processor terminateActive]
		ifFalse: [Processor activeProcess suspend]! !

!Project class methodsFor: 'squeaklet on server' stamp: 'RAA 8/8/2000 10:42'!
spawnNewProcessIfThisIsUI: suspendedProcess

	self uiProcess == suspendedProcess ifTrue: [
		self spawnNewProcess.
		^true
	].
	^false		"no new process was created"
! !

!Project class methodsFor: 'squeaklet on server' stamp: 'RAA 2/19/2001 07:37'!
squeakletDirectory

	| squeakletDirectoryName |
	squeakletDirectoryName := 'Squeaklets'.
	(FileDirectory default directoryExists: squeakletDirectoryName) ifFalse: [
		FileDirectory default createDirectory: squeakletDirectoryName
	].
	^FileDirectory default directoryNamed: squeakletDirectoryName! !

!Project class methodsFor: 'instance creation' stamp: 'RAA 9/27/2000 13:41'!
uiProcess

	^ UIProcess! !

!Project methodsFor: 'docking bars support' stamp: 'dgd 9/20/2004 16:29'!
assureMainDockingBarPresenceMatchesPreference
	"Syncronize the state of the receiver's dockings with the  
	preference"
	(self showWorldMainDockingBar)
		ifTrue: [self createOrUpdateMainDockingBar]
		ifFalse: [self removeMainDockingBar]! !

!Project methodsFor: 'accessing' stamp: 'tk 4/5/2000 16:29'!
changeSet
	^ changeSet! !

!Project methodsFor: 'language' stamp: 'marcus.denker 11/19/2008 13:43'!
chooseNaturalLanguage
	"Put up a menu allowing the user to choose the natural language for the project"

	| aMenu availableLanguages |
	aMenu := MenuMorph new defaultTarget: self.
	aMenu addTitle: 'choose language' translated.
	aMenu lastItem setBalloonText: 'This controls the human language in which tiles should be viewed.  It is potentially extensible to be a true localization mechanism, but initially it only works in the classic tile scripting system.  Each project has its own private language choice' translated.
	aMenu addStayUpItem.

	availableLanguages := NaturalLanguageTranslator availableLanguageLocaleIDs asSortedCollection:[:x :y | x displayName < y displayName].

	availableLanguages do: [:localeID |
			aMenu addUpdating: #stringForLanguageNameIs: target: Locale selector:  #switchAndInstallFontToID: argumentList: {localeID}].
	aMenu popUpInWorld

"Project current chooseNaturalLanguage"! !

!Project methodsFor: 'isolation layers' stamp: 'di 4/1/2000 09:22'!
compileAll: newClass from: oldClass
	"Make sure that shadowed methods in isolation layers get recompiled.
	Traversal is done elsewhere.  This simply handles the current project."

	isolatedHead == true ifFalse: [^ self].   "only isolated projects need to act on this."
	
	changeSet compileAll: newClass from: oldClass! !

!Project methodsFor: 'docking bars support' stamp: 'dgd 9/6/2004 18:47'!
createOrUpdateMainDockingBar
	"Private - create a new main docking bar or update the current one"
	| w mainDockingBars |
	w := self world.
	mainDockingBars := w mainDockingBars.
	mainDockingBars isEmpty
		ifTrue: ["no docking bar, just create a new one"
			TheWorldMainDockingBar instance createDockingBar openInWorld: w.
			^ self].
	""
	"update if nedeed"
	mainDockingBars
		do: [:each | TheWorldMainDockingBar instance updateIfNeeded: each]! !

!Project methodsFor: 'menu messages' stamp: 'StephaneDucasse 11/13/2009 18:43'!
enterForEmergencyRecovery
	"This version of enter invokes an absolute minimum of mechanism.
	An unrecoverable error has been detected in an isolated project.
	It is assumed that the old changeSet has already been revoked.
	No new process gets spawned here. This will happen in the debugger."
	self isCurrentProject
		ifTrue: [^ self].
	CurrentProject := self.
	Display newDepthNoRestore: displayDepth.
	ChangeSet newChanges: changeSet.
	TranscriptStream newTranscript: transcript.
	World pauseEventRecorder.
	"Entering a Morphic project"
	World := world.
	world install.
	UIProcess := Processor activeProcess! !

!Project methodsFor: 'initialization' stamp: 'alain.plantec 5/28/2009 10:15'!
initialize
	"Initialize the project, seting the CurrentProject as my parentProject and initializing my project preferences from those of the CurrentProject"

	super initialize.
	changeSet := ChangeSet new.
	transcript := TranscriptStream new.
	displayDepth := Display depth.
	parentProject := CurrentProject.
	isolatedHead := false.
	self initializeProjectPreferences
! !

!Project methodsFor: 'project parameters' stamp: 'sw 4/24/2001 11:58'!
initializeProjectParameters
	"Initialize the project parameters."

	projectParameters := IdentityDictionary new.
	^ projectParameters! !

!Project methodsFor: 'project parameters' stamp: 'AlainPlantec 10/17/2009 18:16'!
initializeProjectPreferences
	"Initialize the project's preferences from currently-prevailing preferences that are currently being held in projects in this system"
	
	projectPreferenceFlagDictionary := Project current projectPreferenceFlagDictionary deepCopy.    "Project overrides in the new project start out being the same set of overrides in the calling project"

	Preferences allPreferenceObjects do:  "in case we missed some"
		[:aPreference |
			aPreference localToProject ifTrue:
				[(projectPreferenceFlagDictionary includesKey: aPreference name) ifFalse:
			[projectPreferenceFlagDictionary at: aPreference name put: aPreference preferenceValue]]].

! !

!Project methodsFor: 'isolation layers' stamp: 'di 3/29/2000 16:04'!
invoke
	"Install all methods changed here into method dictionaries.
	Make my versions be the ones that will be called."

	isolatedHead ifFalse: [^ self error: 'This isnt an isolation layer.'].
	inForce ifTrue: [^ self error: 'This layer is already in force.'].
	changeSet invoke.	
	inForce := true.! !

!Project methodsFor: 'accessing' stamp: 'RAA 6/3/2000 10:23'!
isCurrentProject

	^self == CurrentProject! !

!Project methodsFor: 'accessing' stamp: 'di 3/29/2000 15:37'!
isTopProject
	"Return true only if this is the top project (its own parent).
	Also include the test here for malformed project hierarchy."

	parentProject == self ifTrue: [^ true].
	parentProject == nil ifTrue: [self error: 'No project should have a nil parent'].
	^ false! !

!Project methodsFor: 'isolation layers' stamp: 'di 4/4/2000 21:10'!
isolationHead
	"Go up the parent chain and find the nearest isolated project."

	isolatedHead == true ifTrue: [^ self].
	self isTopProject ifTrue: [^ nil].
	^ parentProject isolationHead! !

!Project methodsFor: 'isolation layers' stamp: 'di 3/29/2000 17:00'!
isolationSet

	"Return the changeSet for this isolation layer or nil"
	isolatedHead == true ifTrue: [^ changeSet].
	self isTopProject ifTrue: [^ nil].  "At the top, but not isolated"
	^ parentProject isolationSet

! !

!Project methodsFor: 'accessing' stamp: 'tk 6/26/1998 22:17'!
labelString
	^ changeSet name! !

!Project methodsFor: 'language' stamp: 'tak 8/4/2005 16:30'!
localeChanged
	"Set the project's natural language as indicated"
	self projectParameterAt: #localeID put: LocaleID current.
	self updateLocaleDependents! !

!Project methodsFor: 'language' stamp: 'StephaneDucasse 11/6/2009 21:06'!
localeID
	"Answer the natural language for the project"

	^ LocaleID current! !

!Project methodsFor: 'accessing' stamp: 'di 7/19/1999 15:06'!
name
	changeSet == nil ifTrue: [^ 'no name'].
	^ changeSet name! !

!Project methodsFor: 'project parameters' stamp: 'sw 2/16/2001 22:35'!
noteThatParameter: prefSymbol justChangedTo: aBoolean
	"Provides a hook so that a user's toggling of a project parameter might precipitate some immediate action"

! !

!Project methodsFor: 'project parameters' stamp: 'ar 5/25/2000 23:23'!
parameterAt: aSymbol
	^self parameterAt: aSymbol ifAbsent:[nil]! !

!Project methodsFor: 'project parameters' stamp: 'ar 5/25/2000 23:23'!
parameterAt: aSymbol ifAbsent: aBlock
	projectParameters ifNil:[^aBlock value].
	^projectParameters at: aSymbol ifAbsent: aBlock! !

!Project methodsFor: 'accessing' stamp: 'di 3/6/98 10:22'!
parent
	^ parentProject! !

!Project methodsFor: 'accessing' stamp: ''!
projectChangeSet
	^ changeSet! !

!Project methodsFor: 'project parameters' stamp: 'sw 10/30/2000 11:14'!
projectParameterAt: aSymbol
	"Answer the project parameter stored at the given symbol, or nil if none"

	^ self projectParameters at: aSymbol ifAbsent: [nil]! !

!Project methodsFor: 'project parameters' stamp: 'sw 2/17/2001 21:36'!
projectParameterAt: aSymbol put: aValue
	"Set the given project parameter to the given value"

	self projectParameters at: aSymbol put: aValue.
	self noteThatParameter: aSymbol justChangedTo: aValue.
	^ aValue! !

!Project methodsFor: 'project parameters' stamp: 'sw 4/22/1999 15:14'!
projectParameters
	^ projectParameters ifNil: [self initializeProjectParameters]! !

!Project methodsFor: 'project parameters' stamp: 'sw 2/16/2001 22:25'!
projectPreferenceAt: aSymbol ifAbsent: aBlock
	"Answer the project preference stored at the given symbol, or the result of evaluating the block"

	^ self projectPreferenceFlagDictionary at: aSymbol ifAbsent: [aBlock value]! !

!Project methodsFor: 'project parameters' stamp: 'sw 2/16/2001 22:23'!
projectPreferenceFlagDictionary
	"Answer the dictionary that holds the project preferences, creating it if necessary"

	^ projectPreferenceFlagDictionary ifNil: [projectPreferenceFlagDictionary := IdentityDictionary new]! !

!Project methodsFor: 'docking bars support' stamp: 'dgd 9/6/2004 18:46'!
removeMainDockingBar
	"Remove the receiver's main docking bars"
	self world mainDockingBars
		do: [:each | each delete]! !

!Project methodsFor: 'resources' stamp: 'ar 2/27/2001 17:02'!
resourceManager
	^resourceManager ifNil:[resourceManager := ResourceManager new]! !

!Project methodsFor: 'isolation layers' stamp: 'di 3/29/2000 16:06'!
revoke
	"Take back all methods changed here.
	Install the original method dictionaries and organizations.
	The orignal method versions will now be the ones used."

	isolatedHead ifFalse: [^ self error: 'This isnt an isolation layer.'].
	inForce ifFalse: [^ self error: 'This layer should have been in force.'].
	changeSet revoke.	
	inForce := false.
! !

!Project methodsFor: 'initialization' stamp: 'RAA 6/21/2000 22:59'!
setChangeSet: aChangeSet

	isolatedHead == true ifTrue: [^ self].  "ChangeSet of an isolated project cannot be changed"
	changeSet := aChangeSet
! !

!Project methodsFor: 'accessing' stamp: 'di 4/14/1999 13:59'!
setParent: newParent

	parentProject := newParent.
	nextProject := previousProject := nil.! !

!Project methodsFor: 'accessing' stamp: 'RAA 5/10/2001 14:57'!
setThumbnail: aForm

	self flag: #bob.		"no longer used??"
	thumbnail := aForm! !

!Project methodsFor: 'docking bars support' stamp: 'dgd 9/20/2004 16:30'!
showWorldMainDockingBar

	^ self projectPreferenceFlagDictionary
		at: #showWorldMainDockingBar
		ifAbsent: [Preferences showWorldMainDockingBar]! !

!Project methodsFor: 'docking bars support' stamp: 'dgd 9/6/2004 18:47'!
showWorldMainDockingBar: aBoolean 
	"Change ther receiver to show the main docking bar"
	self projectPreferenceFlagDictionary at: #showWorldMainDockingBar put: aBoolean.
	""
	self == Project current
		ifTrue: [""
			aBoolean == Preferences showWorldMainDockingBar
				ifFalse: [Preferences setPreference: #showWorldMainDockingBar toValue: aBoolean]].
	""
	self assureMainDockingBarPresenceMatchesPreference! !

!Project methodsFor: 'docking bars support' stamp: 'dgd 9/20/2004 19:28'!
showWorldMainDockingBarString
	^ (self showWorldMainDockingBar
		ifTrue: ['<yes>']
		ifFalse: ['<no>'])
		, 'show main docking bar (M)' translated! !

!Project methodsFor: 'file in/out' stamp: 'yo 12/14/2004 18:18'!
storeAttributeKey: key value: value on: aStream

	(key includes: $:) ifTrue: [self error: 'key should not contain :'].
	aStream nextPutAll: key.
	aStream nextPutAll: ': '.
	aStream nextPutAll: value.
	aStream cr.
! !

!Project methodsFor: 'file in/out' stamp: 'KR 12/16/2005 11:07'!
storeAttributesOn: aStream

	| details |
	self storeAttributeKey: 'Squeak-Version' value: SystemVersion current version on: aStream.
	self storeAttributeKey: 'Squeak-LatestUpdate' value: SystemVersion current highestUpdate printString on: aStream.
	self storeAttributeKey: 'File-Name-Encoding' value: LanguageEnvironment defaultFileNameConverter class encodingNames first on: aStream.

	details := self world valueOfProperty: #ProjectDetails ifAbsent: [Dictionary new].
	details associationsDo: [:assoc |
		self storeAttributeKey: assoc key asString value: assoc value asString on: aStream.].
! !

!Project methodsFor: 'file in/out' stamp: 'tk 8/21/1999 07:31'!
storeDataOn: aDataStream
	"Write me out.  All references to other projects are weak references.  They only go out if they are written for another reason."
	| cntInstVars cntIndexedVars localInstVars offset |

	cntInstVars := self class instSize.
	cntIndexedVars := self basicSize.
	localInstVars := Project instVarNames.
	offset := Project superclass instSize.
	aDataStream
		beginInstance: self class
		size: cntInstVars + cntIndexedVars.
	1 to: cntInstVars do:
		[:ii | 
		(ii between: offset+1 and: offset + localInstVars size) 
			ifTrue: [(#('parentProject' 'previousProject' 'nextProject') includes: 
				(localInstVars at: ii-offset)) 
					ifTrue: [aDataStream nextPutWeak: (self instVarAt: ii)]
								"owner only written if in our tree"
					ifFalse: [aDataStream nextPut: (self instVarAt: ii)]]
			ifFalse: [aDataStream nextPut: (self instVarAt: ii)]].

	1 to: cntIndexedVars do:
		[:i | aDataStream nextPut: (self basicAt: i)]! !

!Project methodsFor: 'docking bars support' stamp: 'dgd 9/20/2004 16:31'!
toggleShowWorldMainDockingBar
	self showWorldMainDockingBar: self showWorldMainDockingBar not! !

!Project methodsFor: 'language' stamp: 'AlainPlantec 10/17/2009 20:32'!
updateLocaleDependents
	"Set the project's natural language as indicated"
	MenuIcons initializeTranslations.
	LanguageEnvironment localeChanged.
! !

!Project methodsFor: 'accessing' stamp: ''!
world
	^ world! !

Model subclass: #SARInstaller
	instanceVariableNames: 'zip directory fileName installed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!SARInstaller commentStamp: 'AlainPlantec 10/17/2009 18:58' prior: 0!
I am an object that handles the loading of SAR (Squeak ARchive) files.

A SAR file is a Zip file that follows certain simple conventions:

* it may have a member named "install/preamble".

This member, if present, will be filed in as Smalltalk source code at the beginning of installation.
Typically, the code in the preamble will make whatever installation preparations are necessary,
and will then call methods in the "client services" method category to extract or install other zip members.

* It may have a member named "install/postscript".

This member, if present, will be filed in as Smalltalk source code at the end of installation.
Typically, the code in the postscript will set up the operating environment,
and will popen projects or README files, or launch samples.

Within the code in the preamble and postscript, "self" is set to the instance of the SARInstaller.

If neither an "install/preamble" nor an "install/postscript" file is present,
all the members will be installed after prompting the user,
based on a best guess of the member file types that is based on member filename extensions.

This is new behavior.!

!SARInstaller class methodsFor: 'change set utilities' stamp: 'nk 10/27/2002 12:44'!
basicNewChangeSet: newName
	Smalltalk at: #ChangeSorter ifPresentAndInMemory: [ :cs | ^cs basicNewChangeSet: newName ].
	(self changeSetNamed: newName) ifNotNil: [ self inform: 'Sorry that name is already used'. ^nil ].
	^ChangeSet basicNewNamed: newName.! !

!SARInstaller class methodsFor: 'squeakmap' stamp: 'marcus.denker 11/10/2008 10:04'!
cardForSqueakMap: aSqueakMap
	"Answer the current card or a new card."

	(aSqueakMap cardWithId: self squeakMapPackageID)
		ifNotNil: [ :card |
			(card installedVersion = self squeakMapPackageVersion) ifTrue: [ ^card ]
		].

	^self newCardForSqueakMap: aSqueakMap
! !

!SARInstaller class methodsFor: 'change set utilities' stamp: 'nk 10/27/2002 12:44'!
changeSetNamed: newName
	Smalltalk at: #ChangeSorter ifPresentAndInMemory: [ :cs | ^cs changeSetNamed: newName ].
	^ChangeSet allInstances detect: [ :cs | cs name = newName ] ifNone: [ nil ].! !

!SARInstaller class methodsFor: 'change set utilities' stamp: 'nk 7/5/2003 22:49'!
currentChangeSet
	"Answer the current change set, in a way that should work in 3.5 as well"

	"SARInstaller currentChangeSet"

	^[ ChangeSet current ]
		on: MessageNotUnderstood
		do: [ :ex | ex return: Smalltalk changes ]! !

!SARInstaller class methodsFor: 'instance creation' stamp: 'nk 10/27/2002 10:29'!
directory: dir fileName: fn
	^(self new) directory: dir; fileName: fn; yourself.! !

!SARInstaller class methodsFor: 'package format support' stamp: 'ar 9/27/2005 20:11'!
ensurePackageWithId: anIdString

	self squeakMapDo: [ :sm | | card newCS |
		self withCurrentChangeSetNamed: 'updates' do: [ :cs |
			newCS := cs.
			card := sm cardWithId: anIdString.
			(card isNil or: [ card isInstalled not or: [ card isOld ]])
				ifTrue: [ sm installPackageWithId: anIdString ]
		].
		newCS isEmpty ifTrue: [ ChangeSet removeChangeSet: newCS ]
	].! !

!SARInstaller class methodsFor: 'change set utilities' stamp: 'yo 8/17/2004 10:04'!
fileIntoChangeSetNamed: aString fromStream: stream 
	"We let the user confirm filing into an existing ChangeSet
	or specify another ChangeSet name if
	the name derived from the filename already exists.
	Duplicated from SMSimpleInstaller.
	Should be a class-side method."

	^self withCurrentChangeSetNamed: aString
		do: [ :cs | | newName |
			newName := cs name.
			stream setConverterForCode.
			stream 
				fileInAnnouncing: 'Loading ' , newName , ' into change set ''' , newName, ''''.
			stream close]! !

!SARInstaller class methodsFor: 'initialization' stamp: 'nk 11/13/2002 07:33'!
fileReaderServicesForFile: fullName suffix: suffix 

	^(suffix = 'sar') | (suffix = '*') 
		ifTrue: [Array with: self serviceFileInSAR]
		ifFalse: [#()]
! !

!SARInstaller class methodsFor: 'initialization' stamp: 'nk 7/5/2003 22:22'!
initialize
	"SARInstaller initialize"
	(FileList respondsTo: #registerFileReader:)
		ifTrue: [ FileList registerFileReader: self ]! !

!SARInstaller class methodsFor: 'initialization' stamp: 'nk 7/5/2003 21:05'!
installSAR: relativeOrFullName
	FileDirectory splitName: (FileDirectory default fullNameFor: relativeOrFullName)
		to: [ :dir :fileName | (self directory: (FileDirectory on: dir) fileName: fileName) fileIn ]! !

!SARInstaller class methodsFor: 'package format support' stamp: 'nk 7/25/2003 14:05'!
loadDVS
	"Load the DVS support from SqueakMap"

	self ensurePackageWithId: '100d59d0-bf81-4e74-a4fe-5a2fd0c6b4ec'! !

!SARInstaller class methodsFor: 'package format support' stamp: 'nk 9/9/2003 12:08'!
loadMonticello
	"Load Monticello support (MCInstaller and Monticello) from SqueakMap"

	self ensurePackageWithId: 'af9d090d-2896-4a4e-82d0-c61cf2fdf40e'.
	self ensurePackageWithId: '66236497-7026-45f5-bcf6-ad00ba7a8a4e'.! !

!SARInstaller class methodsFor: 'package format support' stamp: 'nk 7/25/2003 14:39'!
loadMonticelloCVS
	"Load MonticelloCVS support from SqueakMap"

	self ensurePackageWithId: '2be9f7e2-1de2-4eb6-89bd-ec9b60593a93'.
! !

!SARInstaller class methodsFor: 'squeakmap' stamp: 'nk 7/21/2003 17:17'!
newCardForSqueakMap: aSqueakMap
	"Answer a new card."

	^(aSqueakMap newCardWithId: self squeakMapPackageID)
	created: 3236292323
	updated:3236292323
	name: 'SARInstaller for 3.6'
	currentVersion:'16'
	summary: 'Lets you load SAR (Squeak ARchive) files from SqueakMap and the File List. For 3.6 and later images.'
	description:'Support for installing SAR (Squeak ARchive) packages from SqueakMap and the File List.
For 3.6 and later images.

SMSARInstaller will use this if it''s present to load SAR packages.

Use SARBuilder for making these packages easily.'
	url: 'http://bike-nomad.com/squeak/'
	downloadUrl:'http://bike-nomad.com/squeak/SARInstallerFor36-nk.16.cs.gz'
	author: 'Ned Konz <ned@bike-nomad.com>'
	maintainer:'Ned Konz <ned@bike-nomad.com>'
	registrator:'Ned Konz <ned@bike-nomad.com>'
	password:240495131608326995113451940367316491071470713347
	categories: #('6ba57b6e-946a-4009-beaa-0ac93c08c5d1' '94277ca9-4d8f-4f0e-a0cb-57f4b48f1c8a' 'a71a6233-c7a5-4146-b5e3-30f28e4d3f6b' '8209da9b-8d6e-40dd-b23a-eb7e05d4677b' );
	modulePath: ''
	moduleVersion:''
	moduleTag:''
	versionComment:'v16: same as v16 of SARInstaller for 3.4 but doesn''t include any classes other than SARInstaller.

To be loaded into 3.6 images only. Will de-register the 3.4 version if it''s registered.

Added a default (DWIM) mode in which SAR files that are missing both a preamble and postscript have all their members loaded in a default manner.

Changed the behavior of #extractMemberWithoutPath: to use the same directory as the SAR itself.

Added #extractMemberWithoutPath:inDirectory:

Moved several change set methods to the class side.

Made change set methods work with 3.5 or 3.6a/b

Now supports the following file types:

Projects (with or without construction of a ViewMorph)
Genie gesture dictionaries
Change sets
DVS packages
Monticello packages
Graphics files (loaded as SketchMorphs)
Text files (loaded as text editor windows)
Morph(s) in files

Now keeps track of installed members.'! !

!SARInstaller class methodsFor: 'change set utilities' stamp: 'nk 7/5/2003 22:51'!
newChanges: aChangeSet
	"Change the current change set, in a way that should work in 3.5 as well"
	"SARInstaller newChanges: SARInstaller currentChangeSet"

	^[ ChangeSet newChanges: aChangeSet ]
		on: MessageNotUnderstood
		do: [ :ex | ex return: (Smalltalk newChanges: aChangeSet) ]! !

!SARInstaller class methodsFor: 'initialization' stamp: 'nk 11/13/2002 07:35'!
serviceFileInSAR
	"Answer a service for opening a changelist browser on a file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'install SAR'
		selector: #installSAR:
		description: 'install this Squeak ARchive into the image.'
		buttonLabel: 'install'! !

!SARInstaller class methodsFor: 'initialization' stamp: 'nk 11/21/2002 09:46'!
services
	^Array with: self serviceFileInSAR
! !

!SARInstaller class methodsFor: 'package format support' stamp: 'nk 7/25/2003 08:27'!
squeakMapDo: aBlock
	"If SqueakMap is installed, evaluate aBlock with the default map.
	Otherwise, offer to install SqueakMap and continue."

	Smalltalk at: #SMSqueakMap ifPresent: [ :smClass | ^aBlock value: smClass default ].

	(self confirm: 'SqueakMap is not installed in this image.
Would you like to load it from the network?')
		ifTrue: [ TheWorldMenu loadSqueakMap.
			^self squeakMapDo: aBlock ].

	^nil! !

!SARInstaller class methodsFor: 'squeakmap' stamp: 'nk 7/21/2003 17:16'!
squeakMapPackageID
	^'75c970ab-dca7-48ee-af42-5a013912c880'! !

!SARInstaller class methodsFor: 'squeakmap' stamp: 'nk 7/21/2003 17:18'!
squeakMapPackageVersion
	^'16'! !

!SARInstaller class methodsFor: 'initialization' stamp: 'nk 7/5/2003 22:22'!
unload

	(FileList respondsTo: #unregisterFileReader:)
		ifTrue: [ FileList unregisterFileReader: self ]! !

!SARInstaller class methodsFor: 'change set utilities' stamp: 'DamienCassou 9/29/2009 13:08'!
withCurrentChangeSetNamed: aString do: aOneArgumentBlock 
	"Evaluate the one-argument block aOneArgumentBlock while the named change set is active.
	We let the user confirm operating on an existing ChangeSet 
	or specify another ChangeSet name if 
	the name derived from the filename already exists. 
	Duplicated from SMSimpleInstaller. 
	Returns change set."

	| changeSet newName oldChanges |
	newName := aString.
	changeSet := self changeSetNamed: newName.
	changeSet ifNotNil: 
			[newName := UIManager default 
						request: 'ChangeSet already present, just confirm to overwrite or enter a new name:'
						initialAnswer: newName.
			newName isEmptyOrNil ifTrue: [self error: 'Cancelled by user'].
			changeSet := self changeSetNamed: newName].
	changeSet ifNil: [changeSet := self basicNewChangeSet: newName].
	changeSet 
		ifNil: [self error: 'User did not specify a valid ChangeSet name'].
	oldChanges := self currentChangeSet.
	
	[ self newChanges: changeSet.
	aOneArgumentBlock value: changeSet] 
			ensure: [ self newChanges: oldChanges].
	^changeSet! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 10/25/2002 12:16'!
directory
	^directory! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 10/25/2002 12:16'!
directory: anObject
	directory := anObject! !

!SARInstaller methodsFor: 'private' stamp: 'nk 10/13/2003 12:56'!
errorNoSuchMember: aMemberName
	(self confirm: 'No member named ', aMemberName, '. Do you want to stop loading?')
		== true ifTrue: [ self error: 'aborted' ].! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 7/5/2003 22:25'!
extractMember: aMemberOrName
	"Extract aMemberOrName to a file using its filename"
	(self zip extractMember: aMemberOrName)
		ifNil: [ self errorNoSuchMember: aMemberOrName ]
		ifNotNil: [ self installed: aMemberOrName ].! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 7/5/2003 22:25'!
extractMember: aMemberOrName toFileNamed: aFileName
	"Extract aMemberOrName to a specified filename"
	(self zip extractMember: aMemberOrName toFileNamed: aFileName)
		ifNil: [ self errorNoSuchMember: aMemberOrName ]
		ifNotNil: [ self installed: aMemberOrName ].! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 7/5/2003 22:40'!
extractMemberWithoutPath: aMemberOrName
	"Extract aMemberOrName to its own filename, but ignore any directory paths, using my directory instead."
	self extractMemberWithoutPath: aMemberOrName inDirectory: self directory.
! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 7/5/2003 22:40'!
extractMemberWithoutPath: aMemberOrName inDirectory: aDirectory
	"Extract aMemberOrName to its own filename, but ignore any directory paths, using aDirectory instead"
	| member |
	member := self memberNamed: aMemberOrName.
	member ifNil: [ ^self errorNoSuchMember: aMemberOrName ].
	self zip extractMemberWithoutPath: member inDirectory: aDirectory.
	self installed: member.! !

!SARInstaller methodsFor: 'filein' stamp: 'ar 9/27/2005 20:10'!
fileIn
	"File in to a change set named like my file"
	| stream newCS |
	stream := directory readOnlyFileNamed: fileName.
	self class withCurrentChangeSetNamed: fileName
		do: [:cs | newCS := cs. self fileInFrom: stream].
	newCS isEmpty ifTrue: [ ChangeSet removeChangeSet: newCS ]! !

!SARInstaller methodsFor: 'filein' stamp: 'yo 8/17/2004 00:33'!
fileInFrom: stream
	"The zip has been saved already by the download.
	Read the zip into my instvar, then file in the correct members"

	| preamble postscript |

	[
		stream position: 0.
		zip := ZipArchive new readFrom: stream.

		preamble := zip memberNamed: 'install/preamble'.
		preamble ifNotNil: [
			preamble contentStream text setConverterForCode fileInFor: self announcing: 'Preamble'.
			self class currentChangeSet preambleString: preamble contents.
		].

		postscript := zip memberNamed: 'install/postscript'.
		postscript ifNotNil: [
			postscript contentStream text setConverterForCode fileInFor: self announcing: 'Postscript'.
			self class currentChangeSet postscriptString: postscript contents.
		].

		preamble isNil & postscript isNil ifTrue: [
			(self confirm: 'No install/preamble or install/postscript member were found.
	Install all the members automatically?') ifTrue: [ self installAllMembers ]
		].

	] ensure: [ stream close ].

! !

!SARInstaller methodsFor: 'private' stamp: 'ar 9/27/2005 20:10'!
fileInMCVersion: member withBootstrap: mcBootstrap
	"This will use the MCBootstrapLoader to load a (non-compressed) Monticello file (.mc or .mcv)"
	| newCS |
	self class withCurrentChangeSetNamed: member localFileName
		do: [ :cs | 
			newCS := cs.
			mcBootstrap loadStream: member contentStream ascii ].

	newCS isEmpty ifTrue: [ ChangeSet removeChangeSet: newCS ].

	World doOneCycle.

	self installed: member.! !

!SARInstaller methodsFor: 'client services' stamp: 'yo 8/17/2004 10:01'!
fileInMemberNamed: csName
	"This is to be used from preamble/postscript code to file in zip members as ChangeSets."
	| cs |
	cs := self memberNamed: csName.
	cs ifNil: [ ^self errorNoSuchMember: csName ].
	self class fileIntoChangeSetNamed: csName fromStream: cs contentStream text setConverterForCode.
	self installed: cs.
! !

!SARInstaller methodsFor: 'client services' stamp: 'ar 9/27/2005 20:10'!
fileInMonticelloPackageNamed: memberName 
	"This is to be used from preamble/postscript code to file in zip 
	members as Monticello packages (.mc)."

	| member file mcPackagePanel mcRevisionInfo mcSnapshot mcFilePackageManager mcPackage info snapshot newCS mcBootstrap |

	mcPackagePanel := Smalltalk at: #MCPackagePanel ifAbsent: [ ].
	mcRevisionInfo := Smalltalk at: #MCRevisionInfo ifAbsent: [ ].
	mcSnapshot := Smalltalk at: #MCSnapshot ifAbsent: [ ].
	mcFilePackageManager := Smalltalk at: #MCFilePackageManager ifAbsent: [ ].
	mcPackage := Smalltalk at: #MCPackage ifAbsent: [ ].
	member := self memberNamed: memberName.
	member ifNil: [ ^self errorNoSuchMember: memberName ].

	"We are missing MCInstaller, Monticello and/or MonticelloCVS.
	If the bootstrap is present, use it. Otherwise interact with the user."
	({ mcPackagePanel. mcRevisionInfo. mcSnapshot. mcFilePackageManager. mcPackage } includes: nil)
		ifTrue: [
			mcBootstrap := self getMCBootstrapLoaderClass.
			mcBootstrap ifNotNil: [ ^self fileInMCVersion: member withBootstrap: mcBootstrap ].

			(self confirm: ('Monticello support is not installed, but must be to load member named ', memberName, '.
Load it from SqueakMap?'))
				ifTrue: [ self class loadMonticello; loadMonticelloCVS.
					^self fileInMonticelloPackageNamed: memberName ]
				ifFalse: [ ^false ] ].

	member extractToFileNamed: member localFileName inDirectory: self directory.
	file := (Smalltalk at: #MCFile)
				name: member localFileName
				directory: self directory.

	self class withCurrentChangeSetNamed: file name do: [ :cs |
		newCS := cs.
		file readStreamDo: [ :stream |
			info := mcRevisionInfo readFrom: stream nextChunk.
			snapshot := mcSnapshot fromStream: stream ].
			snapshot install.
			(mcFilePackageManager forPackage:
				(mcPackage named: info packageName))
					file: file
		].

	newCS isEmpty ifTrue: [ ChangeSet removeChangeSet: newCS ].

	mcPackagePanel allSubInstancesDo: [ :ea | ea refresh ].
	World doOneCycle.

	self installed: member.
! !

!SARInstaller methodsFor: 'client services' stamp: 'ar 9/27/2005 20:10'!
fileInMonticelloVersionNamed: memberName 
	"This is to be used from preamble/postscript code to file in zip 
	members as Monticello version (.mcv) files."

	| member newCS mcMcvReader |
	mcMcvReader := Smalltalk at: #MCMcvReader ifAbsent: [].
	member := self memberNamed: memberName.
	member ifNil: [^self errorNoSuchMember: memberName].

	"If we don't have Monticello, offer to get it."
	mcMcvReader ifNil:  [
		(self confirm: 'Monticello is not installed, but must be to load member named ', memberName , '.
Load it from SqueakMap?') 
			ifTrue:  [ self class loadMonticello.
						^self fileInMonticelloVersionNamed: memberName]
					ifFalse: [^false]].

	self class withCurrentChangeSetNamed: member localFileName
		do: 
			[:cs | 
			newCS := cs.
			(mcMcvReader versionFromStream: member contentStream ascii) load ].
	newCS isEmpty ifTrue: [ChangeSet removeChangeSet: newCS].
	World doOneCycle.
	self installed: member! !

!SARInstaller methodsFor: 'client services' stamp: 'ar 9/27/2005 20:10'!
fileInMonticelloZipVersionNamed: memberName 
	"This is to be used from preamble/postscript code to file in zip 
	members as Monticello version (.mcz) files."

	| member mczInstaller newCS mcMczReader |
	mcMczReader := Smalltalk at: #MCMczReader ifAbsent: [].
	mczInstaller := Smalltalk at: #MczInstaller ifAbsent: [].
	member := self memberNamed: memberName.
	member ifNil: [^self errorNoSuchMember: memberName].

	"If we don't have Monticello, but have the bootstrap, use it silently."
	mcMczReader ifNil:  [
		mczInstaller ifNotNil: [ ^mczInstaller installStream: member contentStream ].
		(self confirm: 'Monticello is not installed, but must be to load member named ', memberName , '.
Load it from SqueakMap?') 
			ifTrue:  [ self class loadMonticello.
						^self fileInMonticelloZipVersionNamed: memberName]
					ifFalse: [^false]].

	self class withCurrentChangeSetNamed: member localFileName
		do: 
			[:cs | 
			newCS := cs.
			(mcMczReader versionFromStream: member contentStream) load ].
	newCS isEmpty ifTrue: [ChangeSet removeChangeSet: newCS].
	World doOneCycle.
	self installed: member! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 7/5/2003 22:27'!
fileInMorphsNamed: memberName addToWorld: aBoolean
	"This will load the Morph (or Morphs) from the given member.
	Answers a Morph, or a list of Morphs, or nil if no such member or error.
	If aBoolean is true, also adds them and their models to the World."

	| member morphOrList |
	member := self memberNamed: memberName.
	member ifNil: [ ^self errorNoSuchMember: memberName ].
	self installed: member.

	morphOrList := member contentStream fileInObjectAndCode.
	morphOrList ifNil: [ ^nil ].
	aBoolean ifTrue: [ ActiveWorld addMorphsAndModel: morphOrList ].

	^morphOrList
! !

!SARInstaller methodsFor: 'client services' stamp: 'yo 8/17/2004 10:05'!
fileInPackageNamed: memberName 
	"This is to be used from preamble/postscript code to file in zip 
	members as DVS packages."
	| member current new baseName imagePackageLoader packageInfo streamPackageLoader packageManager |
	member := self zip memberNamed: memberName.
	member ifNil: [ ^self errorNoSuchMember: memberName ].

	imagePackageLoader := Smalltalk at: #ImagePackageLoader ifAbsent: [].
	streamPackageLoader := Smalltalk at: #StreamPackageLoader ifAbsent: [].
	packageInfo := Smalltalk at: #PackageInfo ifAbsent: [].
	packageManager := Smalltalk at: #FilePackageManager ifAbsent: [].

	"If DVS isn't present, do a simple file-in"
	(packageInfo isNil or: [imagePackageLoader isNil or: [streamPackageLoader isNil]])
		ifTrue: [ ^ self fileInMemberNamed: memberName ].

	baseName := memberName copyReplaceAll: '.st' with: '' asTokens: false.
	(packageManager allManagers anySatisfy: [ :pm | pm packageName = baseName ])
		ifTrue: [
			current := imagePackageLoader new package: (packageInfo named: baseName).
			new := streamPackageLoader new stream: member contentStream ascii.
			(new changesFromBase: current) fileIn ]
		ifFalse: [ self class fileIntoChangeSetNamed: baseName fromStream: member contentStream ascii setConverterForCode. ].

	packageManager registerPackage: baseName.

	self installed: member.! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 9/26/2003 17:17'!
fileInTrueTypeFontNamed: memberOrName

	| member description |
	member := self memberNamed: memberOrName.
	member ifNil: [^self errorNoSuchMember: memberOrName].

	description := TTFontDescription addFromTTStream: member contentStream.
	TTCFont newTextStyleFromTT: description.

	World doOneCycle.
	self installed: member! !

!SARInstaller methodsFor: 'filein' stamp: 'nk 7/27/2003 14:02'!
fileIntoChangeSetNamed: aString fromStream: stream
	"Not recommended for new code"
	^self class fileIntoChangeSetNamed: aString fromStream: stream! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 10/25/2002 12:16'!
fileName
	^fileName! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 10/25/2002 12:16'!
fileName: anObject
	fileName := anObject! !

!SARInstaller methodsFor: 'client services' stamp: 'marcus.denker 11/10/2008 10:04'!
getMCBootstrapLoaderClass
	^Smalltalk at: #MCBootstrapLoader
		ifAbsent: 
			[(self memberNamed: 'MCBootstrapLoader.st') 
				ifNotNil: [:m | self fileInMemberNamed: m.
					Smalltalk at: #MCBootstrapLoader ifAbsent: []]]! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 10/14/2003 15:40'!
importImage: memberOrName
	| member form |
	member := self memberNamed: memberOrName.
	member ifNil: [ ^self errorNoSuchMember: memberOrName ].
	form := ImageReadWriter formFromStream: member contentStream binary.
	form ifNil: [ ^self ].
	Imports default importImage: form named: (FileDirectory localNameFor: member fileName) sansPeriodSuffix.
	self installed: member.! !

!SARInstaller methodsFor: 'initialization' stamp: 'alain.plantec 5/28/2009 10:19'!
initialize
	super initialize.
	installed := OrderedCollection new.! !

!SARInstaller methodsFor: 'filein' stamp: 'nk 10/12/2003 20:41'!
installAllMembers
	"Try to install all the members, in order, based on their filenames and/or contents."
	| uninstalled |
	uninstalled := OrderedCollection new.
	zip members do: [ :member | self installMember: member ].
	uninstalled := self uninstalledMembers.
	uninstalled isEmpty ifTrue: [ ^self ].
	uninstalled inspect.! !

!SARInstaller methodsFor: 'client services' stamp: 'stephane.ducasse 7/10/2009 17:30'!
installMember: memberOrName
	| memberName extension isGraphic stream member |
	member := self memberNamed: memberOrName.
	member ifNil: [ ^false ].
	memberName := member fileName.
	extension := (FileDirectory extensionFor: memberName) asLowercase.
	extension caseOf: {
		[ FileStream st ] -> [ self fileInPackageNamed: memberName ].
		[ FileStream cs ] -> [  self fileInMemberNamed: memberName  ].
"		[ FileStream multiSt ] -> [  self fileInMemberNamedAsUTF8: memberName  ].
		[ FileStream multiCs ] -> [  self fileInMemberNamedAsUTF8: memberName  ].
"
		[ 'mc' ] -> [ self fileInMonticelloPackageNamed: memberName ].
		[ 'mcv' ] -> [ self fileInMonticelloVersionNamed: memberName ].
		[ 'mcz' ] -> [ self fileInMonticelloZipVersionNamed: memberName ].
		[ 'morph' ] -> [ self fileInMorphsNamed: member addToWorld: true ].
		[ 'ttf' ] -> [ self fileInTrueTypeFontNamed: memberName ].
		[ 'translation' ] -> [  self fileInMemberNamed: memberName  ].
	} otherwise: [
		('t*xt' match: extension) ifTrue: [ self openTextFile: memberName ]
			ifFalse: [ stream := member contentStream.
		isGraphic := ImageReadWriter understandsImageFormat: stream.
		stream reset.
		isGraphic
			ifTrue: [ self openGraphicsFile: member ]
			ifFalse: [ "now what?" ^false ]]
	].
	^true
! !

!SARInstaller methodsFor: 'private' stamp: 'nk 7/10/2003 16:55'!
installed: aMemberOrName
	self installedMembers add: (self zip member: aMemberOrName)! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 7/5/2003 23:01'!
installedMemberNames
	"Answer the names of the zip members that have been installed already."
	^self installedMembers collect: [ :ea | ea fileName ]! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 7/10/2003 16:53'!
installedMembers
	"Answer the zip members that have been installed already."
	^installed ifNil: [ installed := OrderedCollection new ]! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 7/5/2003 10:02'!
memberNameForProjectNamed: projectName
	"Answer my member name for the given project, or nil.
	Ignores version numbers and suffixes, and also unescapes percents in filenames."

	^self zip memberNames detect: [ :memberName | | triple |
		triple := Project parseProjectFileName: memberName unescapePercents.
		triple first asLowercase = projectName asLowercase
	] ifNone: [ nil ].! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 10/14/2003 18:58'!
memberNamed: aString
	^(zip member: aString)
		ifNil: [ | matching |
			matching := zip membersMatching: aString.
			matching isEmpty ifFalse: [ matching last ]].! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 7/5/2003 21:57'!
memberNames
	^self zip memberNames! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 10/27/2002 10:34'!
membersMatching: aString
	^self zip membersMatching: aString! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 6/12/2004 10:03'!
openGraphicsFile: memberOrName
	| member morph |
	member := self memberNamed: memberOrName.
	member ifNil: [ ^self errorNoSuchMember: memberOrName ].
	morph := (World drawingClass fromStream: member contentStream binary).
	morph ifNotNil: [ morph openInWorld ].
	self installed: member.! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 7/5/2003 22:28'!
openTextFile: memberOrName
	"Open a text window on the given member"
	| member |
	member := self memberNamed: memberOrName.
	member ifNil: [ ^self errorNoSuchMember: memberOrName ].
	StringHolder new
		acceptContents: member contents;
		openLabel: member fileName.
	self installed: member.! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 10/27/2002 10:36'!
prependedDataSize
	^self zip prependedDataSize! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 7/5/2003 23:00'!
uninstalledMemberNames
	"Answer the names of the zip members that have not yet been installed."
	^self uninstalledMembers collect: [ :ea | ea fileName ]! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 7/10/2003 16:55'!
uninstalledMembers
	"Answer the zip members that haven't been installed or extracted yet."
	^zip members copyWithoutAll: self installedMembers! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 10/25/2002 12:16'!
zip
	^zip! !

!SARInstaller methodsFor: 'accessing' stamp: 'nk 10/25/2002 12:16'!
zip: anObject
	^zip := anObject! !

!SARInstaller methodsFor: 'client services' stamp: 'nk 10/27/2002 10:35'!
zipFileComment
	^self zip zipFileComment! !

Categorizer subclass: #SystemOrganizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!SystemOrganizer commentStamp: '<historical>' prior: 0!
My instances provide an organization for the classes in the system, just as a ClassOrganizer organizes the messages within a class. The only difference is the methods for fileIn/Out.!

!SystemOrganizer methodsFor: 'accessing' stamp: 'ab 2/10/2005 16:33'!
addCategory: newCategory
	| r |
	r := super addCategory: newCategory.
	SystemChangeNotifier uniqueInstance classCategoryAdded: newCategory.
	^ r! !

!SystemOrganizer methodsFor: 'remove' stamp: 'di 3/3/2001 16:07'!
categoriesMatching: matchString
	"Return all matching categories"
	^ self categories select: [:c | matchString match: c]! !

!SystemOrganizer methodsFor: 'accessing' stamp: 'Noury 10/26/2008 15:46'!
classesInCategory: category
	^(self listAtCategoryNamed: category) collect: [:className|
		Smalltalk at: className]! !

!SystemOrganizer methodsFor: 'query' stamp: 'dtl 8/26/2004 11:18'!
commentInventory: categoryName

	"SystemOrganization commentInventory: 'Morphic*'"

	| classes commentedClasses |
	classes := OrderedCollection new.
	self categories withIndexCollect: [:cat :idx |
		(categoryName match: cat)
			ifTrue: [classes addAll: (self listAtCategoryNumber: idx)]
			ifFalse: [nil]].
	commentedClasses := classes select: [:catCls | (Smalltalk at: catCls) hasComment].
	^ 'There are ' , classes size asString , ' classes in ' , categoryName ,
		' of which ' , commentedClasses size asString , ' have comments and ',
		(classes size - commentedClasses size) asString , ' do not yet have comments.'
! !

!SystemOrganizer methodsFor: 'filein/out' stamp: 'PeterHugossonMiller 9/3/2009 11:32'!
fileOut  
	"SystemOrganization fileOut"

	| internalStream |
	internalStream := (String new: 30000) writeStream.
	internalStream nextPutAll: 'SystemOrganization changeFromCategorySpecs: #('; cr;
		print: SystemOrganization;  "ends with a cr"
		nextPutAll: ')!!'; cr.

	FileStream writeSourceCodeFrom: internalStream baseName: (FileDirectory default nextNameFor: 'SystemOrganization' extension: 'st') isSt: true
! !

!SystemOrganizer methodsFor: 'filein/out' stamp: 'PeterHugossonMiller 9/3/2009 11:32'!
fileOutCategory: category 
	"Store on the file named category (a string) concatenated with '.st' all the 
	classes associated with the category."
	
	| internalStream |
	internalStream := (String new: 1000) writeStream.
	self fileOutCategory: category on: internalStream initializing: true.
	^ FileStream writeSourceCodeFrom: internalStream baseName: category isSt: true ! !

!SystemOrganizer methodsFor: 'filein/out' stamp: 'ar 12/22/1999 17:28'!
fileOutCategory: category on: aFileStream 
	"Store on the file associated with aFileStream, all the classes associated 
	with the category and any requested shared pools."
	^self fileOutCategory: category on: aFileStream initializing: true! !

!SystemOrganizer methodsFor: 'filein/out' stamp: 'al 4/25/2004 11:41'!
fileOutCategory: category on: aFileStream initializing: aBool
	"Store on the file associated with aFileStream, all the traits and classes associated 
	with the category and any requested shared pools in the right order."

	| first poolSet tempClass classes traits |
	traits := self orderedTraitsIn: category.
	classes := self superclassOrder: category.
	poolSet := Set new.
	classes do:  [:class | class sharedPools do: [:eachPool | poolSet add: eachPool]].
	poolSet size > 0 ifTrue: [
		tempClass := Class new.
		tempClass shouldFileOutPools ifTrue: [
			poolSet := poolSet select: [:aPool |
				tempClass shouldFileOutPool: (Smalltalk keyAtIdentityValue: aPool)].
			poolSet do: [:aPool | tempClass fileOutPool: aPool onFileStream: aFileStream]]].
	first := true.
	traits, classes do: [:each | 
		first
			ifTrue: [first := false]
			ifFalse: [aFileStream cr; nextPut: Character newPage; cr].
		each
			fileOutOn: aFileStream
			moveSource: false
			toFile: 0
			initializing: false].
	aBool ifTrue: [classes do: [:cls | cls fileOutInitializerOn: aFileStream]].! !

!SystemOrganizer methodsFor: 'private' stamp: 'rw 7/31/2003 17:23'!
ifClassOrganizerDo: aBlock
	"Do nothing, since this is not a class organizer"! !

!SystemOrganizer methodsFor: 'filein/out' stamp: 'tk 9/28/2000 15:50'!
objectForDataStream: refStrm
	| dp |
	"I am about to be written on an object file.  Write a path to me in the other system instead."

self == SystemOrganization ifTrue: [
	dp := DiskProxy global: #SystemOrganization selector: #yourself args: #().
	refStrm replace: self with: dp.
	^ dp].
^ self
! !

!SystemOrganizer methodsFor: 'filein/out' stamp: 'dvf 9/27/2005 19:06'!
orderedTraitsIn: category 
	"Answer an OrderedCollection containing references to the traits in the 
	category whose name is the argument, category (a string). The traits 
	are ordered so they can be filed in."

	| behaviors traits |
	behaviors := (self listAtCategoryNamed: category asSymbol) 
			collect: [:title | Smalltalk at: title].
	traits := behaviors reject: [:each | each isBehavior].
	traits := traits asSortedCollection: [:t1 :t2 |
		(t2 traitComposition allTraits includes: t1)
			or: [(t1 traitComposition allTraits includes: t2) not]].
	^traits asArray! !

!SystemOrganizer methodsFor: 'remove' stamp: 'di 3/3/2001 16:08'!
removeCategoriesMatching: matchString
	"Remove all matching categories with their classes"
	(self categoriesMatching: matchString) do:
		[:c | self removeSystemCategory: c]! !

!SystemOrganizer methodsFor: 'accessing' stamp: 'ab 2/10/2005 16:36'!
removeCategory: cat 
	| r |
	r := super removeCategory: cat.
	SystemChangeNotifier uniqueInstance classCategoryRemoved: cat.
	^ r! !

!SystemOrganizer methodsFor: 'remove' stamp: 'jm 5/20/1998 19:38'!
removeMissingClasses
	"Remove any class names that are no longer in the Smalltalk dictionary. Used for cleaning up after garbage collecting user-generated classes."
	"SystemOrganization removeMissingClasses"

	elementArray copy do: [:el |
		(Smalltalk includesKey: el) ifFalse: [self removeElement: el]].
! !

!SystemOrganizer methodsFor: 'remove' stamp: 'al 4/25/2004 11:38'!
removeSystemCategory: category
	"remove all the classes and traits associated with the category"

	 (self orderedTraitsIn: category) , (self superclassOrder: category)
		reverseDo: [:each | each removeFromSystem].

	self removeCategory: category.
! !

!SystemOrganizer methodsFor: 'accessing' stamp: 'ab 2/10/2005 16:47'!
renameCategory: oldCatString toBe: newCatString
	| r |
	r := super renameCategory: oldCatString toBe: newCatString.
	SystemChangeNotifier uniqueInstance 
		classCategoryRenamedFrom: oldCatString to: newCatString.
	^ r! !

!SystemOrganizer methodsFor: 'filein/out' stamp: 'dvf 9/27/2005 19:06'!
superclassOrder: category 
	"Answer an OrderedCollection containing references to the classes in the 
	category whose name is the argument, category (a string). The classes 
	are ordered with superclasses first so they can be filed in."

	| behaviors classes |
	behaviors := (self listAtCategoryNamed: category asSymbol) 
			collect: [:title | Smalltalk at: title].
	classes := behaviors select: [:each | each isBehavior].
	^ChangeSet superclassOrder: classes! !

!SystemOrganizer methodsFor: 'query' stamp: 'dtl 8/26/2004 11:23'!
uncommentedClassesIn: categoryName

	"SystemOrganization uncommentedClassesIn: 'Morphic*'"

	| classes |
	classes := OrderedCollection new.
	self categories withIndexCollect: [:cat :idx |
		(categoryName match: cat)
			ifTrue: [classes addAll: (self listAtCategoryNumber: idx)]
			ifFalse: [nil]].
	^ (classes collect: [:clsName | Smalltalk at: clsName]
		thenSelect: [:cls | cls hasComment not]) asArray
! !

!Behavior methodsFor: '*system-support' stamp: 'tpr 12/17/2003 16:04'!
allCallsOn
	"Answer a SortedCollection of all the methods that refer to me by name or as part of an association in a global dict."


	^ (self  systemNavigation allCallsOn:  (self environment associationAt: self theNonMetaClass name)), (self  systemNavigation allCallsOn:  self theNonMetaClass name)	! !

!Behavior methodsFor: '*system-support' stamp: 'dvf 8/23/2003 12:43'!
allCallsOn: aSymbol
	"Answer a SortedCollection of all the methods that call on aSymbol."


	^ self  systemNavigation allCallsOn: aSymbol from: self .
	! !

!Behavior methodsFor: '*system-support' stamp: 'stephane.ducasse 10/12/2008 21:02'!
allUnsentMessages
	"Answer an array of all the messages defined by the receiver that are not sent anywhere in the system."

	^ SystemNavigation default allUnsentMessagesIn: self selectors! !

IdentityDictionary subclass: #SystemDictionary
	instanceVariableNames: 'cachedClassNames'
	classVariableNames: 'LastImageName LastQuitLogPosition LowSpaceProcess LowSpaceSemaphore MemoryHogs ShutDownList SpecialSelectors StartUpList StartupStamp SystemChanges'
	poolDictionaries: ''
	category: 'System-Support'!

!SystemDictionary commentStamp: '<historical>' prior: 0!
I represent a special dictionary that supports protocol for asking questions about the structure of the system. Other than class names, I contain (print this)...
	Smalltalk keys select: [:k | ((Smalltalk at: k) isKindOf: Class) not]
			thenCollect: [:k | k -> (Smalltalk at: k) class]
!

!SystemDictionary class methodsFor: 'initialization' stamp: 'mir 11/20/2008 16:43'!
initialize
	"SystemDictionary initialize"

	| oldList |
	oldList := StartUpList.
	StartUpList := OrderedCollection new.
	"These get processed from the top down..."
	#(
		Delay
		DisplayScreen
		Cursor
		InputEventFetcher
		ProcessorScheduler  "Starts low space watcher and bkground."
		LanguageEnvironment
		FileDirectory  "Enables file stack dump and opens sources."
		NaturalLanguageTranslator
		ShortIntegerArray
		ShortRunArray
		CrLfFileStream
	) do:[:clsName|
		Smalltalk at: clsName ifPresent:[:cls| Smalltalk addToStartUpList: cls].
	].
	oldList ifNotNil: [oldList do: [:className | Smalltalk at: className
						ifPresent: [:theClass | Smalltalk addToStartUpList: theClass]]].
	#(
		ImageSegment
		PasteUpMorph
		"ControlManager"
	) do:[:clsName|
		Smalltalk at: clsName ifPresent:[:cls| Smalltalk addToStartUpList: cls].
	].
		

	oldList := ShutDownList.
	ShutDownList := OrderedCollection new.
	"These get processed from the bottom up..."
	#(
		Delay
		DisplayScreen
		InputEventFetcher
		Form
		"ControlManager"
		PasteUpMorph
		StrikeFont
		Color
		FileDirectory
		SoundPlayer
		HttpUrl
		Password
		PWS
		MailDB
		ImageSegment
	) do:[:clsName|
		Smalltalk at: clsName ifPresent:[:cls| Smalltalk addToShutDownList: cls].
	].

	oldList ifNotNil: [oldList reverseDo: [:className | Smalltalk at: className
						ifPresent: [:theClass | Smalltalk addToShutDownList: theClass]]].
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'eem 7/1/2009 14:00'!
abandonSources
	"Smalltalk abandonSources"
	"Replaces every method by a copy with the 4-byte source pointer 
	 replaced by a string of all arg and temp names, followed by its
	 length. These names can then be used to inform the decompiler."
	"wod 11/3/1998: zap the organization before rather than after
	 condensing changes."
	"eem 7/1/2009 13:59 update for the closure schematic temp names regime"
	| oldMethods newMethods bTotal bCount |
	(self confirm: 'This method will preserve most temp names
(up to about 15k characters of temporaries)
while allowing the sources file to be discarded.
-- CAUTION --
If you have backed up your system and
are prepared to face the consequences of
abandoning source code files, choose Yes.
If you have any doubts, you may choose No
to back out with no harm done.')
			== true
		ifFalse: [^ self inform: 'Okay - no harm done'].
	self forgetDoIts.
	oldMethods := OrderedCollection new: CompiledMethod instanceCount.
	newMethods := OrderedCollection new: CompiledMethod instanceCount.
	bTotal := 0.
	bCount := 0.
	self systemNavigation allBehaviorsDo: [:b | bTotal := bTotal + 1].
	'Saving temp names for better decompilation...'
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: bTotal
		during:
			[:bar |
			self systemNavigation allBehaviorsDo:
				[:cl |  "for test: (Array with: Arc with: Arc class) do:"
				bar value: (bCount := bCount + 1).
				cl selectors do:
					[:selector | | m oldCodeString methodNode |
					m := cl compiledMethodAt: selector.
					m fileIndex > 0 ifTrue:
						[oldCodeString := cl sourceCodeAt: selector.
						methodNode := cl compilerClass new
											parse: oldCodeString
											in: cl
											notifying: nil.
						oldMethods addLast: m.
						newMethods addLast: (m copyWithTempsFromMethodNode: methodNode)]]]].
	oldMethods asArray elementsExchangeIdentityWith: newMethods asArray.
	self systemNavigation allBehaviorsDo: [:b | b zapOrganization].
	self condenseChanges.
	Preferences disable: #warnIfNoSourcesFile! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'sd 9/29/2004 18:18'!
abandonTempNames
	"Replaces every method by a copy with no source pointer or
	encoded temp names."
	"Smalltalk abandonTempNames"
	| continue oldMethods newMethods n m |
	continue := self confirm: '-- CAUTION --
If you have backed up your system and
are prepared to face the consequences of
abandoning all source code, hit Yes.
If you have any doubts, hit No,
to back out with no harm done.'.
	continue
		ifFalse: [^ self inform: 'Okay - no harm done'].
	self forgetDoIts; garbageCollect.
	oldMethods := OrderedCollection new.
	newMethods := OrderedCollection new.
	n := 0.
	'Removing temp names to save space...'
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: CompiledMethod instanceCount
		during: [:bar | self systemNavigation
				allBehaviorsDo: [:cl | cl selectors
						do: [:sel | 
							bar value: (n := n + 1).
							m := cl compiledMethodAt: sel.
							oldMethods addLast: m.
							newMethods
								addLast: (m copyWithTrailerBytes: #(0 ))]]].
	oldMethods asArray elementsExchangeIdentityWith: newMethods asArray.
	SmalltalkImage current closeSourceFiles.
	self flag: #shouldUseAEnsureBlockToBeSureThatTheFileIsClosed.
	"sd: 17 April 2003"
	Preferences disable: #warnIfNoChangesFile.
	Preferences disable: #warnIfNoSourcesFile! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'ar 11/19/1999 22:36'!
add: aClass toList: startUpOrShutDownList after: predecessor
	"Add the name of aClass to the startUp or shutDown list.
	Add it after the name of predecessor, or at the end if predecessor is nil."

	| name earlierName |
	name := aClass name.
	(self at: name ifAbsent: [nil]) == aClass ifFalse:
		[self error: name , ' cannot be found in Smalltalk dictionary.'].
	predecessor == nil
		ifTrue: ["No-op if alredy in the list."
				(startUpOrShutDownList includes: name) ifFalse:
					[startUpOrShutDownList == StartUpList
						ifTrue: ["Add to end of startUp list"
								startUpOrShutDownList addLast: name]
						ifFalse: ["Add to front of shutDown list"
								startUpOrShutDownList addFirst: name]]]
		ifFalse: ["Add after predecessor, moving it if already there."
				earlierName := predecessor name.
				(self at: earlierName) == predecessor ifFalse:
					[self error: earlierName , ' cannot be found in Smalltalk dictionary.'].
				(startUpOrShutDownList includes: earlierName) ifFalse:
					[self error: earlierName , ' cannot be found in the list.'].
				startUpOrShutDownList remove: name ifAbsent:[].
				startUpOrShutDownList add: name after: earlierName]! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'di 2/4/1999 15:38'!
addToShutDownList: aClass
	"This will add a ref to this class at the BEGINNING of the shutDown list."

	self addToShutDownList: aClass after: nil! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'di 2/3/1999 22:04'!
addToShutDownList: aClass after: predecessor

	self add: aClass toList: ShutDownList after: predecessor! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'di 2/4/1999 15:37'!
addToStartUpList: aClass
	"This will add a ref to this class at the END of the startUp list."

	self addToStartUpList: aClass after: nil! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'di 2/3/1999 22:04'!
addToStartUpList: aClass after: predecessor

	self add: aClass toList: StartUpList after: predecessor! !

!SystemDictionary methodsFor: 'retrieving' stamp: 'sd 4/17/2003 21:15'!
allClasses  
	"Return all the class defines in the Smalltalk SystemDictionary"
	"Smalltalk allClasses"

	^ self classNames collect: [:name | self at: name]! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'al 1/12/2006 23:59'!
allClassesAndTraits
	"Return all the classes and traits defined in the Smalltalk SystemDictionary"

	^ self classNames , self traitNames collect: [:each | self at: each]! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'al 1/13/2006 00:15'!
allClassesAndTraitsDo: aBlock
	^self allClassesAndTraits do: aBlock! !

!SystemDictionary methodsFor: 'retrieving' stamp: 'sd 4/17/2003 21:18'!
allClassesDo: aBlock
	"Evaluate the argument, aBlock, for each class in the system."

	(self classNames collect: [:name | self at: name]) do: aBlock! !

!SystemDictionary methodsFor: 'retrieving' stamp: 'al 2/23/2006 21:39'!
allTraits
	"Return all traits defined in the Smalltalk SystemDictionary"

	^ self traitNames collect: [:each | self at: each]! !

!SystemDictionary methodsFor: 'dictionary access' stamp: 'md 3/2/2006 22:01'!
associationOrUndeclaredAt: key 
	"return an association or install in undeclared.  Used for mating up ImageSegments."

	^ self associationAt: key ifAbsent: [
		Undeclared at: key put: nil.
		Undeclared associationAt: key]! !

!SystemDictionary methodsFor: 'dictionary access' stamp: ''!
at: aKey put: anObject 
	"Override from Dictionary to check Undeclared and fix up
	references to undeclared variables."
	| index element |
	(self includesKey: aKey) ifFalse: 
		[self declare: aKey from: Undeclared.
		self flushClassNameCache].
	super at: aKey put: anObject.
	^ anObject! !

!SystemDictionary methodsFor: 'memory space' stamp: ''!
bytesLeft
	"Answer the number of bytes of space available. Does a full garbage collection."

	^ self garbageCollect
! !

!SystemDictionary methodsFor: 'memory space' stamp: 'ar 2/25/2001 17:55'!
bytesLeft: aBool
	"Return the amount of available space. If aBool is true, include possibly available swap space. If aBool is false, include possibly available physical memory. For a report on the largest free block currently availabe within Squeak memory but not counting extra memory use #primBytesLeft."
	<primitive: 112>
	^self primBytesLeft! !

!SystemDictionary methodsFor: 'memory space' stamp: 'ar 2/25/2001 18:00'!
bytesLeftString
	"Return a string describing the amount of memory available"
	| availInternal availPhysical availTotal |
	self garbageCollect.
	availInternal := self primBytesLeft.
	availPhysical := self bytesLeft: false.
	availTotal := self bytesLeft: true.
	(availTotal > (availInternal + 10000)) "compensate for mini allocations inbetween"
		ifFalse:[^availInternal asStringWithCommas, ' bytes available'].
	^String streamContents:[:s|
		s nextPutAll: availInternal asStringWithCommas, 	' bytes (internal) '; cr.
		s nextPutAll: availPhysical asStringWithCommas,	' bytes (physical) '; cr.
		s nextPutAll: availTotal asStringWithCommas, 	' bytes (total)     '].! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'al 1/12/2006 23:57'!
classNamed: className 
	^self classOrTraitNamed: className.! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'al 1/13/2006 00:51'!
classNames
	"Answer a SortedCollection of all class names."
	| names |
	cachedClassNames == nil ifTrue:
		[names := OrderedCollection new: self size.
		self do: 
			[:cl | (cl isInMemory
				and: [(cl isKindOf: Class)
					and: [(cl name beginsWith: 'AnObsolete') not]])
				ifTrue: [names add: cl name]].
		cachedClassNames := names asSortedCollection].
	^ cachedClassNames! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'al 1/12/2006 23:56'!
classOrTraitNamed: aString 
	"aString is either a class or trait name or a class or trait name followed by ' class' or 'classTrait' respectively.
	Answer the class or metaclass it names."

	| meta baseName baseClass |
	(aString endsWith: ' class')
		ifTrue: [meta := true.
				baseName := aString copyFrom: 1 to: aString size - 6]
		ifFalse: [
			(aString endsWith: ' classTrait')
				ifTrue: [
					meta := true.
					baseName := aString copyFrom: 1 to: aString size - 11]
				ifFalse: [
					meta := false.
					baseName := aString]].
	baseClass := Smalltalk at: baseName asSymbol ifAbsent: [^ nil].
	meta
		ifTrue: [^ baseClass classSide]
		ifFalse: [^ baseClass]! !

!SystemDictionary methodsFor: 'housekeeping' stamp: ''!
cleanOutUndeclared 
	Undeclared removeUnreferencedKeys! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'di 3/3/2001 08:31'!
cleanUpUndoCommands
	"Smalltalk cleanUpUndoCommands"  "<== print this to get classes involved"

	| classes i p |
	classes := Bag new.
	'Ferreting out obsolete undo commands'
		displayProgressAt: Sensor cursorPoint
		from: 0 to: Morph withAllSubclasses size
		during:
	[:bar | i := 0.
	Morph withAllSubclassesDo:
		[:c | bar value: (i := i+1).
		c allInstancesDo:
			[:m | (p := m otherProperties) ifNotNil:
				[p keys do:
					[:k | (p at: k) class == Command ifTrue:
						[classes add: c name.
						m removeProperty: k]]]]]].
	^ classes! !

!SystemDictionary methodsFor: 'special objects' stamp: 'JMM 6/6/2000 20:36'!
clearExternalObjects
	"Clear the array of objects that have been registered for use in non-Smalltalk code."
	"Smalltalk clearExternalObjects"

	ExternalSemaphoreTable clearExternalObjects
! !

!SystemDictionary methodsFor: 'special objects' stamp: 'sd 9/29/2004 18:30'!
compactClassesArray
	"Smalltalk compactClassesArray"
	"Return the array of 31 classes whose instances may be
	represented compactly"
	^ self specialObjectsArray at: 29! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'adrian_lienhard 7/18/2009 16:02'!
compressSources	
	"Copy all the source file to a compressed file. Usually preceded by Smalltalk condenseSources."
	"The new file will be created in the default directory, and the code in openSources
	will try to open it if it is there, otherwise it will look for normal sources."
	"Smalltalk compressSources"

	| f cfName cf |
	f := SourceFiles first.
	(SmalltalkImage current sourcesName endsWith: 'sources')
		ifTrue: [cfName := (SmalltalkImage current sourcesName allButLast: 7) , 'stc']
		ifFalse: [self error: 'Hey, I thought the sources name ended with ''.sources''.'].
	cf := (CompressedSourceStream on: (FileStream newFileNamed: cfName))
				segmentSize: 20000 maxSize: f size.

	"Copy the sources"
'Compressing Sources File...'
	displayProgressAt: Sensor cursorPoint
	from: 0 to: f size
	during:
		[:bar | f position: 0.
		[f atEnd] whileFalse:
			[cf nextPutAll: (f next: 20000).
			bar value: f position]].
	cf close.
	self setMacFileInfoOn: cfName.
	self inform: 'You now have a compressed sources file!!
Pharo will use it the next time you start.'! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'md 8/12/2008 22:15'!
computeImageSegmentation
	"Smalltalk computeImageSegmentation"
	"Here's how the segmentation works:
	For each partition, we collect the classes involved, and also all
	messages no longer used in the absence of this partition. We
	start by computing a 'Miscellaneous' segment of all the
	unused classes in the system as is."
	| partitions unusedCandM newClasses expandedCandM |
	partitions := Dictionary new.
	unusedCandM := self unusedClassesAndMethodsWithout: {{}. {}}.
	partitions at: 'Miscellaneous' put: unusedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'VMConstruction-*')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'VMConstruction' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'ST80-*')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'ST80' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'Morphic-Games')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Games' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'Morphic-Remote')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Nebraska' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | ((SystemOrganization categoriesMatching: 'Network-*')
						copyWithoutAll: #('Network-Kernel' 'Network-Url' 'Network-Protocols' 'Network-ObjectSocket' ))
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := Smalltalk unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Network' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'Balloon3D-*')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Balloon3D' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'FFI-*')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'FFI' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'Genie-*')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Genie' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'Speech-*')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Speech' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | #('Morphic-Components' )
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := Smalltalk unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Components' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | #('Sound-Scores' 'Sound-Interface' )
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	newClasses := newClasses , #(#WaveletCodec #Sonogram #FWT #AIFFFileReader ).
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Sound' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | ((SystemOrganization categoriesMatching: 'Tools-*')
						copyWithout: 'Tools-Menus')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	newClasses := newClasses copyWithoutAll: #(#Debugger #Inspector #ContextVariablesInspector #SyntaxError #ChangeSet #ChangeRecord #ClassChangeRecord #ChangeList #VersionsBrowser ).
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Tools' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'Balloon-MMFlash*')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	newClasses := newClasses , #(#ADPCMCodec ).
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'Flash' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'Balloon-TrueType*')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'TrueType' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	newClasses := Array
				streamContents: [:s | (SystemOrganization categoriesMatching: 'Graphics-Files')
						do: [:cat | (SystemOrganization superclassOrder: cat)
								do: [:c | s nextPut: c name]]].
	expandedCandM := self unusedClassesAndMethodsWithout: {unusedCandM first asArray , newClasses. unusedCandM second}.
	partitions at: 'GraphicFiles' put: {(expandedCandM first copyWithoutAll: unusedCandM first) addAll: newClasses;
			 yourself. expandedCandM second copyWithoutAll: unusedCandM second}.
	unusedCandM := expandedCandM.
	#(#AliceConstants 'Balloon3D' #B3DEngineConstants 'Balloon3D' #WonderlandConstants 'Balloon3D' #FFIConstants 'FFI' #KlattResonatorIndices 'Speech' )
		pairsDo: [:poolName :part | (partitions at: part) first add: poolName].
	partitions
		keysDo: [:k | k = 'Miscellaneous'
				ifFalse: [(partitions at: 'Miscellaneous') first removeAllFoundIn: (partitions at: k) first]].
	^ partitions! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'nice 11/7/2009 23:24'!
condenseChanges
	"Move all the changes onto a compacted sources file."
	"Smalltalk condenseChanges"
	| f oldChanges count |
	f := FileStream fileNamed: 'ST80.temp'.
	f header; timeStamp.
	'Condensing Changes File...'
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: self classNames size + self traitNames size
		during: [:bar | 
			count := 0.
			self
				allClassesAndTraitsDo: [:classOrTrait | 
					bar value: (count := count + 1).
					classOrTrait moveChangesTo: f.
					classOrTrait putClassCommentToCondensedChangesFile: f.
					classOrTrait classSide moveChangesTo: f]].
	SmalltalkImage current lastQuitLogPosition: f position.
	f trailer; close.
	CompiledMethod allInstancesDo: [:e | e isInstalled ifFalse: [e destroySourcePointer]].
	oldChanges := SourceFiles at: 2.
	oldChanges close.
	FileDirectory default deleteFileNamed: oldChanges name , '.old';
		 rename: oldChanges name toBe: oldChanges name , '.old';
		 rename: f name toBe: oldChanges name.
	self setMacFileInfoOn: oldChanges name.
	SourceFiles
		at: 2
		put: (FileStream oldFileNamed: oldChanges name)! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'nice 11/7/2009 23:24'!
condenseSources	
	"Move all the changes onto a compacted sources file."
	"Smalltalk condenseSources"

	| f dir newVersionString count |
	Utilities fixUpProblemsWithAllCategory.
	"The above removes any concrete, spurious '-- all --' categories, which mess up the process."
	dir := FileDirectory default.
	newVersionString := UIManager default request: 'Please designate the version
for the new source code file...' initialAnswer: SmalltalkImage current sourceFileVersionString.
	newVersionString ifNil: [^ self].
	newVersionString = SmalltalkImage current sourceFileVersionString ifTrue:
		[^ self error: 'The new source file must not be the same as the old.'].
	SmalltalkImage current sourceFileVersionString: newVersionString.

	"Write all sources with fileIndex 1"
	f := FileStream newFileNamed: SmalltalkImage current sourcesName.
	f header; timeStamp.
'Condensing Sources File...'
	displayProgressAt: Sensor cursorPoint
	from: 0 to: self classNames size + self traitNames size
	during:
		[:bar | count := 0.
		Smalltalk allClassesAndTraitsDo:
			[:classOrTrait | bar value: (count := count + 1).
			classOrTrait fileOutOn: f moveSource: true toFile: 1]].
	f trailer; close.
	CompiledMethod allInstancesDo: [:e | e isInstalled ifFalse: [e destroySourcePointer]].

	"Make a new empty changes file"
	SmalltalkImage current closeSourceFiles.
	dir rename: SmalltalkImage current changesName
		toBe: SmalltalkImage current changesName , '.old'.
	(FileStream newFileNamed: SmalltalkImage current changesName)
		header; timeStamp; close.
	SmalltalkImage current lastQuitLogPosition: 0.

	self setMacFileInfoOn: SmalltalkImage current changesName.
	self setMacFileInfoOn: SmalltalkImage current sourcesName.
	SmalltalkImage current openSourceFiles.
	self inform: 'Source files have been rewritten!!
Check that all is well,
and then save/quit.'! !

!SystemDictionary methodsFor: 'memory space' stamp: ''!
createStackOverflow
	"For testing the low space handler..."
	"Smalltalk installLowSpaceWatcher; createStackOverflow"

	self createStackOverflow.  "infinite recursion"! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'em 3/31/2005 11:48'!
currentChangeSetString
	"Smalltalk currentChangeSetString"
	^ 'Current Change Set: ' translated, ChangeSet current name! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'sd 9/29/2004 18:27'!
currentProjectDo: aBlock 
	"So that code can work after removal of Projects"
	self
		at: #Project
		ifPresent: [:projClass | aBlock value: projClass current]! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'alain.plantec 6/10/2008 18:27'!
discardDiscards
	"Discard all discard* methods - including this one."

	(self class selectors select: [:each | each beginsWith: 'discard']) 
		do: [:each | self class removeSelector: each].
	#(lastRemoval majorShrink)
		do: [:each | self class removeSelector: each]! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'sd 9/29/2004 18:20'!
discardFFI
	"Discard the complete foreign function interface.
	NOTE: Recreates specialObjectsArray to prevent obsolete
	references. Has to specially remove external structure
	hierarchy before ExternalType"
	self
		at: #ExternalStructure
		ifPresent: [:cls | (ChangeSet superclassOrder: cls withAllSubclasses asArray)
				reverseDo: [:c | c removeFromSystem]].
	SystemOrganization removeCategoriesMatching: 'FFI-*'.
	self recreateSpecialObjectsArray.
	"Remove obsolete refs"
	ByteArray removeSelector: #asExternalPointer.
	ByteArray removeSelector: #pointerAt:! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'TPR 8/5/2000 01:32'!
discardFlash
	"Discard Flash support."

	SystemOrganization removeCategoriesMatching: 'Balloon-MMFlash*'
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'AlainPlantec 11/29/2009 21:38'!
discardOddsAndEnds
	"This method throws out lots of classes that are not frequently
	used."
	"Smalltalk discardOddsAndEnds"
	self organization removeSystemCategory: 'System-Serial Port'.
	self organization removeSystemCategory: 'ST80-Symbols'.
	self organization removeSystemCategory: 'Tools-File Contents Browser'.
	self organization removeSystemCategory: 'System-Compression'.
	self organization removeSystemCategory: 'Tools-Explorer'.
	self organization removeSystemCategory: 'System-Digital Signatures'.
	self organization removeSystemCategory: 'ST80-Paths'.
	"bit editor (remove Form editor first):"
	Form removeSelector: #bitEdit.
	StrikeFont removeSelector: #edit:.
	self removeClassNamed: #FormButtonCache.
	self removeClassNamed: #FormMenuController.
	self removeClassNamed: #FormMenuView.
	"self removeClassNamed: #BitEditor."
	"inspector for Dictionaries of Forms"
	Dictionary removeSelector: #inspectFormsWithLabel:.
	SystemDictionary removeSelector: #viewImageImports.
	"experimental updating object viewer:"
	Object removeSelector: #evaluate:wheneverChangeIn:.
	self removeClassNamed: #ObjectViewer.
	self removeClassNamed: #ObjectTracer.
	"miscellaneous classes:"
	self removeClassNamed: #Array2D.
	self removeClassNamed: #DriveACar.
	self removeClassNamed: #EventRecorder.
	self removeClassNamed: #FindTheLight.
	self removeClassNamed: #PluggableTest.
	self removeClassNamed: #SystemMonitor.
	self removeClassNamed: #ProtocolBrowser.
	self removeClassNamed: #ObjectExplorerWrapper.
	self removeClassNamed: #HierarchyBrowser.
	self removeClassNamed: #LinkedMessageSet.
	self removeClassNamed: #ObjectExplorer.
	self removeClassNamed: #PackageBrowser.
	self removeClassNamed: #AbstractHierarchicalList.
	self removeClassNamed: #ChangeList.
	self removeClassNamed: #VersionsBrowser.
	self removeClassNamed: #ChangeRecord.
	self removeClassNamed: #SelectorBrowser.
	self removeClassNamed: #HtmlFileStream.
	self removeClassNamed: #CrLfFileStream.
	self removeClassNamed: #FXGrafPort.
	self removeClassNamed: #FXBlt.
	self
		at: #SampledSound
		ifPresent: [:c | c initialize].
	#(#Helvetica #Palatino #Courier #ComicBold #ComicPlain )
		do: [:k | TextConstants
				removeKey: k
				ifAbsent: []].
	StandardFonts buttonFont: (StrikeFont familyName: #NewYork size: 12).
	#(#GZipConstants #ZipConstants #KlattResonatorIndices )
		do: [:k | self
				removeKey: k
				ifAbsent: []]! !

!SystemDictionary methodsFor: 'miscellaneous' stamp: ''!
exitToDebugger
	"Primitive. Enter the machine language debugger, if one exists. Essential.
	See Object documentation whatIsAPrimitive."

	<primitive: 114>
	self primitiveFailed! !

!SystemDictionary methodsFor: 'special objects' stamp: 'JMM 6/6/2000 21:01'!
externalObjects
	"Return an array of objects that have been registered for use in non-Smalltalk code. Smalltalk objects should be referrenced by external code only via indirection through this array, thus allowing the objects to move during compaction. This array can be cleared when the VM re-starts, since variables in external code do not survive snapshots. Note that external code should not attempt to access a Smalltalk object, even via this mechanism, while garbage collection is in progress."
	"Smalltalk externalObjects"

	^ ExternalSemaphoreTable externalObjects
! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'sd 11/16/2003 12:55'!
externalizeSources   
	"Write the sources and changes streams onto external files."
 	"Smalltalk externalizeSources"
	"the logic of this method is complex because it uses changesName and self changesName
	may be this is normal - sd"
	
	| sourcesName changesName aFile |
	sourcesName := SmalltalkImage current sourcesName.
	(FileDirectory default fileExists: sourcesName)
		ifTrue: [^ self inform:
'Sorry, you must first move or remove the
file named ', sourcesName].
	changesName := SmalltalkImage current changesName.
	(FileDirectory default fileExists: changesName)
		ifTrue: [^ self inform:
'Sorry, you must first move or remove the
file named ', changesName].

	aFile :=  FileStream newFileNamed: sourcesName.
	aFile nextPutAll: SourceFiles first originalContents.
	aFile close.
	self setMacFileInfoOn: sourcesName.
	SourceFiles at: 1 put: (FileStream readOnlyFileNamed: sourcesName).

	aFile := FileStream newFileNamed: SmalltalkImage current changesName.
	aFile nextPutAll: SourceFiles last contents.
	aFile close.
	"On Mac, set the file type and creator (noop on other platforms)"
	FileDirectory default
		setMacFileNamed: SmalltalkImage current changesName
		type: 'STch'
		creator: 'FAST'.
	SourceFiles at: 2 put: (FileStream oldFileNamed: changesName).

	self inform: 'Sources successfully externalized'.
! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'di 2/16/2000 10:28'!
flushClassNameCache
	"Smalltalk flushClassNameCache"
	"Forse recomputation of the cached list of class names."

	cachedClassNames := nil! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'ar 2/6/2001 18:42'!
forceChangesToDisk
	"Ensure that the changes file has been fully written to disk by closing and re-opening it. This makes the system more robust in the face of a power failure or hard-reboot."

	| changesFile |
	changesFile := SourceFiles at: 2.
	(changesFile isKindOf: FileStream) ifTrue: [
		changesFile flush.
		SecurityManager default hasFileAccess ifTrue:[
			changesFile close.
			changesFile open: changesFile name forWrite: true].
		changesFile setToEnd.
	].
! !

!SystemDictionary methodsFor: 'memory space' stamp: 'JMM 1/27/2005 13:23'!
forceTenure
	"Primitive. Tell the GC logic to force a tenure on the next increment GC."
	<primitive: 'primitiveForceTenure'>
	^self primitiveFailed! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'NS 1/27/2004 12:08'!
forgetClass: aClass logged: aBool 
	"Delete the class, aClass, from the system.
	Note that this doesn't do everything required to dispose of a class - to do that use Class>>removeFromSystem."

	aBool ifTrue: [SystemChangeNotifier uniqueInstance classRemoved: aClass fromCategory: aClass category].		
	SystemOrganization removeElement: aClass name.
	self removeFromStartUpList: aClass.
	self removeFromShutDownList: aClass.
	self removeKey: aClass name ifAbsent: [].
	self flushClassNameCache! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'sd 4/17/2003 20:59'!
forgetDoIts	
	"Smalltalk forgetDoIts"
	 "get rid of old DoIt methods"

	self systemNavigation allBehaviorsDo:
		[:cl | cl forgetDoIts]

! !

!SystemDictionary methodsFor: 'memory space' stamp: 'ar 2/11/2001 02:36'!
garbageCollect
	"Primitive. Reclaims all garbage and answers the number of bytes of available space."
	Object flushDependents.
	Object flushEvents.
	^self primitiveGarbageCollect! !

!SystemDictionary methodsFor: 'memory space' stamp: ''!
garbageCollectMost
	"Primitive. Reclaims recently created garbage (which is usually most of it) fairly quickly and answers the number of bytes of available space."

	<primitive: 131>
	^ self primBytesLeft! !

!SystemDictionary methodsFor: 'miscellaneous' stamp: 'alain.plantec 6/1/2008 20:41'!
handleUserInterrupt
	Preferences cmdDotEnabled ifTrue:
		[[Project interruptName: 'User Interrupt'] fork]! !

!SystemDictionary methodsFor: 'class and trait names' stamp: ''!
hasClassNamed: aString
	"Answer whether there is a class of the given name, but don't intern aString if it's not alrady interned.  4/29/96 sw"

	Symbol hasInterned: aString ifTrue: 
		[:aSymbol | ^ (self at: aSymbol ifAbsent: [nil]) isKindOf: Class].
	^ false! !

!SystemDictionary methodsFor: 'special objects' stamp: ''!
hasSpecialSelector: aLiteral ifTrueSetByte: aBlock

	1 to: self specialSelectorSize do:
		[:index | 
		(self specialSelectorAt: index) == aLiteral
			ifTrue: [aBlock value: index + 16rAF. ^true]].
	^false! !

!SystemDictionary methodsFor: 'ui' stamp: 'sd 1/16/2004 20:49'!
inspectGlobals
	"Smalltalk  inspectGlobals"
	
	| associations aDict |
	associations := ((self  keys select: [:aKey | ((self  at: aKey) isKindOf: Class) not]) asSortedArray collect:[:aKey | self associationAt: aKey]).
	aDict := IdentityDictionary new.
	associations do: [:as | aDict add: as].
	aDict inspectWithLabel: 'The Globals'! !

!SystemDictionary methodsFor: 'memory space' stamp: ''!
installLowSpaceWatcher
	"Start a process to watch for low-space conditions."
	"Smalltalk installLowSpaceWatcher"

	self primSignalAtBytesLeft: 0.  "disable low-space interrupts"
	LowSpaceProcess == nil ifFalse: [LowSpaceProcess terminate].
	LowSpaceProcess := [self lowSpaceWatcher] newProcess.
	LowSpaceProcess priority: Processor lowIOPriority.
	LowSpaceProcess resume.

! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'sd 11/16/2003 12:55'!
internalizeChangeLog    
		"Smalltalk internalizeChangeLog"
	"Bring the changes file into a memory-resident filestream, for faster access and freedom from external file system.  1/31/96 sw"

	| reply aName aFile |
	reply := self confirm:  'CAUTION -- do not undertake this lightly!!
If you have backed up your system and
are prepared to face the consequences of
the requested internalization of sources,
hit Yes.  If you have any doubts, hit No
to back out with no harm done.'.

	(reply ==  true) ifFalse:
		[^ self inform: 'Okay - abandoned'].

	aName := SmalltalkImage current changesName.
	(aFile := SourceFiles last) == nil ifTrue:
		[(FileDirectory default fileExists: aName)
			ifFalse: [^ self halt: 'Cannot locate ', aName, ' so cannot proceed.'].
		aFile := FileStream readOnlyFileNamed: aName].
	SourceFiles at: 2 put: (ReadWriteStream with: aFile contentsOfEntireFile).

	self inform: 'Okay, changes file internalized'! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'sd 11/16/2003 12:55'!
internalizeSources    
		"Smalltalk internalizeSources"
	"Bring the sources and changes files into memory-resident filestreams, for faster access and freedom from file-system interface.  1/29/96 sw"

	| reply aName aFile |
	reply := self confirm:  'CAUTION -- do not undertake this lightly!!
If you have backed up your system and
are prepared to face the consequences of
the requested internalization of sources,
hit Yes.  If you have any doubts, hit No
to back out with no harm done.'.

	(reply ==  true) ifFalse:
		[^ self inform: 'Okay - abandoned'].

	aName := SmalltalkImage current sourcesName.
	(aFile := SourceFiles first) == nil ifTrue:
		[(FileDirectory default fileExists: aName)
			ifFalse: [^ self halt: 'Cannot locate ', aName, ' so cannot proceed.'].
		aFile := FileStream readOnlyFileNamed: aName].
	SourceFiles at: 1 put: (ReadWriteStream with: aFile contentsOfEntireFile).

	aName := SmalltalkImage current changesName.
	(aFile := SourceFiles last) == nil ifTrue:
		[(FileDirectory default fileExists: aName)
			ifFalse: [^ self halt: 'Cannot locate ', aName, ' so cannot proceed.'].
		aFile := FileStream readOnlyFileNamed: aName].
	SourceFiles at: 2 put: (ReadWriteStream with: aFile contentsOfEntireFile).

	self inform: 'Okay, sources internalized'! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'alain.plantec 6/11/2008 07:36'!
isMorphic
	"Answer true if the user interface is running in Morphic rathern than 
	MVC. By convention the gloabl variable World is set to nil when MVC is 
	running. ScheduledControllers could be set to nil when Morphic is 
	running, but this symmetry is not yet in effect."
	self deprecated: #mvcIsRemoved.
	^ World ~~ nil"or: [RequestCurrentWorldNotification signal notNil]"! !

!SystemDictionary methodsFor: 'memory space' stamp: 'ar 1/18/2005 16:46'!
isRoot: oop
	"Primitive. Answer whether the object is currently a root for youngSpace."
	<primitive: 'primitiveIsRoot'>
	^self primitiveFailed! !

!SystemDictionary methodsFor: 'printing' stamp: 'stephane.ducasse 9/10/2008 21:36'!
isSelfEvaluating
	self == Smalltalk ifTrue: [^true].
	^super isSelfEvaluating! !

!SystemDictionary methodsFor: 'memory space' stamp: 'ar 1/18/2005 16:47'!
isYoung: oop
	"Primitive. Answer whether the object currently resides in youngSpace."
	<primitive: 'primitiveIsYoung'>
	^self primitiveFailed! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'stephane.ducasse 10/12/2008 20:55'!
lastRemoval
	"Smalltalk lastRemoval"
	"Some explicit removals - add unwanted methods keeping
	other methods."
	| oldDicts newDicts |
	#(#abandonSources )
		do: [:each | self class removeSelector: each].
	"Get rid of all unsent methods."
	[self removeAllUnsentMessages > 0] whileTrue.
	"Shrink method dictionaries."
	self garbageCollect.
	oldDicts := MethodDictionary allInstances.
	newDicts := Array new: oldDicts size.
	oldDicts
		withIndexDo: [:d :index | newDicts at: index put: d rehashWithoutBecome].
	oldDicts elementsExchangeIdentityWith: newDicts.
	oldDicts := newDicts := nil.
	self
		allClassesDo: [:c | c zapOrganization].
	SystemOrganization := nil.
	ChangeSet current initialize! !

!SystemDictionary methodsFor: 'miscellaneous' stamp: 'Bill Schwab 9/8/2009 18:07'!
logError: errMsg inContext: aContext to: aFilename
	"Log the error message and a stack trace to the given file."

	| ff |

	"wks - 9-09 - do not delete existing errors.  Note that this could be made
	conditional or left here in this state for those who want it.
	FileDirectory default deleteFileNamed: aFilename ifAbsent: []."
	(ff := FileStream fileNamed: aFilename) ifNil: [^ self "avoid recursive errors"].

	[
		"9-09 - move to end."
		ff setToEnd.
		
		"9-09 - write something easy to find to verify correct operation."
		ff nextPutAll:'THERE_BE_DRAGONS_HERE'; cr.
	
	  	ff nextPutAll: errMsg; cr.
		aContext errorReportOn: ff.
		
		"wks 9-09 - write some type of separator"
		ff nextPutAll:( String new:60 withAll:$- ); cr; cr.
	
	] ensure:[
		ff close.
	].
! !

!SystemDictionary methodsFor: 'memory space' stamp: 'di 8/18/2000 16:49'!
lowSpaceThreshold 
	"Return the low space threshold. When the amount of free memory (after garbage collection) falls below this limit, the system is in serious danger of completely exhausting memory and crashing. This limit should be made high enough to allow the user open a debugger to diagnose a problem or to save the image."

	thisContext isPseudoContext
		ifTrue: [^ 400000  "Enough for JIT compiler"]
		ifFalse: [^ 200000  "Enough for interpreter"]! !

!SystemDictionary methodsFor: 'memory space' stamp: 'marcus.denker 2/19/2009 21:00'!
lowSpaceWatcher
	"Wait until the low space semaphore is signalled, then take appropriate
	actions. "
	| free preemptedProcess |
	self garbageCollectMost <= self lowSpaceThreshold
		ifTrue: [self garbageCollect <= self lowSpaceThreshold
				ifTrue: ["free space must be above threshold before
					starting low space watcher"
					^ Beeper beep]].
	Smalltalk specialObjectsArray at: 23 put: nil.
	"process causing low space will be saved here"
	LowSpaceSemaphore := Semaphore new.
	self primLowSpaceSemaphore: LowSpaceSemaphore.
	self primSignalAtBytesLeft: self lowSpaceThreshold.
	"enable low space interrupts"
	LowSpaceSemaphore wait.
	"wait for a low space condition..."
	self primSignalAtBytesLeft: 0.
	"disable low space interrupts"
	self primLowSpaceSemaphore: nil.
	LowSpaceProcess := nil.
	"The process that was active at the time of the low space interrupt."
	preemptedProcess := Smalltalk specialObjectsArray at: 23.
	Smalltalk specialObjectsArray at: 23 put: nil.
	"Note: user now unprotected until the low space watcher is re-installed"
	self memoryHogs isEmpty
		ifFalse: [free := self bytesLeft.
			self memoryHogs
				do: [:hog | hog freeSomeSpace].
			self bytesLeft > free
				ifTrue: [^ self installLowSpaceWatcher]].
	Project interruptName: 'Space is low' preemptedProcess: preemptedProcess! !

!SystemDictionary methodsFor: 'memory space' stamp: 'nk 10/28/2000 20:37'!
lowSpaceWatcherProcess
	^LowSpaceProcess! !

!SystemDictionary methodsFor: 'miscellaneous' stamp: 'yo 7/2/2004 13:32'!
m17nVersion

	^ 'M17n 5.0' copy
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'MarcusDenker 11/28/2009 14:58'!
majorShrink
	"Undertake a major shrinkage of the image.
	This method throws out lots of the system that is not needed
	for, eg, operation in a hand-held PC. majorShrink produces a
	999k image in Squeak 2.8
	Smalltalk majorShrink; abandonSources; lastRemoval"
	| oldDicts newDicts |
	Project current setParent: Project current.
	MorphicModel removeUninstantiatedModels.
	Utilities classPool at: #ScrapsBook put: nil.
	Utilities zapUpdateDownloader.
	"Smalltalk discardVMConstruction."
	"755k"
	self discardSoundSynthesis.
	"544k"
	self discardOddsAndEnds.
	"234k"
	"Smalltalk discard3D."
	"407k"
	self discardFFI.
	"33k"
	self discardMorphic.
	"1372k"
	Symbol rehash.
	"40k"
	"Above by itself saves about 4,238k"
	"Remove references to a few classes to be deleted, so that they
	won't leave obsolete versions around."
	ChangeSet class compile: 'defaultName
		^ ''Changes'' ' classified: 'initialization'.
	"Now delete various other classes.."
	SystemOrganization removeSystemCategory: 'Graphics-Files'.
	SystemOrganization removeSystemCategory: 'System-Object Storage'.
	"Smalltalk removeClassNamed: #Project."
	self removeClassNamed: #FormSetFont.
	self removeClassNamed: #FontSet.
	self removeClassNamed: #InstructionPrinter.
	self removeClassNamed: #ChangeSorter.
	self removeClassNamed: #DualChangeSorter.
	self removeClassNamed: #EmphasizedMenu.
	self removeClassNamed: #MessageTally.
	StringHolder class removeSelector: #originalWorkspaceContents.
	CompiledMethod removeSelector: #symbolic.
	RemoteString removeSelector: #makeNewTextAttVersion.
	self removeClassNamed: #PenPointRecorder.
	self removeClassNamed: #Path.
	self removeClassNamed: #Base64MimeConverter.
	self removeClassNamed: #RWBinaryOrTextStream.
	self removeClassNamed: #AttributedTextStream.
	self removeClassNamed: #WordNet.
	self removeClassNamed: #SelectorBrowser.
	TextStyle
		allSubInstancesDo: [:ts | ts
				newFontArray: (ts fontArray
						copyFrom: 1
						to: (2 min: ts fontArray size))].
	ListParagraph initialize.
	PopUpMenu initialize.
	ChangeSet noChanges.
	ChangeSet classPool
		at: #AllChangeSets
		put: (OrderedCollection with: ChangeSet current).
	SystemDictionary removeSelector: #majorShrink.
	[self removeAllUnsentMessages > 0]
		whileTrue: [Smalltalk unusedClasses
				do: [:c | (Smalltalk at: c) removeFromSystem]].
	SystemOrganization removeEmptyCategories.
	self
		allClassesDo: [:c | c zapOrganization].
	self garbageCollect.
	'Rehashing method dictionaries . . .'
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: MethodDictionary instanceCount
		during: [:bar | 
			oldDicts := MethodDictionary allInstances.
			newDicts := Array new: oldDicts size.
			oldDicts
				withIndexDo: [:d :index | 
					bar value: index.
					newDicts at: index put: d rehashWithoutBecome].
			oldDicts elementsExchangeIdentityWith: newDicts].
	oldDicts := newDicts := nil.
	ChangeSet current initialize.
	"seems to take more than one try to gc all the weak refs in
	SymbolTable "
	3
		timesRepeat: [self garbageCollect.
			Symbol compactSymbolTable]! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'AlainPlantec 10/19/2009 18:12'!
makeExternalRelease
	"Smalltalk makeExternalRelease"
	(self confirm: SystemVersion current version , '
Is this the correct version designation?
If not, choose no, and fix it.')
		ifFalse: [^ self].
	"Object classPool at: #DependentsFields"
	self reclaimDependents.
	Preferences enable: #fastDragWindowForMorphic.
	Smalltalk at: #Browser ifPresent:[:br| br initialize].
	Undeclared isEmpty
		ifFalse: [self halt].
	#(#Helvetica #Palatino #Courier )
		do: [:n | TextConstants
				removeKey: n
				ifAbsent: []].
	(Utilities classPool at: #UpdateUrlLists) copy
		do: [:pair | (pair first includesSubstring: 'Disney' caseSensitive: false)
				ifTrue: [(Utilities classPool at: #UpdateUrlLists)
						remove: pair]].
	(ServerDirectory serverNames copyWithoutAll: #('UCSBCreateArchive' 'UIUCArchive' 'UpdatesExtUIUC' 'UpdatesExtWebPage' ))
		do: [:sn | ServerDirectory removeServerNamed: sn].
	self  garbageCollect.
	self obsoleteClasses isEmpty
		ifFalse: [self halt].
	Symbol rehash.
	self halt: 'Ready to condense changes or sources'! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'ar 9/27/2005 21:45'!
makeInternalRelease
	"Smalltalk makeInternalRelease"
	(self confirm: SystemVersion current version , '
Is this the correct version designation?
If not, choose no, and fix it.')
		ifFalse: [^ self].
	(Object classPool at: #DependentsFields) size > 1
		ifTrue: [self halt].
	Smalltalk at: #Browser ifPresent:[:br| br initialize].
	Undeclared isEmpty
		ifFalse: [self halt].
	self garbageCollect.
	self obsoleteClasses isEmpty
		ifFalse: [self halt].
	Symbol rehash.
	self halt: 'Ready to condense changes'.
	self condenseChanges! !

!SystemDictionary methodsFor: 'memory space' stamp: 'sma 4/22/2000 19:03'!
memoryHogs
	"Answer the list of objects to notify with #freeSomeSpace if memory gets full."

	^ MemoryHogs ifNil: [MemoryHogs := OrderedCollection new]! !

!SystemDictionary methodsFor: 'miscellaneous' stamp: 'yo 7/2/2004 13:32'!
nihongoVersion

	^ 'Nihongo7.0' copy
! !

!SystemDictionary methodsFor: 'objects from disk' stamp: 'tk 9/28/2000 15:50'!
objectForDataStream: refStrm
	| dp |
	"I am about to be written on an object file.  Write a reference to Smalltalk instead."

	dp := DiskProxy global: #Smalltalk selector: #yourself
			args: #().
	refStrm replace: self with: dp.
	^ dp! !

!SystemDictionary methodsFor: 'memory space' stamp: ''!
okayToProceedEvenIfSpaceIsLow
	"Return true if either there is enough memory to do so safely or if the user gives permission after being given fair warning."

	self garbageCollectMost > self lowSpaceThreshold ifTrue: [^ true].  "quick"
	self garbageCollect > self lowSpaceThreshold ifTrue: [^ true].  "work harder"

	^ self confirm:
'WARNING: There is not enough space to start the low space watcher.
If you proceed, you will not be warned again, and the system may
run out of memory and crash. If you do proceed, you can start the
low space notifier when more space becomes available simply by
opening and then closing a debugger (e.g., by hitting Cmd-period.)
Do you want to proceed?'
! !

!SystemDictionary methodsFor: 'accessing' stamp: 'ar 7/11/1999 21:56'!
organization
	"Return the organizer for the receiver"
	^SystemOrganization! !

!SystemDictionary methodsFor: 'retrieving' stamp: 'sd 9/29/2004 18:17'!
poolUsers
	"Answer a dictionary of pool name -> classes that refer to it.
	Also includes any globally know dictionaries (such as
	Smalltalk, Undeclared etc) which although not strictly
	accurate is potentially useful information"
	"Smalltalk poolUsers"
	| poolUsers pool refs |
	poolUsers := Dictionary new.
	self keys
		do: [:k | "yes, using isKindOf: is tacky but for reflective code like
			this it is very useful. If you really object you can:-
			a) go boil your head.
			b) provide a better answer.
			your choice."
			(((pool := self at: k) isKindOf: Dictionary)
					or: [pool isKindOf: SharedPool class])
				ifTrue: [refs := self systemNavigation allClasses
								select: [:c | c sharedPools identityIncludes: pool]
								thenCollect: [:c | c name].
					refs
						add: (self systemNavigation
								allCallsOn: (self associationAt: k)).
					poolUsers at: k put: refs]].
	^ poolUsers! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'StephaneDucasse 11/7/2009 10:05'!
presumedSentMessages   | sent |
"Smalltalk presumedSentMessages"

	"The following should be preserved for doIts, etc"
	sent := IdentitySet new.
	#( rehashWithoutBecome compactSymbolTable 
		browseAllSelect:  lastRemoval
		scrollBarValue: vScrollBarValue: scrollBarMenuButtonPressed: 
		withSelectionFrom:  to: removeClassNamed:
		dragon: hilberts: mandala: web factorial tinyBenchmarks benchFib
		newDepth: restoreAfter: forgetDoIts zapAllMethods obsoleteClasses
		removeAllUnsentMessages abandonSources removeUnreferencedKeys
		reclaimDependents zapOrganization condenseChanges browseObsoleteReferences
		subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:
		methodsFor:stamp: methodsFor:stamp:prior: instanceVariableNames:
		startTimerInterruptWatcher unusedClasses) do:
		[:sel | sent add: sel].
	"The following may be sent by perform: in dispatchOnChar..."
	(ParagraphEditor classPool at: #CmdActions) asSet do:
		[:sel | sent add: sel].
	(ParagraphEditor classPool at: #ShiftCmdActions) asSet do:
		[:sel | sent add: sel].
	^ sent! !

!SystemDictionary methodsFor: 'memory space' stamp: ''!
primBytesLeft
	"Primitive. Answer the number of bytes available for new object data.
	Not accurate unless preceded by
		Smalltalk garbageCollectMost (for reasonable accuracy), or
		Smalltalk garbageCollect (for real accuracy).
	See Object documentation whatIsAPrimitive."

	<primitive: 112>
	^ 0! !

!SystemDictionary methodsFor: 'image, changes name' stamp: 'yo 3/29/2004 09:36'!
primImageName
	"Answer the full path name for the current image."
	"Smalltalk imageName"

	<primitive: 121>
	self primitiveFailed! !

!SystemDictionary methodsFor: 'image, changes name' stamp: 'yo 3/29/2004 09:36'!
primImageName: newName
	"Set the the full path name for the current image.  All further snapshots will use this."

	<primitive: 121>
	^ self primitiveFailed! !

!SystemDictionary methodsFor: 'memory space' stamp: ''!
primLowSpaceSemaphore: aSemaphore
	"Primitive. Register the given Semaphore to be signalled when the
	number of free bytes drops below some threshold. Disable low-space
	interrupts if the argument is nil."

	<primitive: 124>
	self primitiveFailed! !

!SystemDictionary methodsFor: 'memory space' stamp: ''!
primSignalAtBytesLeft: numBytes
	"Tell the interpreter the low-space threshold in bytes. When the free
	space falls below this threshold, the interpreter will signal the low-space
	semaphore, if one has been registered.  Disable low-space interrupts if the
	argument is zero.  Fail if numBytes is not an Integer."

	<primitive: 125>
	self primitiveFailed! !

!SystemDictionary methodsFor: 'image, changes name' stamp: 'yo 3/29/2004 09:36'!
primVmPath
	"Answer the path for the directory containing the Smalltalk virtual machine. Return the empty string if this primitive is not implemented."
	"Smalltalk vmPath"

	<primitive: 142>
	^ ''! !

!SystemDictionary methodsFor: 'memory space' stamp: 'ar 2/11/2001 02:16'!
primitiveGarbageCollect
	"Primitive. Reclaims all garbage and answers the number of bytes of available space."

	<primitive: 130>
	^ self primBytesLeft! !

!SystemDictionary methodsFor: 'printing' stamp: 'sma 6/1/2000 09:53'!
printElementsOn: aStream
	aStream nextPutAll:'(lots of globals)'! !

!SystemDictionary methodsFor: 'printing' stamp: 'stephane.ducasse 9/10/2008 21:36'!
printOn: aStream
	self == Smalltalk ifFalse: [^super printOn: aStream].
	aStream nextPutAll: 'Smalltalk'! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'ar 11/16/1999 20:12'!
processShutDownList: quitting
	"Send #shutDown to each class that needs to wrap up before a snapshot."

	self send: #shutDown: toClassesNamedIn: ShutDownList with: quitting.
! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'ar 11/16/1999 20:12'!
processStartUpList: resuming
	"Send #startUp to each class that needs to run initialization after a snapshot."

	self send: #startUp: toClassesNamedIn: StartUpList with: resuming.
! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: ''!
quitPrimitive
	"Primitive. Exit to another operating system on the host machine, if one
	exists. All state changes in the object space since the last snapshot are lost.
	Essential. See Object documentation whatIsAPrimitive."

	<primitive: 113>
	self primitiveFailed! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'sd 9/29/2004 18:15'!
reclaimDependents
	"No-opped due to weak dictionary in use"
	self garbageCollect! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'al 1/13/2006 00:19'!
reconstructChanges	
	"Move all the changes and its histories onto another sources file."
	"Smalltalk reconstructChanges"

	| f oldChanges classCount |
	f := FileStream fileNamed: 'ST80.temp'.
	f header; timeStamp.
'Condensing Changes File...'
	displayProgressAt: Sensor cursorPoint
	from: 0 to: self classNames size + self traitNames size
	during:
		[:bar | classCount := 0.
		Smalltalk allClassesAndTraitsDo:
			[:classOrTrait | bar value: (classCount := classCount + 1).
			classOrTrait moveChangesWithVersionsTo: f.
			classOrTrait putClassCommentToCondensedChangesFile: f.
			classOrTrait classSide moveChangesWithVersionsTo: f]].
	SmalltalkImage current lastQuitLogPosition: f position.
	f trailer; close.
	oldChanges := SourceFiles at: 2.
	oldChanges close.
	FileDirectory default 
		deleteFileNamed: oldChanges name , '.old';
		rename: oldChanges name toBe: oldChanges name , '.old';
		rename: f name toBe: oldChanges name.
	self setMacFileInfoOn: oldChanges name.
	SourceFiles at: 2
			put: (FileStream oldFileNamed: oldChanges name)! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'sw 2/3/2000 15:59'!
recover: nCharacters
	"Schedule an editable text view on the last n characters of changes."
	self writeRecentCharacters: nCharacters toFileNamed: 'st80.recent'! !

!SystemDictionary methodsFor: 'special objects' stamp: 'marcus.denker 6/5/2009 11:16'!
recreateSpecialObjectsArray
	"Smalltalk recreateSpecialObjectsArray"
	"The Special Objects Array is an array of object pointers used
	by the
	Squeak virtual machine. Its contents are critical and
	unchecked, so don't even think of playing here unless you
	know what you are doing."
	| newArray |
	newArray := Array new: 50.
	"Nil false and true get used throughout the interpreter"
	newArray at: 1 put: nil.
	newArray at: 2 put: false.
	newArray at: 3 put: true.
	"This association holds the active process (a ProcessScheduler)"
	newArray at: 4 put: (self associationAt: #Processor).
	"Numerous classes below used for type checking and instantiation"
	newArray at: 5 put: Bitmap.
	newArray at: 6 put: SmallInteger.
	newArray at: 7 put: ByteString.
	newArray at: 8 put: Array.
	newArray at: 9 put: Smalltalk.
	newArray at: 10 put: Float.
	newArray at: 11 put: MethodContext.
	newArray at: 12 put: BlockContext.
	newArray at: 13 put: Point.
	newArray at: 14 put: LargePositiveInteger.
	newArray at: 15 put: Display.
	newArray at: 16 put: Message.
	newArray at: 17 put: CompiledMethod.
	newArray at: 18 put: (self specialObjectsArray at: 18).
	"(low space Semaphore)"
	newArray at: 19 put: Semaphore.
	newArray at: 20 put: Character.
	newArray at: 21 put: #doesNotUnderstand:.
	newArray at: 22 put: #cannotReturn:.
	newArray at: 23 put: nil.
	"An array of the 32 selectors that are compiled as special bytecodes,
	 paired alternately with the number of arguments each takes."
	newArray at: 24 put: #(	#+ 1 #- 1 #< 1 #> 1 #<= 1 #>= 1 #= 1 #~= 1
							#* 1 #/ 1 #\\ 1 #@ 1 #bitShift: 1 #// 1 #bitAnd: 1 #bitOr: 1
							#at: 1 #at:put: 2 #size 0 #next 0 #nextPut: 1 #atEnd 0 #== 1 #class 0
							#blockCopy: 1 #value 0 #value: 1 #do: 1 #new 0 #new: 1 #x 0 #y 0 ).
	"An array of the 255 Characters in ascii order."
	newArray at: 25 put: ((0 to: 255) collect: [:ascii | Character value: ascii]).
	newArray at: 26 put: #mustBeBoolean.
	newArray at: 27 put: ByteArray.
	newArray at: 28 put: Process.
	"An array of up to 31 classes whose instances will have compact headers"
	newArray at: 29 put: self compactClassesArray.
	newArray at: 30 put: (self specialObjectsArray at: 30).
	"(delay Semaphore)"
	newArray at: 31 put: (self specialObjectsArray at: 31).
	"(user interrupt Semaphore)"
	"Prototype instances that can be copied for fast initialization"
	newArray at: 32 put: (Float new: 2).
	newArray at: 33 put: (LargePositiveInteger new: 4).
	newArray at: 34 put: Point new.
	newArray at: 35 put: #cannotInterpret:.
	"Note: This must be fixed once we start using context prototypes (yeah, right)"
	"(MethodContext new: CompiledMethod fullFrameSize)."
	newArray at: 36 put: (self specialObjectsArray at: 36). "Is the prototype MethodContext (unused by the VM)"
	newArray at: 37 put: BlockClosure.
	"(BlockContext new: CompiledMethod fullFrameSize)."
	newArray at: 38 put: (self specialObjectsArray at: 38). "Is the prototype BlockContext (unused by the VM)"
	newArray at: 39 put: (self specialObjectsArray at: 39).	"preserve external semaphores"
	"array of objects referred to by external code"
	newArray at: 40 put: PseudoContext.
	"newArray at: 41 put: TranslatedMethod."
	"finalization Semaphore"
	newArray at: 42 put: ((self specialObjectsArray at: 42) ifNil: [Semaphore new]).
	newArray at: 43 put: LargeNegativeInteger.
	"External objects for callout.
	 Note: Written so that one can actually completely remove the FFI."
	newArray at: 44 put: (self at: #ExternalAddress ifAbsent: []).
	newArray at: 45 put: (self at: #ExternalStructure ifAbsent: []).
	newArray at: 46 put: (self at: #ExternalData ifAbsent: []).
	newArray at: 47 put: (self at: #ExternalFunction ifAbsent: []).
	newArray at: 48 put: (self at: #ExternalLibrary ifAbsent: []).
	newArray at: 49 put: #aboutToReturn:through:.
	newArray at: 50 put: #run:with:in:.
	"Now replace the interpreter's reference in one atomic operation"
	self specialObjectsArray become: newArray! !

!SystemDictionary methodsFor: 'special objects' stamp: 'JMM 6/6/2000 20:39'!
registerExternalObject: anObject
	"Register the given object in the external objects array and return its index. If it is already there, just return its index."

	^ExternalSemaphoreTable registerExternalObject: anObject! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'nk 6/2/2006 10:22'!
removeAllLineFeeds
	"Smalltalk removeAllLineFeeds"
	"Scan all methods for source code with lineFeeds.
	Replaces all occurrences of <CR><LF> or <LF> by <CR>.
	When done, offers to display an Inspector containing the message
	names grouped by author initials.
	In this dictionary, the key 'OK' contains the methods that had literals that contained <LF> characters."
	| n authors totalStripped totalOK |
	'Scanning sources for LineFeeds.
This will take a few minutes...'
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: CompiledMethod instanceCount
		during: [:bar | 
			n := 0.
			authors := self
						removeAllLineFeedsQuietlyCalling: [:cls :sel | (n := n + 1) \\ 100 = 0
								ifTrue: [bar value: n]]].
	totalStripped := authors
				inject: 1
				into: [:sum :set | sum + set size].
	totalOK := (authors at: 'OK') size.
	totalStripped := totalStripped - totalOK.
	Transcript cr; show: totalStripped printString , ' methods stripped of LFs.'.
	Transcript cr; show: totalOK printString , ' methods still correctly contain LFs.'.
	(self confirm: 'Do you want to see the affected methods?')
		ifTrue: [authors inspect]! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'nk 6/2/2006 08:35'!
removeAllLineFeedsQuietly
	"Smalltalk removeAllLineFeedsQuietly"
	"Scan all methods for source code with lineFeeds.
	Replaces all occurrences of <CR><LF> or <LF> by <CR>.
	Answer a Dictionary keyed by author name containing sets of affected method names,
	as well as (at the key 'OK') a list of methods that still contain LF characters inside literal strings or characters."
	^self removeAllLineFeedsQuietlyCalling: [ :cls :sel | ].! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'nk 6/2/2006 09:22'!
removeAllLineFeedsQuietlyCalling: aBlock
	"Smalltalk removeAllLineFeedsQuietly"
	"Scan all methods for source code with lineFeeds.
	Replaces all occurrences of <CR><LF> or <LF> by <CR>.
	Answer a Dictionary keyed by author name containing sets of affected method names,
	as well as (at the key 'OK') a list of methods that still contain LF characters inside literal strings or characters.
	Evaluate aBlock for each method so that status can be updated."
	| oldCodeString newCodeString oldStamp oldCategory authors nameString |
	self forgetDoIts.
	authors := Dictionary new.
	authors at: 'OK' put: Set new.
	self systemNavigation
		allBehaviorsDo: [:cls | cls selectors
				do: [:selector | 
					aBlock value: cls value: selector.
					oldCodeString := cls sourceCodeAt: selector.
					(oldCodeString includes: Character lf)
						ifTrue: [
							newCodeString := oldCodeString withSqueakLineEndings.
							nameString := cls name , '>>' , selector.
							((cls compiledMethodAt: selector) hasLiteralSuchThat: [ :lit | lit asString includes: Character lf ])
								ifTrue: [(authors at: 'OK')
										add: nameString]
								ifFalse: [oldStamp := (Utilities
												timeStampForMethod: (cls compiledMethodAt: selector))
												copy replaceAll: Character cr
												with: Character space.
									(authors
										at: (oldStamp copyFrom: 1 to: (oldStamp findFirst: [ :c | c isAlphaNumeric not ]))
										ifAbsentPut: [Set new])
										add: nameString.
									oldCategory := cls whichCategoryIncludesSelector: selector.
									cls
										compile: newCodeString
										classified: oldCategory
										withStamp: oldStamp
										notifying: nil ]]]].
	^ authors! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'MarcusDenker 11/28/2009 14:57'!
removeAllUnsentMessages
	"Smalltalk removeAllUnsentMessages"
	"[Smalltalk unusedClasses do: [:c | (Smalltalk at: c) removeFromSystem]. 
	Smalltalk removeAllUnsentMessages > 0] whileTrue."
	"Remove all implementations of unsent messages."
	| sels n |
	sels := self systemNavigation allUnSentMessages.
	"The following should be preserved for doIts, etc"
	"needed even after #majorShrink is pulled"
	#(#rehashWithoutBecome #compactSymbolTable #browseAllSelect:  #lastRemoval #scrollBarValue: vScrollBarValue: #scrollBarMenuButtonPressed: #withSelectionFrom: #to: #removeClassNamed: #dragon: #hilberts: #mandala: #web #factorial #tinyBenchmarks #benchFib #newDepth: #restoreAfter: #forgetDoIts #zapAllMethods #obsoleteClasses #removeAllUnSentMessages #abandonSources #removeUnreferencedKeys #reclaimDependents #zapOrganization #condenseChanges #browseObsoleteReferences #subclass:instanceVariableNames:classVariableNames:poolDictionaries:category: #methodsFor:stamp: #methodsFor:stamp:prior: #instanceVariableNames: #startTimerInterruptWatcher #unusedClasses )
		do: [:sel | sels
				remove: sel
				ifAbsent: []].
	"The following may be sent by perform: in dispatchOnChar..."
	(ParagraphEditor classPool at: #CmdActions) asSet
		do: [:sel | sels
				remove: sel
				ifAbsent: []].
	(ParagraphEditor classPool at: #ShiftCmdActions) asSet
		do: [:sel | sels
				remove: sel
				ifAbsent: []].
	sels size = 0
		ifTrue: [^ 0].
	n := 0.
	self systemNavigation
		allBehaviorsDo: [:x | n := n + 1].
	'Removing ' , sels size printString , ' messages . . .'
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: n
		during: [:bar | 
			n := 0.
			self systemNavigation
				allBehaviorsDo: [:class | 
					bar value: (n := n + 1).
					sels
						do: [:sel | class basicRemoveSelector: sel]]].
	^ sels size! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'sw 9/5/97 18:30'!
removeClassNamed: aName
	"Invoked from fileouts:  if there is currently a class in the system named aName, then remove it.  If anything untoward happens, report it in the Transcript.  "

	| oldClass |
	(oldClass := self at: aName asSymbol ifAbsent: [nil]) == nil
		ifTrue:
			[Transcript cr; show: 'Removal of class named ', aName, ' ignored because ', aName, ' does not exist.'.
			^ self].

	oldClass removeFromSystem! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'sd 9/29/2004 18:15'!
removeEmptyMessageCategories
	"Smalltalk removeEmptyMessageCategories"
	self garbageCollect.
	(ClassOrganizer allInstances copyWith: SystemOrganization)
		do: [:org | org removeEmptyCategories]! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'di 2/3/1999 22:22'!
removeFromShutDownList: aClass

	ShutDownList remove: aClass name ifAbsent: []! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'di 2/3/1999 22:22'!
removeFromStartUpList: aClass

	StartUpList remove: aClass name ifAbsent: []! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'MarcusDenker 11/28/2009 14:23'!
removeNormalCruft
	"Remove various graphics, uniclasses, references. Caution: see
	comment at bottom of method"
	"Smalltalk removeNormalCruft"
	self classNames
		do: [:cName | #('Player' 'CardPlayer' 'Component' 'WonderlandActor' 'MorphicModel' 'PlayWithMe' )
				do: [:superName | ((cName ~= superName
								and: [cName beginsWith: superName])
							and: [(cName allButFirst: superName size)
									allSatisfy: [:ch | ch isDigit]])
						ifTrue: [self removeClassNamed: cName]]].
	ChangeSet current clear! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'sd 9/29/2004 18:26'!
removeSelector: descriptor 
	"Safely remove a selector from a class (or metaclass). If the
	class or the method doesn't exist anymore, never mind and
	answer nil.
	This method should be used instead of 'Class removeSelector:
	#method' to omit global class references."
	| class sel |
	class := self
				at: descriptor first
				ifAbsent: [^ nil].
	(descriptor size > 2
			and: [descriptor second == #class])
		ifTrue: [class := class class.
			sel := descriptor third]
		ifFalse: [sel := descriptor second].
	^ class removeSelector: sel! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'rw 10/7/2006 08:34'!
renameClass: aClass as: newName 
	"Rename the class, aClass, to have the title newName."
	"Original one I want to keep but needs to be fixed"
	
	| oldref i oldName category |
	oldName := aClass name.
	category := aClass category.
	SystemOrganization classify: newName under: aClass category.
	SystemOrganization removeElement: aClass name.
	oldref := self associationAt: aClass name.
	self removeKey: aClass name.
	oldref key: newName.
	self add: oldref.  "Old association preserves old refs"
	(Array with: StartUpList with: ShutDownList) do:
		[:list |  i := list indexOf: aClass name ifAbsent: [0].
		i > 0 ifTrue: [list at: i put: newName]].
	self flushClassNameCache.
	SystemChangeNotifier uniqueInstance classRenamed: aClass from: oldName to: newName inCategory: category! !

!SystemDictionary methodsFor: 'class names' stamp: 'rw 10/17/2006 23:07'!
renameClass: aClass from: oldName 
	"Rename the class, aClass, to have the title newName."
	| oldref i newName category |
	newName := aClass name.
	category := SystemOrganization categoryOfElement: oldName.
	SystemOrganization classify: newName under: category.
	SystemOrganization removeElement: oldName.
	oldref := self associationAt: oldName.
	self removeKey: oldName.
	oldref key: newName.
	self add: oldref.  "Old association preserves old refs"
	(Array with: StartUpList with: ShutDownList) do:
		[:list |  i := list indexOf: oldName ifAbsent: [0].
		i > 0 ifTrue: [list at: i put: newName]].
	self flushClassNameCache.

	SystemChangeNotifier uniqueInstance classRenamed: aClass from: oldName to: newName inCategory: category! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'sw 10/28/96'!
renameClassNamed: oldName as: newName
	"Invoked from fileouts:  if there is currently a class in the system named oldName, then rename it to newName.  If anything untoward happens, report it in the Transcript.  "

	| oldClass |
	(oldClass := self at: oldName asSymbol ifAbsent: [nil]) == nil
		ifTrue:
			[Transcript cr; show: 'Class-rename for ', oldName, ' ignored because ', oldName, ' does not exist.'.
			^ self].

	oldClass rename: newName! !

!SystemDictionary methodsFor: 'memory space' stamp: 'ar 1/18/2005 16:48'!
rootTable
	"Primitive. Answer a snapshot of the VMs root table. 
	Keep in mind that the primitive may itself cause GC."
	<primitive: 'primitiveRootTable'>
	^self primitiveFailed! !

!SystemDictionary methodsFor: 'memory space' stamp: 'ar 1/18/2005 16:49'!
rootTableAt: index
	"Primitive. Answer the nth element of the VMs root table"
	<primitive: 'primitiveRootTableAt'>
	^nil! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'adrian_lienhard 7/18/2009 16:02'!
send: startUpOrShutDown toClassesNamedIn: startUpOrShutDownList with: argument
	"Send the message #startUp: or #shutDown: to each class named in the list.
	The argument indicates if the system is about to quit (for #shutDown:) or if
	the image is resuming (for #startUp:).
	If any name cannot be found, then remove it from the list."

	| removals class |
	removals := OrderedCollection new.
	startUpOrShutDownList do:
		[:name |
		class := self at: name ifAbsent: [nil].
		class == nil
			ifTrue: [removals add: name]
			ifFalse: [class isInMemory ifTrue:
						[[class perform: startUpOrShutDown with: argument]
								on: Exception
								do: [:ex | SmalltalkImage current hasDisplay
										ifTrue: [ex pass]
										ifFalse: [Smalltalk
												at: #Console
												ifPresent: [:console | console printNl: ex description].
											Smalltalk
												logError: ex printString
												inContext: thisContext
												to: 'PharoDebug.log']]]]].

	"Remove any obsolete entries, but after the iteration"
	startUpOrShutDownList removeAll: removals! !

!SystemDictionary methodsFor: 'memory space' stamp: 'JMM 1/27/2005 13:12'!
setGCBiasToGrow: aNumber
	"Primitive. Indicate that the GC logic should be bias to grow"
	<primitive: 'primitiveSetGCBiasToGrow'>
	^self primitiveFailed
"Example:
	Smalltalk setGCBiasToGrowGCLimit: 16*1024*1024.
	Smalltalk setGCBiasToGrow: 1.
"! !

!SystemDictionary methodsFor: 'memory space' stamp: 'JMM 1/27/2005 12:27'!
setGCBiasToGrowGCLimit: aNumber
	"Primitive. Indicate that the bias to grow logic should do a GC after aNumber Bytes"
	<primitive: 'primitiveSetGCBiasToGrowGCLimit'>
	^self primitiveFailed
"Example:
	Smalltalk setGCBiasToGrowGCLimit: 16*1024*1024.
"! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'marcus.denker 11/8/2008 17:10'!
setGCParameters
	"Adjust the VM's default GC parameters to avoid premature tenuring."

	SmalltalkImage current  vmParameterAt: 5 put: 4000.  "do an incremental GC after this many allocations"
	SmalltalkImage current  vmParameterAt: 6 put: 2000.  "tenure when more than this many objects survive the GC"
	  Smalltalk setGCBiasToGrowGCLimit: 16*1024*1024.
       Smalltalk setGCBiasToGrow: 1.
! !

!SystemDictionary methodsFor: 'memory space' stamp: 'ar 1/18/2005 16:54'!
setGCSemaphore: semaIndex
	"Primitive. Indicate the GC semaphore index to be signaled on GC occurance."
	<primitive: 'primitiveSetGCSemaphore'>
	^self primitiveFailed
"Example:

	| index sema process |
	sema := Semaphore new.
	index := Smalltalk registerExternalObject: sema.
	Smalltalk setGCSemaphore: index.
	process := [
		[[true] whileTrue:[
			sema wait.
			Smalltalk beep.
		]] ensure:[
			Smalltalk setGCSemaphore: 0.
			Smalltalk unregisterExternalObject: sema.
		].
	] fork.
	process inspect.
"! !

!SystemDictionary methodsFor: 'miscellaneous' stamp: 'MPH 10/24/2000 14:27'!
setMacFileInfoOn: aString
	"On Mac, set the file type and creator (noop on other platforms)"
	FileDirectory default
		setMacFileNamed: aString
		type: 'STch'
		creator: 'FAST'.! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'sd 11/16/2003 13:14'!
shutDown
	^ SmalltalkImage current closeSourceFiles! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'gk 2/23/2004 20:51'!
shutDownSound
	"No longer used in the release, but retained for backward compatibility."

	SoundService default shutDown
! !

!SystemDictionary methodsFor: 'memory space' stamp: ''!
signalLowSpace
	"Signal the low-space semaphore to alert the user that space is running low."

	LowSpaceSemaphore signal.! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'JMM 11/21/2000 21:02'!
snapshotEmbeddedPrimitive
	<primitive: 247>
	^nil "indicates error writing embedded image file"! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'ar 7/22/2000 14:34'!
snapshotPrimitive
	"Primitive. Write the current state of the object memory on a file in the
	same format as the Smalltalk-80 release. The file can later be resumed,
	returning you to this exact state. Return normally after writing the file.
	Essential. See Object documentation whatIsAPrimitive."

	<primitive: 97>
	^nil "indicates error writing image file"! !

!SystemDictionary methodsFor: 'special objects' stamp: ''!
specialNargsAt: anInteger 
	"Answer the number of arguments for the special selector at: anInteger."

	^ (self specialObjectsArray at: 24) at: anInteger * 2! !

!SystemDictionary methodsFor: 'special objects' stamp: ''!
specialObjectsArray  "Smalltalk specialObjectsArray at: 1"
	<primitive: 129>
	^ self primitiveFailed! !

!SystemDictionary methodsFor: 'special objects' stamp: ''!
specialSelectorAt: anInteger 
	"Answer the special message selector from the interleaved specialSelectors array."

	^ (self specialObjectsArray at: 24) at: anInteger * 2 - 1! !

!SystemDictionary methodsFor: 'special objects' stamp: ''!
specialSelectorSize
	"Answer the number of special selectors in the system."

	^ (self specialObjectsArray at: 24) size // 2! !

!SystemDictionary methodsFor: 'special objects' stamp: ''!
specialSelectors
	"Used by SystemTracer only."

	^SpecialSelectors! !

!SystemDictionary methodsFor: 'objects from disk' stamp: 'tk 3/7/2000 18:40'!
storeDataOn: aDataStream
	"I don't get stored.  Use a DiskProxy"

	self error: 'use a DiskProxy to store me'! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'alain.plantec 5/18/2009 16:00'!
testFormatter
	"Smalltalk testFormatter"

	"Reformats the source for every method in the system, and
	then compiles that source and verifies that it generates
	identical code"

	| newCodeString methodNode oldMethod newMethod badOnes n |
	badOnes := OrderedCollection new.
	self forgetDoIts.
	'Formatting all classes...' 
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: CompiledMethod instanceCount
		during: 
			[:bar | 
			n := 0.
			self systemNavigation allBehaviorsDo: 
					[:cls | 
					"Transcript cr; show: cls name."

					cls selectors do: 
							[:selector | 
							(n := n + 1) \\ 100 = 0 ifTrue: [bar value: n].
							newCodeString := cls prettyPrinterClass 
										format: (cls sourceCodeAt: selector)
										in: cls
										notifying: nil.
							methodNode := cls compilerClass new 
										compile: newCodeString
										in: cls
										notifying: nil
										ifFail: [].
							newMethod := methodNode generate: #(0 0 0 0).
							oldMethod := cls compiledMethodAt: selector.
							oldMethod = newMethod 
								ifFalse: 
									[Transcript
										cr;
										show: '***' , cls name , ' ' , selector.
									badOnes add: cls name , ' ' , selector]]]].
	self systemNavigation browseMessageList: badOnes asSortedCollection
		name: 'Formatter Discrepancies'! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'alain.plantec 5/18/2009 16:00'!
testFormatter2
	"Smalltalk testFormatter2"

	"Reformats the source for every method in the system, and
	then verifies that the order of source tokens is unchanged."

	| newCodeString badOnes n oldCodeString oldTokens newTokens |
	badOnes := OrderedCollection new.
	self forgetDoIts.
	'Formatting all classes...' 
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: CompiledMethod instanceCount
		during: 
			[:bar | 
			n := 0.
			self systemNavigation allBehaviorsDo: 
					[:cls | 
					"Transcript cr; show: cls name."

					cls selectors do: 
							[:selector | 
							(n := n + 1) \\ 100 = 0 ifTrue: [bar value: n].
							oldCodeString := (cls sourceCodeAt: selector) asString.
							newCodeString := cls prettyPrinterClass 
										format: oldCodeString
										in: cls
										notifying: nil.
							oldTokens := oldCodeString findTokens: Character separators.
							newTokens := newCodeString findTokens: Character separators.
							oldTokens = newTokens 
								ifFalse: 
									[Transcript
										cr;
										show: '***' , cls name , ' ' , selector.
									badOnes add: cls name , ' ' , selector]]]].
	self systemNavigation browseMessageList: badOnes asSortedCollection
		name: 'Formatter Discrepancies'! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'al 1/12/2006 23:54'!
traitNames
	"Answer a SortedCollection of all traits (not including class-traits) names."
	| names |
	names := OrderedCollection new.
	self do: 
		[:cl | (cl isInMemory
			and: [(cl isKindOf: Trait)
			and: [(cl name beginsWith: 'AnObsolete') not]])
				ifTrue: [names add: cl name]].
	^ names! !

!SystemDictionary methodsFor: 'special objects' stamp: 'JMM 6/6/2000 20:40'!
unregisterExternalObject: anObject
	"Unregister the given object in the external objects array. Do nothing if it isn't registered."

	ExternalSemaphoreTable unregisterExternalObject: anObject! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'sd 4/29/2003 19:06'!
unusedClasses
	"Enumerates all classes in the system and returns a list of those that are 
	apparently unused. A class is considered in use if it (a) has subclasses 
	or (b) is referred to by some method or (c) has its name in use as a 
	literal. "
	"Smalltalk unusedClasses asSortedCollection"
	^ self systemNavigation allUnusedClassesWithout: {{}. {}}! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'alain.plantec 2/8/2009 22:52'!
unusedClassesAndMethodsWithout: classesAndMessagesPair 
	"Accepts and returns a pair: {set of class names. set of selectors}. 
	It is expected these results will be diff'd with the normally unused 
	results. "
	| classRemovals messageRemovals nClasses nMessages |
	(classRemovals := IdentitySet new) addAll: classesAndMessagesPair first.
	(messageRemovals := IdentitySet new) addAll: classesAndMessagesPair second.
	nClasses := nMessages := -1.
	
	[ "As long as we keep making progress..."
	classRemovals size > nClasses or: [ messageRemovals size > nMessages ] ] whileTrue: 
		[ "...keep trying for bigger sets of unused classes and selectors."
		nClasses := classRemovals size.
		nMessages := messageRemovals size.
		UIManager default 
			informUser: 'Iterating removals ' translated, (classesAndMessagesPair first isEmpty 
					ifTrue: [ 'for baseline...' translated]
					ifFalse: [ 'for ' translated, classesAndMessagesPair first first , ' etc...' ]) , Character cr asString , nClasses printString , ' classes, ' , nMessages printString , ' messages.
|
|'
			during: 
				[ "spacers move menu off cursor"
				classRemovals addAll: (self systemNavigation allUnusedClassesWithout: { 
							classRemovals.
							messageRemovals
						 }).
				messageRemovals addAll: (self systemNavigation allUnSentMessagesWithout: { 
							classRemovals.
							messageRemovals
						 }) ] ].
	^ { 
		classRemovals.
		(self systemNavigation allUnSentMessagesWithout: {  classRemovals. messageRemovals  })
	 }! !

!SystemDictionary methodsFor: 'memory space' stamp: 'apb 10/3/2000 16:40'!
useUpMemory
	"For testing the low space handler..."
	"Smalltalk installLowSpaceWatcher; useUpMemory"

	| lst |
	lst := nil.
	[true] whileTrue: [
		lst := Link nextLink: lst.
	].! !

!SystemDictionary methodsFor: 'memory space' stamp: 'di 8/18/2000 21:15'!
useUpMemoryWithArrays 
	"For testing the low space handler..."
	"Smalltalk installLowSpaceWatcher; useUpMemoryWithArrays"

	| b |  "First use up most of memory."
	b := String new: self bytesLeft - self lowSpaceThreshold - 100000.
	b := b.  "Avoid unused value warning"
	(1 to: 10000) collect: [:i | Array new: 10000]! !

!SystemDictionary methodsFor: 'memory space' stamp: 'di 8/18/2000 16:49'!
useUpMemoryWithContexts 
	"For testing the low space handler..."
	"Smalltalk installLowSpaceWatcher; useUpMemoryWithContexts"

	self useUpMemoryWithContexts! !

!SystemDictionary methodsFor: 'memory space' stamp: 'di 8/18/2000 16:50'!
useUpMemoryWithTinyObjects 
	"For testing the low space handler..."
	"Smalltalk installLowSpaceWatcher; useUpMemoryWithTinyObjects"

	| b |  "First use up most of memory."
	b := String new: self bytesLeft - self lowSpaceThreshold - 100000.
	b := b.  "Avoid unused value warning"
	(1 to: 10000) collect: [:i | BitBlt new]! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'sd 4/17/2003 21:01'!
verifyChanges		"Smalltalk verifyChanges"
	"Recompile all methods in the changes file."
	self systemNavigation allBehaviorsDo: [:class | class recompileChanges].
! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'md 5/16/2006 12:34'!
version
	"Answer the version of this release."

	^SystemVersion current version! !

!SystemDictionary methodsFor: 'copying' stamp: 'tk 10/20/2000 11:35'!
veryDeepCopyWith: deepCopier
	"Return self.  I can't be copied.  Do not record me."! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'JMM 4/13/2005 20:35'!
wordSize
	"Answer the size (in bytes) of an object pointer."
	"Smalltalk wordSize"

	^[SmalltalkImage current vmParameterAt: 40] on: Error do: [4]! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'marcus.denker 12/15/2008 12:18'!
writeImageSegmentsFrom: segmentDictionary withKernel: kernel 
	"segmentDictionary is associates segmentName -> {classNames. methodNames},
	and kernel is another set of classNames determined to be essential.
	Add a partition, 'Secondary' with everything not in partitions and not in the kernel.
	Then write segments based on this partitioning of classes."
	"First, put all classes that are in no other partition, and not in kernel into a new partition called 'Secondary'.  Also remove any classes in kernel from putative partitions."
	| metas secondary dups segDict overlaps classes n symbolHolder |
	secondary := Smalltalk classNames asIdentitySet.
	segmentDictionary keysDo: 
		[ :segName | 
		secondary removeAllFoundIn: (segmentDictionary at: segName) first.
		(segmentDictionary at: segName) first removeAllFoundIn: kernel ].
	secondary removeAllFoundIn: kernel.
	secondary removeAllFoundIn: #(
			#PseudoContext
			#Utilities
			#Preferences
			#OutOfScopeNotification
			#FakeClassPool
			#BlockCannotReturn
			#FormSetFont
			#ExternalSemaphoreTable
			#NetNameResolver
			#ScreenController
			#InterpreterPlugin
			#Command
			#WeakSet
		).
	FileDirectory allSubclassesDo: 
		[ :c | 
		secondary 
			remove: c name
			ifAbsent: [  ] ].
	segmentDictionary 
		at: 'Secondary'
		put: {  secondary. {   }  }.

	"Now build segDict giving className -> segName, and report any duplicates."
	dups := Dictionary new.
	segDict := IdentityDictionary new: 3000.
	segmentDictionary keysDo: 
		[ :segName | 
		(segmentDictionary at: segName) first do: 
			[ :className | 
			(segDict includesKey: className) ifTrue: 
				[ (dups includesKey: className) ifFalse: 
					[ dups 
						at: className
						put: Array new ].
				dups 
					at: className
					put: (dups at: className) , {  segName  } ].
			segDict 
				at: className
				put: segName ] ].
	dups size > 0 ifTrue: 
		[ dups inspect.
		^ self error: 'Duplicate entries' ].

	"Then for every class in every partition, make sure that neither it
	nor any of its superclasses are in any other partition.  If they are,
	enter them in a dictionary of overlaps.
	If the dictionary is not empty, then stop and report it."
	overlaps := Dictionary new.
	segmentDictionary keysDo: 
		[ :segName | 
		classes := (segmentDictionary at: segName) first asArray collect: [ :k | Smalltalk at: k ].
		classes do: 
			[ :c | 
			(c isKindOf: Class) ifTrue: 
				[ c withAllSuperclasses do: 
					[ :sc | 
					n := segDict 
						at: sc name
						ifAbsent: [ segName ].
					n ~= segName ifTrue: 
						[ n = 'Secondary' 
							ifTrue: 
								[ (segmentDictionary at: 'Secondary') first 
									remove: sc name
									ifAbsent: [  ] ]
							ifFalse: 
								[ overlaps 
									at: c name
									put: (c withAllSuperclasses collect: 
										[ :cc | 
										segDict 
											associationAt: cc name
											ifAbsent: [ cc name -> 'Kernel' ] ]) ] ] ] ] ] ].
	overlaps size > 0 ifTrue: 
		[ overlaps inspect.
		^ self error: 'Superclasses in separate segments' ].

	"If there are no overlaps, then proceed to write the partitioned classes."
	symbolHolder := Symbol allSymbols.	"Hold onto Symbols with strong pointers, 
		so they will be in outPointers"
	segmentDictionary keysDo: 
		[ :segName | 
		UIManager default 
			informUser: segName
			during: 
				[ classes := (segmentDictionary at: segName) first asArray collect: [ :k | Smalltalk at: k ].
				metas := classes 
					select: [ :c | c isKindOf: Class ]
					thenCollect: [ :c | c class ].
				(ImageSegment new 
					copyFromRoots: classes , metas
					sizeHint: 0)
					extract;
					writeToFile: segName ] ].
	symbolHolder	"Keep compiler for getting uppity."! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'adrian-lienhard 5/27/2009 21:28'!
writeRecentCharacters: nCharacters toFileNamed: aFilename
	"Schedule an editable text view on the last n characters of changes."
	| changes |
	changes := SourceFiles at: 2.
	changes setToEnd; skip: nCharacters negated.
	(FileStream newFileNamed: aFilename) nextPutAll: (changes next: nCharacters); close; open; edit! !

!SystemDictionary methodsFor: 'sources, change log' stamp: 'ar 9/27/2005 22:38'!
writeRecentToFile
	"Smalltalk writeRecentToFile"
	| numChars aDirectory aFileName |
	aDirectory := FileDirectory default.
	aFileName := Utilities
				keyLike: 'squeak-recent.01'
				withTrailing: '.log'
				satisfying: [:aKey | (aDirectory includesKey: aKey) not].
	numChars := ChangeSet getRecentLocatorWithPrompt: 'copy logged source as far back as...'.
	numChars
		ifNotNil: [self writeRecentCharacters: numChars toFileNamed: aFileName]! !

Object subclass: #AbstractLauncher
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!AbstractLauncher commentStamp: '<historical>' prior: 0!
The class AutoStart in combination with the Launcher classes provides a mechanism for starting Squeak from the command line or a web page. Parameters on the command line or in the embed tag in the web page a parsed and stored in the lauchner's parameter dictionary.
Subclasses can access these parameters to determine what to do.

CommandLineLauncherExample provides an example for a command line application. if you start squeak with a command line 'class Integer' it will launch a class browser on class Integer.
To enable this execute
CommandLineLauncherExample activate
before you save the image.
To disable execute
CommandLineLauncherExample deactivate

The PluginLauchner is an example how to use this framework to start Squeak as a browser plugin. It looks for a parameter 'src' which should point to a file containing a squeak script.!

!AbstractLauncher class methodsFor: 'activation' stamp: 'mir 8/6/1999 18:14'!
activate
	"Register this launcher with the auto start class"

	self autoStarter addLauncher: self! !

!AbstractLauncher class methodsFor: 'private' stamp: 'mir 8/4/1999 13:57'!
autoStarter
	^AutoStart! !

!AbstractLauncher class methodsFor: 'activation' stamp: ''!
deactivate
	"Unregister this launcher with the auto start class"
	self autoStarter removeLauncher: self! !

!AbstractLauncher class methodsFor: 'private' stamp: 'sd 9/30/2003 13:55'!
extractParameters
	
	^ SmalltalkImage current extractParameters! !

!AbstractLauncher methodsFor: 'private' stamp: 'dc 5/30/2008 10:17'!
commandLine: aString 
	"Start up this launcher from within Squeak as if it Squeak been launched the given command line."
	| dict tokens cmd arg |
	dict := Dictionary new.
	tokens := (aString findTokens: ' ') readStream.
	
	[ cmd := tokens next.
	arg := tokens next.
	cmd ~~ nil and: [ arg ~~ nil ] ] whileTrue: 
		[ dict 
			at: cmd
			put: arg ].
	self parameters: dict.
	self startUp! !

!AbstractLauncher methodsFor: 'private' stamp: 'mir 9/23/1999 13:18'!
determineParameterNameFrom: alternateParameterNames
	"Determine which of the given alternate parameter names is actually used."

	^alternateParameterNames detect: [:each | self includesParameter: each asUppercase] ifNone: [nil] ! !

!AbstractLauncher methodsFor: 'private' stamp: 'mir 1/11/2000 16:35'!
includesParameter: parName
	"Return if the parameter named parName exists."
	^self parameters
		includesKey: parName asUppercase! !

!AbstractLauncher methodsFor: 'private' stamp: 'mdr 4/10/2001 10:50'!
numericParameterAtOneOf: alternateParameterNames ifAbsent: aBlock
	"Return the parameter named using one of the alternate names or an empty string"

	| parameterValue |
	parameterValue := self parameterAtOneOf: alternateParameterNames.
	parameterValue isEmpty
		ifTrue: [^aBlock value].
	^[Number readFrom: parameterValue] ifError: aBlock 

! !

!AbstractLauncher methodsFor: 'private' stamp: 'mir 8/4/1999 14:19'!
parameterAt: parName
	"Return the parameter named parName or an empty string"
	^self
		parameterAt: parName
		ifAbsent: ['']! !

!AbstractLauncher methodsFor: 'private' stamp: 'mir 1/11/2000 16:36'!
parameterAt: parName ifAbsent: aBlock
	"Return the parameter named parName.
	Evaluate the block if parameter does not exist."
	^self parameters
		at: parName asUppercase
		ifAbsent: [aBlock value]! !

!AbstractLauncher methodsFor: 'private' stamp: 'mir 9/23/1999 12:09'!
parameterAtOneOf: alternateParameterNames
	| parameterName |
	"Return the parameter named using one of the alternate names or an empty string"

	parameterName := self determineParameterNameFrom: alternateParameterNames.
	^parameterName isNil
		ifTrue: ['']
		ifFalse: [self parameterAt: parameterName ifAbsent: ['']]! !

!AbstractLauncher methodsFor: 'private' stamp: 'marcus.denker 9/14/2008 18:54'!
parameters
	parameters ifNil: [parameters := self class extractParameters].
	^parameters! !

!AbstractLauncher methodsFor: 'private' stamp: 'mir 7/29/1999 10:21'!
parameters: startupParameters
	parameters := startupParameters! !

!AbstractLauncher methodsFor: 'running' stamp: 'tk 10/24/2001 06:40'!
startUp
	"A backstop for subclasses.  Note that this is not a class message (most startUps are class messages)."

! !

Object subclass: #AbstractSoundSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!AbstractSoundSystem commentStamp: 'gk 2/24/2004 08:34' prior: 0!
This is the abstract base class for a sound system. A sound system offers a small protocol for playing sounds and making beeps and works like a facade towards the rest of Squeak. A sound system is registered in the application registry SoundService and can be accessed by "SoundService default" like for example:

SoundService default playSoundNamed: 'croak'

The idea is that as much sound playing as possible should go through this facade. This way we decouple the sound system from the rest of Squeak and make it pluggable. It also is a perfect spot to check for the Preference class>>soundsEnabled.

Two basic subclasses exist at the time of this writing, the BaseSoundSystem which represents the standard Squeak sound system, and the DummySoundSystem which is a dummy implementation that can be used when there is no sound card available, or when the base sound system isn't in the image, or when you simply don't want to use the available sound card.!

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/24/2004 23:27'!
beep
	"Make a primitive beep."

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:47'!
playSampledSound: samples rate: rate

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:50'!
playSoundNamed: soundName

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:51'!
playSoundNamed: soundName ifAbsentReadFrom: aifFileName

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:51'!
playSoundNamedOrBeep: soundName

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:52'!
randomBitsFromSoundInput: bitCount

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:52'!
sampledSoundChoices

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:53'!
shutDown
	"Default is to do nothing."! !

!AbstractSoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:56'!
soundNamed: soundName

	self subclassResponsibility! !

AbstractSoundSystem subclass: #DummySoundSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!DummySoundSystem commentStamp: 'gk 2/24/2004 23:14' prior: 0!
This is a dummy sound system registered in SoundService to absorb all sound playing and to use the primitive beep instead of sampled sounds when playing a beep.!

!DummySoundSystem class methodsFor: 'initialization' stamp: 'gk 2/23/2004 21:08'!
initialize
	SoundService register: self new.! !

!DummySoundSystem class methodsFor: 'initialization' stamp: 'gk 2/23/2004 21:08'!
unload
	SoundService registeredClasses do: [:ss |
		(ss isKindOf: self) ifTrue: [SoundService unregister: ss]].! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 2/24/2004 23:53'!
beep
	"Make a primitive beep."

	Beeper beepPrimitive! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:53'!
playSampledSound: samples rate: rate
	"Do nothing."
	! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:54'!
playSoundNamed: soundName
	"Do nothing."! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:54'!
playSoundNamed: soundName ifAbsentReadFrom: aifFileName
	"Do nothing."! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 4/8/2005 14:15'!
playSoundNamedOrBeep: soundName 
	"There is no sound support, so we make the beep."

	self beep! !

!DummySoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 20:48'!
randomBitsFromSoundInput: bitCount
 	"I'm not sure what the right thing to do here is."

 	self error: 'Can not provide random data.'! !

!DummySoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:54'!
sampledSoundChoices
	"No choices other than this."

	^ #('silence')! !

!DummySoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:55'!
soundNamed: soundName
	"There are no sounds to look up."

	^ nil! !

Object subclass: #Author
	instanceVariableNames: 'fullName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

Author class
	instanceVariableNames: 'uniqueInstance'!

!Author commentStamp: 'MiguelCoba 7/25/2009 01:09' prior: 0!
I am responsible for the full name used to identify the current code author.!

Author class
	instanceVariableNames: 'uniqueInstance'!

!Author class methodsFor: 'convenience' stamp: 'MiguelCoba 7/25/2009 01:58'!
fullName
	^ Author uniqueInstance fullName! !

!Author class methodsFor: 'convenience' stamp: 'MiguelCoba 7/25/2009 01:58'!
fullName: aString
	^ Author uniqueInstance fullName: aString! !

!Author class methodsFor: 'convenience' stamp: 'MiguelCoba 7/25/2009 01:58'!
fullNamePerSe
	^ Author uniqueInstance fullNamePerSe! !

!Author class methodsFor: 'deprecated' stamp: 'AndrewBlack 9/1/2009 06:24'!
initials
	self deprecated: 'Use ''Author fullName'' instead.'.
	^ Author uniqueInstance fullName! !

!Author class methodsFor: 'deprecated' stamp: 'AndrewBlack 9/1/2009 06:25'!
initials: aString
	self deprecated: 'Use ''Author fullName:'' instead.'.
	^ Author uniqueInstance fullName: aString! !

!Author class methodsFor: 'deprecated' stamp: 'AndrewBlack 9/1/2009 06:25'!
initialsPerSe
	self deprecated: 'Use ''Author fullNamePerSe'' instead.'.
	^ Author uniqueInstance fullNamePerSe! !

!Author class methodsFor: 'instance creation' stamp: 'on 5/10/2008 13:05'!
new
	self error: 'Author is a singleton -- send uniqueInstance instead'! !

!Author class methodsFor: 'convenience' stamp: 'MiguelCoba 7/25/2009 01:10'!
requestFullName
	^ Author uniqueInstance requestFullName! !

!Author class methodsFor: 'deprecated' stamp: 'AndrewBlack 9/1/2009 06:26'!
requestInitials
	self deprecated: 'Use ''Author requestFullName'' instead.'.
	^ Author uniqueInstance requestFullName! !

!Author class methodsFor: 'deprecated' stamp: 'AndrewBlack 9/1/2009 06:27'!
requestName
	self deprecated: 'Use ''Author requestFullName'' instead.'.
	^ Author uniqueInstance requestFullName! !

!Author class methodsFor: 'instance creation' stamp: 'on 5/10/2008 13:10'!
reset
	^ uniqueInstance := nil! !

!Author class methodsFor: 'instance creation' stamp: 'on 5/10/2008 13:09'!
uniqueInstance
	^ uniqueInstance ifNil: [ uniqueInstance := super new ]! !

!Author class methodsFor: 'instance creation' stamp: 'on 5/10/2008 13:27'!
uniqueInstance: anInstance
	"Needed by AuthorTest to restore saved instance"
	^ uniqueInstance := anInstance! !

!Author class methodsFor: 'testing-support' stamp: 'oscar.nierstrasz 10/18/2009 18:31'!
useAuthor: aString during: aBlock
	^ self uniqueInstance useAuthor: aString during: aBlock! !

!Author methodsFor: 'accessing' stamp: 'MiguelCoba 7/25/2009 02:41'!
fullName
	"Answer the full name to be used to identify the current code author."

	[fullName isEmptyOrNil ] whileTrue: [self requestFullName].
	^ fullName! !

!Author methodsFor: 'accessing' stamp: 'MiguelCoba 7/25/2009 00:55'!
fullName: aString
	fullName := aString.! !

!Author methodsFor: 'accessing' stamp: 'MiguelCoba 7/25/2009 00:58'!
fullNamePerSe

	"Answer the currently-prevailing author full name, such as it is, empty or not"

	^ fullName! !

!Author methodsFor: 'initialization' stamp: 'MiguelCoba 7/25/2009 03:07'!
initialize
	super initialize.
	fullName := ''.! !

!Author methodsFor: 'ui-requests' stamp: 'MiguelCoba 9/16/2009 12:10'!
messagePrompt
	^
'Please type your full name. 
It will be used to sign the changes you make to the image.
Avoid spaces, accents, dashes, underscore and similar characters.

Vincent van Gogh -> VincentVanGogh
Miguel Cob -> MiguelCoba
Gran Krampe -> GoranKrampe
Gran Krampe -> GoeranKrampe
Stphane DUCASSE -> StephaneDucasse
Yoshiki Oshima -> YoshikiOshima
'! !

!Author methodsFor: 'ui-requests' stamp: 'MiguelCoba 7/25/2009 01:54'!
requestFullName
	| initialAnswer |
	initialAnswer := fullName isEmptyOrNil
						ifTrue: ['FirstnameLastname' translated]
						ifFalse: [fullName].
	fullName := UIManager default 
			request: self messagePrompt
			initialAnswer: initialAnswer! !

!Author methodsFor: 'testing-support' stamp: 'oscar.nierstrasz 10/18/2009 18:27'!
useAuthor: aString during: aBlock
	| previous |
	previous := fullName.
	fullName := aString.
	^ aBlock ensure: [ fullName := previous ]! !

Object subclass: #AutoStart
	instanceVariableNames: 'parameters'
	classVariableNames: 'Active InstalledLaunchers'
	poolDictionaries: ''
	category: 'System-Support'!

!AutoStart class methodsFor: 'private' stamp: 'mir 9/7/2004 13:34'!
active
	^ Active == true! !

!AutoStart class methodsFor: 'private' stamp: 'mir 9/7/2004 13:36'!
active: aBoolean
	Active := aBoolean! !

!AutoStart class methodsFor: 'accessing' stamp: 'mir 7/28/1999 17:47'!
addLauncher: launcher
	self installedLaunchers add: launcher! !

!AutoStart class methodsFor: 'accessing' stamp: ''!
addLauncherFirst: launcher
	self installedLaunchers addFirst: launcher! !

!AutoStart class methodsFor: 'updating' stamp: 'sd 3/20/2008 22:26'!
checkForPluginUpdate
	| pluginVersion updateURL |
	World 
		ifNotNil: [
			World install.
			ActiveHand position: 100@100].
	HTTPClient isRunningInBrowser
		ifFalse: [^false].
	pluginVersion := AbstractLauncher extractParameters
		at: (SmalltalkImage current platformName copyWithout: Character space) asUppercase
		ifAbsent: [^false].
	updateURL := AbstractLauncher extractParameters
		at: 'UPDATE_URL'
		ifAbsent: [^false].
	^SystemVersion check: pluginVersion andRequestPluginUpdate: updateURL! !

!AutoStart class methodsFor: 'updating' stamp: 'sd 3/20/2008 22:26'!
checkForUpdates
	| availableUpdate updateServer |
	World 
		ifNotNil: [
			World install.
			ActiveHand position: 100@100].
	HTTPClient isRunningInBrowser
		ifFalse: [^self processUpdates].
	availableUpdate := (AbstractLauncher extractParameters
		at: 'UPDATE'
		ifAbsent: [''] ) asInteger.
	availableUpdate
		ifNil: [^false].
	updateServer := AbstractLauncher extractParameters
		at: 'UPDATESERVER'
		ifAbsent: [AbstractLauncher extractParameters
		at: 'UPDATE_SERVER'
		ifAbsent: ['Squeakland']].
	Utilities setUpdateServer: updateServer.
	^SystemVersion checkAndApplyUpdates: availableUpdate! !

!AutoStart class methodsFor: 'initialization' stamp: 'mir 7/28/1999 17:44'!
deinstall
	"AutoStart deinstall"

	Smalltalk removeFromStartUpList: AutoStart.
	InstalledLaunchers := nil! !

!AutoStart class methodsFor: 'initialization' stamp: 'mir 9/30/2004 15:05'!
initialize
	"AutoStart initialize"
	"Order: ExternalSettings, SecurityManager, AutoStart"
	Smalltalk addToStartUpList: AutoStart after: SecurityManager.
	Smalltalk addToShutDownList: AutoStart after: SecurityManager.! !

!AutoStart class methodsFor: 'private' stamp: 'mir 7/28/1999 17:43'!
installedLaunchers
	InstalledLaunchers ifNil: [
		InstalledLaunchers := OrderedCollection new].
	^InstalledLaunchers! !

!AutoStart class methodsFor: 'updating' stamp: 'CdG 10/17/2005 19:34'!
processUpdates
	"Process update files from a well-known update server.  This method is called at system startup time,   Only if the preference #updateFromServerAtStartup is true is the actual update processing undertaken automatically"
	| choice |
	(Preferences valueOfFlag: #updateFromServerAtStartup) ifTrue:
		[choice := UIManager default chooseFrom: #('Yes, Update' 'No, Not now' 'Don''t ask again')
			title: 'Shall I look for new code\updates on the server?' withCRs.
		choice = 1 ifTrue: [
			Utilities updateFromServer].
		choice = 3 ifTrue: [
			Preferences setPreference: #updateFromServerAtStartup toValue: false.
			self inform: 'Remember to save you image to make this setting permant.']].
	^false! !

!AutoStart class methodsFor: 'accessing' stamp: 'mir 7/28/1999 17:47'!
removeLauncher: launcher
	self installedLaunchers remove: launcher ifAbsent: []! !

!AutoStart class methodsFor: 'initialization' stamp: 'mir 9/30/2004 15:06'!
shutDown: quitting
	self active: false! !

!AutoStart class methodsFor: 'initialization' stamp: 'stephane.ducasse 4/13/2009 21:14'!
startUp: resuming
	"The image is either being newly started (resuming is true), or it's just been snapshotted.
	If this has just been a snapshot, skip all the startup stuff."

	| startupParameters launchers |
	self active ifTrue: [^self].
	self active: true.
	resuming ifFalse: [^self].

	HTTPClient determineIfRunningInBrowser.
	startupParameters := AbstractLauncher extractParameters.
	(startupParameters includesKey: 'apiSupported' asUppercase )
		ifTrue: [
			HTTPClient browserSupportsAPI: ((startupParameters at: 'apiSupported' asUppercase) asUppercase = 'TRUE').
			HTTPClient isRunningInBrowser
				ifFalse: [HTTPClient isRunningInBrowser: true]].
	self checkForUpdates
		ifTrue: [^self].
	self checkForPluginUpdate.
	launchers := self installedLaunchers collect: [:launcher |
		launcher new].
	launchers do: [:launcher |
		launcher parameters: startupParameters].
	launchers do: [:launcher |
		Smalltalk at: #WorldState ifPresent: [ :ws | ws addDeferredUIMessage: [launcher startUp]]]! !

Object subclass: #Beeper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

Beeper class
	instanceVariableNames: 'default'!

!Beeper commentStamp: 'gk 2/26/2004 22:44' prior: 0!
Beeper provides simple audio (or in some other way) feedback to the user.

The recommended use is "Beeper beep" to give the user the equivalence of a beep. If you want to force the beep to use the primitive in the VM for beeping, then use "Beeper beepPrimitive". In either case, if sounds are disabled there will be no beep.

The actual beeping, when you use "Beeper beep", is done by sending a #play message to a registered playable object. You can register your own playable object by invoking the class side method #setDefault: passing in an object that responds to the #play message.

The default playable object is an instance of Beeper itself which implements #play on the instance side. That implementation delegates the playing of the beep to the default SoundService.

Note that #play is introduced as a common interface between AbstractSound and Beeper.
This way we can register instances of AbstractSound as playable entities, for example:

	Beeper setDefault: (SampledSound new
						setSamples: self coffeeCupClink
						samplingRate: 12000).

Then "Beeper beep" will play the coffeeCup sound.!

Beeper class
	instanceVariableNames: 'default'!

!Beeper class methodsFor: 'beeping' stamp: 'pavel.krivanek 3/11/2009 07:39'!
beep
	"The preferred way of producing an audible feedback.
	The default playable entity (an instance of Beeper)
	also uses the pluggable SoundService
	mechanism, so it will use the primitive beep only
	if there is no other sound mechanism available."
	
	self default 
		ifNil: [self beepPrimitive]
		ifNotNil: [ self default play].
! !

!Beeper class methodsFor: 'beeping' stamp: 'gk 2/24/2004 08:38'!
beepPrimitive
	"Make a primitive beep. Only use this if
	you want to force this to be a primitive beep.
	Otherwise use Beeper class>>beep
	since this method bypasses the current
	registered playable entity."

	Preferences soundsEnabled ifTrue: [
		self primitiveBeep]! !

!Beeper class methodsFor: 'customize' stamp: 'gk 2/22/2004 17:51'!
clearDefault
	"Clear the default playable.
	Will be lazily initialized in Beeper class >>default."

	default := nil! !

!Beeper class methodsFor: 'customize' stamp: 'gk 2/22/2004 17:55'!
default
	"When the default is not defined it is
	initialized using #newDefault."

	default isNil 
		ifTrue: [default := self newDefault ].
	^ default! !

!Beeper class methodsFor: 'customize' stamp: 'gk 2/24/2004 22:12'!
newDefault
	"Subclasses may override me to provide a default beep.
	This base implementation returns an instance of Beeper
	which uses the pluggable sound service."

	^ self new! !

!Beeper class methodsFor: 'private' stamp: 'gk 2/24/2004 23:51'!
primitiveBeep
	"Make a primitive beep. Not to be called directly.
	It is much better to use Beeper class>>beep
	or Beeper class>>beepPrimitive
	since this method bypasses the current
	registered playable entity and does not
	check Preferences class>>soundsEnabled."

	<primitive: 140>
	self primitiveFailed! !

!Beeper class methodsFor: 'customize' stamp: 'gk 2/22/2004 17:54'!
setDefault: aPlayableEntity
	"Set the playable entity used when making a beep.
	The playable entity should implement the message #play."

	default := aPlayableEntity! !

!Beeper methodsFor: 'play interface' stamp: 'gk 2/24/2004 23:25'!
play
	"This is how the default Beeper makes a beep,
	by sending beep to the default sound service.
	The sound system will check if sounds are enabled."

	SoundService default beep! !

Object subclass: #ColorTheme
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!ColorTheme classSide methodsFor: 'accessing' stamp: 'marcus.denker 11/29/2008 22:53'!
current
	^ self new! !

!ColorTheme methodsFor: 'theme' stamp: 'marcus.denker 11/29/2008 22:55'!
baseColors
	^ Array
		with: (Color fromArray: #(0.2 0.3 0.9 ))
		with: (Color fromArray: #(0.6 0.7 1.0 ))
		with: (Color fromArray: #(0.85 0.9 1.0 ))! !

!ColorTheme methodsFor: 'theme' stamp: 'dgd 11/2/2004 21:23'!
cancelColor
	^ Color lightRed! !

!ColorTheme methodsFor: 'theme - dialogs' stamp: 'dgd 11/3/2004 19:27'!
dialog3DTitles
	^ true! !

!ColorTheme methodsFor: 'theme - dialogs' stamp: 'dgd 11/2/2004 19:56'!
dialogBorderColor
	^ Color fromArray: #(0.355 0.516 1.0 )! !

!ColorTheme methodsFor: 'theme - dialogs' stamp: 'dgd 11/2/2004 19:54'!
dialogBorderWidth
	^ 4! !

!ColorTheme methodsFor: 'theme - dialogs' stamp: 'dgd 11/2/2004 19:59'!
dialogButtonBorderWidth
	^ 0! !

!ColorTheme methodsFor: 'theme - dialogs' stamp: 'dgd 11/2/2004 21:09'!
dialogColor
	^ Color paleYellow! !

!ColorTheme methodsFor: 'theme - dialogs' stamp: 'dgd 11/3/2004 19:49'!
dialogPaneBorderColor
	^ Color black
! !

!ColorTheme methodsFor: 'theme - dialogs' stamp: 'dgd 11/3/2004 19:44'!
dialogPaneBorderWidth
	^ 0! !

!ColorTheme methodsFor: 'theme - dialogs' stamp: 'dgd 11/3/2004 19:30'!
dialogPaneRampOrColor
	^ {0.0 -> (Color r: 0.742 g: 0.871 b: 1.0).
		1.0 -> (Color r: 0.516 g: 0.645 b: 1.0)}! !

!ColorTheme methodsFor: 'theme - dialogs' stamp: 'dgd 11/3/2004 19:38'!
dialogRampOrColor
	^ {0.0 -> (Color r: 0.516 g: 0.645 b: 1.0).
		1.0 -> (Color r: 0.742 g: 0.871 b: 1.0)}! !

!ColorTheme methodsFor: 'theme - dockingbar' stamp: 'marcus.denker 11/29/2008 22:52'!
dockingBarAutoGradient
	^ true! !

!ColorTheme methodsFor: 'theme - dockingbar' stamp: 'marcus.denker 11/29/2008 22:52'!
dockingBarColor
	^self normal:1! !

!ColorTheme methodsFor: 'theme - dockingbar' stamp: 'marcus.denker 11/29/2008 22:52'!
dockingBarGradientRamp
	^ {0.0 -> Color white. 1.0
		-> (self normal:1)}! !

!ColorTheme methodsFor: 'theme' stamp: 'marcus.denker 11/29/2008 22:57'!
helpColor
^ self okColor! !

!ColorTheme methodsFor: 'theme - menus' stamp: 'dgd 11/2/2004 17:30'!
menuBorderColor
	^ self subclassResponsibility! !

!ColorTheme methodsFor: 'theme - menus' stamp: 'dgd 11/2/2004 17:31'!
menuBorderWidth
	^ self subclassResponsibility! !

!ColorTheme methodsFor: 'theme - menus' stamp: 'dgd 11/2/2004 17:31'!
menuColor
	^ self subclassResponsibility! !

!ColorTheme methodsFor: 'theme - menus' stamp: 'dgd 11/2/2004 17:32'!
menuLineColor
	^ self subclassResponsibility! !

!ColorTheme methodsFor: 'theme - menus' stamp: 'dgd 11/2/2004 17:32'!
menuSelectionColor
	^ self subclassResponsibility! !

!ColorTheme methodsFor: 'theme - menus' stamp: 'dgd 11/2/2004 17:32'!
menuTitleBorderColor
	^ self subclassResponsibility! !

!ColorTheme methodsFor: 'theme - menus' stamp: 'dgd 11/2/2004 17:32'!
menuTitleBorderWidth
	^ self subclassResponsibility! !

!ColorTheme methodsFor: 'theme - menus' stamp: 'dgd 11/2/2004 17:32'!
menuTitleColor
	^ self subclassResponsibility! !

!ColorTheme methodsFor: 'theme' stamp: 'marcus.denker 11/29/2008 23:06'!
normal: index 
	^  (self baseColors second wheel: 8) at: index! !

!ColorTheme methodsFor: 'theme' stamp: 'dgd 11/2/2004 21:22'!
okColor
	^ Color lightGreen! !

Object subclass: #DefaultExternalDropHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!DefaultExternalDropHandler commentStamp: 'dgd 4/5/2004 19:07' prior: 0!
An alternative default handler that uses the file-list services to process files.
!

!DefaultExternalDropHandler class methodsFor: 'initialization' stamp: 'dgd 4/5/2004 19:10'!
initialize
	"initialize the receiver"
	ExternalDropHandler defaultHandler: self new! !

!DefaultExternalDropHandler class methodsFor: 'initialization' stamp: 'dgd 4/5/2004 19:09'!
unload
	"initialize the receiver"
	ExternalDropHandler defaultHandler: nil! !

!DefaultExternalDropHandler methodsFor: 'private' stamp: 'alain.plantec 2/8/2009 22:02'!
chooseServiceFrom: aCollection
	"private - choose a service from aCollection asking the user if  
	needed"
	aCollection size = 1
		ifTrue: [^ aCollection anyOne].
	^ UIManager default chooseFrom: (aCollection collect: [:each | each label]) values: aCollection.
! !

!DefaultExternalDropHandler methodsFor: 'event handling' stamp: 'bf 9/21/2004 18:44'!
handle: dropStream in: pasteUp dropEvent: anEvent 
	"the file was just droped, let's do our job"
	| fileName services theOne |
	fileName := dropStream name.
	""
	services := self servicesForFileNamed: fileName.
	""
	"no service, default behavior"
	services isEmpty
		ifTrue: [""
			dropStream edit.
			^ self].
	""
	theOne := self chooseServiceFrom: services.
	theOne isNil

		ifFalse: [theOne performServiceFor: dropStream]! !

!DefaultExternalDropHandler methodsFor: 'private' stamp: 'dgd 4/5/2004 19:23'!
servicesForFileNamed: aString 
	"private - answer a collection of file-services for the file named  
	aString"
	| allServices |
	allServices := FileList itemsForFile: aString.
	^ allServices
		reject: [:svc | self unwantedSelectors includes: svc selector]! !

!DefaultExternalDropHandler methodsFor: 'private' stamp: 'dgd 4/5/2004 19:23'!
unwantedSelectors
	"private - answer a collection well known unwanted selectors "
	^ #(#removeLineFeeds: #addFileToNewZip: #compressFile: #putUpdate: )! !

Object subclass: #ExternalDropHandler
	instanceVariableNames: 'action type extension'
	classVariableNames: 'DefaultHandler RegisteredHandlers'
	poolDictionaries: ''
	category: 'System-Support'!

!ExternalDropHandler class methodsFor: 'private' stamp: 'alain.plantec 2/8/2009 22:06'!
chooseServiceFrom: aCollection
	"private - choose a service from aCollection asking the user if  
	needed"
	aCollection size = 1
		ifTrue: [^ aCollection anyOne].
	""
	^ UIManager default chooseFrom: (aCollection collect: [:each | each label]) values: aCollection.
! !

!ExternalDropHandler class methodsFor: 'private' stamp: 'mir 1/10/2002 17:23'!
defaultGZipHandler
	^ExternalDropHandler
		type: nil
		extension: 'gz'
		action: [:stream :pasteUp :event |
			stream viewGZipContents]! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: 'mir 1/10/2002 17:17'!
defaultHandler
	DefaultHandler ifNil: [DefaultHandler := ExternalDropHandler type: nil extension: nil action: [:dropStream | dropStream edit]].
	^DefaultHandler! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: 'mir 1/10/2002 16:54'!
defaultHandler: externalDropHandler
	DefaultHandler := externalDropHandler! !

!ExternalDropHandler class methodsFor: 'private' stamp: 'stephane.ducasse 4/13/2009 21:13'!
defaultImageHandler
	| image sketch |
	^ExternalDropHandler
		type: 'image/'
		extension: nil
		action: [:stream :pasteUp :event |
			stream binary.
			image := Form fromBinaryStream: ((RWBinaryOrTextStream with: stream contents) reset).
			Project current resourceManager 
				addResource: image 
				url: (FileDirectory urlForFileNamed: stream name) asString.
			sketch := World drawingClass withForm: image.
			pasteUp addMorph: sketch centeredNear: event position.
			image := sketch := nil]! !

!ExternalDropHandler class methodsFor: 'initialization' stamp: 'StephaneDucasse 11/6/2009 17:43'!
initialize
	"ExternalDropHandler initialize"

	self resetRegisteredHandlers.
	self
		registerHandler: self defaultImageHandler;
		registerHandler: self defaultGZipHandler.
	! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: 'mir 8/24/2004 15:37'!
lookupExternalDropHandler: stream

	| types extension serviceHandler |
	types := stream mimeTypes.

	types ifNotNil: [
		self registeredHandlers do: [:handler | 
			(handler matchesTypes: types)
				ifTrue: [^handler]]].

	extension := FileDirectory extensionFor: stream name.
	self registeredHandlers do: [:handler | 
		(handler matchesExtension: extension)
				ifTrue: [^handler]].
	serviceHandler := self lookupServiceBasedHandler: stream.
	^serviceHandler
		ifNil: [self defaultHandler]! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: 'mir 8/24/2004 17:15'!
lookupServiceBasedHandler: dropStream
	"the file was just droped, let's do our job"
	| fileName services theOne |
	fileName := dropStream name.

	services := (FileList itemsForFile: fileName)
		reject: [:svc | self unwantedSelectors includes: svc selector].

	"no service, default behavior"
	services isEmpty
		ifTrue: [^nil].

	theOne := self chooseServiceFrom: services.
	^theOne
		ifNotNil: [ExternalDropHandler type: nil extension: nil action: [:stream | theOne performServiceFor: stream]]! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: 'mir 1/10/2002 17:19'!
registerHandler: aHandler
	self registeredHandlers add: aHandler! !

!ExternalDropHandler class methodsFor: 'initialization' stamp: 'nk 6/12/2004 16:15'!
registerStandardExternalDropHandlers
	"ExternalDropHandler registerStandardExternalDropHandlers"

	self registeredHandlers add: (
		ExternalDropHandler
			type: 'image/'
			extension: nil
			action: [:stream :pasteUp :event |
				pasteUp addMorph: (World drawingClass withForm: (Form fromBinaryStream: stream binary)) centeredNear: event position])! !

!ExternalDropHandler class methodsFor: 'private' stamp: 'mir 1/10/2002 15:57'!
registeredHandlers
	RegisteredHandlers ifNil: [RegisteredHandlers := OrderedCollection new].
	^RegisteredHandlers! !

!ExternalDropHandler class methodsFor: 'private' stamp: 'mir 1/10/2002 15:57'!
resetRegisteredHandlers
	RegisteredHandlers := nil! !

!ExternalDropHandler class methodsFor: 'instance creation' stamp: 'mir 1/10/2002 17:16'!
type: aType extension: anExtension action: anAction 
	^self new type: aType extension: anExtension action: anAction ! !

!ExternalDropHandler class methodsFor: 'private' stamp: 'mir 8/24/2004 15:28'!
unwantedSelectors
	"private - answer a collection well known unwanted selectors "
	^ #(#removeLineFeeds: #addFileToNewZip: #compressFile: #putUpdate: )! !

!ExternalDropHandler methodsFor: 'accessing' stamp: 'mir 1/10/2002 15:54'!
extension
	^extension! !

!ExternalDropHandler methodsFor: 'accessing' stamp: 'mir 1/10/2002 17:29'!
handle: dropStream in: pasteUp dropEvent: anEvent
	| numArgs |
	numArgs := action numArgs.
	numArgs == 1
		ifTrue: [^action value: dropStream].
	numArgs == 2
		ifTrue: [^action value: dropStream value: pasteUp].
	numArgs == 3
		ifTrue: [^action value: dropStream value: pasteUp value: anEvent].
	self error: 'Wrong number of args for dop action.'! !

!ExternalDropHandler methodsFor: 'testing' stamp: 'spfa 5/25/2004 13:38'!
matchesExtension: aExtension
	(self extension isNil or: [aExtension isNil])
		ifTrue: [^false].
	FileDirectory activeDirectoryClass isCaseSensitive
		ifTrue: [^extension = aExtension]
		ifFalse: [^extension sameAs: aExtension]! !

!ExternalDropHandler methodsFor: 'testing' stamp: 'mir 1/10/2002 16:35'!
matchesTypes: types
	(self type isNil or: [types isNil])
		ifTrue: [^false].
	^types anySatisfy: [:mimeType | mimeType beginsWith: self type]! !

!ExternalDropHandler methodsFor: 'accessing' stamp: 'mir 1/10/2002 15:54'!
type
	^type! !

!ExternalDropHandler methodsFor: 'initialize' stamp: 'mir 1/10/2002 17:17'!
type: aType extension: anExtension action: anAction 
	action := anAction.
	type := aType.
	extension := anExtension! !

Object subclass: #ExternalSemaphoreTable
	instanceVariableNames: ''
	classVariableNames: 'ProtectTable'
	poolDictionaries: ''
	category: 'System-Support'!

!ExternalSemaphoreTable commentStamp: '<historical>' prior: 0!
By John M McIntosh johnmci@smalltalkconsulting.com
This class was written to mange the external semaphore table. When I was writing a Socket test server I discovered various race conditions on the access to the externalSemaphore table. This new class uses class side methods to restrict access using a mutex semaphore. It seemed cleaner to deligate the reponsibility here versus adding more code and another class variable to SystemDictionary 

Note that in Smalltalk recreateSpecialObjectsArray we still directly play with the table.!

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'JMM 6/6/2000 20:36'!
clearExternalObjects
	"Clear the array of objects that have been registered for use in non-Smalltalk code."

	ProtectTable critical: [Smalltalk specialObjectsArray at: 39 put: Array new].
! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'JMM 6/6/2000 21:01'!
externalObjects
	^ProtectTable critical: [Smalltalk specialObjectsArray at: 39].! !

!ExternalSemaphoreTable class methodsFor: 'initialize' stamp: 'JMM 6/6/2000 20:32'!
initialize
	ProtectTable := Semaphore forMutualExclusion! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'JMM 6/6/2000 20:44'!
registerExternalObject: anObject
	^ ProtectTable critical: [self safelyRegisterExternalObject: anObject]
! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'JMM 6/6/2000 20:57'!
safelyRegisterExternalObject: anObject
	"Register the given object in the external objects array and return its index. If it is already there, just return its index."

	| objects firstEmptyIndex obj sz newObjects |
	objects := Smalltalk specialObjectsArray at: 39.

	"find the first empty slot"
	firstEmptyIndex := 0.
	1 to: objects size do: [:i |
		obj := objects at: i.
		obj == anObject ifTrue: [^ i].  "object already there, just return its index"
		(obj == nil and: [firstEmptyIndex = 0]) ifTrue: [firstEmptyIndex := i]].

	"if no empty slots, expand the array"
	firstEmptyIndex = 0 ifTrue: [
		sz := objects size.
		newObjects := objects species new: sz + 20.  "grow linearly"
		newObjects replaceFrom: 1 to: sz with: objects startingAt: 1.
		firstEmptyIndex := sz + 1.
		Smalltalk specialObjectsArray at: 39 put: newObjects.
		objects := newObjects].

	objects at: firstEmptyIndex put: anObject.
	^ firstEmptyIndex
! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'JMM 6/6/2000 20:59'!
safelyUnregisterExternalObject: anObject
	"Unregister the given object in the external objects array. Do nothing if it isn't registered.
	JMM change to return if we clear the element, since it should only appear once in the array"

	| objects |
	anObject ifNil: [^ self].
	objects := Smalltalk specialObjectsArray at: 39.
	1 to: objects size do: [:i |
		(objects at: i) == anObject ifTrue: 
		[objects at: i put: nil.
		^self]].
! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'JMM 6/6/2000 20:45'!
unregisterExternalObject: anObject
	ProtectTable critical: [self safelyUnregisterExternalObject: anObject]
! !

Object subclass: #ExternalSettings
	instanceVariableNames: ''
	classVariableNames: 'RegisteredClients'
	poolDictionaries: ''
	category: 'System-Support'!

!ExternalSettings commentStamp: '<historical>' prior: 0!
ExternalSettings manages settings kept externally, e.g. files.
Objects can register themselves as clients to be notified at startup time to read their settings.

Eventually all the preferences should be managed through this mechanism.
!

!ExternalSettings class methodsFor: 'accessing' stamp: 'sw 1/25/2002 12:39'!
assuredPreferenceDirectory
	"Answer the preference directory, creating it if necessary"

	|  prefDir |
	prefDir := self preferenceDirectory.
	prefDir
		ifNil:
			[prefDir := FileDirectory default directoryNamed: self preferenceDirectoryName.
			prefDir assureExistence].
	^ prefDir! !

!ExternalSettings class methodsFor: 'initialization' stamp: 'ar 8/23/2001 22:56'!
initialize
	"ExternalSettings initialize"
	"Order: ExternalSettings, SecurityManager, AutoStart"
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self! !

!ExternalSettings class methodsFor: 'accessing' stamp: 'dc 5/30/2008 10:17'!
parseServerEntryArgsFrom: stream 
	"Args are in the form <argName>: <argValueString> delimited by end of line.
	It's not a very robust format and should be replaced by something like XML later.
	But it avoids evaluating the entries for security reasons."
	| entries lineStream entryName entryValue |
	entries := Dictionary new.
	stream skipSeparators.
	[ stream atEnd ] whileFalse: 
		[ lineStream := stream nextLine readStream.
		entryName := lineStream upTo: $:.
		lineStream skipSeparators.
		entryValue := lineStream upToEnd.
		(entryName isEmptyOrNil or: [ entryValue isEmptyOrNil ]) ifFalse: 
			[ entries 
				at: entryName
				put: entryValue withoutTrailingBlanks ].
		stream skipSeparators ].
	^ entries! !

!ExternalSettings class methodsFor: 'accessing' stamp: 'sd 9/30/2003 14:01'!
preferenceDirectory
	| prefDirName path |
	prefDirName := self preferenceDirectoryName.
	path := SmalltalkImage current vmPath.
	^(FileDirectory default directoryExists: prefDirName)
		ifTrue: [FileDirectory default directoryNamed: prefDirName]
		ifFalse: [
			((FileDirectory on: path) directoryExists: prefDirName)
				ifTrue: [(FileDirectory on: path) directoryNamed: prefDirName]
				ifFalse: [nil]]
! !

!ExternalSettings class methodsFor: 'accessing' stamp: 'mir 11/16/2001 13:33'!
preferenceDirectoryName
	^'prefs'! !

!ExternalSettings class methodsFor: 'accessing' stamp: 'mir 6/25/2001 18:45'!
registerClient: anObject
	"Register anObject as a settings client to be notified on startup."

	self registeredClients add: anObject! !

!ExternalSettings class methodsFor: 'private' stamp: 'mir 6/25/2001 18:46'!
registeredClients
	RegisteredClients ifNil: [RegisteredClients := Set new].
	^RegisteredClients! !

!ExternalSettings class methodsFor: 'initialization' stamp: 'mir 8/22/2001 15:17'!
shutDown
	"Look for external defs and load them."
	"ExternalSettings shutDown"

	self registeredClients do: [:client | 
		client releaseExternalSettings]! !

!ExternalSettings class methodsFor: 'initialization' stamp: 'mir 11/16/2001 13:29'!
startUp
	"Look for external defs and load them."
	"ExternalSettings startUp"

	| prefDir |
	prefDir := self preferenceDirectory.
	prefDir
		ifNil: [^self].
	self registeredClients do: [:client | 
		client fetchExternalSettingsIn: prefDir]! !

Object subclass: #HTTPClient
	instanceVariableNames: ''
	classVariableNames: 'BrowserSupportsAPI RunningInBrowser'
	poolDictionaries: ''
	category: 'System-Support'!

!HTTPClient class methodsFor: 'initialization' stamp: 'mir 4/2/2002 15:37'!
browserSupportsAPI
	^BrowserSupportsAPI == true! !

!HTTPClient class methodsFor: 'initialization' stamp: 'mir 4/2/2002 15:36'!
browserSupportsAPI: aBoolean
	BrowserSupportsAPI := aBoolean! !

!HTTPClient class methodsFor: 'utilities' stamp: 'PeterHugossonMiller 9/3/2009 09:59'!
composeMailTo: address subject: subject body: body
	"HTTPClient composeMailTo: 'michael.rueger@squeakland.org' subject: 'test subject' body: 'message' "
	| mailTo |
	mailTo := String new writeStream.
	mailTo nextPutAll: 'mailto:'.
	mailTo
		nextPutAll: address;
		nextPut: $?.
	subject isEmptyOrNil
		ifFalse: [mailTo nextPutAll: 'subject='; nextPutAll: subject; nextPut: $&].
	body isEmptyOrNil
		ifFalse: [mailTo nextPutAll: 'body='; nextPutAll: body].

	self httpGet: mailTo contents! !

!HTTPClient class methodsFor: 'initialization' stamp: 'mir 2/2/2001 17:27'!
determineIfRunningInBrowser
	"HTTPClient determineIfRunningInBrowser"

	RunningInBrowser := StandardFileStream isRunningAsBrowserPlugin
! !

!HTTPClient class methodsFor: 'examples' stamp: 'adrian_lienhard 7/18/2009 15:56'!
exampleMailTo
	"HTTPClient exampleMailTo"

	HTTPClient mailTo: 'm.rueger@acm.org' message: 'A test message from within Pharo'
! !

!HTTPClient class methodsFor: 'examples' stamp: 'md 7/28/2005 10:33'!
examplePostArgs
	"HTTPClient examplePostArgs"

	| args result |
	args := Dictionary new.
	args
		at: 'arg1' put: #('val1');
		at: 'arg2' put: #('val2');
		yourself.
	result := HTTPClient httpPostDocument: 'http://www.squeaklet.com/cgi-bin/thrd.pl [^]' args: args.
	Transcript show: result content; cr; cr.

! !

!HTTPClient class methodsFor: 'examples' stamp: 'mir 2/2/2001 17:44'!
examplePostMultipart
	"HTTPClient examplePostMultipart"

	| args result |
	args := Dictionary new.
	args
		at: 'arg1' put: #('val1');
		at: 'arg2' put: #('val2');
		yourself.
	result := HTTPClient httpPostMultipart: 'http://www.squeaklet.com/cgi-bin/thrd.pl'  args: args.
	Transcript show: result content; cr; cr.

! !

!HTTPClient class methodsFor: 'utilities' stamp: 'mir 5/13/2003 10:43'!
getDirectoryListing: dirListURL
	"HTTPClient getDirectoryListing: 'http://www.squeakalpha.org/uploads' "
	| answer ftpEntries |
"	answer := self 
		httpPostDocument: dirListURL
		args: Dictionary new."
	"Workaround for Mac IE problem"
	answer := self httpGetDocument: dirListURL.
	answer isString
		ifTrue: [^self error: 'Listing failed: ' , answer]
		ifFalse: [answer := answer content].
	answer first == $<
		ifTrue: [self error: 'Listing failed: ' , answer].
	ftpEntries := answer findTokens: String crlf.
	^ ftpEntries 
		collect:[:ftpEntry | ServerDirectory parseFTPEntry: ftpEntry]
		thenSelect: [:entry | entry notNil]! !

!HTTPClient class methodsFor: 'post/get' stamp: 'nk 8/30/2004 07:50'!
httpGet: url
	| document |
	document := self httpGetDocument: url.
	^(document isString)
		ifTrue: [
			"strings indicate errors"
			document]
		ifFalse: [(RWBinaryOrTextStream with: document content) reset]! !

!HTTPClient class methodsFor: 'post/get' stamp: 'mir 5/11/2001 12:55'!
httpGetDocument: url
	| stream content | 
	^self shouldUsePluginAPI
		ifTrue: [
			stream := FileStream requestURLStream: url ifError: [self error: 'Error in get from ' , url printString].
			stream ifNil: [^''].
			stream position: 0.
			content := stream upToEnd.
			stream close.
			MIMEDocument content: content]
		ifFalse: [HTTPSocket httpGetDocument: url]! !

!HTTPClient class methodsFor: 'post/get' stamp: 'mir 5/1/2001 15:04'!
httpPostDocument: url args: argsDict
	^self httpPostDocument: url target: nil args: argsDict! !

!HTTPClient class methodsFor: 'post/get' stamp: 'mir 5/1/2001 15:06'!
httpPostDocument: url target: target args: argsDict
	| argString stream content |
	^self shouldUsePluginAPI
		ifTrue: [
			argString := argsDict
				ifNotNil: [argString := HTTPSocket argString: argsDict]
				ifNil: [''].
			stream := FileStream post: argString , ' ' target: target url: url , argString ifError: [self error: 'Error in post to ' , url printString].
			stream position: 0.
			content := stream upToEnd.
			stream close.
			MIMEDocument content: content]
		ifFalse: [HTTPSocket httpPostDocument: url  args: argsDict]! !

!HTTPClient class methodsFor: 'post/get' stamp: 'mir 5/1/2001 12:51'!
httpPostMultipart: url args: argsDict
	" do multipart/form-data encoding rather than x-www-urlencoded "

	^self shouldUsePluginAPI
		ifTrue: [self pluginHttpPostMultipart: url args: argsDict]
		ifFalse: [HTTPSocket httpPostMultipart: url args: argsDict accept: nil request: '']! !

!HTTPClient class methodsFor: 'testing' stamp: 'ccn 3/14/2001 19:56'!
isRunningInBrowser

	RunningInBrowser isNil
		ifTrue: [self determineIfRunningInBrowser].
	^RunningInBrowser! !

!HTTPClient class methodsFor: 'testing' stamp: 'mir 8/4/2003 13:44'!
isRunningInBrowser: aBoolean
 	"Override the automatic process.
 	This should be used with caution.
 	One way to determine it without using the primitive is to check for parameters typically only encountered when running as a plugin."
 
 	RunningInBrowser := aBoolean! !

!HTTPClient class methodsFor: 'utilities' stamp: 'mir 5/1/2001 12:51'!
mailTo: address message: aString
	HTTPClient shouldUsePluginAPI
		ifFalse: [^self error: 'You need to run inside a web browser.'].
	FileStream post: aString url: 'mailto:' , address ifError: [self error: 'Can not send mail']! !

!HTTPClient class methodsFor: 'private' stamp: 'PeterHugossonMiller 9/3/2009 09:59'!
pluginHttpPostMultipart: url args: argsDict
	| mimeBorder argsStream crLf fieldValue resultStream result |
	" do multipart/form-data encoding rather than x-www-urlencoded "

	crLf := String crlf.
	mimeBorder := '----pharo-', Time millisecondClockValue printString, '-stuff-----'.
	"encode the arguments dictionary"
	argsStream := String new writeStream.
	argsDict associationsDo: [:assoc |
		assoc value do: [ :value |
		"print the boundary"
		argsStream nextPutAll: '--', mimeBorder, crLf.
		" check if it's a non-text field "
		argsStream nextPutAll: 'Content-disposition: form-data; name="', assoc key, '"'.
		(value isKindOf: MIMEDocument)
			ifFalse: [fieldValue := value]
			ifTrue: [argsStream nextPutAll: ' filename="', value url pathForFile, '"', crLf, 'Content-Type: ', value contentType.
				fieldValue := (value content
					ifNil: [(FileStream fileNamed: value url pathForFile) contentsOfEntireFile]
					ifNotNil: [value content]) asString].
" Transcript show: 'field=', key, '; value=', fieldValue; cr. "
		argsStream nextPutAll: crLf, crLf, fieldValue, crLf.
	]].
	argsStream nextPutAll: '--', mimeBorder, '--'.
	resultStream := FileStream
		post: 
			('ACCEPT: text/html', crLf,
			'User-Agent: Pharo', crLf,
			'Content-type: multipart/form-data; boundary=', mimeBorder, crLf,
			'Content-length: ', argsStream contents size printString, crLf, crLf, 
			argsStream contents)
		url: url ifError: [^'Error in post ' url asString].
	"get the header of the reply"
	result := resultStream
		ifNil: ['']
		ifNotNil: [resultStream upToEnd].
	^MIMEDocument content: result! !

!HTTPClient class methodsFor: 'post/get' stamp: 'mir 4/2/2002 15:52'!
requestURL: url target: target
	^self shouldUsePluginAPI
		ifTrue: [FileStream requestURL: url target: target]
		ifFalse: [self error: 'Requesting a new URL target is not supported.']! !

!HTTPClient class methodsFor: 'testing' stamp: 'RobRothwell 2/23/2009 22:41'!
shouldUsePluginAPI
	"HTTPClient shouldUsePluginAPI" 

	self isRunningInBrowser
		ifFalse: [^false].
	self browserSupportsAPI
		ifFalse: [^false].
	"The Mac plugin calls do not work in full screen mode"
	^((SmalltalkImage current  platformName = 'Mac OS')
		and: [Display isFullScreen]) not! !

!HTTPClient class methodsFor: 'utilities' stamp: 'mir 2/2/2001 17:59'!
uploadFileNamed: aFilename to: baseUrl user: user passwd: passwd

	| fileContents remoteFilename |
	remoteFilename := (baseUrl endsWith: '/')
		ifTrue: [baseUrl , '/' , aFilename]
		ifFalse: [baseUrl , aFilename].
	fileContents := (StandardFileStream readOnlyFileNamed: aFilename) contentsOfEntireFile.
	HTTPSocket httpPut: fileContents to: remoteFilename user: user passwd: passwd! !

Object subclass: #Imports
	instanceVariableNames: 'imports'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

Imports class
	instanceVariableNames: 'default'!

!Imports commentStamp: 'sd 5/11/2003 20:34' prior: 0!
I represent imported resources such as images, sounds, and other kind of files. 
For now I only store images in a simple way. 

To access my default instance use: Imports default. However I'm not a strict singleton
and clients may create several of me using new. !

Imports class
	instanceVariableNames: 'default'!

!Imports class methodsFor: 'instance creation' stamp: 'nk 7/12/2003 10:38'!
default
	"Answer my default instance, creating one if necessary."
	"Imports default"
	^default ifNil: [ default := self new ]! !

!Imports class methodsFor: 'instance creation' stamp: 'nk 7/12/2003 10:36'!
default: anImports
	"Set my default instance. Returns the old value if any."
	| old |
	old := default.
	default := anImports.
	^old! !

!Imports methodsFor: 'images' stamp: 'sd 5/11/2003 20:36'!
images
	"returns all the imported images"

	^ imports values

	! !

!Imports methodsFor: 'images' stamp: 'nk 6/12/2004 12:49'!
importImage: anImage named: aName 
	imports
		at: (Utilities
				keyLike: aName
				satisfying: [:ea | (imports includesKey: ea) not])
		put: anImage! !

!Imports methodsFor: 'icons' stamp: 'marcus.denker 11/10/2008 10:04'!
importImageDirectory: directoryOrName 
	| dir extensions forms |
	dir := directoryOrName isString
		ifFalse: [ directoryOrName ]
		ifTrue: [ FileDirectory default directoryNamed: directoryOrName ].
	dir exists
		ifFalse: [self error: dir fullName , ' does not exist'. ^ #()].
	extensions := (ImageReadWriter allTypicalFileExtensions add: 'form';
				 yourself)
				collect: [:ex | '.' , ex].
	forms := OrderedCollection new.
	dir fileNames
		do: [:fileName | | fullName | (fileName endsWithAnyOf: extensions)
				ifTrue: [fullName := dir fullNameFor: fileName.
					(self importImageFromFileNamed: fullName)
						ifNotNil: [:form | forms add: form]]].
	^ forms! !

!Imports methodsFor: 'icons' stamp: 'nk 6/12/2004 12:44'!
importImageDirectoryWithSubdirectories: directoryOrName 
	| dir forms |
	dir := directoryOrName isString
		ifFalse: [ directoryOrName ]
		ifTrue: [ FileDirectory default directoryNamed: directoryOrName ].
	dir exists
		ifFalse: [self error: dir fullName , ' does not exist'. ^ #()].
	forms := OrderedCollection new.
	dir withAllSubdirectoriesCollect: [ :subdir | forms addAll: (self importImageDirectory: dir) ].
	^ forms! !

!Imports methodsFor: 'icons' stamp: 'nk 6/12/2004 12:25'!
importImageFromFileNamed: fullName 
	| localName pathParts form imageName |
	FileDirectory
		splitName: fullName
		to: [:dirPath :lname | 
			localName := lname.
			pathParts := dirPath findTokens: FileDirectory slash].
	form := [Form fromFileNamed: fullName]
				on: Error
				do: [:ex | ex return: nil].
	form
		ifNil: [^ nil].
	imageName := FileDirectory baseNameFor: localName.
	[imports includesKey: imageName]
		whileTrue: [imageName := pathParts isEmpty
						ifTrue: [Utilities
								keyLike: imageName
								satisfying: [:ea | (imports includesKey: ea) not]]
						ifFalse: [pathParts removeLast , '-' , imageName]].
	imports at: imageName put: form.
	^ form! !

!Imports methodsFor: 'images' stamp: 'yo 7/17/2003 00:17'!
imports

	^ imports
! !

!Imports methodsFor: 'initialize' stamp: 'alain.plantec 5/28/2009 09:58'!
initialize
	super initialize.
	imports := Dictionary new.! !

!Imports methodsFor: 'images' stamp: 'sd 5/11/2003 22:26'!
namesAndImagesDo: aBlock
	"iterate over all the names and image"

	^ imports keysAndValuesDo: aBlock

	! !

Object subclass: #LRUCache
	instanceVariableNames: 'size factory calls hits values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!LRUCache commentStamp: '<historical>' prior: 0!
I'm a cache of values, given a key I return a Value from the cache or from the factory!

!LRUCache class methodsFor: 'instance creation' stamp: 'dgd 3/26/2003 22:29'!
size: aNumber factory: aBlock 
	"answer an instance of the receiver"
	^ self new initializeSize: aNumber factory: aBlock! !

!LRUCache class methodsFor: 'testing' stamp: 'dgd 3/26/2003 22:22'!
test
	" 
	LRUCache test 
	"
	| c |
	c := LRUCache
				size: 5
				factory: [:key | key * 2].
	c at: 1.
	c at: 2.
	c at: 3.
	c at: 4.
	c at: 1.
	c at: 5.
	c at: 6.
	c at: 7.
	c at: 8.
	c at: 1.
	^ c! !

!LRUCache class methodsFor: 'testing' stamp: 'dgd 3/26/2003 22:22'!
test2
	" 
	LRUCache test2.  
	Time millisecondsToRun:[LRUCache test2]. 
	MessageTally spyOn:[LRUCache test2].  
	"
	| c |
	c := LRUCache
				size: 600
				factory: [:key | key * 2].
	1
		to: 6000
		do: [:each | c at: each].
	^ c! !

!LRUCache methodsFor: 'initialization' stamp: 'dgd 3/28/2003 19:42'!
initializeSize: aNumber factory: aBlock 
	"initialize the receiver's size and factory"
	size := aNumber.
	values := Array new: aNumber withAll: {nil. nil. nil}.
	factory := aBlock.
	calls := 0.
	hits := 0! !

!LRUCache methodsFor: 'printing' stamp: 'dgd 3/28/2003 19:41'!
printOn: aStream 
	"Append to the argument, aStream, a sequence of characters  
	that identifies the receiver."
	aStream nextPutAll: self class name;
		 nextPutAll: ' size:';
		 nextPutAll: size asString;
		 nextPutAll: ', calls:';
		 nextPutAll: calls asString;
		 nextPutAll: ', hits:';
		 nextPutAll: hits asString;
		 nextPutAll: ', ratio:';
nextPutAll: 
	(hits / calls) asFloat asString! !

Object subclass: #MczInstaller
	instanceVariableNames: 'stream zip'
	classVariableNames: 'Versions'
	poolDictionaries: ''
	category: 'System-Support'!

!MczInstaller class methodsFor: 'versioninfo' stamp: 'avi 1/19/2004 13:13'!
clearVersionInfo
	Versions := Dictionary new! !

!MczInstaller class methodsFor: 'services' stamp: 'cwp 8/7/2003 18:49'!
extension
	^ 'mcz'! !

!MczInstaller class methodsFor: 'services' stamp: 'nk 6/8/2004 17:29'!
fileReaderServicesForFile: fileName suffix: suffix
	^({ self extension. '*' } includes: suffix)
		ifTrue: [ self services ]
		ifFalse: [#()].
! !

!MczInstaller class methodsFor: 'services' stamp: 'avi 3/7/2004 14:51'!
initialize
	self clearVersionInfo.
	self registerForFileList.! !

!MczInstaller class methodsFor: 'installing' stamp: 'cwp 8/7/2003 18:13'!
installFileNamed: aFileName
	self installStream: (FileStream readOnlyFileNamed: aFileName)! !

!MczInstaller class methodsFor: 'installing' stamp: 'cwp 8/7/2003 17:56'!
installStream: aStream
	(self on: aStream) install! !

!MczInstaller class methodsFor: 'services' stamp: 'cwp 8/7/2003 18:54'!
loadVersionFile: fileName
	self installFileNamed: fileName
! !

!MczInstaller class methodsFor: 'instance creation' stamp: 'cwp 8/7/2003 17:56'!
on: aStream
	^ self new stream: aStream! !

!MczInstaller class methodsFor: 'services' stamp: 'avi 3/7/2004 14:49'!
registerForFileList
	Smalltalk at: #MCReader ifAbsent: [FileList registerFileReader: self]! !

!MczInstaller class methodsFor: 'services' stamp: 'cwp 8/7/2003 18:53'!
serviceLoadVersion
	^ SimpleServiceEntry
		provider: self
		label: 'load'
		selector: #loadVersionFile:
		description: 'load a package version'! !

!MczInstaller class methodsFor: 'services' stamp: 'ab 8/8/2003 18:01'!
services
	^ Array with: self serviceLoadVersion! !

!MczInstaller class methodsFor: 'versioninfo' stamp: 'cwp 8/11/2003 23:49'!
storeVersionInfo: aVersion
	Versions 
		at: aVersion package name
		put: aVersion info asDictionary! !

!MczInstaller class methodsFor: 'versioninfo' stamp: 'avi 3/7/2004 14:51'!
unloadMonticello
	"self unloadMonticello"
	Utilities breakDependents.
	
	Smalltalk at: #MCWorkingCopy ifPresent:
		[:wc | 
		wc allInstances do:
			[:ea | 
			Versions at: ea package name put: ea currentVersionInfo asDictionary.
			ea breakDependents.
			Smalltalk at: #SystemChangeNotifier ifPresent: [:scn | scn uniqueInstance noMoreNotificationsFor: ea]]
	displayingProgress: 'Saving version info...'].
	
	"keep things simple and don't unload any class extensions"
	(ChangeSet superclassOrder: ((PackageInfo named: 'Monticello') classes)) reverseDo:
		[:ea | 
		ea removeFromSystem].
	
	self registerForFileList.! !

!MczInstaller class methodsFor: 'versioninfo' stamp: 'avi 2/17/2004 02:49'!
versionInfo
	^ Versions! !

!MczInstaller methodsFor: 'utilities' stamp: 'stephane.ducasse 3/31/2009 21:10'!
associate: tokens
	| result |
	result := Dictionary new.
	tokens pairsDo: [:key :value | 
					| tmp |
					tmp := value.
					value isString ifFalse: [tmp := value collect: [:ea | self associate: ea]].
					value = 'nil' ifTrue: [tmp := ''].
					result at: key put: tmp].
	^ result! !

!MczInstaller methodsFor: 'utilities' stamp: 'avi 2/17/2004 02:53'!
checkDependencies
	| dependencies unmet |
	dependencies := (zip membersMatching: 'dependencies/*') 
			collect: [:member | self extractInfoFrom: (self parseMember: member)].
	unmet := dependencies reject: [:dep |
		self versions: Versions anySatisfy: (dep at: #id)].
	^ unmet isEmpty or: [
		self confirm: (String streamContents: [:s|
			s nextPutAll: 'The following dependencies seem to be missing:'; cr.
			unmet do: [:each | s nextPutAll: (each at: #name); cr].
			s nextPutAll: 'Do you still want to install this package?'])]! !

!MczInstaller methodsFor: 'utilities' stamp: 'avi 2/17/2004 03:26'!
extractInfoFrom: dict
	dict at: #id put: (UUID fromString: (dict at: #id)).
	dict at: #date ifPresent: [:d | d isEmpty ifFalse: [dict at: #date put: (Date fromString: d)]].
	dict at: #time ifPresent: [:t | t isEmpty ifFalse: [dict at: #time put: (Time readFrom: t readStream)]].
	dict at: #ancestors ifPresent: [:a | dict at: #ancestors put: (a collect: [:ea | self extractInfoFrom: ea])].
	^ dict! !

!MczInstaller methodsFor: 'accessing' stamp: 'cwp 8/13/2003 01:58'!
extractPackageName
	^ (self parseMember: 'package') at: #name.
	! !

!MczInstaller methodsFor: 'accessing' stamp: 'cwp 8/13/2003 02:17'!
extractVersionInfo
	^ self extractInfoFrom: (self parseMember: 'version')! !

!MczInstaller methodsFor: 'installation' stamp: 'avi 2/17/2004 02:56'!
install
	| sources |
	zip := ZipArchive new.
	zip readFrom: stream.
	self checkDependencies ifFalse: [^false].
	self recordVersionInfo.
	sources := (zip membersMatching: 'snapshot/*') 
				asSortedCollection: [:a :b | a fileName < b fileName].
	sources do: [:src | self installMember: src].! !

!MczInstaller methodsFor: 'installation' stamp: 'yo 8/17/2004 10:03'!
installMember: member
	 | str |
	self useNewChangeSetDuring:
		[str := member contentStream text.
		str setConverterForCode.
		str fileInAnnouncing: 'loading ', member fileName]! !

!MczInstaller methodsFor: 'utilities' stamp: 'cwp 8/13/2003 01:58'!
parseMember: fileName
	| tokens |
	tokens := (self scanner scanTokens: (zip contentsOf: fileName)) first.
	^ self associate: tokens! !

!MczInstaller methodsFor: 'accessing' stamp: 'cwp 8/7/2003 19:18'!
recordVersionInfo
	Versions 
		at: self extractPackageName 
		put: self extractVersionInfo! !

!MczInstaller methodsFor: 'accessing' stamp: 'cwp 8/13/2003 02:04'!
scanner
	^ Scanner new! !

!MczInstaller methodsFor: 'accessing' stamp: 'avi 2/17/2004 02:55'!
stream: aStream
	stream := aStream! !

!MczInstaller methodsFor: 'utilities' stamp: 'bf 2/9/2004 13:56'!
useNewChangeSetDuring: aBlock
	| changeHolder oldChanges newChanges |
	changeHolder := (ChangeSet respondsTo: #newChanges:)
						ifTrue: [ChangeSet]
						ifFalse: [Smalltalk].
	oldChanges := (ChangeSet respondsTo: #current)
						ifTrue: [ChangeSet current]
						ifFalse: [Smalltalk changes].

	newChanges := ChangeSet new name: (ChangeSet uniqueNameLike: self extractPackageName).
	changeHolder newChanges: newChanges.
	[aBlock value] ensure: [changeHolder newChanges: oldChanges].! !

!MczInstaller methodsFor: 'utilities' stamp: 'bf 2/9/2004 15:00'!
versions: aVersionList anySatisfy: aDependencyID
	^ aVersionList anySatisfy: [:version | 
			aDependencyID = (version at: #id)
				or: [self versions: (version at: #ancestors) anySatisfy: aDependencyID]]! !

!Object methodsFor: '*system-support' stamp: 'dvf 8/23/2003 12:27'!
systemNavigation

	^ SystemNavigation default! !

Object subclass: #Preference
	instanceVariableNames: 'name value defaultValue helpString localToProject categoryList changeInformee changeSelector viewRegistry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!Preference commentStamp: '<historical>' prior: 0!
Represents a true/false flag that is under user control and which can be interrogated by a call to Preferences
	viewRegistry		the registry of the classes responsible for building my view
	name 				a symbol, the formal name of the preference.
	value				a boolean, the current value
	defaultValue		the default value of the preference
	helpString 			string or text, constituting the help message
	localToProject		boolean, whether each project holds its own version
	categoryList			list of categories under which to offer this
	changeInformee 	whom, if anyone, to inform if the value changes:
	changeSelector 		what selector to send to the changeInformee when the value changes!

!Preference methodsFor: 'menu' stamp: 'sw 4/12/2001 23:42'!
categoryList
	"Answer the categoryList"

	^ categoryList! !

!Preference methodsFor: 'initialization' stamp: 'sw 4/29/2001 23:51'!
categoryList: aList
	"Set the receiver's categoryList"

	categoryList := aList! !

!Preference methodsFor: 'change notification' stamp: 'michael.rueger 4/27/2009 18:00'!
changeInformee

	^changeInformee! !

!Preference methodsFor: 'initialization' stamp: 'michael.rueger 4/27/2009 17:54'!
changeInformee: informee
	"Set the object to be informed when my value changes"

	changeInformee := (informee == nil or: [informee == #nil])
						ifTrue: [nil]
						ifFalse:	[(informee isKindOf: Symbol)
							ifTrue:
								[Smalltalk at: informee]
							ifFalse:
								[informee]]! !

!Preference methodsFor: 'change notification' stamp: 'sw 4/12/2001 01:39'!
changeInformee: informee changeSelector: aSelector
	"Set the changeInformee and changeSelector as specified"

	changeInformee := informee.
	changeSelector := aSelector! !

!Preference methodsFor: 'menu' stamp: 'sw 4/13/2001 00:01'!
copyName
	"Copy the name of the given preference to the clipboard"

	Clipboard clipboardText: name asString asText! !

!Preference methodsFor: 'value' stamp: 'sw 4/10/2001 15:01'!
defaultValue
	"Answer this preference's defaultValue"

	^ defaultValue! !

!Preference methodsFor: 'value' stamp: 'sw 4/18/2002 12:15'!
defaultValue: aValue
	"Set the receiver's defaultValue"

	defaultValue := aValue.! !

!Preference methodsFor: 'menu' stamp: 'sw 4/13/2001 00:04'!
helpString
	"Answer the help string provided for the receiver"

	^ helpString ifNil: ['no help available']! !

!Preference methodsFor: 'testing' stamp: 'ar 9/27/2005 21:50'!
isObsolete
	^(changeInformee class isObsolete or:[changeInformee isBehavior and:[changeInformee isObsolete]])! !

!Preference methodsFor: 'local to project' stamp: 'sw 4/10/2001 12:37'!
isProjectLocalString
	"Answer a string representing whether sym is a project-local preference or not"

	| aStr |
	aStr :=  'each project has its own setting'.
	^ localToProject
		ifTrue:
			['<yes>', aStr]
		ifFalse:
			['<no>', aStr]! !

!Preference methodsFor: 'local to project' stamp: 'sw 4/10/2001 01:14'!
localToProject
	"Answer whether this preference is project-local"

	^ localToProject! !

!Preference methodsFor: 'menu' stamp: 'sw 4/10/2001 15:02'!
name
	"Answer this preference's name"

	^ name! !

!Preference methodsFor: 'initialization' stamp: 'hpt 9/26/2004 15:59'!
name: aName defaultValue: aValue helpString: aString localToProject: projectBoolean categoryList: aList changeInformee: informee changeSelector:  aChangeSelector viewRegistry: aViewRegistry
	"Initialize the preference from the given values.  There is an extra tolerence here for the symbols #true, #false, and #nil, which are interpreted, when appropriate, as meaning true, false, and nil"

	name := aName asSymbol.
	defaultValue := aValue.
	aValue = #true ifTrue: [defaultValue := true].
	aValue = #false ifTrue: [defaultValue := false].
	value := defaultValue.
	helpString := aString.
	localToProject := projectBoolean == true or: [projectBoolean = #true].
	viewRegistry := aViewRegistry.
	categoryList := (aList ifNil: [OrderedCollection with: #unclassified]) collect:
		[:elem | elem asSymbol].

	changeInformee := (informee == nil or: [informee == #nil])
						ifTrue: [nil]
						ifFalse:	[(informee isKindOf: Symbol)
							ifTrue:
								[Smalltalk at: informee]
							ifFalse:
								[informee]].
	changeSelector  := aChangeSelector! !

!Preference methodsFor: 'change notification' stamp: 'sw 4/12/2001 00:03'!
notifyInformeeOfChange
	"If there is a changeInformee, notify her that I have changed value"

	changeInformee ifNotNil: [changeInformee perform: changeSelector]! !

!Preference methodsFor: 'value' stamp: 'sw 4/10/2001 15:35'!
preferenceValue
	"Answer the current value of the preference"

	^ value! !

!Preference methodsFor: 'value' stamp: 'sw 4/12/2001 23:28'!
preferenceValue: aValue
	"set the value as indicated, and invoke the change selector if appropriate"

	| oldValue |
	oldValue := value.
	value := aValue.
	oldValue ~~ value ifTrue:
		[self notifyInformeeOfChange]! !

!Preference methodsFor: 'debugging' stamp: 'sw 4/13/2001 00:05'!
printOn: aStream
	"Print a string decribing the receiver to the given stream"

	super printOn: aStream.
	aStream nextPutAll: name storeString, ' ', value storeString! !

!Preference methodsFor: 'value' stamp: 'sw 4/12/2001 23:28'!
rawValue: aValue
	"set the value as indicated, with no side effects"

	value := aValue! !

!Preference methodsFor: 'user interface' stamp: 'hpt 9/26/2004 16:58'!
representativeButtonWithColor: aColor inPanel: aPanel
	| view |
	view := self viewForPanel: aPanel.
	^view ifNotNil: [view representativeButtonWithColor: aColor inPanel: aPanel]! !

!Preference methodsFor: 'value' stamp: 'sw 4/12/2001 00:04'!
restoreDefaultValue
	"restore the default value to the preference"

	value := defaultValue! !

!Preference methodsFor: 'value' stamp: 'hpt 9/26/2004 16:51'!
togglePreferenceValue
	"Toggle whether the value of the preference. Self must be a boolean preference."
	value := value not.
	self notifyInformeeOfChange! !

!Preference methodsFor: 'local to project' stamp: 'alain.plantec 6/6/2009 14:23'!
toggleProjectLocalness
	"Toggle whether the preference should be held project-by-project or globally"

	localToProject := localToProject not.
! !

!Preference methodsFor: 'user interface' stamp: 'hpt 9/26/2004 15:42'!
viewClassForPanel: aPreferencePanel
	^self viewRegistry viewClassFor: aPreferencePanel! !

!Preference methodsFor: 'user interface' stamp: 'hpt 9/26/2004 16:58'!
viewForPanel: aPreferencePanel
	| viewClass |
	viewClass := self viewClassForPanel: aPreferencePanel.
	^viewClass ifNotNil: [viewClass preference: self]! !

!Preference methodsFor: 'user interface' stamp: 'hpt 9/26/2004 15:40'!
viewRegistry
	^viewRegistry! !

!Preference methodsFor: 'user interface' stamp: 'hpt 9/26/2004 15:40'!
viewRegistry: aRegistry
	viewRegistry := aRegistry! !

Object subclass: #PreferenceView
	instanceVariableNames: 'preference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

PreferenceView class
	instanceVariableNames: 'registeredClasses'!

!PreferenceView commentStamp: '<historical>' prior: 0!
My subclasses instances are responsible for building the visual representation of each kind of preference.!

PreferenceView class
	instanceVariableNames: 'registeredClasses'!

PreferenceView subclass: #BooleanPreferenceView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!BooleanPreferenceView commentStamp: '<historical>' prior: 0!
I am responsible for building the visual representation of a preference that accepts true and false values!

!BooleanPreferenceView classSide methodsFor: 'view registry' stamp: 'alain.plantec 6/6/2009 22:38'!
handlesPanel: aPreferencePanel
	^false! !

!BooleanPreferenceView classSide methodsFor: 'initialization' stamp: 'hpt 9/26/2004 15:55'!
initialize
	PreferenceViewRegistry ofBooleanPreferences register: self.! !

!BooleanPreferenceView classSide methodsFor: 'initialization' stamp: 'hpt 9/26/2004 15:55'!
unload
	PreferenceViewRegistry ofBooleanPreferences unregister: self.! !

!BooleanPreferenceView methodsFor: 'user interface' stamp: 'alain.plantec 5/30/2008 09:57'!
offerPreferenceNameMenu: aPanel with: ignored1 in: ignored2
	"the user clicked on a preference name -- put up a menu"

	| aMenu |
	ActiveHand showTemporaryCursor: nil.
	aMenu := MenuMorph new defaultTarget: self preference.
	aMenu addTitle: self preference name.

	(Preferences okayToChangeProjectLocalnessOf: self preference name) ifTrue:
		[aMenu addUpdating: #isProjectLocalString target: self preference action: #toggleProjectLocalness.
		aMenu balloonTextForLastItem: 'Some preferences are best applied uniformly to all projects, and others are best set by each individual project.  If this item is checked, then this preference will be printed in bold and will have a separate value for each project'].

	aMenu add: 'browse senders' target: self systemNavigation selector: #browseAllCallsOn: argument: self preference name.
	aMenu balloonTextForLastItem: 'This will open a method-list browser on all methods that the send the preference "', self preference name, '".'. 
	aMenu add: 'show category...' target: aPanel selector: #findCategoryFromPreference: argument: self preference name.
	aMenu balloonTextForLastItem: 'Allows you to find out which category, or categories, this preference belongs to.'.

	aMenu add: 'hand me a button for this preference' target: self selector: #tearOffButton.
	aMenu balloonTextForLastItem: 'Will give you a button that governs this preference, which you may deposit wherever you wish'.

	aMenu add: 'copy this name to clipboard' target: self preference selector: #copyName.
	aMenu balloonTextForLastItem: 'Copy the name of the preference to the text clipboard, so that you can paste into code somewhere'.

	aMenu popUpInWorld! !

!BooleanPreferenceView methodsFor: 'user interface' stamp: 'md 9/5/2005 15:40'!
representativeButtonWithColor: aColor inPanel: aPreferencesPanel
	"Return a button that controls the setting of prefSymbol.  It will keep up to date even if the preference value is changed in a different place"

	| outerButton aButton str miniWrapper |
	
	outerButton := AlignmentMorph newRow height: 24.
	outerButton color:  (aColor ifNil: [Color r: 0.645 g: 1.0 b: 1.0]).
	outerButton hResizing: (aPreferencesPanel ifNil: [#shrinkWrap] ifNotNil: [#spaceFill]).
	outerButton vResizing: #shrinkWrap.
	outerButton addMorph: (aButton := UpdatingThreePhaseButtonMorph checkBox).
	aButton
		target: self preference;
		actionSelector: #togglePreferenceValue;
		getSelector: #preferenceValue.

	outerButton addTransparentSpacerOfSize: (2 @ 0).
	str := StringMorph contents: self preference name font: (StrikeFont familyName: 'NewYork' size: 12).

	self preference localToProject ifTrue:
		[str emphasis: TextEmphasis bold emphasisCode].

	miniWrapper := AlignmentMorph newRow hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	miniWrapper beTransparent addMorphBack: str lock.
	aPreferencesPanel
		ifNotNil:  "We're in a Preferences panel"
			[miniWrapper on: #mouseDown send: #offerPreferenceNameMenu:with:in: to: self withValue: aPreferencesPanel.
			miniWrapper on: #mouseEnter send: #menuButtonMouseEnter: to: miniWrapper.
			miniWrapper on: #mouseLeave send: #menuButtonMouseLeave: to: miniWrapper.
			miniWrapper setBalloonText: 'Click here for a menu of options regarding this preference.  Click on the checkbox to the left to toggle the setting of this preference']

		ifNil:  "We're a naked button, not in a panel"
			[miniWrapper setBalloonText: self preference helpString; setProperty: #balloonTarget toValue: aButton].

	outerButton addMorphBack: miniWrapper.
	outerButton setNameTo: self preference name.

	aButton setBalloonText: self preference helpString.

	^ outerButton

	"(Preferences preferenceAt: #balloonHelpEnabled) view tearOffButton"! !

PreferenceView subclass: #HaloThemePreferenceView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!HaloThemePreferenceView commentStamp: '<historical>' prior: 0!
I am responsible for building the view for the preference that choose the halo theme!

!HaloThemePreferenceView classSide methodsFor: 'view registry' stamp: 'alain.plantec 6/6/2009 22:38'!
handlesPanel: aPreferencePanel
	^false! !

!HaloThemePreferenceView classSide methodsFor: 'initialization' stamp: 'hpt 9/26/2004 15:58'!
initialize
	"adding the halo theme preference to Preferences and registering myself as its view"
	PreferenceViewRegistry ofHaloThemePreferences register: self.
	Preferences 
		addPreference: #haloTheme 
		categories: {#halos} 
		default: #iconicHaloSpecifications
		balloonHelp: ''
		projectLocal: false
		changeInformee: nil
		changeSelector: nil
		viewRegistry: PreferenceViewRegistry ofHaloThemePreferences.! !

!HaloThemePreferenceView classSide methodsFor: 'initialization' stamp: 'hpt 9/26/2004 15:58'!
unload
	PreferenceViewRegistry ofHaloThemePreferences unregister: self.! !

!HaloThemePreferenceView methodsFor: 'user interface' stamp: 'md 12/18/2008 16:22'!
haloThemeRadioButtons
	"Answer a column of butons representing the choices of halo theme"

	| buttonColumn aRow aRadioButton aStringMorph |
	buttonColumn := AlignmentMorph newColumn beTransparent.
	#(	(iconicHaloSpecifications iconic iconicHalosInForce	'circular halos with icons inside')
		(classicHaloSpecs	classic	classicHalosInForce		'plain circular halos')
		(customHaloSpecs	custom	customHalosInForce		'customizable halos')) do:

		[:quad |
			aRow := AlignmentMorph newRow beTransparent.
			aRow addMorph: (aRadioButton := UpdatingThreePhaseButtonMorph radioButton).
			aRadioButton target: Preferences.
			aRadioButton setBalloonText: quad fourth.
			aRadioButton actionSelector: #installHaloTheme:.
			aRadioButton getSelector: quad third.
			aRadioButton arguments: (Array with: quad first).
			aRow addTransparentSpacerOfSize: (4 @ 0).
			aRow addMorphBack: (aStringMorph := StringMorph contents: quad second asString).
			aStringMorph setBalloonText: quad fourth.
			buttonColumn addMorphBack: aRow].
	^ buttonColumn

	"(Preferences preferenceAt: #haloTheme) view tearOffButton"! !

!HaloThemePreferenceView methodsFor: 'user interface' stamp: 'hpt 9/24/2004 23:11'!
representativeButtonWithColor: aColor inPanel: aPreferencesPanel
	| outerButton editButton |
	editButton := SimpleButtonMorph new 
					target: Preferences; 
					color: Color transparent; 
					actionSelector: #editCustomHalos; 
					label: 'Edit custom halos' translated;
					setBalloonText: 'Click here to edit the method that defines the custom halos' translated.
	
	outerButton := AlignmentMorph newColumn.
	outerButton
		color:  (aColor ifNil: [Color r: 0.645 g: 1.0 b: 1.0]);
		hResizing: (aPreferencesPanel ifNil: [#shrinkWrap] ifNotNil: [#spaceFill]);
		vResizing: #shrinkWrap;	
		addTransparentSpacerOfSize: (0@4);
		addMorphBack: self haloThemeRadioButtons;
		addTransparentSpacerOfSize: (0@4);
		addMorphBack: editButton.
		
	^outerButton.
	
	"(Preferences preferenceAt: #haloTheme) view tearOffButton"	! !

!PreferenceView classSide methodsFor: 'view registry' stamp: 'hpt 9/26/2004 16:09'!
handlesPanel: aPreferencePanel
	self subclassResponsibility ! !

!PreferenceView classSide methodsFor: 'instance creation' stamp: 'hpt 9/24/2004 22:25'!
preference: aPreference
	^self new
		initializeWithPreference: aPreference;
		yourself! !

!PreferenceView methodsFor: 'initialization' stamp: 'hpt 9/24/2004 22:25'!
initializeWithPreference: aPreference
	preference := aPreference! !

!PreferenceView methodsFor: 'accessing' stamp: 'hpt 9/24/2004 22:25'!
preference
	^preference! !

!PreferenceView methodsFor: 'user interface' stamp: 'hpt 9/24/2004 22:56'!
representativeButtonWithColor: aColor inPanel: aPreferencesPanel
	self subclassResponsibility ! !

!PreferenceView methodsFor: 'user interface' stamp: 'hpt 9/26/2004 16:14'!
tearOffButton
	"Hand the user a button the can control this"

	| aButton |
	aButton := self representativeButtonWithColor: self preference defaultBackgroundColor inPanel: nil.
	aButton borderWidth: 1; borderColor:  Color black; useRoundedCorners.
	aButton openInHand! !

Object subclass: #PreferenceViewRegistry
	instanceVariableNames: 'registeredClasses viewOrder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

PreferenceViewRegistry class
	instanceVariableNames: 'registries'!

!PreferenceViewRegistry commentStamp: '<historical>' prior: 0!
PreferenceViewRegistry is much like the AppRegistry classes.  Its purpose is to allow PreferenceBrowser implementers to register its own views for each kind of preference.!

PreferenceViewRegistry class
	instanceVariableNames: 'registries'!

!PreferenceViewRegistry classSide methodsFor: 'instance creation' stamp: 'hpt 9/26/2004 16:23'!
ofBooleanPreferences
	^(self registryOf: #booleanPreferences)
		viewOrder: 1; 
		yourself.! !

!PreferenceViewRegistry classSide methodsFor: 'instance creation' stamp: 'hpt 9/26/2004 16:24'!
ofColorPreferences
	^(self registryOf: #colorPreferences)
		viewOrder: 5;
		yourself.! !

!PreferenceViewRegistry classSide methodsFor: 'instance creation' stamp: 'hpt 9/26/2004 16:24'!
ofFontPreferences
	^(self registryOf: #fontPreferences)
		viewOrder: 4;
		yourself.! !

!PreferenceViewRegistry classSide methodsFor: 'instance creation' stamp: 'hpt 9/26/2004 16:23'!
ofHaloThemePreferences
	^(self registryOf: #haloThemePreferences)
		viewOrder: 2;
		yourself.! !

!PreferenceViewRegistry classSide methodsFor: 'instance creation' stamp: 'hpt 12/9/2004 22:16'!
ofNumericPreferences
	^(self registryOf: #numericPreferences)
		viewOrder: 3;
		yourself.! !

!PreferenceViewRegistry classSide methodsFor: 'instance creation' stamp: 'hpt 9/26/2004 16:23'!
ofTextPreferences
	^(self registryOf: #textPreferences)
		viewOrder: 3;
		yourself.! !

!PreferenceViewRegistry classSide methodsFor: 'instance creation' stamp: 'hpt 9/26/2004 15:28'!
registries
	^registries ifNil: [registries := Dictionary new]! !

!PreferenceViewRegistry classSide methodsFor: 'instance creation' stamp: 'hpt 9/26/2004 15:33'!
registryOf: aSymbol
	^self registries at: aSymbol ifAbsentPut: [self new]! !

!PreferenceViewRegistry methodsFor: 'initialization' stamp: 'alain.plantec 5/28/2009 10:15'!
initialize
	super initialize.
	viewOrder := 1.! !

!PreferenceViewRegistry methodsFor: 'view registry' stamp: 'hpt 9/26/2004 15:26'!
register: aProviderClass
	(self registeredClasses includes: aProviderClass) 
		ifFalse: [self registeredClasses add: aProviderClass].! !

!PreferenceViewRegistry methodsFor: 'view registry' stamp: 'hpt 9/26/2004 15:26'!
registeredClasses
	^registeredClasses ifNil: [registeredClasses := OrderedCollection new]! !

!PreferenceViewRegistry methodsFor: 'view registry' stamp: 'hpt 9/26/2004 15:26'!
unregister: aProviderClass
	self registeredClasses remove: aProviderClass ifAbsent: []! !

!PreferenceViewRegistry methodsFor: 'view registry' stamp: 'hpt 9/26/2004 15:26'!
viewClassFor: aPreferencePanel
	^self registeredClasses 
		detect: [:aViewClass| aViewClass handlesPanel: aPreferencePanel]
		ifNone: [].! !

!PreferenceViewRegistry methodsFor: 'view order' stamp: 'hpt 9/26/2004 16:22'!
viewOrder
	"answer the order in which the registered views should appear relative to the other views"
	^viewOrder! !

!PreferenceViewRegistry methodsFor: 'view order' stamp: 'hpt 9/26/2004 16:22'!
viewOrder: aNumber
	viewOrder := aNumber! !

Object subclass: #Preferences
	instanceVariableNames: ''
	classVariableNames: 'DesktopColor DictionaryOfPreferences Parameters'
	poolDictionaries: ''
	category: 'System-Support'!

!Preferences commentStamp: '<historical>' prior: 0!
A general mechanism to store preference choices.  The default setup treats any symbol as a potential boolean flag; flags unknown to the preference dictionary are always returned as false.  

	To open the control panel:		Preferences openFactoredPanel
	To read how to use the panel (and how to make a preference be per-project):
		 Preferences giveHelpWithPreferences

All messages are on the class side.

To query a a preference:
	Preferences logDebuggerStackToFile
or some people prefer the more verbose
	Preferences valueOfFlag: #logDebuggerStackToFile

You can make up a new preference any time.  Do not define a new message in Preferences class. Accessor methods are compiled automatically when you add a preference as illustrated below:

To add a preference (e.g. in the Postscript of a fileout):
	Preferences addPreference: #samplePreference categories: #(general browsing)
		default: true balloonHelp: 'This is an example of a preference added by a do-it'
		projectLocal: false changeInformee: nil changeSelector: nil.

To change a preference programatically:
	Preferences disable: #logDebuggerStackToFile.
Or to turn it on,
	Preferences enable: #logDebuggerStackToFile.
!

!Preferences class methodsFor: 'standard queries' stamp: ''!
Curvier
	^ self
		valueOfFlag: #Curvier
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPConflict
	^ self
		valueOfFlag: #DPConflict
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPDebugging
	^ self
		valueOfFlag: #DPDebugging
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPDuplicated
	^ self
		valueOfFlag: #DPDuplicated
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPLocal
	^ self
		valueOfFlag: #DPLocal
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPLong
	^ self
		valueOfFlag: #DPLong
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPOverride
	^ self
		valueOfFlag: #DPOverride
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPRecentlyModified
	^ self
		valueOfFlag: #DPRecentlyModified
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPRequired
	^ self
		valueOfFlag: #DPRequired
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPSuperSend
	^ self
		valueOfFlag: #DPSuperSend
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPSupplied
	^ self
		valueOfFlag: #DPSupplied
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
DPUncommented
	^ self
		valueOfFlag: #DPUncommented
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
FreeTypeCacheSize
	^ self
		valueOfFlag: #FreeTypeCacheSize
		ifAbsent: [5000]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
GlyphContrast
	^ self
		valueOfFlag: #GlyphContrast
		ifAbsent: [50]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
HintingFull
	^ self
		valueOfFlag: #HintingFull
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
HintingLight
	^ self
		valueOfFlag: #HintingLight
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
HintingNone
	^ self
		valueOfFlag: #HintingNone
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
HintingNormal
	^ self
		valueOfFlag: #HintingNormal
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
MonitorTypeCRT
	^ self
		valueOfFlag: #MonitorTypeCRT
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
MonitorTypeLCD
	^ self
		valueOfFlag: #MonitorTypeLCD
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: 'adrian-lienhard 6/4/2009 21:29'!
UpdateFontsAtImageStartup
	^ self
		valueOfFlag: #UpdateFontsAtImageStartup
		ifAbsent: [false]! !

!Preferences class methodsFor: 'fonts' stamp: 'jmv 8/4/2009 15:09'!
aaFontsColormapDepth
	"Adjust balance between colored AA text quality (especially if subpixel AA is used) and space / performance.
	5 is optimal quality. Each colorMap takes 128kB of RAM, and takes several seconds to build.
	4 is a reasonable balance. Each colorMap takes 16kB of RAM and builds fast on a fast machine.
	3 is good for slow hardware or memory restrictions. Each colorMap takes 2 kb of RAM."
	^self
		valueOfFlag: #aaFontsColormapDepth
		ifAbsent: [4]! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/24/1999 12:26'!
acceptAnnotationsFrom: aSystemWindow
	"This intricate extraction is based on the precise structure of the annotation-request window.  Kindly avert your eyes."
	| aList |
	aList := aSystemWindow paneMorphs first firstSubmorph submorphs collect:
		[:m |  m contents asSymbol].
	self defaultAnnotationRequests: aList
	! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:00'!
addBooleanPreference: prefSymbol categories: categoryList default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"

	self addPreference: prefSymbol  categories: categoryList  default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofBooleanPreferences ! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 12/5/2004 13:28'!
addBooleanPreference: prefSymbol categories: categoryList default: aValue balloonHelp: helpString projectLocal: localBoolean changeInformee: informeeSymbol  changeSelector: aChangeSelector
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean"

	self addPreference: prefSymbol  categories: categoryList default:  aValue balloonHelp: helpString  projectLocal: localBoolean  changeInformee: informeeSymbol changeSelector: aChangeSelector viewRegistry: PreferenceViewRegistry ofBooleanPreferences ! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:01'!
addBooleanPreference: prefSymbol category: categorySymbol default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"

	self addPreference: prefSymbol  categories: {categorySymbol} default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofBooleanPreferences ! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:03'!
addColorPreference: prefSymbol categories: categoryList default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"

	self addPreference: prefSymbol  categories: categoryList  default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofColorPreferences ! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:03'!
addColorPreference: prefSymbol category: categorySymbol default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"

	self addPreference: prefSymbol  categories: {categorySymbol} default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofColorPreferences ! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:03'!
addFontPreference: prefSymbol categories: categoryList default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"

	self addPreference: prefSymbol  categories: categoryList  default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofFontPreferences ! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:02'!
addFontPreference: prefSymbol category: categorySymbol default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"

	self addPreference: prefSymbol  categories: {categorySymbol} default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofFontPreferences ! !

!Preferences class methodsFor: 'misc' stamp: 'sw 10/6/1999 15:20'!
addModelItemsToWindowMenu: aMenu
	aMenu addLine.
	aMenu add: 'restore default preference settings' target: self action: #chooseInitialSettings.
	aMenu add: 'restore default text highlighting' target: self action: #initializeTextHighlightingParameters! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 12/9/2004 22:15'!
addNumericPreference: prefSymbol categories: categoryList default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. "

	self addPreference: prefSymbol  categories: categoryList  default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofNumericPreferences ! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 12/9/2004 22:15'!
addNumericPreference: prefSymbol category: categorySymbol default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system."

	self addPreference: prefSymbol  categories: {categorySymbol} default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofNumericPreferences ! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:05'!
addPreference: prefSymbol categories: categoryList default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"
	self addBooleanPreference: prefSymbol categories: categoryList default: defaultValue balloonHelp: helpString.! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 12/5/2004 13:29'!
addPreference: prefSymbol categories: categoryList default: aValue balloonHelp: helpString projectLocal: localBoolean changeInformee: informeeSymbol  changeSelector: aChangeSelector
	"Add an item representing the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"

	self addBooleanPreference: prefSymbol categories: categoryList default: aValue balloonHelp: helpString projectLocal: localBoolean changeInformee: informeeSymbol  changeSelector: aChangeSelector
! !

!Preferences class methodsFor: 'add preferences' stamp: 'rr 10/1/2005 15:14'!
addPreference: aName categories: categoryList default: aValue balloonHelp: helpString projectLocal: localBoolean changeInformee: informeeSymbol changeSelector: aChangeSelector viewRegistry: aViewRegistry 
	"Add or replace a preference as indicated.  Reuses the preexisting Preference object for this symbol, if there is one, so that UI artifacts that interact with it will remain valid."

	| aPreference aPrefSymbol |
	aPrefSymbol := aName asSymbol.
	aPreference := self dictionaryOfPreferences  at:aPrefSymbol
				 ifAbsent:[Preference new].
	aPreference 
		 name:aPrefSymbol
		 defaultValue:aValue
		 helpString:helpString
		 localToProject:localBoolean
		 categoryList:categoryList
		 changeInformee:informeeSymbol
		 changeSelector:aChangeSelector
		 viewRegistry:aViewRegistry.
	self dictionaryOfPreferences  at:aPrefSymbol  put:aPreference.
	self  compileAccessMethodForPreference:aPreference! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:05'!
addPreference: prefSymbol category: categorySymbol default: defaultValue balloonHelp: helpString 
	"Add the given preference, putting it in the given category, with the given default value, and with the given balloon help. It assumes boolean preference for backward compatibility"

	self addBooleanPreference: prefSymbol category: categorySymbol default: defaultValue balloonHelp: helpString.! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:02'!
addTextPreference: prefSymbol categories: categoryList default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"

	self addPreference: prefSymbol  categories: categoryList  default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofTextPreferences ! !

!Preferences class methodsFor: 'add preferences' stamp: 'hpt 9/26/2004 16:02'!
addTextPreference: prefSymbol category: categorySymbol default: defaultValue balloonHelp: helpString 
	"Add an item repreesenting the given preference symbol to the system. Default view for this preference is boolean to keep backward compatibility"

	self addPreference: prefSymbol  categories: {categorySymbol} default:  defaultValue balloonHelp: helpString  projectLocal: false  changeInformee: nil changeSelector: nil viewRegistry: PreferenceViewRegistry ofTextPreferences ! !

!Preferences class methodsFor: 'preference-object access' stamp: 'rr 10/1/2005 15:14'!
allPreferenceObjects
	"Answer a list of all the Preference objects registered in the system"

	^ self dictionaryOfPreferences values! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
allowBlockArgumentAssignment
	^ self
		valueOfFlag: #allowBlockArgumentAssignment
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
allowUnderscoreAssignment
	^ self
		valueOfFlag: #allowUnderscoreAssignment
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
allwaysShowVScrollBar
	^ self
		valueOfFlag: #allwaysShowVScrollBar
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
alternateHandlesLook
	^ self
		valueOfFlag: #alternateHandlesLook
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
alternativeButtonsInScrollBars
	^ self
		valueOfFlag: #alternativeButtonsInScrollBars
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
alwaysHideHScrollbar
	^ self
		valueOfFlag: #alwaysHideHScrollbar
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
alwaysShowConnectionVocabulary
	^ self
		valueOfFlag: #alwaysShowConnectionVocabulary
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
alwaysShowHScrollbar
	^ self
		valueOfFlag: #alwaysShowHScrollbar
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
alwaysShowVScrollbar
	^ self
		valueOfFlag: #alwaysShowVScrollbar
		ifAbsent: [true]! !

!Preferences class methodsFor: 'parameters' stamp: 'alain.plantec 5/30/2008 14:17'!
annotationEditingWindow
	"Answer a window affording editing of annotations"
	| aPanel ins outs current aMorph aWindow aButton info pair standardHeight standardWidth |
	standardHeight := 180.
	standardWidth := (2 sqrt reciprocal * standardHeight) rounded.
	aPanel := AlignmentMorph newRow extent: 2 * standardWidth @ standardHeight.
	ins := AlignmentMorph newColumn extent: standardWidth @ standardHeight.
	ins color: Color green muchLighter.
	ins enableDrop: true;
		 beSticky.
	outs := AlignmentMorph newColumn extent: standardWidth @ standardHeight.
	outs color: Color red muchLighter.
	outs enableDrop: true;
		 beSticky.
	aPanel addMorph: outs;
		 addMorphFront: ins.
	outs position: ins position + (standardWidth @ 0).
	current := self defaultAnnotationRequests.
	info := self annotationInfo.
	current
		do: [:sym | 
			pair := info
						detect: [:aPair | aPair first == sym].
			aMorph := StringMorph new contents: pair first.
			aMorph setBalloonText: pair last.
			aMorph enableDrag: true.
			aMorph
				on: #startDrag
				send: #startDrag:with:
				to: aMorph.
			ins addMorphBack: aMorph].
	info
		do: [:aPair | (current includes: aPair first)
				ifFalse: [aMorph := StringMorph new contents: aPair first.
					aMorph setBalloonText: aPair last.
					aMorph enableDrag: true.
					aMorph
						on: #startDrag
						send: #startDrag:with:
						to: aMorph.
					outs addMorph: aMorph]].
	aPanel layoutChanged.
	aWindow := SystemWindowWithButton new setLabel: 'Annotations'.
	aButton := SimpleButtonMorph new target: Preferences;
				 actionSelector: #acceptAnnotationsFrom:;
				
				arguments: (Array with: aWindow);
				 label: 'apply';
				 borderWidth: 0;
				 borderColor: Color transparent;
				 color: Color transparent.
	aButton submorphs first color: Color blue.
	aButton setBalloonText: 'After moving all the annotations you want to the left (green) side, and all the ones you do NOT want to the right (pink) side, hit this "apply" button to have your choices take effect.'.
	aWindow buttonInTitle: aButton;
		 adjustExtraButton.
	^ aPanel wrappedInWindow: aWindow"Preferences annotationEditingWindow openInHand"! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 7/12/2001 18:18'!
annotationInfo 
	"Answer a list of pairs characterizing all the available kinds of annotations; in each pair, the first element is a symbol representing the info type, and the second element is a string providing the corresponding balloon help"

	^ #(

		(timeStamp			'The time stamp of the last submission of the method.')
		(firstComment		'The first comment in the method, if any.')
		(masterComment		'The comment at the beginning of the supermost implementor of the method if any.')
		(documentation		'Comment at beginning of the method or, if it has none, comment at the beginning of a superclass''s implementation of the method')
		(messageCategory	'Which method category the method lies in')
		(sendersCount		'A report of how many senders there of the message.')
		(implementorsCount	'A report of how many implementors there are of the message.')
		(recentChangeSet	'The most recent change set bearing the method.')
		(allChangeSets		'A list of all change sets bearing the method.')
		(priorVersionsCount	'A report of how many previous versions there are of the method' )
		(priorTimeStamp		'The time stamp of the penultimate submission of the method, if any'))! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
annotationPanes
	^ self
		valueOfFlag: #annotationPanes
		ifAbsent: [false]! !

!Preferences class methodsFor: 'reacting to change' stamp: 'AlainPlantec 10/17/2009 18:14'!
annotationPanesChanged
	"The setting of the annotationPanes preference changed; react"! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
areaFillsAreTolerant
	^ self
		valueOfFlag: #areaFillsAreTolerant
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
areaFillsAreVeryTolerant
	^ self
		valueOfFlag: #areaFillsAreVeryTolerant
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
autoFocusForColumns
	^ self
		valueOfFlag: #autoFocusForColumns
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
autoIndent
	^ self
		valueOfFlag: #autoIndent
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
automaticKeyGeneration
	^ self
		valueOfFlag: #automaticKeyGeneration
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
automaticPlatformSettings
	^ self
		valueOfFlag: #automaticPlatformSettings
		ifAbsent: [true]! !

!Preferences class methodsFor: 'misc' stamp: 'dgd 9/7/2004 18:35'!
balloonHelpDelayTime
	"Answer the number of milliseconds before a balloon help 
	should be put up on morphs."
	^ Parameters
		at: #balloonHelpDelayTime
		ifAbsent: [800]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
balloonHelpEnabled
	^ self
		valueOfFlag: #balloonHelpEnabled
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
beDynamic
	^ self
		valueOfFlag: #beDynamic
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
bigDisplay
	^ self
		valueOfFlag: #bigDisplay
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
biggerHandles
	^ self
		valueOfFlag: #biggerHandles
		ifAbsent: [true]! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/15/1999 19:44'!
borderColorWhenRunning
	^ Color green! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'ar 9/28/2005 15:04'!
browseToolClass
	"This method is used for returning the appropiate class for the #browserShowsPackagePane preference. Now that preference modifies the registry so here we query directly to the registry"
	^ SystemBrowser default.! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
browseWithPrettyPrint
	^ self
		valueOfFlag: #browseWithPrettyPrint
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
browserWindowColor
	^ self
		valueOfFlag: #browserWindowColor
		ifAbsent: [Color
				r: 0.8
				g: 1.0
				b: 0.6]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
canRecordWhilePlaying
	^ self
		valueOfFlag: #canRecordWhilePlaying
		ifAbsent: [false]! !

!Preferences class methodsFor: 'text highlighting' stamp: 'lr 7/12/2006 09:25'!
caretWidth
	^ Parameters at: #caretWidth! !

!Preferences class methodsFor: 'text highlighting' stamp: 'lr 7/12/2006 09:25'!
caretWidth: anInteger
	^ Parameters at: #caretWidth put: anInteger! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
caseSensitiveFinds
	^ self
		valueOfFlag: #caseSensitiveFinds
		ifAbsent: [false]! !

!Preferences class methodsFor: 'factored pref panel' stamp: 'sw 4/10/2001 14:29'!
categoriesContainingPreference: prefSymbol
	"Return a list of all categories in which the preference occurs"

	^ (self preferenceAt: prefSymbol ifAbsent: [^ #(unclassified)]) categoryList! !

!Preferences class methodsFor: 'preferences panel' stamp: 'rr 10/1/2005 15:14'!
categoryNames
	| aSet |
	aSet := Set new.
	self dictionaryOfPreferences  do:
			[:aPreference | 
			aSet  addAll:(aPreference categoryList 
						 collect:[:aCategory | aCategory asSymbol])].
	^ aSet! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
cautionBeforeClosing
	^ self
		valueOfFlag: #cautionBeforeClosing
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
changeListWindowColor
	^ self
		valueOfFlag: #changeListWindowColor
		ifAbsent: [Color
				r: 0.8
				g: 1.0
				b: 1.0]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
changeSorterWindowColor
	^ self
		valueOfFlag: #changeSorterWindowColor
		ifAbsent: [Color
				r: 0.8
				g: 1.0
				b: 1.0]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
chasingBrowsers
	^ self
		valueOfFlag: #chasingBrowsers
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
checkForSlips
	^ self
		valueOfFlag: #checkForSlips
		ifAbsent: [true]! !

!Preferences class methodsFor: 'window colors' stamp: 'hpt 12/6/2004 22:49'!
checkForWindowColors
	(self allPreferenceObjects noneSatisfy:  [:aPref | aPref name endsWith: 'WindowColor'])
		ifTrue: [self installBrightWindowColors].! !

!Preferences class methodsFor: 'fonts' stamp: 'bp 6/13/2004 17:20'!
chooseBalloonHelpFont

	BalloonMorph chooseBalloonFont! !

!Preferences class methodsFor: 'fonts' stamp: 'laza 3/25/2004 23:11'!
chooseFontWithPrompt: aPrompt andSendTo: aReceiver withSelector: aSelector
	self chooseFontWithPrompt: aPrompt andSendTo: aReceiver withSelector: aSelector highlight: nil
! !

!Preferences class methodsFor: 'fonts' stamp: 'alain.plantec 5/30/2008 14:18'!
chooseFontWithPrompt: aPrompt andSendTo: aReceiver withSelector: aSelector highlight: currentFont
	TextStyle promptForFont: aPrompt andSendTo: aReceiver withSelector: aSelector highlight: currentFont! !

!Preferences class methodsFor: 'initialization' stamp: 'StephaneDucasse 11/6/2009 20:03'!
chooseInitialSettings
	"Restore the default choices for all of the standard Preferences."

	self allPreferenceObjects do:
		[:aPreference |
			aPreference restoreDefaultValue].
	! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
classCommentOnClassSelect
	^ self
		valueOfFlag: #classCommentOnClassSelect
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
classCommentVersionsBrowserWindowColor
	^ self
		valueOfFlag: #classCommentVersionsBrowserWindowColor
		ifAbsent: [Color
				r: 0.769
				g: 0.653
				b: 1.0]! !

!Preferences class methodsFor: 'halos' stamp: 'AlainPlantec 10/19/2009 18:41'!
classicHaloSpecs
	"Non-iconic halos with traditional placements"

	"Preferences installClassicHaloSpecs"
	"Preferences resetHaloSpecifications"  "  <-  will result in the standard default halos being reinstalled"
	"NB: listed below in clockwise order"

		^ #(
	"  	selector				horiz		vert			color info						icon key
		---------				------		-----------		-------------------------------		---------------"
	(addMenuHandle:		left			top				(red)							none)
	(addDismissHandle:		leftCenter	top				(red		muchLighter)			#haloDismissIcon)
	(addGrabHandle:			center		top				(black)							none)
	(addDragHandle:			rightCenter	top				(brown)							none)
	(addDupHandle:			right		top				(green)							none)	
	(addDebugHandle:		right		topCenter		(blue	veryMuchLighter)		none)
	(addRepaintHandle:		right		center			(lightGray)						none)
	(addGrowHandle:		right		bottom			(yellow)						none)
	(addScaleHandle:		right		bottom			(lightOrange)					none)
	(addFontEmphHandle:	rightCenter	bottom			(lightBrown darker)				none)
	(addFontStyleHandle:		center		bottom			(lightRed)						none)
	(addFontSizeHandle:		leftCenter	bottom			(lightGreen)						none)

	(addRecolorHandle:		right		bottomCenter	(magenta darker)				none)

	(addRotateHandle:		left			bottom			(blue)							none))

! !

!Preferences class methodsFor: 'halos' stamp: 'hpt 9/24/2004 23:34'!
classicHalosInForce
	^ (self preferenceAt: #haloTheme) preferenceValue == #classicHaloSpecs! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
cmdDotEnabled
	^ self
		valueOfFlag: #cmdDotEnabled
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: 'programmatic 7/15/1999 09:55'!
cmdGesturesEnabled
	"compiled programatically -- return hard-coded preference value"
	^ true! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'programmatic 7/15/1999 09:55'!
cmdKeysInText
	"compiled programatically -- return hard-coded preference value"
	^ true! !

!Preferences class methodsFor: 'initialization' stamp: 'NS 1/28/2004 14:43'!
compileAccessMethodForPreference: aPreference
	"Compile an accessor method for the given preference"

	self class compileSilently: (aPreference name, '
	^ self valueOfFlag: #', aPreference name, ' ifAbsent: [', aPreference defaultValue storeString, ']') classified: 'standard queries'! !

!Preferences class methodsFor: 'personalization' stamp: 'NS 1/28/2004 14:43'!
compileHardCodedPref: prefName enable: aBoolean
	"Compile a method that returns a simple true or false (depending on the value of aBoolean) when Preferences is sent prefName as a message"

	self class compileSilently: (prefName asString, '
	"compiled programatically -- return hard-coded preference value"
	^ ', aBoolean storeString) classified: 'hard-coded prefs'.
	
"Preferences compileHardCodedPref: #testing enable: false"! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
conversionMethodsAtFileOut
	^ self
		valueOfFlag: #conversionMethodsAtFileOut
		ifAbsent: [false
			"Form literalScannedAs: 14 notifying: nil 14
			Form literalScannedAs: #OneBitForm notiEfying: nil OneBitForm
			Form literalScannedAs: ##OneBitForm notifying: nil
			OneBitForm->a Form
			Form literalScannedAs: ##Form notifying: nil Form->Form
			Form literalScannedAs: ###Form notifying: nil nilE->Form clas
			"]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
cpuWatcherEnabled
	^ self
		valueOfFlag: #cpuWatcherEnabled
		ifAbsent: [false]! !

!Preferences class methodsFor: 'halos' stamp: 'AlainPlantec 10/19/2009 18:40'!
customHaloSpecs
	"Intended for you to modify to suit your personal preference.  What is implemented in the default here is just a skeleton; in comment at the bottom of this method are some useful lines you may wish to paste in to the main body here, possibly modifying positions, colors, etc..
	Note that in this example, we include:
			Dismiss handle, at top-left
			Menu handle, at top-right
			Resize handle, at bottom-right
			Rotate handle, at bottom-left
			Drag handle, at top-center
			Recolor handle, at left-center.  (this one is NOT part of the standard formulary --
											it is included here to illustrate how to
 											add non-standard halos)
			Note that the optional handles for specialized morphs, such as Sketch, Text, PasteUp, are also included"

	^ #(
	(addDismissHandle:		left			top				(red		muchLighter)			#haloDismissIcon)
	(addMenuHandle:		right		top				(red)							#haloMenuIcon)
	(addDragHandle:			center	top					(brown)							#haloDragIcon)
	(addGrowHandle:		right		bottom			(yellow)						#haloScaleIcon)
	(addScaleHandle:		right		bottom			(lightOrange)					#haloScaleIcon)

	(addRecolorHandle:		left			center			(green muchLighter lighter)		#haloRecolorIcon)

	(addRepaintHandle:		right		center			(lightGray)						#haloPaintIcon)
	(addFontSizeHandle:		leftCenter	bottom			(lightGreen)						#haloFontSizeIcon)
	(addFontStyleHandle:		center		bottom			(lightRed)						#haloFontStyleIcon)
	(addFontEmphHandle:	rightCenter	bottom			(lightBrown darker)				#haloFontEmphIcon)
	(addRotateHandle:		left			bottom			(blue)							#haloRotIcon)

	(addDebugHandle:		right		topCenter		(blue	veryMuchLighter)		#haloDebugIcon)
			)

	"  Other useful handles...

  		selector				horiz		vert			color info						icon key
		---------				------		-----------		-------------------------------		---------------

	(addTileHandle:			left			bottomCenter	(lightBrown)					#haloTileIcon)
	(addViewHandle:			left			center			(cyan)							#haloViewIcon)
	(addGrabHandle:			center		top				(black)							#haloGrabIcon)
	(addDragHandle:			rightCenter	top				(brown)							#haloDragIcon)
	(addDupHandle:			right		top				(green)							#haloDupIcon)	
	(addHelpHandle:			center		bottom			(lightBlue)						#haloHelpIcon)
	(addFewerHandlesHandle:	left		topCenter		(paleBuff)						#haloFewerHandlesIcon)
	(addPaintBgdHandle:		right		center			(lightGray)						#haloPaintIcon)
	(addRepaintHandle:		right		center			(lightGray)						#haloPaintIcon)
	"
! !

!Preferences class methodsFor: 'halos' stamp: 'hpt 9/24/2004 23:34'!
customHalosInForce
	^ (self preferenceAt: #haloTheme) preferenceValue == #customHaloSpecs! !

!Preferences class methodsFor: 'window colors' stamp: 'hpt 12/8/2004 19:34'!
darkenStandardWindowPreferences
	"Make all window-color preferences one shade darker"

	(self allPreferenceObjects 
		select: [:aPref | (aPref name endsWith: 'WindowColor')
								and: [aPref preferenceValue isColor]])
		do: [:aPref | aPref preferenceValue: aPref preferenceValue darker].

"Preferences darkenStandardWindowPreferences"
! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
debugHaloHandle
	^ self
		valueOfFlag: #debugHaloHandle
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: 'mir 3/5/2004 19:22'!
debugLogTimestamp
	^ self
		valueOfFlag: #debugLogTimestamp
		ifAbsent: [false]! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 12/1/1999 13:04'!
debugMenuItemsInvokableFromScripts
	"If true, then items occurring in an object's debug menu will be included in the alternatives offered as arguments to a doMenuItem: tile in the scripting system"
	^ false! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
debugPrintSpaceLog
	^ self
		valueOfFlag: #debugPrintSpaceLog
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
debugShowDamage
	^ self
		valueOfFlag: #debugShowDamage
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
debuggerWindowColor
	^ self
		valueOfFlag: #debuggerWindowColor
		ifAbsent: [Color
				r: 1.0
				g: 0.8
				b: 0.8]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
decorateBrowserButtons
	^ self
		valueOfFlag: #decorateBrowserButtons
		ifAbsent: [false]! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/17/1999 00:40'!
defaultAnnotationRequests
	^ Parameters at: #MethodAnnotations ifAbsent:
		[self setDefaultAnnotationInfo]
	"Preferences annotationInfo"! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/8/1999 10:14'!
defaultAnnotationRequests: newList
	^ Parameters at: #MethodAnnotations put: newList! !

!Preferences class methodsFor: 'parameters' stamp: 'on 5/10/2008 15:10'!
defaultAuthorName
	"Answer the author name to be planted, by default, in a changeset-preamble template.  You can hard-code this to hold your name, thus saving you time when writing the preambles of subsequent changesets"

	^ Author fullName! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/1/2000 14:05'!
defaultPaintingExtent
	"Answer the preferred size for the onion-skin paint area when launching a new painting within a paste-up morph.  Feel free to change the parameters to suit your configuration."

	^ 800 @ 600! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
defaultWindowColor
	^ self
		valueOfFlag: #defaultWindowColor
		ifAbsent: [Color
				r: 1.0
				g: 1.0
				b: 1.0]! !

!Preferences class methodsFor: 'menu parameters' stamp: 'dgd 9/5/2004 16:17'!
defaultWorldColor
	^ Parameters
		at: #defaultWorldColor
		ifAbsent: [ Color r: 0.937 g: 0.937 b: 0.937 ].
! !

!Preferences class methodsFor: 'parameters' stamp: 'jhm 10/15/97 17:31'!
desktopColor
	"Answer the desktop color. Initialize it if necessary."
	
	DesktopColor == nil ifTrue: [DesktopColor := Color gray].
	^ DesktopColor
! !

!Preferences class methodsFor: 'parameters' stamp: 'jhm 10/15/97 17:31'!
desktopColor: aColor
	"Record a new desktop color preference."

	DesktopColor := aColor.
! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 9/6/2000 05:26'!
desktopMenuTitle
	"Answer the title to be used for the 'meta menu'.  For now, you can hard-code this, later someone should make this be a parameter the user can easily change.  sw 9/6/2000"

	^ 'World'    "This is what it has always been"

	"^ 'Desktop'
	^ 'Squeak'
	^ 'Mike''s Control Panel'"! !

!Preferences class methodsFor: 'accessing' stamp: 'rr 10/1/2005 15:14'!
dictionaryOfPreferences
	^DictionaryOfPreferences! !

!Preferences class methodsFor: 'accessing' stamp: 'rr 10/1/2005 15:14'!
dictionaryOfPreferences: anObject
	DictionaryOfPreferences := anObject! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
diffsInChangeList
	^ self
		valueOfFlag: #diffsInChangeList
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
diffsWithPrettyPrint
	^ self
		valueOfFlag: #diffsWithPrettyPrint
		ifAbsent: [false]! !

!Preferences class methodsFor: 'get/set' stamp: 'sw 4/12/2001 23:29'!
disable: aSymbol
	"Shorthand access to enabling a preference of the given name.  If there is none in the image, conjure one up"

	| aPreference |
	aPreference := self preferenceAt: aSymbol ifAbsent:
		[self addPreference: aSymbol category: 'unclassified' default: false balloonHelp: 'this preference was added idiosyncratically and has no help message.'.
		self preferenceAt: aSymbol].
	aPreference preferenceValue: false! !

!Preferences class methodsFor: 'get/set' stamp: 'sw 1/19/2000 13:51'!
disableGently: preferenceNameSymbol
	"Unlike #disable:, this on does not reset the CategoryInfo cache"
	self setPreference: preferenceNameSymbol toValue: false! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
dismissAllOnOptionClose
	^ self
		valueOfFlag: #dismissAllOnOptionClose
		ifAbsent: [false]! !

!Preferences class methodsFor: 'get/set' stamp: 'sw 11/11/1998 11:40'!
doesNotUnderstand: aMessage
	"Look up the message selector as a flag."
	aMessage arguments size > 0 ifTrue: [^ super doesNotUnderstand: aMessage].
	^ self valueOfFlag: aMessage selector
! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
dualChangeSorterWindowColor
	^ self
		valueOfFlag: #dualChangeSorterWindowColor
		ifAbsent: [Color
				r: 0.8
				g: 1.0
				b: 1.0]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
duplicateAllControlAndAltKeys
	^ self
		valueOfFlag: #duplicateAllControlAndAltKeys
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
duplicateControlAndAltKeys
	^ self
		valueOfFlag: #duplicateControlAndAltKeys
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
dynamicProtocolActivation
	^ self
		valueOfFlag: #dynamicProtocolActivation
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
easySelection
	^ self
		valueOfFlag: #easySelection
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
ecompletionCaseSensitive
	^ self
		valueOfFlag: #ecompletionCaseSensitive
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
ecompletionEnabled
	^ self
		valueOfFlag: #ecompletionEnabled
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
ecompletionSmartCharacters
	^ self
		valueOfFlag: #ecompletionSmartCharacters
		ifAbsent: [true]! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 6/13/2001 19:40'!
editAnnotations
	"Put up a window that allows the user to edit annotation specifications"

	| aWindow |
	self currentWorld addMorphCentered: (aWindow := self annotationEditingWindow).
	aWindow activateAndForceLabelToShow

	"Preferences editAnnotations"

! !

!Preferences class methodsFor: 'halos' stamp: 'ar 9/27/2005 20:32'!
editCustomHalos

	ToolSet browse: Preferences class
		selector: #customHaloSpecs! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
editableStringMorphs
	^ self
		valueOfFlag: #editableStringMorphs
		ifAbsent: [false]! !

!Preferences class methodsFor: 'get/set' stamp: 'sw 4/12/2001 23:29'!
enable: aSymbol
	"Shorthand access to enabling a preference of the given name.  If there is none in the image, conjure one up"

	| aPreference |
	aPreference := self preferenceAt: aSymbol ifAbsent:
		[self addPreference: aSymbol category: 'unclassified' default: true balloonHelp: 'this preference was added idiosyncratically and has no help message.'.
		self preferenceAt: aSymbol].
	aPreference preferenceValue: true! !

!Preferences class methodsFor: 'get/set' stamp: 'sw 1/19/2000 13:53'!
enableGently: preferenceNameSymbol
	"Unlike #enable:, this one does not reset the CategoryInfo cache"
	self setPreference: preferenceNameSymbol toValue: true! !

!Preferences class methodsFor: 'get/set' stamp: 'sw 8/12/2000 01:26'!
enableOrDisable: preferenceNameSymbol asPer: aBoolean
	"either enable or disable the given Preference, depending on the value of aBoolean"

	aBoolean ifTrue: [self enable: preferenceNameSymbol] ifFalse: [self disable: preferenceNameSymbol]! !

!Preferences class methodsFor: 'get/set' stamp: 'sw 7/13/2001 21:34'!
enableProjectNavigator
	"Answer whether the project-navigator menu item should be enabled"

	^ true! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 5/16/2003 00:27'!
expungeParameter: aKey
	"If Parameters holds an entry under the given key, remove the entry.  No senders in the current system, but called from the postscript of the change-set that defines it, and potentially useful otherwise."

	Parameters removeKey: aKey ifAbsent: []! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
externalFocusForPluggableText
	^ self
		valueOfFlag: #externalFocusForPluggableText
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
extraDebuggerButtons
	^ self
		valueOfFlag: #extraDebuggerButtons
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
fadedBackgroundWindows
	^ self
		valueOfFlag: #fadedBackgroundWindows
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
fastDragWindowForMorphic
	^ self
		valueOfFlag: #fastDragWindowForMorphic
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
fenceSoundEnabled
	^ self
		valueOfFlag: #fenceSoundEnabled
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: 'mir 6/7/2002 17:10'!
fenceSoundEnabled: aBoolean
	self setPreference: #fenceSoundEnabled toValue: aBoolean! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
fileContentsBrowserWindowColor
	^ self
		valueOfFlag: #fileContentsBrowserWindowColor
		ifAbsent: [Color
				r: 0.8
				g: 0.8
				b: 0.5]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
fileListWindowColor
	^ self
		valueOfFlag: #fileListWindowColor
		ifAbsent: [Color
				r: 1.0
				g: 0.8
				b: 1.0]! !

!Preferences class methodsFor: 'fonts' stamp: 'AlainPlantec 11/29/2009 21:54'!
fontConfigurationMenu
	| aMenu |
	aMenu := MenuMorph new defaultTarget: Preferences.
	aMenu addTitle: 'Standard System Fonts' translated.
	
	aMenu addStayUpIcons.
	
	aMenu add: 'default text font...' translated action: #chooseSystemFont.
	aMenu balloonTextForLastItem: 'Choose the default font to be used for code and  in workspaces, transcripts, etc.' translated.
	aMenu lastItem font: StandardFonts defaultFont.
	
	aMenu add: 'list font...' translated action: #chooseListFont.
	aMenu lastItem font: Preferences standardListFont.
	aMenu balloonTextForLastItem: 'Choose the font to be used in list panes' translated.
	
	aMenu add: 'halo label font...' translated action: #chooseHaloLabelFont.
	aMenu lastItem font: Preferences standardHaloLabelFont.
	aMenu balloonTextForLastItem: 'Choose the font to be used on labels ih halo' translated.

	aMenu add: 'menu font...' translated action: #chooseMenuFont.
	aMenu lastItem font: Preferences standardMenuFont.
	aMenu balloonTextForLastItem: 'Choose the font to be used in menus' translated.
	
	aMenu add: 'window-title font...' translated action: #chooseWindowTitleFont.
	aMenu lastItem font: Preferences windowTitleFont.
	aMenu balloonTextForLastItem: 'Choose the font to be used in window titles.' translated.

	aMenu add: 'balloon-help font...' translated action: #chooseBalloonHelpFont.
	aMenu lastItem font: Preferences standardBalloonHelpFont.
	aMenu balloonTextForLastItem: 'choose the font to be used when presenting balloon help.' translated.
	
	aMenu add: 'code font...' translated action: #chooseCodeFont. 
	aMenu lastItem font: Preferences standardCodeFont. 
	aMenu balloonTextForLastItem: 'Choose the font to be used in code panes.' translated.

	aMenu add: 'button font...' translated action: #chooseStandardButtonFont.
	aMenu lastItem font: Preferences standardButtonFont.
	aMenu balloonTextForLastItem: 'Choose the font to be used in buttons.' translated.

	aMenu addLine.
	aMenu add: 'demo mode' translated action: #setDemoFonts.
	aMenu balloonTextForLastItem: 'Set Fonts usable for giving a presentation' translated.

	aMenu addLine.
	aMenu add: 'restore default font choices' translated action: #restoreDefaultFonts.
	aMenu balloonTextForLastItem: 'Use the standard system font defaults' translated.
	
	aMenu add: 'print default font choices' translated action: #printStandardSystemFonts.
	aMenu balloonTextForLastItem: 'Print the standard system font defaults to the Transcript' translated.

	^ aMenu! !

!Preferences class methodsFor: 'scrollbar parameters' stamp: 'dgd 3/25/2003 19:58'!
fontFactor
	"answer the convertion factor for resizing element based on font  
	size"
	| factor |
	factor := TextStyle defaultFont height / 12.0.
	^ factor > 1.0
		ifTrue: [1 + (factor - 1.0 * 0.5)]
		ifFalse: [factor]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
fullScreenLeavesDeskMargins
	^ self
		valueOfFlag: #fullScreenLeavesDeskMargins
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
generalizedYellowButtonMenu
	^ self
		valueOfFlag: #generalizedYellowButtonMenu
		ifAbsent: [true]! !

!Preferences class methodsFor: 'misc' stamp: 'AlainPlantec 10/17/2009 18:18'!
giveHelpWithPreferences
	"Open up a workspace with explanatory info in it about Preferences"

	| aString aHelpString |
	aString := String streamContents: [:aStream | 
		aStream nextPutAll:

'Many aspects of the system are governed by the settings of various "Preferences".  

Click on any of brown tabs at the top of the panel to see all the preferences in that category.  
Or type in to the box above the Search button, then hit Search, and all Preferences matching whatever you typed in will appear in the "search results" category.  A preference is considered to match your search if either its name matches the characters *or* if anything in the balloon help provided for the preferences matches the search text.

To find out more about any particular Preference, hold the mouse over it for a moment and balloon help will appear.  Also, a complete list of all the Preferences, with documentation for each, is included below.

Preferences whose names are in shown in bold in the Preferences Panel are designated as being allowed to vary from project to project; those whose name are not in bold are "global", which is to say, they apply equally whatever project you are in.

Click on the name of any preference to get a menu which allows you to *change* whether the preference should vary from project to project or should be global, and also allows you to browse all the senders of the preference, and to discover all the categories under which the preference has been classified, and to be handed a button that you can drop wherever you please that will control the preference.

If you like all your current Preferences settings, you may wish to hit the "Save Current Settings as my Personal Preferences" button.  Once you have done that, you can at any point in the future hit "Restore my Personal Preferences" and all your saved settings will get restored immediately.

Also, you can use "themes" to set multiple preferences all at once; click on the "change theme..." button in the Preferences panel, or seek out the themes item in the Appearance menu.' translated.

	aStream cr; cr; nextPutAll: '-----------------------------------------------------------------';
		cr; cr; nextPutAll:  'Alphabetical listing of all Preferences' translated; cr; cr.
   (Preferences allPreferenceObjects asSortedCollection: [:a :b | a name < b name]) do:
	[:pref |
		aStream nextPutAll: pref name; cr.
		aHelpString := pref helpString translated.
		(aHelpString beginsWith: pref name) ifTrue:
			[aHelpString := aHelpString copyFrom: (pref name size + 3) to: aHelpString size].
		aHelpString := (aHelpString copyReplaceAll: String cr with: ' ')  copyWithout: Character tab.
		aStream nextPutAll: aHelpString capitalized.
		(aHelpString last == $.) ifFalse: [aStream nextPut: $.].
        aStream cr; cr]].

	UIManager default edit: aString label: 'About Preferences' translated

"Preferences giveHelpWithPreferences"! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
gradientButtonLook
	^ self
		valueOfFlag: #gradientButtonLook
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
gradientMenu
	^ self
		valueOfFlag: #gradientMenu
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
gradientScrollBars
	^ self
		valueOfFlag: #gradientScrollBars
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
gradientScrollbarLook
	^ self
		valueOfFlag: #gradientScrollbarLook
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
haloEnclosesFullBounds
	^ self
		valueOfFlag: #haloEnclosesFullBounds
		ifAbsent: [false]! !

!Preferences class methodsFor: 'halos' stamp: 'sw 10/30/2000 13:32'!
haloSpecifications
	"Answer a list of HaloSpecs that describe which halos are to be used, what they should look like, and where they should be situated"

	^ Parameters at: #HaloSpecs ifAbsent:
			[self installHaloTheme: #iconicHaloSpecifications.
			^ Parameters at: #HaloSpecs]

	"Preferences haloSpecifications"
	"Preferences resetHaloSpecifications"
! !

!Preferences class methodsFor: 'halos' stamp: 'michael.rueger 3/9/2009 18:46'!
haloSpecificationsForWorld
	| desired |
	"Answer a list of HaloSpecs that describe which halos are to be used on a world halo, what they should look like, and where they should be situated"
	"Preferences resetHaloSpecifications"

	desired := #(addDebugHandle: addMenuHandle:   addHelpHandle:  addRecolorHandle:).
	^ self haloSpecifications select:
		[:spec | desired includes: spec addHandleSelector]! !

!Preferences class methodsFor: 'halos' stamp: ''!
haloTheme
	^ self
		valueOfFlag: #haloTheme
		ifAbsent: [#iconicHaloSpecifications]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
haloTransitions
	^ self
		valueOfFlag: #haloTransitions
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
hierarchyBrowserIsRB
	^ self
		valueOfFlag: #hierarchyBrowserIsRB
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
higherPerformance
	^ self
		valueOfFlag: #higherPerformance
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
honorDesktopCmdKeys
	^ self
		valueOfFlag: #honorDesktopCmdKeys
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
httpProxyPort
	^ self
		valueOfFlag: #httpProxyPort
		ifAbsent: [80]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
httpProxyServer
	^ self
		valueOfFlag: #httpProxyServer
		ifAbsent: ['']! !

!Preferences class methodsFor: 'halos' stamp: 'AlainPlantec 10/19/2009 18:45'!
iconicHaloSpecifications
	"Answer an array that characterizes the locations, colors, icons, and selectors of the halo handles that may be used in the iconic halo scheme"

	"Preferences resetHaloSpecifications"

	^ #(
	"  	selector				horiz		vert			color info						icon key
		---------				------		-----------		-------------------------------		---------------"
	(addCollapseHandle:		left			topCenter		(tan)							#haloCollapseIcon)
	(addDebugHandle:		right		topCenter		(blue	veryMuchLighter)		#haloDebugIcon)
	(addDismissHandle:		left			top				(red		muchLighter)			#haloDismissIcon)
	(addRotateHandle:		left			bottom			(blue)							#haloRotIcon)
	(addMenuHandle:		leftCenter	top				(red)							#haloMenuIcon)
	(addGrabHandle:			center		top				(black)							#haloGrabIcon)
	(addDragHandle:			rightCenter	top				(brown)							#haloDragIcon)
	(addDupHandle:			right		top				(green)							#haloDupIcon)	
	(addHelpHandle:			center		bottom			(lightBlue)						#haloHelpIcon)
	(addGrowHandle:		right		bottom			(yellow)						#haloScaleIcon)
	(addScaleHandle:		right		bottom			(lightOrange)					#haloScaleIcon)
	(addRepaintHandle:		right		center			(lightGray)						#haloPaintIcon)
	(addFontSizeHandle:		leftCenter	bottom			(lightGreen)						#haloFontSizeIcon)
	(addFontStyleHandle:		center		bottom			(lightRed)						#haloFontStyleIcon)
	(addFontEmphHandle:	rightCenter	bottom			(lightBrown darker)				#haloFontEmphIcon)
	(addRecolorHandle:		right		bottomCenter	(magenta darker)				#haloRecolorIcon)
	(addChooseGraphicHandle:	right	bottomCenter	(green muchLighter)			#haloChooseGraphicIcon)
		) ! !

!Preferences class methodsFor: 'halos' stamp: 'hpt 9/24/2004 23:34'!
iconicHalosInForce
	^ (self preferenceAt: #haloTheme) preferenceValue == #iconicHaloSpecifications! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
includeSoundControlInNavigator
	^ self
		valueOfFlag: #includeSoundControlInNavigator
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
infiniteUndo
	^ self
		valueOfFlag: #infiniteUndo
		ifAbsent: [false]! !

!Preferences class methodsFor: 'reacting to change' stamp: 'sw 4/12/2001 01:32'!
infiniteUndoChanged
	"The infiniteUndo preference changed; react"
	
	self infiniteUndo ifFalse:
		[CommandHistory resetAllHistory]! !

!Preferences class methodsFor: 'initialization' stamp: 'rr 10/1/2005 15:14'!
initializeDictionaryOfPreferences
	"Initialize the DictionaryOfPreferences to be an empty IdentityDictionary"

	"Preferences initializeDictionaryOfPreferences"

	self  dictionaryOfPreferences:IdentityDictionary new! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 9/7/1999 12:45'!
initializeParameters
	"Preferences initializeParameters"
	Parameters := IdentityDictionary new.
	self restoreDefaultMenuParameters.
	Parameters at: #maxBalloonHelpLineLength put: 28.
	self initializeTextHighlightingParameters! !

!Preferences class methodsFor: 'text highlighting' stamp: 'lr 7/12/2006 09:42'!
initializeTextHighlightingParameters
	"Preferences initializeTextHighlightingParameters"

	self
		caretWidth: 2;
		insertionPointColor: (TranslucentColor r: 0.0 g: 0.0 b: 0.8 alpha: 0.8);
		textHighlightColor: (TranslucentColor r: 0.0 g: 0.0 b: 0.8 alpha: 0.2)! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
inlineServicesInMenu
	^ self
		valueOfFlag: #inlineServicesInMenu
		ifAbsent: [true]! !

!Preferences class methodsFor: 'text highlighting' stamp: 'sw 9/7/1999 12:53'!
insertionPointColor
	^ Parameters at: #insertionPointColor! !

!Preferences class methodsFor: 'text highlighting' stamp: 'sw 9/7/1999 12:54'!
insertionPointColor: aColor
	Parameters at: #insertionPointColor put: aColor! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 1/24/2001 21:44'!
inspectParameters
	"Open up an inspector on the Parameters of Preferences.  This is crude!!"

	Parameters inspectWithLabel: 'Parameters'! !

!Preferences class methodsFor: 'window colors' stamp: 'sw 2/26/2002 13:56'!
installBrightWindowColors
	"Install the factory-provided default window colors for all tools"

	"Preferences installBrightWindowColors"

	self installWindowColorsVia: [:aSpec | aSpec brightColor]! !

!Preferences class methodsFor: 'halos' stamp: 'sw 1/28/2000 10:35'!
installClassicHaloSpecs
	"Install an alternative set of halos,  rather more based on the old placements, and without icons, , and lacking the scripting-relating handles.."
	"Preferences installClassicHaloSpecs"
	"Preferences resetHaloSpecifications"  "  <-  will result in the standard default halos being reinstalled"
	self installHaloTheme: #classicHaloSpecs! !

!Preferences class methodsFor: 'halos' stamp: 'sw 1/28/2000 10:36'!
installCustomHaloSpecs
	"Install an alternative set of halos, as customized by the user"
	"Preferences installCustomHaloSpecs"
	self installHaloTheme: #customHaloSpecs! !

!Preferences class methodsFor: 'halos' stamp: 'sw 1/27/2000 16:45'!
installHaloSpecsFromArray: anArray

	| aColor |
	^ Parameters at: #HaloSpecs put: 
		(anArray collect:
			[:quin |
				aColor := Color.
				quin fourth do: [:sel | aColor := aColor perform: sel].
				HaloSpec new 
					horizontalPlacement: quin second
					verticalPlacement: quin third 
					color: aColor
					iconSymbol: quin fifth
					addHandleSelector: quin first])! !

!Preferences class methodsFor: 'halos' stamp: 'hpt 9/24/2004 23:35'!
installHaloTheme: themeSymbol
	self installHaloSpecsFromArray: (self perform: themeSymbol).
	(self preferenceAt: #haloTheme) preferenceValue: themeSymbol.
	! !

!Preferences class methodsFor: 'halos' stamp: 'sw 1/28/2000 10:36'!
installIconicHaloSpecs
	"Install an alternative set of halos,  rather more based on the old placements, and without icons, , and lacking the scripting-relating handles.."
	"Preferences installIconicHaloSpecs"
	self installHaloTheme: #iconicHaloSpecifications! !

!Preferences class methodsFor: 'window colors' stamp: 'hpt 12/8/2004 23:17'!
installMissingWindowColors
	"Install the factory-provided bright window colors  -- a one-time bootstrap"
	"Preferences installMissingWindowColors"
	| color |
	self windowColorTable do:
		[:aColorSpec |
			color := (Color colorFrom: aColorSpec brightColor).
			self setWindowColorFor: aColorSpec classSymbol to: color]! !

!Preferences class methodsFor: 'window colors' stamp: 'sw 2/26/2002 13:55'!
installPastelWindowColors
	"Install the factory-provided default pastel window colors for all tools"

	"Preferences installBrightWindowColors"
	self installWindowColorsVia: [:aSpec | aSpec pastelColor]! !

!Preferences class methodsFor: 'halos' stamp: 'sw 1/28/2000 10:36'!
installSimpleHaloSpecs
	"Preferences installSimpleHaloSpecs"
	self installHaloTheme: #simpleFullHaloSpecifications! !

!Preferences class methodsFor: 'misc' stamp: 'yo 7/2/2004 19:44'!
installTheme: aSymbol
	"Install the theme represented by aSymbol.  The code that makes the theme-specific changes is lodged in a method of the same name as aSymbol, which must reside in category #themes in Preferences class"

	self perform: aSymbol.
	self inform: ('Theme {1} is now installed.
Many of the changes will only be
noticeable in new windows that you
create from now on.' translated format: {aSymbol translated}).! !

!Preferences class methodsFor: 'window colors' stamp: 'sw 2/26/2002 12:55'!
installUniformWindowColors
	"Install the factory-provided uniform window colors for all tools"

	"Preferences installUniformWindowColors"
	self installWindowColorsVia: [:aQuad | #white]! !

!Preferences class methodsFor: 'window colors' stamp: 'hpt 12/8/2004 20:10'!
installWindowColorsVia: colorSpecBlock
	"Install windows colors using colorSpecBlock to deliver the color source for each element; the block is handed a WindowColorSpec object"
	"Preferences installBrightWindowColors"
	| color |
	self windowColorTable do:
		[:aColorSpec |
			color := (Color colorFrom: (colorSpecBlock value: aColorSpec)).
			self setWindowColorFor: aColorSpec classSymbol to: color]
! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
instanceBrowserWindowColor
	^ self
		valueOfFlag: #instanceBrowserWindowColor
		ifAbsent: [Color
				r: 0.806
				g: 1.0
				b: 1.0]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
keepTasklistOpen
	^ self
		valueOfFlag: #keepTasklistOpen
		ifAbsent: [false]! !

!Preferences class methodsFor: 'text highlighting' stamp: 'dew 1/8/2002 01:07'!
keyboardFocusColor
	"Answer the keyboard focus color, initializing it if necessary"

	^ Parameters at: #keyboardFocusColor ifAbsentPut: [Color lightGray]

"
Parameters removeKey: #keyboardFocusColor.
Preferences keyboardFocusColor
"! !

!Preferences class methodsFor: 'text highlighting' stamp: 'sw 12/7/2001 00:44'!
keyboardFocusColor: aColor
	"Set the keyboard focus color"

	Parameters at: #keyboardFocusColor put: aColor! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
lexiconWindowColor
	^ self
		valueOfFlag: #lexiconWindowColor
		ifAbsent: [Color
				r: 0.878
				g: 1.0
				b: 0.878]! !

!Preferences class methodsFor: 'window colors' stamp: 'hpt 12/8/2004 19:35'!
lightenStandardWindowPreferences
	"Make all window-color preferences one shade darker"

		(self allPreferenceObjects 
		select: [:aPref | (aPref name endsWith: 'WindowColor')
								and: [aPref preferenceValue isColor]])
		do: [:aPref | aPref preferenceValue: aPref preferenceValue lighter].

"Preferences lightenStandardWindowPreferences"
! !

!Preferences class methodsFor: 'personalization' stamp: 'marcus.denker 11/10/2008 10:04'!
loadPreferencesFrom: aFileName
	| stream params dict desktopColor |
	stream := ReferenceStream fileNamed: aFileName.
	params := stream next.
	self assert: (params isKindOf: IdentityDictionary).
	params removeKey: #PersonalDictionaryOfPreferences.
	dict := stream next.
	self assert: (dict isKindOf: IdentityDictionary).
	desktopColor := stream next.
	stream close.
	dict keysAndValuesDo:
		[:key :value | (self preferenceAt: key ifAbsent: [nil]) ifNotNil:
			[:pref | pref preferenceValue: value preferenceValue]].

	params keysAndValuesDo: [ :key :value | self setParameter: key to: value ].

	World fillStyle: desktopColor ! !

!Preferences class methodsFor: 'initialization' stamp: 'tak 8/3/2005 21:17'!
localeChanged
	LocaleID current isoLanguage = 'ja'
		ifTrue: [Preferences enable: #useFormsInPaintBox]
		ifFalse: [Preferences disable: #useFormsInPaintBox]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
logDebuggerStackToFile
	^ self
		valueOfFlag: #logDebuggerStackToFile
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
magicHalos
	^ self
		valueOfFlag: #magicHalos
		ifAbsent: [false
			"Form literalScannedAs: 14 notifying: nil 14
			Form literalScannedAs: #OneBitForm notiEfying: nil OneBitForm
			Form literalScannedAs: ##OneBitForm notifying: nil
			OneBitForm->a Form
			Form literalScannedAs: ##Form notifying: nil Form->Form
			Form literalScannedAs: ###Form notifying: nil nilE->Form clas
			"]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
maintainHalos
	^ self
		valueOfFlag: #maintainHalos
		ifAbsent: [true]! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 11/5/1998 16:49'!
maxBalloonHelpLineLength
	^ Parameters at: #maxBalloonHelpLineLength! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
menuAppearance3d
	^ self
		valueOfFlag: #menuAppearance3d
		ifAbsent: [true]! !

!Preferences class methodsFor: 'menu parameters' stamp: 'di 1/14/1999 20:16'!
menuBorderColor
	Display depth <= 2 ifTrue: [^ Color black].
	^ Parameters at: #menuBorderColor! !

!Preferences class methodsFor: 'menu parameters' stamp: 'sw 11/3/1998 11:16'!
menuBorderWidth
	^ Parameters at: #menuBorderWidth! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
menuButtonInToolPane
	^ self
		valueOfFlag: #menuButtonInToolPane
		ifAbsent: [false]! !

!Preferences class methodsFor: 'menu parameters' stamp: 'di 1/14/1999 20:17'!
menuColor
	Display depth <= 2 ifTrue: [^ Color white].
	^ Parameters at: #menuColor! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
menuColorFromWorld
	^ self
		valueOfFlag: #menuColorFromWorld
		ifAbsent: [true
			"success"]! !

!Preferences class methodsFor: 'misc' stamp: 'dgd 9/21/2003 13:51'!
menuColorString
	^ ((self valueOfFlag: #menuColorFromWorld)
		ifTrue: ['stop menu-color-from-world']
		ifFalse: ['start menu-color-from-world']) translated! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
menuKeyboardControl
	^ self
		valueOfFlag: #menuKeyboardControl
		ifAbsent: [true]! !

!Preferences class methodsFor: 'menu parameters' stamp: 'dgd 3/23/2003 11:06'!
menuLineColor
	^ Parameters
		at: #menuLineColor
		ifAbsentPut: [Preferences menuBorderColor lighter]! !

!Preferences class methodsFor: 'menu parameters' stamp: 'dgd 8/30/2004 20:59'!
menuSelectionColor
	^ Parameters
		at: #menuSelectionColor
		ifAbsent: [nil]! !

!Preferences class methodsFor: 'menu parameters' stamp: 'di 1/14/1999 20:19'!
menuTitleBorderColor
	Display depth <= 2 ifTrue: [^ Color black].
	^ Parameters at: #menuTitleBorderColor! !

!Preferences class methodsFor: 'menu parameters' stamp: 'sw 11/3/1998 11:16'!
menuTitleBorderWidth
	^ Parameters at: #menuTitleBorderWidth! !

!Preferences class methodsFor: 'menu parameters' stamp: 'di 1/14/1999 20:18'!
menuTitleColor
	Display depth = 1 ifTrue: [^ Color white].
	Display depth = 2 ifTrue: [^ Color gray].
	^ Parameters at: #menuTitleColor! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
menuWithIcons
	^ self
		valueOfFlag: #menuWithIcons
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
mercuryPanel
	^ self
		valueOfFlag: #mercuryPanel
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
messageListWindowColor
	^ self
		valueOfFlag: #messageListWindowColor
		ifAbsent: [Color
				r: 0.8
				g: 1.0
				b: 1.0]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
messageNamesWindowColor
	^ self
		valueOfFlag: #messageNamesWindowColor
		ifAbsent: [Color
				r: 0.645
				g: 1.0
				b: 0.452]! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 8/11/2002 02:18'!
messengersInViewers
	"A coming technology..."

	^ false! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 8/18/2000 13:26'!
metaMenuDisabled
	"If true, then click/cmd-click on the desktop will not bring up the World menu.  Can be changed manually right here, and can be programattically changed via a call of the following form:

	Preferences compileHardCodedPref: #metaMenuDisabled enable: true"

	^ false! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
methodFinderWindowColor
	^ self
		valueOfFlag: #methodFinderWindowColor
		ifAbsent: [Color
				r: 0.4
				g: 1.0
				b: 1.0]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
modalColorPickers
	^ self
		valueOfFlag: #modalColorPickers
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
mouseClickForKeyboardFocus
	^ self
		valueOfFlag: #mouseClickForKeyboardFocus
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
mouseOverForKeyboardFocus
	^ self
		valueOfFlag: #mouseOverForKeyboardFocus
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
mouseOverHalos
	^ self
		valueOfFlag: #mouseOverHalos
		ifAbsent: [false]! !

!Preferences class methodsFor: 'reacting to change' stamp: 'mir 9/12/2001 15:15'!
mouseOverHalosChanged
	World wantsMouseOverHalos: self mouseOverHalos! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
multipleTextUndo
	^ self
		valueOfFlag: #multipleTextUndo
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
noWindowAnimationForClosing
	^ self
		valueOfFlag: #noWindowAnimationForClosing
		ifAbsent: [false]! !

!Preferences class methodsFor: 'misc' stamp: 'sw 4/24/2001 12:02'!
okayToChangeProjectLocalnessOf: prefSymbol
	"Answer whether it would be okay to allow the user to switch the setting of whether or not the preference symbol is local to a project.  Formerly useful and perhaps again will be, though to be sure this is a non-modular design."

	^ (#() includes: prefSymbol) not! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
oliveHandleForScriptedObjects
	^ self
		valueOfFlag: #oliveHandleForScriptedObjects
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
optionalButtons
	^ self
		valueOfFlag: #optionalButtons
		ifAbsent: [false]! !

!Preferences class methodsFor: 'reacting to change' stamp: 'AlainPlantec 10/17/2009 18:19'!
optionalButtonsChanged
	"The setting of the optionalButtons preference changed; react."
! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
packageBrowserWindowColor
	^ self
		valueOfFlag: #packageBrowserWindowColor
		ifAbsent: [Color
				r: 1.0
				g: 1.0
				b: 0.6]! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 1/27/2000 23:02'!
parameterAt: aKey
	^ Parameters at: aKey ifAbsent: [nil]! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 9/28/2001 08:52'!
parameterAt: aKey default: defaultValueBlock
	"Deprecated interface; no surviving senders in the released image, but clients probably still use"

	^ self parameterAt: aKey ifAbsentPut: defaultValueBlock! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/7/2001 14:37'!
parameterAt: aKey ifAbsent: aBlock
	"Answer the parameter saved at the given key; if there is no such key in the Parameters dictionary, evaluate aBlock"

	^ Parameters at: aKey ifAbsent: [aBlock value]! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 9/28/2001 08:40'!
parameterAt: aKey ifAbsentPut: defaultValueBlock
	"Return the Parameter setting at the given key.  If there is no entry for this key in the Parameters dictionary, create one with the value of defaultValueBlock as its value"

	^ Parameters at: aKey ifAbsentPut: defaultValueBlock! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
postscriptStoredAsEPS
	^ self
		valueOfFlag: #postscriptStoredAsEPS
		ifAbsent: [false]! !

!Preferences class methodsFor: 'preference-object access' stamp: 'rr 10/1/2005 15:14'!
preferenceAt: aSymbol 
	"Answer the Preference object at the given symbol, or nil if not there"

	^ self dictionaryOfPreferences  at:aSymbol  ifAbsent:[nil]! !

!Preferences class methodsFor: 'preference-object access' stamp: 'rr 10/1/2005 15:14'!
preferenceAt: aSymbol ifAbsent: aBlock 
	"Answer the Preference object at the given symbol, or the value of aBlock if not present"

	^ self dictionaryOfPreferences  at:aSymbol  ifAbsent:[aBlock value]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
preferenceBrowserWindowColor
	^ self
		valueOfFlag: #preferenceBrowserWindowColor
		ifAbsent: [Color
				r: 0.645
				g: 1.0
				b: 1.0]! !

!Preferences class methodsFor: 'preferences panel' stamp: 'rr 10/1/2005 15:14'!
preferenceObjectsInCategory: aCategorySymbol 
	"Answer a list of Preference objects that reside in the given category, in alphabetical order"

	^ (self dictionaryOfPreferences 
		 select:[:aPreference | aPreference categoryList  includes:aCategorySymbol]) 
			 asSortedCollection:
				[:pref1 :pref2 | 
				pref1 viewRegistry viewOrder  <pref2 viewRegistry viewOrder  or:
						[pref1 viewRegistry viewOrder  =pref2 viewRegistry viewOrder 
							 &(pref1 name  <pref2 name)]]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
preferencesPanelWindowColor
	^ self
		valueOfFlag: #preferencesPanelWindowColor
		ifAbsent: [Color
				r: 0.645
				g: 1.0
				b: 1.0]! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 8/29/2000 15:01'!
preserveCommandExcursions
	"An architecture is in place for storing command excursions to which access is otherwise cut off by having taken a variant branch, but it is not accessible unless you hand-code this preference to true -- which I suggest you do only with fingers crossed."

	^ false! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
preserveTrash
	^ self
		valueOfFlag: #preserveTrash
		ifAbsent: [false]! !

!Preferences class methodsFor: 'fonts' stamp: 'AlainPlantec 10/17/2009 18:20'!
printStandardSystemFonts
	"self printStandardSystemFonts"

	| string |
	string := String streamContents: [ :s |

	#(standardDefaultTextFont standardListFont 
	standardMenuFont windowTitleFont 
	standardBalloonHelpFont standardCodeFont standardButtonFont) do: [:selector |
		| font |
		font := Preferences perform: selector.
		s
			nextPutAll: selector; space;
			nextPutAll: font familyName; space;
			nextPutAll: (AbstractFont emphasisStringFor: font emphasis);
			nextPutAll: ' points: ';
			print: font pointSize;
			nextPutAll: ' height: ';
			print: font height;
			cr
		]].

	(StringHolder new)
		contents: string;
		openLabel: 'Current system font settings' translated.
! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
projectsSentToDisk
	^ self
		valueOfFlag: #projectsSentToDisk
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
promptForUpdateServer
	^ self
		valueOfFlag: #promptForUpdateServer
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
promptOnRefactoring
	^ self
		valueOfFlag: #promptOnRefactoring
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
purgeUndoOnQuit
	^ self
		valueOfFlag: #purgeUndoOnQuit
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
raiseDeprecatedWarnings
	^ self
		valueOfFlag: #raiseDeprecatedWarnings
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
readDocumentAtStartup
	^ self
		valueOfFlag: #readDocumentAtStartup
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
readOnlyMode
	^ self
		valueOfFlag: #readOnlyMode
		ifAbsent: [false]! !

!Preferences class methodsFor: 'fonts' stamp: 'AlainPlantec 10/17/2009 18:21'!
refreshFontSettings
	"Try to update all the current font settings to make things consistent."

	self setWindowTitleFontTo: (self windowTitleFont);
		setListFontTo: (self standardListFont);
		setMenuFontTo: (self standardMenuFont);
		setSystemFontTo: (TextStyle defaultFont);
		setCodeFontTo: (self standardCodeFont);
		setBalloonHelpFontTo: (BalloonMorph balloonFont).

	SystemWindow allSubInstancesDo: [ :s | | rawLabel |
		rawLabel := s getRawLabel.
		rawLabel owner vResizing: #spaceFill.
		rawLabel font: rawLabel font.
		s setLabel: s label.
		s replaceBoxes ].! !

!Preferences class methodsFor: 'initialization' stamp: 'ar 9/27/2005 21:49'!
removeObsolete
	"Remove obsolete preferences"
	Preference allInstancesDo:[:pref|
		pref isObsolete ifTrue:[self removePreference: pref].
	].! !

!Preferences class methodsFor: 'initialization' stamp: 'HenrikSperreJohansen 11/23/2009 17:59'!
removePreference: aSymbol
	"Remove all memory of the given preference symbol."
	
	self dictionaryOfPreferences removeKey: aSymbol ifAbsent: [].
	Parameters removeKey: aSymbol ifAbsent: [].
	(Parameters at: #PersonalDictionaryOfPreferences ifAbsent: []) 
		ifNotNil: [:personalDict | personalDict removeKey: aSymbol ifAbsent: []]! !

!Preferences class methodsFor: 'halos' stamp: 'sw 1/25/2000 20:10'!
resetHaloSpecifications
	"Preferences resetHaloSpecifications"

	^ Parameters removeKey: #HaloSpecs ifAbsent: []! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
restartAlsoProceeds
	^ self
		valueOfFlag: #restartAlsoProceeds
		ifAbsent: [true]! !

!Preferences class methodsFor: 'fonts' stamp: 'MarcusDenker 11/21/2009 12:26'!
restoreDefaultFonts
	"Since this is called from menus, we can take the opportunity to prompt for missing font styles."
	"
	Preferences restoreDefaultFonts
	"

	Preferences setDefaultFonts: #(
		(setSystemFontTo: 'Bitmap DejaVu Sans' 9)
		(setCodeFontTo: 'Bitmap DejaVu Sans' 9)
		(setListFontTo: 'Bitmap DejaVu Sans' 9)
		(setMenuFontTo: 'Bitmap DejaVu Sans' 9)
		(setPaintBoxButtonFontTo: 'Accuny' 12)
		(setWindowTitleFontTo: 'Bitmap DejaVu Sans' 12)
		(setBalloonHelpFontTo: 'Accuny' 10)
		(setButtonFontTo: 'Accuny' 9))! !

!Preferences class methodsFor: 'menu parameters' stamp: 'dgd 3/23/2003 11:11'!
restoreDefaultMenuParameters
	"Restore the four color choices of the original implementors of  
	MorphicMenus"
	" 
	Preferences restoreDefaultMenuParameters
	"
	Parameters
		at: #menuColor
		put: (Color
				r: 0.97
				g: 0.97
				b: 0.97).
	Parameters
		at: #menuBorderColor
		put: (Color
				r: 0.167
				g: 0.167
				b: 1.0).
	Parameters at: #menuBorderWidth put: 2.
	Parameters at: #menuTitleColor put: (Color
			r: 0.4
			g: 0.8
			b: 0.9) twiceDarker.
	Parameters
		at: #menuTitleBorderColor
		put: (Color
				r: 0.333
				g: 0.667
				b: 0.751).
	Parameters at: #menuTitleBorderWidth put: 1.
	Parameters
		at: #menuLineColor
		put: (Preferences menuBorderColor lighter)! !

!Preferences class methodsFor: 'personalization' stamp: 'marcus.denker 11/10/2008 10:04'!
restorePersonalPreferences
	"Restore all the user's saved personal preference settings"

	| savedPrefs |
	savedPrefs := self parameterAt: #PersonalDictionaryOfPreferences ifAbsent: [^ self inform: 'There are no personal preferences saved in this image yet'].

	savedPrefs associationsDo:
		[:assoc | (self preferenceAt: assoc key ifAbsent: [nil]) ifNotNil:
			[:pref | pref preferenceValue: assoc value preferenceValue]]! !

!Preferences class methodsFor: 'personalization' stamp: 'nk 11/17/2002 12:07'!
restorePreferencesFromDisk
	(FileDirectory default fileExists: 'my.prefs')
		ifTrue: [ Cursor wait showWhile: [
			[ self loadPreferencesFrom: 'my.prefs' ] on: Error do: [ :ex | self inform: 'there was an error restoring the preferences' ]
		] ]
		ifFalse: [ self inform: 'you haven''t saved your preferences yet!!' ].
	! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
reverseWindowStagger
	^ self
		valueOfFlag: #reverseWindowStagger
		ifAbsent: [true]! !

!Preferences class methodsFor: 'misc' stamp: 'yo 2/10/2005 16:15'!
roundedCornersString
	^ (((self valueOfFlag: #roundedWindowCorners)
		ifTrue: ['stop']
		ifFalse: ['start']) , ' rounding window corners') translated! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
roundedMenuCorners
	^ self
		valueOfFlag: #roundedMenuCorners
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
roundedWindowCorners
	^ self
		valueOfFlag: #roundedWindowCorners
		ifAbsent: [true]! !

!Preferences class methodsFor: 'reacting to change' stamp: 'sw 4/12/2001 01:11'!
roundedWindowCornersChanged
	"The user changed the value of the roundedWindowCorners preference.  React"

	ActiveWorld fullRepaintNeeded! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
rulesAccessSelector
	^ self
		valueOfFlag: #rulesAccessSelector
		ifAbsent: ['defaultCategorizationRules']! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
rulesGlobalName
	^ self
		valueOfFlag: #rulesGlobalName
		ifAbsent: ['AutomaticMethodCategorizer']! !

!Preferences class methodsFor: 'personalization' stamp: 'rr 10/1/2005 15:14'!
savePersonalPreferences
	"Save the current list of Preference settings as the user's personal choices"

	self  setParameter:#PersonalDictionaryOfPreferences
		 to:self dictionaryOfPreferences deepCopy! !

!Preferences class methodsFor: 'parameters' stamp: ''!
scrollBarColor
	"Answer the preferred color for scroll bar elevators."

	^ Color gray! !

!Preferences class methodsFor: 'parameters' stamp: ''!
scrollBarWidth
	"Answer the preferred width for scroll bars."

	^ 8! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
scrollBarsNarrow
	^ self
		valueOfFlag: #scrollBarsNarrow
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
scrollBarsOnRight
	^ self
		valueOfFlag: #scrollBarsOnRight
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
scrollBarsWithoutMenuButton
	^ self
		valueOfFlag: #scrollBarsWithoutMenuButton
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
securityChecksEnabled
	^ self
		valueOfFlag: #securityChecksEnabled
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
selectiveHalos
	^ self
		valueOfFlag: #selectiveHalos
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
serverMode
	^ self
		valueOfFlag: #serverMode
		ifAbsent: [true]! !

!Preferences class methodsFor: 'misc' stamp: 'sw 3/2/2004 22:11'!
setArrowheads
 	"Let the user edit the size of arrowheads"
 
 	| aParameter result  |
 	aParameter := self parameterAt: #arrowSpec ifAbsent: [5 @ 4].
 	result := Morph obtainArrowheadFor: 'Default size of arrowheads on pen trails ' translated defaultValue: aParameter asString.
 	result ifNotNil:
 			[self setParameter: #arrowSpec to: result]
 		ifNil:
 			[Beeper beep]! !

!Preferences class methodsFor: 'fonts' stamp: 'bp 6/13/2004 17:46'!
setBalloonHelpFontTo: aFont

	Smalltalk at: #BalloonMorph ifPresent:
		[:thatClass | thatClass setBalloonFontTo: aFont]! !

!Preferences class methodsFor: 'fonts' stamp: 'sw 12/8/1999 22:06'!
setButtonFontTo: aFont
	Parameters at: #standardButtonFont put: aFont! !

!Preferences class methodsFor: 'fonts' stamp: 'sw 7/25/2004 17:26'!
setCodeFontTo: aFont
	"Establish the code font."

	Parameters at: #standardCodeFont put: aFont! !

!Preferences class methodsFor: 'parameters' stamp: 'sw 2/17/1999 00:41'!
setDefaultAnnotationInfo
	"Preferences setDefaultAnnotationInfo"
	^ Parameters at: #MethodAnnotations put: #(timeStamp messageCategory implementorsCount allChangeSets)! !

!Preferences class methodsFor: 'fonts' stamp: 'M 8/30/2009 13:21'!
setDefaultFonts: defaultFontsSpec
	"Since this is called from menus, we can take the opportunity to prompt for missing font styles."

	| fontNames map emphases |
	fontNames := defaultFontsSpec collect: [:array | array second].
	map := IdentityDictionary new.
	emphases := IdentityDictionary new.
	fontNames do: [:originalName | | decoded style response |
		decoded := TextStyle decodeStyleName: originalName.
		style := map at: originalName put: (TextStyle named: decoded second).
		emphases at: originalName put: decoded first.
		style ifNil: [

			map at: originalName put: (TextStyle default) ]].

	defaultFontsSpec do: [:triplet | self
		perform: triplet first
		with: (((map at: triplet second) fontOfPointSize: triplet third) emphasized: (emphases at: triplet second))]! !

!Preferences class methodsFor: 'fonts' stamp: 'damiencassou 8/8/2009 20:25'!
setDemoFonts
	"Preferences setDemoFonts"
	|size font codeFont titleFont|
	size := UIManager default request: 'Base font size?' initialAnswer: '14'.
	size isEmptyOrNil ifTrue: [^ self].
	size := size asInteger.
	(size isNil or: [size <= 0]) ifTrue: [^ self].
	font := LogicalFont familyName: 'DejaVu Sans' pointSize: size.
	codeFont := LogicalFont familyName: 'DejaVu Sans Mono' pointSize: size.
	titleFont := LogicalFont familyName: 'DejaVu Serif' pointSize: size.

Preferences
	setListFontTo: font;
	setMenuFontTo: font;
	setCodeFontTo: codeFont;
	setButtonFontTo: font;
	setSystemFontTo: font;
	setWindowTitleFontTo: titleFont.! !

!Preferences class methodsFor: 'misc' stamp: 'sw 1/4/2001 06:56'!
setFlag: prefSymbol toValue: aBoolean during: aBlock
	"Set the flag to the given value for the duration of aBlock"

	| existing |
	existing := self valueOfFlag: prefSymbol.
	existing == aBoolean ifFalse: [self setPreference: prefSymbol toValue: aBoolean].
	aBlock value.
	existing == aBoolean ifFalse: [self setPreference: prefSymbol toValue: existing]! !

!Preferences class methodsFor: 'fonts' stamp: 'mir 8/24/2004 12:34'!
setHaloLabelFontTo: aFont 
	"change the font used in eToys environment"
	Parameters at: #haloLabelFont put: aFont! !

!Preferences class methodsFor: 'fonts' stamp: 'AlainPlantec 10/17/2009 16:57'!
setListFontTo: aFont
	"Set the list font as indicated"

	Parameters at: #standardListFont put: aFont.
	ListParagraph initialize.
! !

!Preferences class methodsFor: 'fonts' stamp: 'dgd 4/3/2006 14:21'!
setMenuFontTo: aFont
	"rbb 2/18/2005 12:54 - How should this be changed to work
	with the UIManager, if at all?"
	Parameters at: #standardMenuFont put: aFont.
	PopUpMenu setMenuFontTo: aFont.
	TheWorldMainDockingBar updateInstances.! !

!Preferences class methodsFor: 'reacting to change' stamp: 'AlainPlantec 10/17/2009 17:06'!
setNotificationParametersForStandardPreferences
	"Set up the notification parameters for the standard preferences that require need them.  When adding new Preferences that require use of the notification mechanism, users declare the notifcation info as part of the call that adds the preference, or afterwards -- the two relevant methods for doing that are:
 	Preferences.addPreference:categories:default:balloonHelp:projectLocal:changeInformee:changeSelector:   and
	Preference changeInformee:changeSelector:"

		"Preferences setNotificationParametersForStandardPreferences"

	| aPreference |
	#(	
		(annotationPanes		annotationPanesChanged)
		(infiniteUndo			infiniteUndoChanged)
		(optionalButtons			optionalButtonsChanged)
		(roundedWindowCorners	roundedWindowCornersChanged)
		(smartUpdating			smartUpdatingChanged)
	)  do:

			[:pair |
				aPreference := self preferenceAt: pair first.
				aPreference changeInformee: self changeSelector: pair second]! !

!Preferences class methodsFor: 'fonts' stamp: 'yo 1/12/2005 22:43'!
setPaintBoxButtonFontTo: aFont 
	"change the font used in the buttons in PaintBox."
	Parameters at: #paintBoxButtonFont put: aFont! !

!Preferences class methodsFor: 'parameters' stamp: 'stp 01/13/2000 13:29'!
setParameter: paramName to: paramValue
	"Set the given field in the parameters dictionary."

	Parameters at: paramName put: paramValue! !

!Preferences class methodsFor: 'get/set' stamp: 'sw 4/12/2001 23:29'!
setPreference: prefSymbol toValue: aBoolean
	"Set the given preference to the given value, and answer that value"

	^ (self preferenceAt: prefSymbol ifAbsent: [^ aBoolean]) preferenceValue: aBoolean! !

!Preferences class methodsFor: 'initialization' stamp: 'sw 4/21/2002 05:13'!
setPreferencesFrom: listOfPairs
	"Given a list of <preferenceName, value> pairs, set preference values.  This method is tolerent of the value being supplied either a Boolean or else one of the symbols #true and #false.  Also, a new-value of #noOpinion will result in that 'preference's value not being changed."

	listOfPairs do:
		[:aPair |
			(aPair second == #noOpinion) ifFalse:
				[Preferences setPreference: aPair first toValue: ((aPair second == #true) or: [aPair second == true])]]

"
Preferences setPreferencesFrom: #(( mouseOverForKeyboardFocus false))
Preferences setPreferencesFrom: {{  #mouseOverForKeyboardFocus. true}}
"! !

!Preferences class methodsFor: 'fonts' stamp: 'AlainPlantec 10/20/2009 10:11'!
setSystemFontTo: aFont
	"Establish the default text font and style"

	| aStyle newDefaultStyle |
	aFont ifNil: [^ self].
	aStyle := aFont textStyle ifNil: [^ self].
	newDefaultStyle := aStyle copy.
	newDefaultStyle defaultFontIndex: (aStyle fontIndexOf: aFont).
	TextConstants at: #DefaultTextStyle put: newDefaultStyle.
! !

!Preferences class methodsFor: 'window colors' stamp: 'sd 9/14/2006 20:12'!
setWindowColorFor: modelSymbol to: incomingColor
	| aColor aPrefSymbol aColorSpec |
	aColorSpec := WindowColorRegistry registeredWindowColorSpecFor: modelSymbol.
	aColorSpec ifNil: [^self].
	aColor := incomingColor asNontranslucentColor.
	(aColor = ColorPickerMorph perniciousBorderColor or: [aColor = Color black]) 
		ifTrue: [^ self].	
	aPrefSymbol :=  self windowColorPreferenceForClassNamed: aColorSpec classSymbol.
	self 
		addPreference: aPrefSymbol  
		categories:  { #'window colors' }
		default:  aColor 
		balloonHelp: aColorSpec helpMessage translated
		projectLocal: false
		changeInformee: nil
		changeSelector: nil
		viewRegistry: (PreferenceViewRegistry registryOf: #windowColorPreferences)! !

!Preferences class methodsFor: 'fonts' stamp: 'AlainPlantec 10/17/2009 16:58'!
setWindowTitleFontTo: aFont
	"Set the window-title font to be as indicated"

	Parameters at: #windowTitleFont put: aFont.
! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
showBoundsInHalo
	^ self
		valueOfFlag: #showBoundsInHalo
		ifAbsent: [false
			"keep scanning"]! !

!Preferences class methodsFor: 'halos' stamp: 'sw 11/6/2000 10:02'!
showChooseGraphicHaloHandle
	"Hard-coded; reimplement to change behavior.  If this preference is set to true, then a choose-graphic halo handle may appear on the halo of SketchMorphs"

	^ false! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
showDeprecationWarnings
	^ self
		valueOfFlag: #showDeprecationWarnings
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
showDirectionForSketches
	^ self
		valueOfFlag: #showDirectionForSketches
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
showDirectionHandles
	^ self
		valueOfFlag: #showDirectionHandles
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
showSecurityStatus
	^ self
		valueOfFlag: #showSecurityStatus
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
showSplitterHandles
	^ self
		valueOfFlag: #showSplitterHandles
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
showTextEditingState
	^ self
		valueOfFlag: #showTextEditingState
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
showWorldMainDockingBar
	^ self
		valueOfFlag: #showWorldMainDockingBar
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
showWorldTaskbar
	^ self
		valueOfFlag: #showWorldTaskbar
		ifAbsent: [true]! !

!Preferences class methodsFor: 'halos' stamp: 'AlainPlantec 10/19/2009 18:32'!
simpleFullHaloSpecifications
	"This method gives the specs for the 'full' handles variant when simple halos are in effect"

	"Preferences resetHaloSpecifications"

	^ #(
	"  	selector				horiz		vert			color info						icon key
		---------				------		-----------		-------------------------------		---------------"
	(addDebugHandle:		right		topCenter		(blue	veryMuchLighter)		#haloDebugIcon)
	(addDismissHandle:		left			top				(red		muchLighter)			#haloDismissIcon)
	(addRotateHandle:		left			bottom			(blue)							#haloRotIcon)
	(addMenuHandle:		leftCenter	top				(red)							#haloMenuIcon)
	(addGrabHandle:			center		top				(black)							#halograbIcon)
	(addDragHandle:			rightCenter	top				(brown)							#haloDragIcon)
	(addDupHandle:			right		top				(green)							#haloDupIcon)	
	(addHelpHandle:			center		bottom			(lightBlue)						#haloHelpIcon')
	(addGrowHandle:		right		bottom			(yellow)						#haloScaleIcon)
	(addScaleHandle:		right		bottom			(lightOrange)					#haloScaleIcon)
	(addFewerHandlesHandle:	left		topCenter		(paleBuff)						#haloFewerHandlesIcon')
	(addRepaintHandle:		right		center			(lightGray)						#haloPaintIcon)
	(addFontSizeHandle:		leftCenter	bottom			(lightGreen)						#haloFontSizeIcon)
	(addFontStyleHandle:		center		bottom			(lightRed)						#haloFontStyleIcon)
	(addFontEmphHandle:	rightCenter	bottom			(lightBrown darker)		  #haloFontEmphIcon)
	(addRecolorHandle:		right		bottomCenter	(magenta darker)				#haloRecolorIcon)

		) ! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
slideDismissalsToTrash
	^ self
		valueOfFlag: #slideDismissalsToTrash
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
smartUpdating
	^ self
		valueOfFlag: #smartUpdating
		ifAbsent: [true]! !

!Preferences class methodsFor: 'reacting to change' stamp: 'sw 4/12/2001 01:30'!
smartUpdatingChanged
	"The smartUpdating preference changed. React"

	SystemWindow allSubInstancesDo:
		[:aWindow | aWindow amendSteppingStatus]

	"NOTE: This makes this preference always behave like a global preference, which is problematical"! !

!Preferences class methodsFor: 'misc' stamp: 'marcus.denker 9/19/2008 17:01'!
soundEnablingString
	^ self soundsEnabled
		ifFalse:
			['Turn sound on' translated]
		ifTrue:
			['Turn sound off' translated]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
soundQuickStart
	^ self
		valueOfFlag: #soundQuickStart
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
soundStopWhenDone
	^ self
		valueOfFlag: #soundStopWhenDone
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
soundTheme
	^ self
		valueOfFlag: #soundTheme
		ifAbsent: [SoundTheme]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
soundsEnabled
	^ self
		valueOfFlag: #soundsEnabled
		ifAbsent: [true]! !

!Preferences class methodsFor: 'misc' stamp: 'dgd 9/21/2003 13:46'!
staggerPolicyString
	"Answer the string to be shown in a menu to represent the 
	stagger-policy status"
	^ ((self valueOfFlag: #reverseWindowStagger)
		ifTrue: ['<yes>']
		ifFalse: ['<no>']), 'stagger windows' translated! !

!Preferences class methodsFor: 'standard queries' stamp: 'mir 11/10/2003 14:28'!
standaloneSecurityChecksEnabled
 	^ self
 		valueOfFlag: #standaloneSecurityChecksEnabled
 		ifAbsent: [false]! !

!Preferences class methodsFor: 'fonts' stamp: 'bp 6/13/2004 17:19'!
standardBalloonHelpFont
	^BalloonMorph balloonFont! !

!Preferences class methodsFor: 'fonts' stamp: 'StephaneDucasse 11/2/2009 09:48'!
standardButtonFont
	"Answer an attractive font to use for buttons"
	^ Parameters at: #standardButtonFont ifAbsentPut:
		[(StrikeFont familyName: #ComicBold size: 16)]! !

!Preferences class methodsFor: 'fonts' stamp: 'Lukas Renggli 11/2/2009 00:29'!
standardCodeFont
	"Answer the font to be used in code"

	^Parameters at: #standardCodeFont ifAbsentPut: [TextStyle defaultFont]! !

!Preferences class methodsFor: 'fonts' stamp: 'bp 6/13/2004 17:24'!
standardDefaultTextFont
	^TextStyle defaultFont! !

!Preferences class methodsFor: 'fonts' stamp: 'Lukas Renggli 11/2/2009 00:29'!
standardHaloLabelFont
	"Answer the font to be used in the eToys environment"

	^Parameters at: #haloLabelFont ifAbsentPut: [TextStyle defaultFont]! !

!Preferences class methodsFor: 'fonts' stamp: 'Lukas Renggli 11/2/2009 00:29'!
standardListFont
	"Answer the font to be used in lists"

	^Parameters at: #standardListFont ifAbsentPut: [TextStyle defaultFont]! !

!Preferences class methodsFor: 'fonts' stamp: 'Lukas Renggli 11/2/2009 00:29'!
standardMenuFont
	"Answer the font to be used in menus"

	^Parameters at: #standardMenuFont ifAbsentPut: [TextStyle defaultFont]! !

!Preferences class methodsFor: 'fonts' stamp: 'Lukas Renggli 11/2/2009 00:29'!
standardPaintBoxButtonFont
	"Answer the font to be used in the eToys environment"

	^Parameters at: #paintBoxButtonFont ifAbsentPut: [self standardButtonFont]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
startInUntrustedDirectory
	^ self
		valueOfFlag: #startInUntrustedDirectory
		ifAbsent: [false]! !

!Preferences class methodsFor: 'personalization' stamp: 'alain.plantec 5/30/2008 14:25'!
storePreferencesIn: aFileName 
	| stream |
	#(#Prevailing #PersonalPreferences )  do:[:ea | Parameters  removeKey:ea  ifAbsent:[]].
	stream := ReferenceStream  fileNamed:aFileName.
	stream  nextPut:Parameters.
	stream  nextPut:self dictionaryOfPreferences.
	stream  nextPut:World fillStyle.
	stream close! !

!Preferences class methodsFor: 'personalization' stamp: 'nk 11/17/2002 12:08'!
storePreferencesToDisk
	Cursor wait showWhile: [
		[ self storePreferencesIn: 'my.prefs' ] on: Error do: [ :ex | self inform: 'there was an error storing your preferences to disk' ]]! !

!Preferences class methodsFor: 'fonts' stamp: 'jmv 8/4/2009 15:09'!
subPixelRenderColorFonts
	^ self
		valueOfFlag: #subPixelRenderColorFonts
		ifAbsent: [true]! !

!Preferences class methodsFor: 'fonts' stamp: 'jmv 8/4/2009 15:09'!
subPixelRenderFonts
	^ self
		valueOfFlag: #subPixelRenderFonts
		ifAbsent: [true]! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 11/15/2001 08:37'!
suppressWindowTitlesInInstanceBrowsers
	"Hard-coded for the moment: answer whether instance browsers should suppresss their window titles"

	^ false! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
swapControlAndAltKeys
	^ self
		valueOfFlag: #swapControlAndAltKeys
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: 'adrian-lienhard 5/22/2009 10:41'!
swapMouseButtons
	^ self
		valueOfFlag: #swapMouseButtons
		ifAbsent: [
			OSPlatform current platformFamily ~= #Windows ]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
syntaxHighlightingAsYouType
	^ self
		valueOfFlag: #syntaxHighlightingAsYouType
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
syntaxHighlightingAsYouTypeAnsiAssignment
	^ self
		valueOfFlag: #syntaxHighlightingAsYouTypeAnsiAssignment
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
syntaxHighlightingAsYouTypeLeftArrowAssignment
	^ self
		valueOfFlag: #syntaxHighlightingAsYouTypeLeftArrowAssignment
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
systemWindowEmbedOK
	^ self
		valueOfFlag: #systemWindowEmbedOK
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
tabAmongFields
	^ self
		valueOfFlag: #tabAmongFields
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
testRunnerWindowColor
	^ self
		valueOfFlag: #testRunnerWindowColor
		ifAbsent: [Color
				r: 0.65
				g: 0.753
				b: 0.976]! !

!Preferences class methodsFor: 'text highlighting' stamp: 'sw 9/7/1999 12:53'!
textHighlightColor
	^ Parameters at: #textHighlightColor! !

!Preferences class methodsFor: 'text highlighting' stamp: 'sw 9/7/1999 12:54'!
textHighlightColor: aColor
	Parameters at: #textHighlightColor put: aColor! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
tinyDisplay
	^ self
		valueOfFlag: #tinyDisplay
		ifAbsent: [false]! !

!Preferences class methodsFor: 'misc' stamp: 'sw 7/13/1999 16:51'!
toggleMenuColorPolicy
	self togglePreference: #menuColorFromWorld! !

!Preferences class methodsFor: 'get/set' stamp: 'hpt 9/26/2004 16:50'!
togglePreference: prefSymbol
	"Toggle the given preference. prefSymbol must be of a boolean preference"
	(self preferenceAt: prefSymbol ifAbsent: [self error: 'unknown preference: ', prefSymbol]) togglePreferenceValue! !

!Preferences class methodsFor: 'misc' stamp: 'sw 7/13/1999 16:52'!
toggleRoundedCorners
	self togglePreference: #roundedWindowCorners! !

!Preferences class methodsFor: 'misc' stamp: 'ssa 2/9/2000 11:01'!
toggleSoundEnabling
     self togglePreference: #soundsEnabled! !

!Preferences class methodsFor: 'misc' stamp: 'sw 6/11/1999 20:49'!
toggleWindowPolicy
	self togglePreference: #reverseWindowStagger! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
transcriptWindowColor
	^ self
		valueOfFlag: #transcriptWindowColor
		ifAbsent: [Color
				r: 1.0
				g: 0.8
				b: 0.4]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
turnOffPowerManager
	^ self
		valueOfFlag: #turnOffPowerManager
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
uiTheme
	^ self
		valueOfFlag: #uiTheme
		ifAbsent: [UIThemeStandardSqueak]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
unlimitedPaintArea
	^ self
		valueOfFlag: #unlimitedPaintArea
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
updateFromServerAtStartup
	^ self
		valueOfFlag: #updateFromServerAtStartup
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
updateSavesFile
	^ self
		valueOfFlag: #updateSavesFile
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
upgradeIsMerge
	^ self
		valueOfFlag: #upgradeIsMerge
		ifAbsent: [false]! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 2/16/1999 11:24'!
useCategoryListsInViewers
	"Temporarily hard-coded pending viewer work underway"
	^ false! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
useFormsInPaintBox
	^ self
		valueOfFlag: #useFormsInPaintBox
		ifAbsent: [false]! !

!Preferences class methodsFor: 'paintbox' stamp: 'yo 1/13/2005 11:05'!
useFormsInPaintBox: aBoolean

	self setPreference: #useFormsInPaintBox toValue: aBoolean
! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
useLocale
	^ self
		valueOfFlag: #useLocale
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
useNewDiffToolsForMC
	^ self
		valueOfFlag: #useNewDiffToolsForMC
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
useOnlyServicesInMenu
	^ self
		valueOfFlag: #useOnlyServicesInMenu
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
useRBASTForPrettyPrint
	^ self
		valueOfFlag: #useRBASTForPrettyPrint
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
useServicesInBrowserButtonBar
	^ self
		valueOfFlag: #useServicesInBrowserButtonBar
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
useSmartLabels
	^ self
		valueOfFlag: #useSmartLabels
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
useThemeSounds
	^ self
		valueOfFlag: #useThemeSounds
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
useUndo
	^ self
		valueOfFlag: #useUndo
		ifAbsent: [true]! !

!Preferences class methodsFor: 'get/set' stamp: 'hpt 9/26/2004 16:49'!
valueOfFlag: aFlagName
	"Utility method for all the preferences that are boolean, and for backward compatibility"
	^self valueOfPreference: aFlagName ifAbsent: [false].! !

!Preferences class methodsFor: 'get/set' stamp: 'hpt 9/26/2004 16:48'!
valueOfFlag: aFlagName ifAbsent: booleanValuedBlock
	"the same as in #valueOfFlag:"
	^self valueOfPreference: aFlagName ifAbsent: booleanValuedBlock.! !

!Preferences class methodsFor: 'get/set' stamp: 'hpt 9/26/2004 16:49'!
valueOfPreference: aPreferenceSymbol
	"Answer the value of the given preference"
	^self valueOfPreference: aPreferenceSymbol ifAbsent: []! !

!Preferences class methodsFor: 'get/set' stamp: 'hpt 9/26/2004 16:49'!
valueOfPreference: aPreferenceSymbol ifAbsent: booleanValuedBlock
	"Answer the value of the given preference"
	^ (self preferenceAt: aPreferenceSymbol ifAbsent: [^ booleanValuedBlock value]) preferenceValue! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
versionsBrowserWindowColor
	^ self
		valueOfFlag: #versionsBrowserWindowColor
		ifAbsent: [Color
				r: 0.869
				g: 0.753
				b: 1.0]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
visualExplorer
	^ self
		valueOfFlag: #visualExplorer
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
visualizations
	^ self
		valueOfFlag: #visualizations
		ifAbsent: [true]! !

!Preferences class methodsFor: 'misc' stamp: 'MiguelCoba 7/25/2009 02:19'!
wantsChangeSetLogging
	"Answer whether method changes in the receiver should be logged to current change set.  This circumlocution avoids such logging for programmatically-compiled methods in Preferences, removing an annoyance"

	^ Author fullNamePerSe  ~= 'programmatic'! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
warnAboutInsecureContent
	^ self
		valueOfFlag: #warnAboutInsecureContent
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
warnIfNoChangesFile
	^ self
		valueOfFlag: #warnIfNoChangesFile
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
warnIfNoSourcesFile
	^ self
		valueOfFlag: #warnIfNoSourcesFile
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
windowAnimation
	^ self
		valueOfFlag: #windowAnimation
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
windowAnimationDelay
	^ self
		valueOfFlag: #windowAnimationDelay
		ifAbsent: [10]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
windowAnimationSteps
	^ self
		valueOfFlag: #windowAnimationSteps
		ifAbsent: [15]! !

!Preferences class methodsFor: 'window colors' stamp: 'sd 9/14/2006 20:13'!
windowColorFor: aModelClassName
	| classToCheck prefSymbol |
	self checkForWindowColors.
	classToCheck := Smalltalk at: aModelClassName.
	prefSymbol := self windowColorPreferenceForClassNamed: classToCheck name.
	[(classToCheck ~~ Object) and: [(self preferenceAt: prefSymbol) isNil]]
		whileTrue: 
				[classToCheck := classToCheck superclass.
				prefSymbol := self windowColorPreferenceForClassNamed: classToCheck name].
	^self valueOfPreference: prefSymbol ifAbsent: [Color white].! !

!Preferences class methodsFor: 'window colors' stamp: 'sw 4/21/2002 02:55'!
windowColorHelp
	"Provide help for the window-color panel"

	| helpString |
	helpString := 
'The "Window Colors" panel lets you select colors for many kinds of standard Squeak windows.

You can change your color preference for any particular tool by clicking on the color swatch and then selecting the desired color from the resulting color-picker.

The three buttons entitled "Bright", "Pastel", and "White" let you revert to any of three different standard color schemes.  

The choices you make in the Window Colors panel only affect the colors of new windows that you open.

You can make other tools have their colors governed by this panel by simply implementing #windowColorSpecification on the class side of the model -- consult implementors of that method to see examples of how to do this.'.

	 (StringHolder new contents: helpString)
		openLabel: 'About Window Colors'

	"Preferences windowColorHelp"! !

!Preferences class methodsFor: 'window colors' stamp: 'hpt 10/9/2005 23:28'!
windowColorPreferenceForClassNamed: aClassName
	| aColorSpec wording |
	aColorSpec := WindowColorRegistry registeredWindowColorSpecFor: aClassName.
	wording := aColorSpec ifNil: [aClassName] ifNotNil: [aColorSpec wording].
	^(wording, 'WindowColor') asLegalSelector asSymbol.! !

!Preferences class methodsFor: 'window colors' stamp: 'hpt 10/9/2005 23:17'!
windowColorTable
	"Answer a list of WindowColorSpec objects, one for each tool to be represented in the window-color panel"
	^ (WindowColorRegistry registeredWindowColorSpecs
		asSortedCollection: 
			[:specOne :specTwo | specOne wording < specTwo wording]) asArray.

"Preferences windowColorTable"! !

!Preferences class methodsFor: 'fonts' stamp: 'Lukas Renggli 11/2/2009 00:29'!
windowTitleFont
	"Answer the standard font to use for window titles"

	^Parameters at: #windowTitleFont
		ifAbsentPut: [StrikeFont familyName: #NewYork size: 15]! !

!Preferences class methodsFor: 'fonts' stamp: 'sw 12/8/1999 22:18'!
windowTitleStyle
	"Answer the standard style to use for window titles"
	^  self windowTitleFont textStyle! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
windowsActiveOnFirstClick
	^ self
		valueOfFlag: #windowsActiveOnFirstClick
		ifAbsent: [false]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
wordStyleCursorMovement
	^ self
		valueOfFlag: #wordStyleCursorMovement
		ifAbsent: [true]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
workspaceWindowColor
	^ self
		valueOfFlag: #workspaceWindowColor
		ifAbsent: [Color
				r: 1.0
				g: 1.0
				b: 0.8]! !

!Preferences class methodsFor: 'standard queries' stamp: ''!
worldTaskbarWindowPreview
	^ self
		valueOfFlag: #worldTaskbarWindowPreview
		ifAbsent: [true]! !

!Preferences methodsFor: 'look in class' stamp: 'di 12/4/1999 15:11'!
seeClassSide
	"All the code for Preferences is on the class side"! !

Object subclass: #ProjectLoading
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!ProjectLoading classSide methodsFor: 'utilities' stamp: 'md 5/11/2008 11:43'!
bestAccessToFileName: aFileName andDirectory: aDirectoryOrUrlString
	"Answer an array with a stream and a directory. The directory can be nil."
	
	| dir url |
	dir := Project squeakletDirectory.
	(dir fileExists: aFileName) ifTrue: [
		^{dir readOnlyFileNamed: aFileName. dir}].

	aDirectoryOrUrlString isString ifFalse: [
		^{aDirectoryOrUrlString readOnlyFileNamed: aFileName. aDirectoryOrUrlString}].

	url := Url absoluteFromFileNameOrUrlString: aDirectoryOrUrlString.

	(url scheme = 'file') ifTrue: [
		dir := FileDirectory on: url pathForDirectory.
		^{dir readOnlyFileNamed: aFileName. dir}].

	^{ServerFile new fullPath: aDirectoryOrUrlString. nil}! !

!ProjectLoading classSide methodsFor: 'loading' stamp: 'RAA 2/19/2001 08:22'!
openFromFile: preStream fromDirectory: aDirectoryOrNil withProjectView: existingView


	self error: 'use #openFromFile:fromDirectory:withProjectView:'
! !

!ProjectLoading classSide methodsFor: 'utilities' stamp: 'ar 9/27/2005 20:10'!
useTempChangeSet
	"While reading the project in, use the temporary change set zzTemp"

	| zz |
	zz := ChangeSet named: 'zzTemp'.
	zz ifNil: [zz := ChangeSet basicNewChangeSet: 'zzTemp'].
	ChangeSet  newChanges: zz.! !

Object subclass: #RealEstateAgent
	instanceVariableNames: ''
	classVariableNames: 'ReverseStaggerOffset StaggerOffset StaggerOrigin StandardSize StandardWindowOrigins'
	poolDictionaries: ''
	category: 'System-Support'!

!RealEstateAgent commentStamp: '<historical>' prior: 0!
Responsible for real-estate management on the screen, which is to say, controlling where new windows appear, with what sizes, etc.  5/20/96 sw!

!RealEstateAgent class methodsFor: 'utilities' stamp: 'AlainPlantec 10/17/2009 17:40'!
assignCollapseFrameFor: aSSView 
	"Offer up a location along the left edge of the screen for a collapsed
	SSView. Make sure it doesn't overlap any other collapsed frames."
	| grid otherFrames topLeft viewBox collapsedFrame extent newFrame verticalBorderDistance top |
	grid := 8.
	verticalBorderDistance := 8.
	otherFrames := ( aSSView world windowsSatisfying: [:w | w ~= aSSView])
				collect: [:w | w collapsedFrame]
				thenSelect: [:rect | rect notNil].
	viewBox := aSSView world viewBox.
	collapsedFrame := aSSView collapsedFrame.
	extent := collapsedFrame notNil
				ifTrue: [collapsedFrame extent]
				ifFalse: [aSSView getRawLabel width + aSSView labelWidgetAllowance @ (aSSView labelHeight + 2)].
	collapsedFrame notNil
		ifTrue: [(otherFrames
					anySatisfy: [:f | collapsedFrame intersects: f])
				ifFalse: ["non overlapping"
					^ collapsedFrame]].
	top := viewBox top + verticalBorderDistance.
	[topLeft := viewBox left @ top.
	newFrame := topLeft extent: extent.
	newFrame bottom <= (viewBox height - verticalBorderDistance)]
		whileTrue: [(otherFrames
					anySatisfy: [:w | newFrame intersects: w])
				ifFalse: ["no overlap"
					^ newFrame].
			top := top + grid].
	"If all else fails... (really to many wins here)"
	^ 0 @ 0 extent: extent! !

!RealEstateAgent class methodsFor: 'utilities' stamp: 'AlainPlantec 10/17/2009 17:40'!
assignCollapsePointFor: aSSView 
	"Offer up a location along the left edge of the screen for a collapsed
	SSView. Make sure it doesn't overlap any other collapsed frames."
	| grid otherFrames y free topLeft viewBox |
	grid := 24.
	"should be mult of 8, since manual move is gridded by 8"
	otherFrames := ( aSSView world   windowsSatisfying: [:w | true])
				collect: [:w | w collapsedFrame]
				thenSelect: [:rect | rect notNil].
	viewBox := aSSView world viewBox.
	y := viewBox top.
	[(y := y + grid) <= (viewBox height - grid)]
		whileTrue: [topLeft := viewBox left @ y.
			free := true.
			otherFrames
				do: [:w | free := free & (topLeft ~= w topLeft)].
			free
				ifTrue: [^ topLeft]].
	"If all else fails..."
	^ 0 @ 0! !

!RealEstateAgent class methodsFor: 'framing' stamp: 'RAA 5/25/2000 08:17'!
initialFrameFor: aView
	"Find a plausible initial screen area for the supplied view.  See called method."

	self error: 'please use #initialFrameFor:world:'! !

!RealEstateAgent class methodsFor: 'framing' stamp: 'RAA 5/25/2000 08:18'!
initialFrameFor: aView initialExtent: initialExtent

	self error: 'please use #initialFrameFor:initialExtent:world:'! !

!RealEstateAgent class methodsFor: 'framing' stamp: 'stephane.ducasse 9/25/2008 13:36'!
initialFrameFor: aView initialExtent: initialExtent world: aWorld
	"Find a plausible initial screen area for the supplied view, which should be a StandardSystemView, taking into account the 'reverseWindowStagger' Preference, the size needed, and other windows currently on the screen."

	| allOrigins screenRight screenBottom putativeOrigin putativeFrame allowedArea staggerOrigin otherFrames |

	Preferences reverseWindowStagger ifTrue:
		[^ self strictlyStaggeredInitialFrameFor: aView initialExtent: initialExtent world: aWorld].

	allowedArea := self maximumUsableAreaInWorld: aWorld.
	screenRight := allowedArea right.
	screenBottom := allowedArea bottom.

	otherFrames := (aWorld windowsSatisfying: [:w | w isCollapsed not])
					collect: [:w | w bounds].

	allOrigins := otherFrames collect: [:f | f origin].
	(self standardPositionsInWorld: aWorld) do:  "First see if one of the standard positions is free"
		[:aPosition | (allOrigins includes: aPosition)
			ifFalse:
				[^ (aPosition extent: initialExtent) translatedAndSquishedToBeWithin: allowedArea]].

	staggerOrigin := (self standardPositionsInWorld: aWorld) first.  "Fallback: try offsetting from top left"
	putativeOrigin := staggerOrigin.

	[putativeOrigin := putativeOrigin + StaggerOffset.
	putativeFrame := putativeOrigin extent: initialExtent.
	(putativeFrame bottom < screenBottom) and:
					[putativeFrame right < screenRight]]
				whileTrue:
					[(allOrigins includes: putativeOrigin)
						ifFalse:
							[^ (putativeOrigin extent: initialExtent) translatedAndSquishedToBeWithin: allowedArea]].
	^ (self scrollBarSetback @ self screenTopSetback extent: initialExtent) translatedAndSquishedToBeWithin: allowedArea! !

!RealEstateAgent class methodsFor: 'framing' stamp: 'RAA 5/25/2000 08:13'!
initialFrameFor: aView world: aWorld
	"Find a plausible initial screen area for the supplied view.  See called method."

	^ self initialFrameFor: aView initialExtent: aView initialExtent world: aWorld! !

!RealEstateAgent class methodsFor: 'class initialization' stamp: 'al 9/21/2008 19:59'!
initialize
	"RealEstateAgent initialize"

	StaggerOffset := 6 @ 20.
	ReverseStaggerOffset := -6 @ 20.
	StaggerOrigin := 200 @ 30.
	StandardSize := 750@500.! !

!RealEstateAgent class methodsFor: 'accessing' stamp: 'alain.plantec 6/1/2008 20:02'!
maximumUsableArea

	| allowedArea |
	allowedArea := Display usableArea.
	allowedArea := allowedArea intersect: ActiveWorld visibleClearArea.
	^allowedArea! !

!RealEstateAgent class methodsFor: 'accessing' stamp: 'dgd 4/4/2006 16:42'!
maximumUsableAreaInWorld: aWorldOrNil

	| allowedArea |
	allowedArea := Display usableArea.
	aWorldOrNil ifNotNil: [
		allowedArea := allowedArea intersect: aWorldOrNil visibleClearArea.
	].
	^allowedArea! !

!RealEstateAgent class methodsFor: 'settings' stamp: 'alain.plantec 6/1/2008 20:02'!
screenTopSetback
	^ 0! !

!RealEstateAgent class methodsFor: 'settings' stamp: 'alain.plantec 6/1/2008 20:06'!
scrollBarSetback
	^ 16-3 "width = 16; inset from border by 3"
! !

!RealEstateAgent class methodsFor: 'accessing' stamp: 'RAA 5/25/2000 08:45'!
standardPositions
	
	self error: 'please use #standardPositionsInWorld:'! !

!RealEstateAgent class methodsFor: 'accessing' stamp: 'RAA 5/25/2000 08:43'!
standardPositionsInWorld: aWorldOrNil
	"Return a list of standard window positions -- this may have one, two, or four of them, depending on the size and shape of the display screen.  "

	| anArea aList  midX midY |

	anArea := self maximumUsableAreaInWorld: aWorldOrNil.

	midX := self scrollBarSetback +   ((anArea width - self scrollBarSetback)  // 2).
	midY := self screenTopSetback + ((anArea height - self screenTopSetback) // 2).
	aList := OrderedCollection with: (self scrollBarSetback @ self screenTopSetback).
	self windowColumnsDesired > 1
		ifTrue:
			[aList add: (midX @ self screenTopSetback)].
	self windowRowsDesired > 1
		ifTrue:
			[aList add: (self scrollBarSetback @ (midY+self screenTopSetback)).
			self windowColumnsDesired > 1 ifTrue:
				[aList add: (midX @ (midY+self screenTopSetback))]].
	^ aList! !

!RealEstateAgent class methodsFor: 'settings' stamp: 'RAA 11/21/1999 22:55'!
standardWindowExtent
	"Answer the standard default extent for new windows.  "

	| effectiveExtent width strips height grid allowedArea maxLevel |
	effectiveExtent := self maximumUsableArea extent
					- (self scrollBarSetback @ self screenTopSetback).
	Preferences reverseWindowStagger ifTrue:
		["NOTE: following copied from strictlyStaggeredInitialFrameFor:"
		allowedArea := self maximumUsableArea insetBy: (
			self scrollBarSetback @ self screenTopSetback extent: 0@0
		).
		"Number to be staggered at each corner (less on small screens)"
		maxLevel := allowedArea area > 300000 ifTrue: [3] ifFalse: [2].
		"Amount by which to stagger (less on small screens)"
		grid := allowedArea area > 500000 ifTrue: [40] ifFalse: [20].
		^ (allowedArea extent - (grid*(maxLevel+1*2) + (grid//2))) min: StandardSize "600@400"].
	width := (strips := self windowColumnsDesired) > 1
		ifTrue:
			[effectiveExtent x // strips]
		ifFalse:
			[(3 * effectiveExtent x) // 4].
	height := (strips := self windowRowsDesired) > 1
		ifTrue:
			[effectiveExtent y // strips]
		ifFalse:
			[(3 * effectiveExtent y) //4].
	^ width @ height

"RealEstateAgent standardWindowExtent"! !

!RealEstateAgent class methodsFor: 'framing' stamp: 'RAA 5/25/2000 09:15'!
strictlyStaggeredInitialFrameFor: aStandardSystemView initialExtent: initialExtent

	self error: 'please use #strictlyStaggeredInitialFrameFor:initialExtent:world:'! !

!RealEstateAgent class methodsFor: 'framing' stamp: 'AlainPlantec 10/17/2009 17:41'!
strictlyStaggeredInitialFrameFor: aStandardSystemView initialExtent: initialExtent world: aWorld
	"This method implements a staggered window placement policy that I (di) like.
	Basically it provides for up to 4 windows, staggered from each of the 4 corners.
	The windows are staggered so that there will always be a corner visible."

	| allowedArea grid initialFrame otherFrames cornerSel corner delta putativeCorner free maxLevel |

	allowedArea :=(self maximumUsableAreaInWorld: aWorld)
		insetBy: (self scrollBarSetback @ self screenTopSetback extent: 0@0).
	"Number to be staggered at each corner (less on small screens)"
	maxLevel := allowedArea area > 300000 ifTrue: [3] ifFalse: [2].
	"Amount by which to stagger (less on small screens)"
	grid := allowedArea area > 500000 ifTrue: [40] ifFalse: [20].
	initialFrame := 0@0 extent: ((initialExtent
							"min: (allowedArea extent - (grid*(maxLevel+1*2) + (grid//2))))
							min: 600@400")).
	otherFrames := (aWorld windowsSatisfying: [:w | w isCollapsed not])
					collect: [:w | w bounds].
	0 to: maxLevel do:
		[:level | 
		1 to: 4 do:
			[:ci | cornerSel := #(topLeft topRight bottomRight bottomLeft) at: ci.
			corner := allowedArea perform: cornerSel.
			"The extra grid//2 in delta helps to keep title tabs distinct"
			delta := (maxLevel-level*grid+(grid//2)) @ (level*grid).
			1 to: ci-1 do: [:i | delta := delta rotateBy: #right centerAt: 0@0]. "slow way"
			putativeCorner := corner + delta.
			free := true.
			otherFrames do:
				[:w |
				free := free & ((w perform: cornerSel) ~= putativeCorner)].
			free ifTrue:
				[^ (initialFrame align: (initialFrame perform: cornerSel)
								with: putativeCorner)
						 translatedAndSquishedToBeWithin: allowedArea]]].
	"If all else fails..."
	^ (self scrollBarSetback @ self screenTopSetback extent: initialFrame extent)
		translatedAndSquishedToBeWithin: allowedArea! !

!RealEstateAgent class methodsFor: 'settings' stamp: 'RAA 11/21/1999 22:54'!
windowColumnsDesired
	"Answer how many separate vertical columns of windows are wanted.  5/22/96 sw"
	^ Preferences reverseWindowStagger
		ifTrue:
			[1]
		ifFalse:
			[(self maximumUsableArea width > 640)
				ifTrue:
					[2]
				ifFalse:
					[1]]! !

!RealEstateAgent class methodsFor: 'settings' stamp: 'RAA 11/21/1999 22:54'!
windowRowsDesired
	"Answer how many separate horizontal rows of windows are wanted.  5/22/96 sw"
	^ Preferences reverseWindowStagger
		ifTrue:
			[1]
		ifFalse:
			[(self maximumUsableArea height > 480)
				ifTrue:
					[2]
				ifFalse:
					[1]]! !

Object subclass: #ResourceCollector
	instanceVariableNames: 'stubMap originalMap locatorMap localDirectory baseUrl resourceDirectory internalStubs'
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'System-Support'!

!ResourceCollector commentStamp: '<historical>' prior: 0!
The ResourceCollector collects resources that are encountered during project loading or publishing. It merely decouples the places where resources are held from the core object enumeration so that resources can be stored independently from what is enumerated for publishing.!

!ResourceCollector classSide methodsFor: 'accessing' stamp: 'ar 2/24/2001 21:41'!
current
	^Current! !

!ResourceCollector classSide methodsFor: 'accessing' stamp: 'ar 2/24/2001 21:41'!
current: aResourceManager
	Current := aResourceManager! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/27/2001 20:32'!
baseUrl
	^baseUrl! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/27/2001 20:39'!
baseUrl: aString
	baseUrl := aString.
	baseUrl isEmpty ifFalse:[
		baseUrl last = $/ ifFalse:[baseUrl := baseUrl copyWith: $/].
	].! !

!ResourceCollector methodsFor: 'initialize' stamp: 'ar 2/27/2001 23:08'!
forgetObsolete
	"Forget obsolete locators, e.g., those that haven't been referenced and not been stored on a file."
	locatorMap keys "copy" do:[:k|
		(locatorMap at: k) localFileName ifNil:[locatorMap removeKey: k]].! !

!ResourceCollector methodsFor: 'initialize' stamp: 'AlainPlantec 10/19/2009 18:10'!
initialize
	| fd pvt |
	super initialize.
	originalMap := IdentityDictionary new.
	stubMap := IdentityDictionary new.
	locatorMap := IdentityDictionary new.
	internalStubs := IdentityDictionary new.
	! !

!ResourceCollector methodsFor: 'initialize' stamp: 'ar 2/27/2001 22:36'!
initializeFrom: aResourceManager
	"Initialize the receiver from aResourceManager."
	aResourceManager resourceMap keysAndValuesDo:[:loc :res|
		(res notNil)
			ifTrue:[locatorMap at: res put:  loc.
					loc localFileName: nil].
	].! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/24/2001 22:23'!
localDirectory
	^localDirectory! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/24/2001 22:24'!
localDirectory: aDirectory
	localDirectory := aDirectory! !

!ResourceCollector methodsFor: 'accessing' stamp: 'tk 6/28/2001 15:58'!
locatorMap
	"allow outsiders to store in it.  For files that are not resources that do want to live in the resource directory locally and on the server.  (.t files for example)"

	^locatorMap! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/27/2001 22:54'!
locators
	^locatorMap values! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/27/2001 20:07'!
locatorsDo: aBlock
	^locatorMap valuesDo: aBlock! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/27/2001 17:01'!
noteResource: aResourceStub replacing: anObject
	"Remember the fact that we need to load aResource which will replace anObject."
	stubMap at: aResourceStub put: anObject.! !

!ResourceCollector methodsFor: 'objects from disk' stamp: 'ar 2/24/2001 22:37'!
objectForDataStream: refStream
	"This should never happen; when projects get written they must be decoupled from the resource collector. If you get the error message below something is seriously broken."
	self error:'Cannot write resource manager'! !

!ResourceCollector methodsFor: 'accessing' stamp: 'mir 10/29/2003 13:33'!
objectForDataStream: refStream fromForm: aForm
 	"Return a replacement for aForm to be stored instead"
 	| stub fName copy loc fullSize nameAndSize |
 
 	"First check if the form is one of the intrinsic Squeak forms"
 	stub := internalStubs at: aForm ifAbsent:[nil].
 	stub ifNotNil:[
 		refStream replace: aForm with: stub. 
 		^stub].
 
 	"Now see if we have created the stub already 
 	(this may happen if for instance some form is shared)"
 	stub := originalMap at: aForm ifAbsent:[nil].
 	stub ifNotNil:[^aForm].
 	aForm hibernate.
 	aForm bits class == FormStub ifTrue:[^nil].	"something is wrong"
 	"too small to be of interest"
 	"(aForm bits byteSize < 4096) ifTrue:[^aForm]."
 	"We'll turn off writing out forms until we figure out how to reliably deal with resources"
 	true ifTrue: [^aForm].
 
 	"Create our stub form"
 	stub := FormStub 
 		extent: (aForm width min: 32) @ (aForm height min: 32) 
 		depth: (aForm depth min: 8).
 	aForm displayScaledOn: stub.
 	aForm hibernate.
 
 	"Create a copy of the original form which we use to store those bits"
 	copy := Form extent: aForm extent depth: aForm depth bits: nil.
 	copy setResourceBits: aForm bits.
 
 	"Get the locator for the form (if we have any)"
 	loc := locatorMap at: aForm ifAbsent:[nil].
 
 	"Store the resource file"
 	nameAndSize := self writeResourceForm: copy locator: loc.
 	fName := nameAndSize first.
 	fullSize := nameAndSize second.
 
 	ProgressNotification signal: '2:resourceFound' extra: stub.
 	stub hibernate.
 	"See if we need to assign a new locator"
 	(loc notNil and:[loc hasRemoteContents not]) ifTrue:[
 		"The locator describes some local resource. 
 		If we're preparing to upload the entire project to a
 		remote server, make it a remote URL instead."
 "		(baseUrl isEmpty not and:[baseUrl asUrl hasRemoteContents])
 			ifTrue:[loc urlString: baseUrl, fName].
 "
 		baseUrl isEmpty not
 			ifTrue:[loc urlString: self resourceDirectory , fName]].
 
 	loc ifNil:[
 		loc := ResourceLocator new urlString: self resourceDirectory , fName.
 		locatorMap at: aForm put: loc].
 	loc localFileName: (localDirectory fullNameFor: fName).
 	loc resourceFileSize: fullSize.
 	stub locator: loc.
 
 	"Map old against stub form"
 	aForm setResourceBits: stub.
 	originalMap at: aForm put: copy.
 	stubMap at: stub put: aForm.
 	locatorMap at: aForm put: loc.
 	"note: *must* force aForm in out pointers if 
 	in IS or else won't get #comeFullyUpOnReload:"
 	refStream replace: aForm with: aForm.
 	^aForm! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/27/2001 22:59'!
removeLocator: loc
	locatorMap keys "copy" do:[:k|
		(locatorMap at: k) = loc ifTrue:[locatorMap removeKey: k]].! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/27/2001 20:21'!
replaceAll
	"Replace all resources by their originals. Done after the resource have been collected to get back to the original state."
	originalMap keysAndValuesDo:[:k :v|
		v ifNotNil:[k replaceByResource: v].
	].! !

!ResourceCollector methodsFor: 'accessing' stamp: 'mir 6/21/2001 14:51'!
resourceDirectory
	resourceDirectory ifNil: [resourceDirectory := self baseUrl copyFrom: 1 to: (self baseUrl lastIndexOf: $/)].
	^resourceDirectory! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/27/2001 20:08'!
resourceFileNames
	"Return a list of all the resource files created"
	^locatorMap values asArray collect:[:loc| loc localFileName].! !

!ResourceCollector methodsFor: 'accessing' stamp: 'ar 2/27/2001 17:01'!
stubMap
	^stubMap! !

!ResourceCollector methodsFor: 'resource writing' stamp: 'yo 11/13/2002 23:30'!
writeResourceForm: aForm fromLocator: aLocator
	"The given form has been externalized before. If it was reasonably compressed, use the bits of the original data - this allows us to recycle GIF, JPEG, PNG etc. data without using the internal compression (which is in most cases inferior). If necessary the data will be retrieved from its URL location. This retrieval is done only if the resouce comes from either
		* the local disk (in which case the file has never been published)
		* the browser cache (in which case we don't cache the resource locally)
	In any other case we will *not* attempt to retrieve it, because doing so can cause the system to connect to the network which is probably not what we want. It should be a rare case anyways; could only happen if one clears the squeak cache selectively."
	| fName fStream url data |
	"Try to be smart about the name of the file"
	fName := (aLocator urlString includes: $:)
		ifTrue: [
			url := aLocator urlString asUrl.
			url path last]
		ifFalse: [aLocator urlString].
	fName isEmptyOrNil ifFalse:[fName := fName asFileName].
	(fName isEmptyOrNil or:[localDirectory isAFileNamed: fName]) ifTrue:[
		"bad luck -- duplicate name"
		fName := localDirectory 
				nextNameFor:'resource' 
				extension: (FileDirectory extensionFor: aLocator urlString)].
	"Let's see if we have cached it locally"
	ResourceManager
		lookupCachedResource: self baseUrl , aLocator urlString
		ifPresentDo:[:stream | data := stream upToEnd].
	"Check if the cache entry is without qualifying baseUrl. Workaround for older versions."
	data ifNil:[
		ResourceManager
			lookupCachedResource: aLocator urlString
			ifPresentDo:[:stream | data := stream upToEnd]].
	data ifNil:[
		"We don't have it cached locally. Retrieve it from its original location."
		((url notNil and: [url hasRemoteContents]) and:[HTTPClient isRunningInBrowser not])
			ifTrue:[^nil]. "see note above"
		(Url schemeNameForString: aLocator urlString)
			ifNil: [^nil].
		data := HTTPLoader default retrieveContentsFor: aLocator urlString.
		data ifNil:[^nil].
		data := data content.
	].
	"data size > aForm bits byteSize ifTrue:[^nil]."
	fStream := localDirectory newFileNamed: fName.
	fStream binary.
	fStream nextPutAll: data.
	fStream close.
	^{fName. data size}! !

!ResourceCollector methodsFor: 'resource writing' stamp: 'ar 9/23/2002 03:34'!
writeResourceForm: aForm locator: aLocator
	"Store the given form on a file. Return an array with the name and the size of the file"
	| fName fStream fullSize result writerClass |
	aLocator ifNotNil:[
		result := self writeResourceForm: aForm fromLocator: aLocator.
		result ifNotNil:[^result]
		"else fall through"
	].
	fName := localDirectory nextNameFor:'resource' extension:'form'.
	fStream := localDirectory newFileNamed: fName.
	fStream binary.
	aForm storeResourceOn: fStream.
false ifTrue:[
	"What follows is a Really, REALLY bad idea. I leave it in as a reminder of what you should NOT do. 
	PART I: Using JPEG or GIF compression on forms where we don't have the original data means loosing both quality and alpha information if present..."
	writerClass := ((Smalltalk includesKey: #JPEGReaderWriter2)
		and: [(Smalltalk at: #JPEGReaderWriter2) new isPluginPresent])
		ifTrue: [(Smalltalk at: #JPEGReaderWriter2)]
		ifFalse: [GIFReadWriter].
	writerClass putForm: aForm onStream: fStream.
	fStream open.
	fullSize := fStream size.
	fStream close.
].

	"Compress contents here"
true ifTrue:[
	"...PART II: Using the builtin compression which combines RLE+ZIP is AT LEAST AS GOOD as PNG and how much more would you want???"
	fStream position: 0.
	fStream compressFile.
	localDirectory deleteFileNamed: fName.
	localDirectory rename: fName, FileDirectory dot, 'gz' toBe: fName.
	fStream := localDirectory readOnlyFileNamed: fName.
	fullSize := fStream size.
	fStream close.
].
	^{fName. fullSize}! !

Object subclass: #ResourceLocator
	instanceVariableNames: 'urlString fileSize localFileName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!ResourceLocator commentStamp: '<historical>' prior: 0!
Describes where a resource can be found.

Instance variables:
	urlString	<String> 	The URL of the resource
	fileSize		<Integer>	The size of the resource
	localFileName	<String>	When non-nil, the place where this resource was/is stored.!

!ResourceLocator classSide methodsFor: 'utilities' stamp: 'fbs 2/2/2005 13:24'!
make: newURLString relativeTo: oldURLString 
	"Local file refs are not handled well, so work around here"
	^((oldURLString includesSubString: '://') not
		and: [(newURLString includesSubString: '://') not])
		ifTrue: [oldURLString , (UnixFileDirectory localNameFor: newURLString)]
		ifFalse: [(newURLString asUrlRelativeTo: oldURLString asUrl) asString]! !

!ResourceLocator methodsFor: 'comparing' stamp: 'tk 7/5/2001 22:10'!
= aLocator

	^ self species == aLocator species and: [self urlString = aLocator urlString]
! !

!ResourceLocator methodsFor: 'accessing' stamp: ''!
adjustToDownloadUrl: downloadUrl
	"Adjust to the fully qualified URL for this resource."
	self urlString: (ResourceLocator make: self urlString relativeTo: downloadUrl) unescapePercents! !

!ResourceLocator methodsFor: 'accessing' stamp: 'mir 6/19/2001 16:55'!
adjustToRename: newName from: oldName
	"Adjust to the fully qualified URL for this resource."
	self urlString: (self urlString copyReplaceAll: oldName with: newName)! !

!ResourceLocator methodsFor: 'testing' stamp: 'ar 2/27/2001 22:11'!
hasRemoteContents
	"Return true if we describe a resource which is non-local, e.g., on some remote server."
	(urlString indexOf: $:) = 0 ifTrue:[^false]. "no scheme"
	^urlString asUrl hasRemoteContents! !

!ResourceLocator methodsFor: 'comparing' stamp: 'ar 2/27/2001 20:02'!
hash
	^urlString hash! !

!ResourceLocator methodsFor: 'accessing' stamp: 'ar 2/27/2001 20:00'!
localFileName
	^localFileName! !

!ResourceLocator methodsFor: 'accessing' stamp: 'ar 2/27/2001 20:01'!
localFileName: aString
	localFileName := aString! !

!ResourceLocator methodsFor: 'printing' stamp: 'ar 2/27/2001 20:02'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(;
		print: urlString;
		nextPut: $)! !

!ResourceLocator methodsFor: 'accessing' stamp: 'ar 3/2/2001 18:13'!
resourceFileSize
	^fileSize! !

!ResourceLocator methodsFor: 'accessing' stamp: 'ar 3/2/2001 18:13'!
resourceFileSize: aNumber
	fileSize := aNumber! !

!ResourceLocator methodsFor: 'comparing' stamp: 'tk 7/5/2001 22:10'!
species
	^ResourceLocator! !

!ResourceLocator methodsFor: 'accessing' stamp: 'ar 2/27/2001 19:57'!
urlString
	^urlString! !

!ResourceLocator methodsFor: 'accessing' stamp: 'ar 2/27/2001 19:57'!
urlString: aString
	urlString := aString.! !

Object subclass: #ResourceManager
	instanceVariableNames: 'resourceMap loaded unloaded stopSemaphore stopFlag loaderProcess'
	classVariableNames: 'CachedResources LocalizedExternalResources'
	poolDictionaries: ''
	category: 'System-Support'!

!ResourceManager classSide methodsFor: 'private-resources' stamp: 'mir 11/29/2001 16:19'!
addCacheLocation: aString for: urlString
	| locations |
	locations := CachedResources at: urlString ifAbsentPut: [#()].
	(locations includes: aString)
		ifFalse: [CachedResources at: urlString put: ({aString} , locations)]! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'ar 5/30/2001 23:21'!
cacheResource: urlString inArchive: archiveName
	"Remember the given url as residing in the given archive"
	| fd file fullName |
	fullName := 'zip://', archiveName.
	((self resourceCache at: urlString ifAbsent:[#()]) 
		anySatisfy:[:cache| cache = fullName]) ifTrue:[^self]. "don't cache twice"
	fd := Project squeakletDirectory.
	"update cache"
	file := [fd oldFileNamed: self resourceCacheName] 
			on: FileDoesNotExistException
			do:[:ex| fd forceNewFileNamed: self resourceCacheName].
	file setToEnd.
	file nextPutAll: urlString; cr.
	file nextPutAll: fullName; cr.
	file close.
	self addCacheLocation: fullName for: urlString.! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'yo 12/20/2003 02:12'!
cacheResource: urlString stream: aStream
	| fd localName file buf |
	HTTPClient shouldUsePluginAPI ifTrue:[^self]. "use browser cache"
	(self resourceCache at: urlString ifAbsent:[#()]) size > 0 
		ifTrue:[^self]. "don't waste space"
	fd := Project squeakletDirectory.
	localName := fd nextNameFor: 'resource' extension:'cache'.
	file := fd forceNewFileNamed: localName.
	buf := ByteArray new: 10000.
	aStream binary.
	file binary.
	[aStream atEnd] whileFalse:[
		buf := aStream next: buf size into: buf.
		file nextPutAll: buf.
	].
	file close.
	"update cache"
	file := [fd oldFileNamed: self resourceCacheName] 
			on: FileDoesNotExistException
			do:[:ex| fd forceNewFileNamed: self resourceCacheName].
	file setToEnd.
	file nextPutAll: urlString; cr.
	file nextPutAll: localName; cr.
	file close.
	self addCacheLocation: localName for: urlString.
	aStream position: 0.
! !

!ResourceManager classSide methodsFor: 'private-resources' stamp: 'AlexandreBergel 7/30/2008 13:39'!
loadResource: urlString fromCacheFileNamed: fileName in: dir
	| archiveName file archive |
	(fileName beginsWith: 'zip://') ifTrue:[
		archiveName := fileName copyFrom: 7 to: fileName size.
		archive := [dir readOnlyFileNamed: archiveName] 
						on: FileDoesNotExistException
						do:[:ex| ^ nil].
		archive isZipArchive ifTrue:[
			archive := ZipArchive new readFrom: archive.
			file := archive members detect: [:any| any fileName = urlString] ifNone: [nil]].
		file ifNotNil:[file := file contentStream].
		archive close.
	] ifFalse:[
		file := [dir readOnlyFileNamed: fileName] 
				on: FileDoesNotExistException
				do:[:ex| ^ nil].
	].
	^file! !

!ResourceManager classSide methodsFor: 'private-resources' stamp: 'mir 8/21/2001 15:50'!
localizedExternalResources
	^LocalizedExternalResources ifNil:[LocalizedExternalResources := Dictionary new]! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'AlexandreBergel 7/30/2008 13:40'!
lookupCachedResource: cachedUrlString ifPresentDo: streamBlock
	"See if we have cached the resource described by the given url and if so, evaluate streamBlock with the cached resource."
	|  urlString candidates url stream |
	CachedResources ifNil:[^self].

	candidates := CachedResources at: cachedUrlString ifAbsent:[nil].
	(self lookupCachedResource: cachedUrlString in: candidates ifPresentDo: streamBlock)
		ifTrue: [^self].

	urlString := self relocatedExternalResource: cachedUrlString.
	urlString ifNil: [^self].
	candidates := CachedResources at: urlString ifAbsent:[nil].
	candidates
		ifNil: [
			(url := urlString asUrl) schemeName = 'file'
				ifTrue: [
					stream := [FileStream readOnlyFileNamed: url pathForFile] 
								on: FileDoesNotExistException do:[:ex| ex return: nil].
					stream
						ifNotNil: [[streamBlock value: stream] ensure: [stream close]]]]
		ifNotNil: [self lookupCachedResource: urlString in: candidates ifPresentDo: streamBlock]! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'mir 8/21/2001 18:31'!
lookupCachedResource: urlString in: candidates ifPresentDo: streamBlock
	"See if we have cached the resource described by the given url and if so, evaluate streamBlock with the cached resource."
	| sortedCandidates dir file |
	(candidates isNil or:[candidates size = 0])
		ifTrue:[^false].
	"First, try non-zip members (faster since no decompression is involved)"
	sortedCandidates := (candidates reject:[:each| each beginsWith: 'zip://']),
					(candidates select:[:each| each beginsWith: 'zip://']).
	dir := Project squeakletDirectory.
	sortedCandidates do:[:fileName|
		file := self loadResource: urlString fromCacheFileNamed: fileName in: dir.
		file ifNotNil:[
			[streamBlock value: file] ensure:[file close].
			^true]].
	^false! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'mir 6/21/2001 22:49'!
lookupOriginalResourceCacheEntry: resourceFileName for: resourceUrl
	"See if we have cached the resource described by the given url in an earlier version of the same project on the same server. In that case we don't need to upload it again but rather link to it."
	| candidates resourceBase resourceMatch matchingUrls |
	
	CachedResources ifNil:[^nil].

	"Strip the version number from the resource url"
	resourceBase := resourceUrl copyFrom: 1 to: (resourceUrl lastIndexOf: $.) .
	"Now collect all urls that have the same resource base"
	resourceMatch := resourceBase , '*/' , resourceFileName.
	matchingUrls := self resourceCache keys
		select: [:entry | (resourceMatch match: entry) and: [(entry beginsWith: resourceUrl) not]].
	matchingUrls isEmpty
		ifTrue: [^nil].
	matchingUrls asSortedCollection do: [:entry | 
			candidates := (self resourceCache at: entry).
			candidates isEmptyOrNil
				ifFalse: [candidates do: [:candidate |
					candidate = resourceFileName
						ifTrue: [^entry]]]].
	^nil! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'dc 5/30/2008 10:17'!
reloadCachedResources
	"ResourceManager reloadCachedResources"
	"Reload cached resources from the disk"
	| fd files stream url localName storeBack archiveName |
	CachedResources := Dictionary new.
	LocalizedExternalResources := nil.
	fd := Project squeakletDirectory.
	files := fd fileNames asSet.
	stream := [ fd readOnlyFileNamed: self resourceCacheName ] 
		on: FileDoesNotExistException
		do: [ :ex | fd forceNewFileNamed: self resourceCacheName ].
	stream size < 50000 ifTrue: [ stream := stream contentsOfEntireFile readStream ].
	storeBack := false.
	[ stream atEnd ] whileFalse: 
		[ url := stream upTo: Character cr.
		localName := stream upTo: Character cr.
		(localName beginsWith: 'zip://') 
			ifTrue: 
				[ archiveName := localName 
					copyFrom: 7
					to: localName size.
				(files includes: archiveName) 
					ifTrue: 
						[ self 
							addCacheLocation: localName
							for: url ]
					ifFalse: [ storeBack := true ] ]
			ifFalse: 
				[ (files includes: localName) 
					ifTrue: 
						[ self 
							addCacheLocation: localName
							for: url ]
					ifFalse: [ storeBack := true ] ] ].
	stream close.
	storeBack ifTrue: 
		[ stream := fd forceNewFileNamed: self resourceCacheName.
		CachedResources keysAndValuesDo: 
			[ :urlString :cacheLocs | 
			cacheLocs do: 
				[ :cacheLoc | 
				stream
					nextPutAll: urlString;
					cr.
				stream
					nextPutAll: cacheLoc;
					cr ] ].
		stream close ]! !

!ResourceManager classSide methodsFor: 'private-resources' stamp: 'mir 8/21/2001 16:06'!
relocatedExternalResource: urlString
	^self localizedExternalResources at: urlString ifAbsent: [nil]! !

!ResourceManager classSide methodsFor: 'private-resources' stamp: 'mir 8/21/2001 16:00'!
relocatedExternalResource: urlString to: newUrlString
	| originalURL |
	originalURL := (self localizedExternalResources includesKey: urlString)
		ifTrue: [self localizedExternalResources at: urlString]
		ifFalse: [urlString].
	self localizedExternalResources at: newUrlString put: originalURL! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'mir 8/21/2001 17:24'!
renameCachedResource: urlString to: newUrlString
	"A project was renamed. Reflect this change by duplicating the cache entry to the new url."
	self renameCachedResource: urlString to: newUrlString external: true! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'mir 12/3/2001 13:14'!
renameCachedResource: urlString to: newUrlString external: isExternal
	"A project was renamed. Reflect this change by duplicating the cache entry to the new url."
	| candidates |
	CachedResources
		ifNil:[
			isExternal
				ifTrue: [self resourceCache "force init" ]
				ifFalse: [^self]].
	candidates := CachedResources at: urlString ifAbsent:[nil].
	(candidates isNil or:[candidates size = 0])
		ifFalse: [
		candidates do: [:candidate |
			self addCacheLocation: candidate for: newUrlString]].
	isExternal
		ifTrue: [self relocatedExternalResource: urlString to: newUrlString]! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'ar 8/23/2001 17:52'!
resourceCache
	^CachedResources ifNil:[
		CachedResources := Dictionary new.
		self reloadCachedResources.
		CachedResources].! !

!ResourceManager classSide methodsFor: 'resource caching' stamp: 'ar 3/3/2001 17:27'!
resourceCacheName
	^'resourceCache.map'! !

!ResourceManager methodsFor: 'private' stamp: 'ar 3/2/2001 19:25'!
abandonResourcesThat: matchBlock
	"Private. Forget resources that match the given argument block"
	resourceMap keys "need copy" do:[:loc|
		(matchBlock value: loc) ifTrue:[
			resourceMap removeKey: loc ifAbsent:[].
			loaded remove: loc ifAbsent:[].
			unloaded remove: loc ifAbsent:[].
		].
	].! !

!ResourceManager methodsFor: 'accessing' stamp: 'ar 2/27/2001 20:50'!
addResource: anObject locator: aLocator
	resourceMap at: aLocator put: anObject.
	loaded add: aLocator.! !

!ResourceManager methodsFor: 'accessing' stamp: 'ar 3/2/2001 20:56'!
addResource: anObject url: urlString
	^self addResource: anObject locator: (ResourceLocator new urlString: urlString)! !

!ResourceManager methodsFor: 'accessing' stamp: 'mir 6/26/2001 17:33'!
adjustToDownloadUrl: downloadUrl
	"Adjust the resource manager to the current download location. A project might have been moved manually to a different location or server."

	downloadUrl isEmptyOrNil ifTrue: [^self].

	self resourceMap
		keysDo:[:locator | locator adjustToDownloadUrl: downloadUrl].
	self resourceMap rehash.
	unloaded rehash! !

!ResourceManager methodsFor: 'accessing' stamp: ''!
adjustToNewServer: newResourceUrl from: oldResourceUrl
	"Adjust the resource manager to the current download location. A project might have been moved manually to a different location or server."
	| urlMap oldUrl newUrl |
	newResourceUrl isEmptyOrNil ifTrue: [^self].
	urlMap := Dictionary new.
	self resourceMap
		keysDo: [:locator | 
			"Local file refs are not handled well, so work around here"
			oldUrl := ResourceLocator make: locator urlString relativeTo: oldResourceUrl.
			newUrl := ResourceLocator make: locator urlString relativeTo: newResourceUrl.
			oldUrl ~= newUrl
				ifTrue: [urlMap at: oldUrl asString unescapePercents put: newUrl asString unescapePercents]].
	self resourceMap rehash.
	unloaded rehash.
	urlMap keysAndValuesDo: [:old :new |
		ResourceManager renameCachedResource: old to: new]! !

!ResourceManager methodsFor: 'accessing' stamp: 'mir 6/21/2001 16:02'!
adjustToRename: newName from: oldName
	"Adjust the resource manager to the current download location. A project might have been moved manually to a different location or server."
	| urlMap oldUrl |
	newName isEmptyOrNil ifTrue: [^self].
	urlMap := Dictionary new.
	self resourceMap
		keysDo: [:locator | 
			oldUrl := locator urlString.
			locator adjustToRename: newName from: oldName.
			urlMap at: oldUrl put: locator urlString].
	self resourceMap rehash.
	unloaded rehash.
	urlMap keysAndValuesDo: [:old :new |
		ResourceManager renameCachedResource: old to: new]! !

!ResourceManager methodsFor: 'backward-compatibility' stamp: 'nk 7/30/2004 21:46'!
convertMapNameForBackwardcompatibilityFrom: aString 
	(SmalltalkImage current platformName = 'Mac OS' 
		and: ['10*' match: SmalltalkImage current osVersion]) 
			ifTrue: [^aString convertFromWithConverter: ShiftJISTextConverter new].
	^aString convertFromSystemString! !

!ResourceManager methodsFor: 'private' stamp: 'yo 1/12/2004 22:54'!
fixJISX0208Resource

	| keys value url |
	keys := resourceMap keys.

	keys do: [:key |
		value := resourceMap at: key.
		url := key urlString copy.
		url isOctetString not ifTrue: [url mutateJISX0208StringToUnicode].
		resourceMap removeKey: key.
		key urlString: url.
		resourceMap at: key put: value.
	].
! !

!ResourceManager methodsFor: 'private' stamp: 'ar 3/3/2001 15:30'!
formChangedReminder
	^[World newResourceLoaded].! !

!ResourceManager methodsFor: 'private' stamp: 'ar 3/2/2001 17:22'!
hackURL: urlString
	(urlString findString: '/SuperSwikiProj/') > 0 
		ifTrue:[^urlString copyReplaceAll: '/SuperSwikiProj/' with: '/uploads/']
		ifFalse:[^urlString]! !

!ResourceManager methodsFor: 'initialize' stamp: 'alain.plantec 5/28/2009 10:17'!
initialize
	"So resources may get garbage collected if possible"
	super initialize.
	self reset.! !

!ResourceManager methodsFor: 'initialize' stamp: 'mir 6/18/2001 22:49'!
initializeFrom: aCollector
	"Initialize the receiver from the given resource collector. None of the resources have been loaded yet, so make register all resources as unloaded."
	| newLoc |
	aCollector stubMap keysAndValuesDo:[:stub :res|
		newLoc := stub locator.
		resourceMap at: newLoc put: res.
		"unloaded add: newLoc."
	].! !

!ResourceManager methodsFor: 'loading' stamp: 'ar 5/30/2001 23:11'!
installResource: aResource from: aStream locator: loc
	| repl |
	aResource ifNil:[^false]. "it went away, so somebody might have deleted it"
	(aStream == nil or:[aStream size = 0]) ifTrue:[^false]. "error?!!"
	repl := aResource clone readResourceFrom: aStream asUnZippedStream.
	repl ifNotNil:[
		aResource replaceByResource: repl.
		unloaded remove: loc.
		loaded add: loc.
		^true
	].
	^false! !

!ResourceManager methodsFor: 'loading' stamp: 'nk 4/17/2004 19:50'!
loadCachedResources
	"Load all the resources that we have cached locally"
	| resource |
	self class reloadCachedResources.
	self prioritizedUnloadedResources do:[:loc|
		self class lookupCachedResource: loc urlString ifPresentDo:[:stream|
			resource := resourceMap at: loc ifAbsent:[nil].
			self installResource: resource
				from: stream
				locator: loc.
			(resource isForm) ifTrue:[
				self formChangedReminder value.
				World displayWorldSafely].
		].
	].! !

!ResourceManager methodsFor: 'loading' stamp: 'nk 4/17/2004 19:50'!
loaderProcess
	| loader requests req locator resource stream |
	loader := HTTPLoader default.
	requests := Dictionary new.
	self prioritizedUnloadedResources do:[:loc|
		req := HTTPLoader httpRequestClass for: (self hackURL: loc urlString) in: loader.
		loader addRequest: req.
		requests at: req put: loc].
	[stopFlag or:[requests isEmpty]] whileFalse:[
		stopSemaphore waitTimeoutMSecs: 500.
		requests keys "need a copy" do:[:r|
			r isSemaphoreSignaled ifTrue:[
				locator := requests at: r.
				requests removeKey: r.
				stream := r contentStream.
				resource := resourceMap at: locator ifAbsent:[nil].
				self class cacheResource: locator urlString stream: stream.
				self installResource: resource
					from: stream
					locator: locator.
				(resource isForm) ifTrue:[
					WorldState addDeferredUIMessage: self formChangedReminder]
ifFalse: [self halt].
			].
		].
	].
	"Either done downloading or terminating process"
	stopFlag ifTrue:[loader abort].
	loaderProcess := nil.
	stopSemaphore := nil.! !

!ResourceManager methodsFor: 'private' stamp: 'mir 8/20/2001 17:12'!
localizeAllExternalResources
	"Should be a preference later."
	^true! !

!ResourceManager methodsFor: 'accessing' stamp: 'mir 8/21/2001 17:07'!
makeAllProjectResourcesLocalTo: resourceUrl
	"Change the urls in the resource locators so project specific resources are stored and referenced locally. Project specific resources are all those that are kept locally in any of the project's versions."

	| locators locUrl locBase lastSlash projectBase localResource isExternal |
 	"Construct the version neutral project base"
	resourceUrl isEmptyOrNil ifTrue: [^self].
	projectBase := resourceUrl copyFrom: 1 to: (resourceUrl lastIndexOf: $.) - 1.
	locators := OrderedCollection new.
	self resourceMap
		keysAndValuesDo:[:loc :res | res ifNotNil: [locators add: loc]].
	locators do: [:locator |
		locUrl := locator urlString.
		locUrl ifNotNil: [
			lastSlash := locUrl lastIndexOf: $/.
			lastSlash > 0
				ifTrue: [
					locBase := locUrl copyFrom: 1 to: lastSlash - 1.
					locBase := locBase copyFrom: 1 to: (((locBase lastIndexOf: $.) - 1) max: 0).
					isExternal := projectBase ~= locBase.
					(isExternal not
						or: [self localizeAllExternalResources])
						ifTrue: [
							localResource := locUrl copyFrom: lastSlash+1 to: locUrl size.
							"Update the cache entry to point to the new resource location"
							ResourceManager renameCachedResource: locUrl to: (resourceUrl , localResource) external: isExternal.
							locator urlString: localResource]]]].
	self resourceMap rehash
! !

!ResourceManager methodsFor: 'loading' stamp: 'tetha 3/6/2004 15:46'!
preLoadFromArchive: aZipArchive cacheName: aFileName
	"Load the resources from the given zip archive"
	| orig nameMap resMap loc stream |
	self class reloadCachedResources.
	resMap := Dictionary new.
	nameMap := Dictionary new.
	unloaded do:[:locator|
		locator localFileName: nil.
		nameMap at: locator urlString put: locator.
		resMap at: locator urlString put: (resourceMap at: locator)].

	aZipArchive members do:[:entry|
		stream := nil.
		orig := resMap at: (self convertMapNameForBackwardcompatibilityFrom: entry fileName ) ifAbsent:[nil].
		loc := nameMap at: (self convertMapNameForBackwardcompatibilityFrom: entry fileName ) ifAbsent:[nil].
		"note: orig and loc may be nil for non-resource members"
		(orig notNil and:[loc notNil]) ifTrue:[
			stream := entry contentStream.
			self installResource: orig from: stream locator: loc.
			stream reset.
			aFileName 
				ifNil:[self class cacheResource: loc urlString stream: stream]
				ifNotNil:[self class cacheResource: loc urlString inArchive: aFileName]].
	].! !

!ResourceManager methodsFor: 'loading' stamp: 'ar 3/2/2001 18:16'!
prioritizedUnloadedResources
	"Return an array of unloaded resource locators prioritized by some means"
	| list |
	list := unloaded asArray.
	^list sort:[:l1 :l2|
		(l1 resourceFileSize ifNil:[SmallInteger maxVal]) <=
			(l2 resourceFileSize ifNil:[SmallInteger maxVal])]! !

!ResourceManager methodsFor: 'loading' stamp: 'mir 6/18/2001 22:49'!
registerUnloadedResources
	resourceMap keys do: [:newLoc |
		unloaded add: newLoc]
! !

!ResourceManager methodsFor: 'initialize' stamp: 'ar 2/27/2001 16:54'!
reset
	"Clean out everything"
	resourceMap := WeakValueDictionary new.
	loaded := Set new.
	unloaded := Set new.! !

!ResourceManager methodsFor: 'accessing' stamp: 'ar 2/27/2001 20:57'!
resourceMap
	^resourceMap! !

!ResourceManager methodsFor: 'loading' stamp: 'ar 3/3/2001 18:01'!
startDownload
	"Start downloading unloaded resources"
	self stopDownload.
	unloaded isEmpty ifTrue:[^self].
	self loadCachedResources.
	unloaded isEmpty ifTrue:[^self].
	stopFlag := false.
	stopSemaphore := Semaphore new.
	loaderProcess := [self loaderProcess] newProcess.
	loaderProcess priority: Processor lowIOPriority.
	loaderProcess resume.! !

!ResourceManager methodsFor: 'loading' stamp: 'ar 3/2/2001 17:09'!
stopDownload
	"Stop downloading unloaded resources"
	loaderProcess ifNil:[^self].
	stopFlag := true.
	stopSemaphore signal.
	[loaderProcess == nil] whileFalse:[(Delay forMilliseconds: 10) wait].
	stopSemaphore := nil.! !

!ResourceManager methodsFor: 'loading' stamp: 'ar 2/27/2001 21:42'!
updateResourcesFrom: aCollector
	"We just assembled all the resources in a project.
	Include all that were newly found"
	self reset. "start clean"
	aCollector stubMap keysAndValuesDo:[:stub :res|
		"update all entries"
		resourceMap at: stub locator put: res.
		loaded add: stub locator.
	].! !

Object subclass: #SecurityManager
	instanceVariableNames: 'privateKeyPair trustedKeys keysFileName'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'System-Support'!

!SecurityManager class methodsFor: 'accessing' stamp: 'nk 7/30/2004 21:50'!
default
	^Default ifNil: [Default := self new]! !

!SecurityManager class methodsFor: 'initialization' stamp: 'nk 7/30/2004 21:50'!
initialize
	"SecurityManager initialize"

	"Order: ExternalSettings, SecurityManager, AutoStart"

	Default := self new.
	Smalltalk addToStartUpList: self after: ExternalSettings.
	Smalltalk addToShutDownList: self! !

!SecurityManager class methodsFor: 'initialization' stamp: 'ar 2/6/2001 16:46'!
shutDown
	self default shutDown.! !

!SecurityManager class methodsFor: 'initialization' stamp: 'ar 2/6/2001 17:12'!
startUp
	self default startUp.! !

!SecurityManager methodsFor: 'accessing' stamp: 'ar 2/6/2001 16:19'!
addTrustedKey: aPublicKey
	"Add a public key to the list of trusted keys"
	trustedKeys := (trustedKeys copyWithout: aPublicKey) copyWith: aPublicKey.! !

!SecurityManager methodsFor: 'testing' stamp: 'ar 2/6/2001 16:14'!
canWriteImage
	"SecurityManager default canWriteImage"
	"Primitive. Return true if the right to write an image hasn't been revoked."
	<primitive: 'primitiveCanWriteImage' module: 'SecurityPlugin'>
	^true "assume so unless otherwise proven"! !

!SecurityManager methodsFor: 'security operations' stamp: 'ar 2/6/2001 16:14'!
disableFileAccess
	"SecurityManager default disableFileAccess"
	"Primitive. Disable unlimited access to files.
	Cannot be revoked from the image."
	<primitive: 'primitiveDisableFileAccess' module: 'FilePlugin'>
	^self primitiveFailed! !

!SecurityManager methodsFor: 'security operations' stamp: 'ar 2/6/2001 16:15'!
disableImageWrite
	"SecurityManager default disableImageWrite"
	"Primitive. Disable writing to an image file.
	Cannot be revoked from the image."
	<primitive: 'primitiveDisableImageWrite' module: 'SecurityPlugin'>
	^self primitiveFailed! !

!SecurityManager methodsFor: 'security operations' stamp: 'ar 2/6/2001 16:15'!
disableSocketAccess
	"SecurityManage default disableSocketAccess"
	"Primitive. Disable access to sockets.
	Cannot be revoked from the image."
	<primitive: 'primitiveDisableSocketAccess' module: 'SocketPlugin'>
	^self primitiveFailed! !

!SecurityManager methodsFor: 'security operations' stamp: 'rbb 2/18/2005 14:27'!
enterRestrictedMode
	"Some insecure contents was encountered. Close all doors and proceed."
	self isInRestrictedMode ifTrue:[^true].
	Preferences securityChecksEnabled ifFalse:[^true]. "it's been your choice..."
	Preferences warnAboutInsecureContent ifTrue:[
		( UIManager default chooseFrom: #('Load it anyways' 'Do not load it')
			title: 
'You are about to load some insecure content.
If you continue, access to files as well as
some other capabilities will be limited.')
			 = 1 ifFalse:[
				"user doesn't really want it"
				^false.
			].
	].
	"here goes the actual restriction"
	self flushSecurityKeys.
	self disableFileAccess.
	self disableImageWrite.
	"self disableSocketAccess."
	FileDirectory setDefaultDirectory: self untrustedUserDirectory.
	^true
! !

!SecurityManager methodsFor: 'security operations' stamp: 'sd 1/30/2004 15:22'!
fileInObjectAndCode: aStream
	| trusted |
	trusted := self positionToSecureContentsOf: aStream.
	trusted ifFalse:[self enterRestrictedMode ifFalse:[
		aStream close.
		^nil]].
	^aStream fileInObjectAndCode! !

!SecurityManager methodsFor: 'initialization' stamp: 'ar 2/6/2001 16:24'!
flushSecurityKey: aKey
	"Flush a security key"
	| n |
	n := aKey first.
	1 to: n basicSize do:[:i| n basicAt: i put: 0].
	n := aKey second.
	1 to: n basicSize do:[:i| n basicAt: i put: 0].
! !

!SecurityManager methodsFor: 'initialization' stamp: 'ar 2/6/2001 16:23'!
flushSecurityKeys
	"Flush all keys"
	privateKeyPair ifNotNil:[
		self flushSecurityKey: privateKeyPair first.
		self flushSecurityKey: privateKeyPair last.
	].
	privateKeyPair := nil.
	trustedKeys do:[:key| self flushSecurityKey: key].
	trustedKeys := #().! !

!SecurityManager methodsFor: 'private' stamp: 'adrian_lienhard 7/18/2009 16:01'!
generateKeyPairInBackground
	"SecurityManager default generateKeyPairInBackground"
	"Silently generate a key set on the local machine while running in the background."
	| guesstimate startTime |
	guesstimate := [ 10 benchmark ] timeToRun * 150.
	startTime := Time millisecondClockValue.
	privateKeyPair := nil.
	[ self generateLocalKeyPair ] fork.
	UIManager default informUserDuring: 
		[ :bar | 
		[ privateKeyPair == nil ] whileTrue: 
			[ bar value: 'Initializing security system (' , ((Time millisecondClockValue - startTime) * 100 // guesstimate) printString , '%)'.
			(Delay forSeconds: 1) wait ] ]! !

!SecurityManager methodsFor: 'private' stamp: 'sd 9/30/2003 13:58'!
generateLocalKeyPair
	"SecurityManager default generateLocalKeyPair"
	"Generate a key set on the local machine."
	| dsa |
	dsa := DigitalSignatureAlgorithm new.
	dsa initRandomFromString: 
		Time millisecondClockValue printString, 
		Date today printString, 
		SmalltalkImage current platformName printString.
	privateKeyPair := dsa generateKeySet.
	self storeSecurityKeys.! !

!SecurityManager methodsFor: 'testing' stamp: 'ar 2/6/2001 16:16'!
hasFileAccess
	"SecurityManager default hasFileAccess"
	"Return true if the right to access arbitrary files hasn't been revoked"
	<primitive: 'primitiveHasFileAccess' module: 'FilePlugin'>
	^true "assume so unless otherwise proven"! !

!SecurityManager methodsFor: 'testing' stamp: 'ar 2/6/2001 16:16'!
hasSocketAccess
	"SecurityManager default hasSocketAccess"
	"Return true if the right to access sockets hasn't been revoked"
	<primitive: 'primitiveHasSocketAccess' module: 'SocketPlugin'>
	^true "assume so unless otherwise proven"! !

!SecurityManager methodsFor: 'initialization' stamp: 'adrian_lienhard 7/18/2009 16:01'!
initialize
	super initialize.
	privateKeyPair := nil.
	trustedKeys := #().
	keysFileName := 'Pharo.keys'.! !

!SecurityManager methodsFor: 'testing' stamp: 'ar 2/6/2001 16:13'!
isInRestrictedMode
	"Return true if we're in restricted mode"
	^(self canWriteImage 
		or:[self hasFileAccess 
		"or:[self hasSocketAccess]"]) not! !

!SecurityManager methodsFor: 'accessing' stamp: 'ar 2/6/2001 16:17'!
keysFileName
	^keysFileName! !

!SecurityManager methodsFor: 'accessing' stamp: 'ar 2/6/2001 16:17'!
keysFileName: aFileName
	keysFileName := aFileName! !

!SecurityManager methodsFor: 'filein/out' stamp: 'AlexandreBergel 7/30/2008 13:41'!
loadSecurityKeys
	"SecurityManager default loadSecurityKeys"
	"Load the keys file for the current user"
	| fd loc file keys |
	self isInRestrictedMode ifTrue: [ ^ self ]. "no point in even trying"
	loc := self secureUserDirectory. "where to get it from"
	loc last = FileDirectory pathNameDelimiter ifFalse:[
		loc := loc copyWith: FileDirectory pathNameDelimiter.
	].

	fd := FileDirectory on: loc.
	file := [fd readOnlyFileNamed: keysFileName] 
				on: FileDoesNotExistException 
				do:[:ex| ^ self "no keys file"].
				
	keys := Object readFrom: file.
	privateKeyPair := keys first.
	trustedKeys := keys last.
	file close.! !

!SecurityManager methodsFor: 'security operations' stamp: 'RAA 3/2/2002 14:33'!
positionToSecureContentsOf: aStream
	| bytes trusted part1 part2 sig hash dsa okay pos |
	aStream binary.
	pos := aStream position.
	bytes := aStream next: 4.
	bytes = 'SPRJ' asByteArray ifFalse:[
		"was not signed"
		aStream position: pos.
		^false].
	part1 := (aStream nextInto: (LargePositiveInteger basicNew: 20)) normalize.
	part2 := (aStream nextInto: (LargePositiveInteger basicNew: 20)) normalize.
	sig := Array with: part1 with: part2.
	hash := SecureHashAlgorithm new hashStream: aStream.
	dsa := DigitalSignatureAlgorithm new.
	trusted := self trustedKeys.
	okay := (trusted detect:[:key| dsa verifySignature: sig ofMessageHash: hash publicKey: key]
			ifNone:[nil]) notNil.
	aStream position: pos+44.
	^okay! !

!SecurityManager methodsFor: 'accessing' stamp: 'tak 3/15/2005 00:45'!
primSecureUserDirectory
	<primitive: 'primitiveGetSecureUserDirectory' module: 'SecurityPlugin'>
	^ nil! !

!SecurityManager methodsFor: 'private' stamp: 'tak 12/17/2004 14:15'!
primUntrustedUserDirectory
	"Primitive. Return the untrusted user directory that is the root directory for files that are visible even in restricted mode."
	<primitive: 'primitiveGetUntrustedUserDirectory' module: 'SecurityPlugin'>
	^ nil! !

!SecurityManager methodsFor: 'private' stamp: 'mir 11/10/2003 16:14'!
printStateOn: stream
 	"Print the current state of myself onto stream.
 	Used to gather information in the debug log."
 
 	stream
 		nextPutAll: 'SecurityManager state:'; cr;
 		nextPutAll: 'Restricted: '; nextPutAll: self isInRestrictedMode asString; cr;
 		nextPutAll: 'FileAccess: '; nextPutAll: self hasFileAccess asString; cr;
 		nextPutAll: 'SocketAccess: '; nextPutAll: self hasSocketAccess asString; cr;
 		nextPutAll: 'Working Dir '; nextPutAll: FileDirectory default pathName asString; cr;
 		nextPutAll: 'Trusted Dir '; nextPutAll: self secureUserDirectory asString; cr;
 		nextPutAll: 'Untrusted Dir '; nextPutAll: self untrustedUserDirectory asString; cr;
 		cr! !

!SecurityManager methodsFor: 'accessing' stamp: 'tak 3/15/2005 00:46'!
secureUserDirectory
	"SecurityManager default secureUserDirectory"
	| dir |
	dir := self primSecureUserDirectory.
	^ dir
		ifNil: [FileDirectory default pathName]
		ifNotNil: [(FilePath pathName: dir isEncoded: true) asSqueakPathName]! !

!SecurityManager methodsFor: 'initialization' stamp: 'ar 2/6/2001 16:22'!
shutDown
	"Flush existing keys"
	self flushSecurityKeys.! !

!SecurityManager methodsFor: 'accessing' stamp: 'ar 2/6/2001 16:20'!
signingKey
	"Return the key used for signing projects"
	^privateKeyPair ifNotNil:[privateKeyPair first]! !

!SecurityManager methodsFor: 'initialization' stamp: 'ar 2/6/2001 18:28'!
startUp
	"Attempt to load existing keys"
	self loadSecurityKeys.
	(privateKeyPair == nil 
		and:[self isInRestrictedMode not
		and:[Preferences automaticKeyGeneration]]) ifTrue:[
			self generateKeyPairInBackground.
	].! !

!SecurityManager methodsFor: 'nil' stamp: 'sw 1/25/2002 12:41'!
storeSecurityKeys
	"Store the keys file for the current user"
	"SecurityManager default storeSecurityKeys"

	| fd loc file |
	self isInRestrictedMode ifTrue:[^self]. "no point in even trying"
	loc := self secureUserDirectory. "where to put it"
	loc last = FileDirectory pathNameDelimiter ifFalse:
		[loc := loc copyWith: FileDirectory pathNameDelimiter].
	fd := FileDirectory on: loc.
	fd assureExistence.
	fd deleteFileNamed: self keysFileName ifAbsent:[].
	file := fd newFileNamed: self keysFileName.
	{privateKeyPair. trustedKeys} storeOn: file.
	file close! !

!SecurityManager methodsFor: 'accessing' stamp: 'ar 2/6/2001 16:42'!
trustedKeys
	"Return an array of trusted public keys for verifying some project"
	privateKeyPair ifNil:[^trustedKeys].
	^{privateKeyPair second}, trustedKeys! !

!SecurityManager methodsFor: 'accessing' stamp: 'tak 12/17/2004 14:19'!
untrustedUserDirectory
	"SecurityManager default untrustedUserDirectory"
	| dir |
	dir := self primUntrustedUserDirectory.
	^ dir
		ifNil: [FileDirectory default pathName]
		ifNotNil: [(FilePath pathName: dir isEncoded: true) asSqueakPathName]! !

Object subclass: #SmalltalkImage
	instanceVariableNames: ''
	classVariableNames: 'EndianCache LastImageName LastQuitLogPosition LastStats SourceFileVersionString StartupStamp'
	poolDictionaries: ''
	category: 'System-Support'!

SmalltalkImage class
	instanceVariableNames: 'current'!

!SmalltalkImage commentStamp: 'sd 7/2/2003 21:50' prior: 0!
I represent the SmalltalkImage and partly the VM. Using my current instance you can 
	- get the value of some VM parameters, system arguments, vm profiling,
	endianess status, external objects,....

	- save the image, manage sources

As you will notice browsing my code I'm a fat class having still too much responsibility.
But this is life. sd-2 July 2003

PS: if someone wants to split me go ahead.!

SmalltalkImage class
	instanceVariableNames: 'current'!

!SmalltalkImage class methodsFor: 'instance creation' stamp: 'sd 9/30/2003 14:28'!
current
	"Note that this could be implemented differently to avoid the test"

	current isNil
		ifTrue: [current := self basicNew].
	^ current! !

!SmalltalkImage class methodsFor: 'initialization' stamp: 'yo 2/18/2004 18:26'!
initialize
"
	self initialize
"
	Smalltalk addToStartUpList: SmalltalkImage.
	SmalltalkImage startUp.
! !

!SmalltalkImage class methodsFor: 'instance creation' stamp: 'sd 9/30/2003 13:39'!
new

	self error: 'Use current'.! !

!SmalltalkImage class methodsFor: 'initialization' stamp: 'yo 2/18/2004 18:25'!
startUp

	EndianCache := nil.
! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'alain.plantec 5/12/2009 12:24'!
aboutThisSystem
	"Identify software version"
	| text dialog width |
	text := SmalltalkImage current systemInformationString withCRs.
	width := 0.
	text linesDo: [:l | width := width	max: (UITheme current textFont widthOfStringOrText: l)].
	dialog := LongMessageDialogWindow new entryText: text.
	dialog open.
	dialog width: (width + 120 min: Display width - 50)! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'NS 1/16/2004 15:34'!
assureStartupStampLogged
	"If there is a startup stamp not yet actually logged to disk, do it now."
	| changesFile |
	StartupStamp ifNil: [^ self].
	(SourceFiles isNil or: [(changesFile := SourceFiles at: 2) == nil]) ifTrue: [^ self].
	changesFile isReadOnly ifTrue:[^self].
	changesFile setToEnd; cr; cr.
	changesFile nextChunkPut: StartupStamp asString; cr.
	StartupStamp := nil.
	self forceChangesToDisk.! !

!SmalltalkImage methodsFor: 'endian' stamp: 'yo 2/18/2004 18:24'!
calcEndianness
	| bytes word blt |
	"What endian-ness is the current hardware?  The String '1234' will be stored into a machine word.  On BigEndian machines (the Mac), $1 will be the high byte if the word.  On LittleEndian machines (the PC), $4 will be the high byte."
	"SmalltalkImage current endianness"

	bytes := ByteArray withAll: #(0 0 0 0).  "(1 2 3 4) or (4 3 2 1)"
	word := WordArray with: 16r01020304.
	blt := (BitBlt toForm: (Form new hackBits: bytes)) 
				sourceForm: (Form new hackBits: word).
	blt combinationRule: Form over.  "store"
	blt sourceY: 0; destY: 0; height: 1; width: 4.
	blt sourceX: 0; destX: 0.
	blt copyBits.  "paste the word into the bytes"
	bytes first = 1 ifTrue: [^ #big].
	bytes first = 4 ifTrue: [^ #little].
	self error: 'Ted is confused'.! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'yo 7/2/2004 13:24'!
changeImageNameTo: aString
	
	self imageName: aString asSqueakPathName.
	LastImageName := self imageName! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'bf 1/12/2006 19:43'!
changesName
	"Answer the name for the changes file corresponding to the image file name."
	"Smalltalk changesName"

	| imName |
	imName := FileDirectory baseNameFor: self imageName.
	^ imName, FileDirectory dot, 'changes'! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'sd 11/16/2003 13:12'!
closeSourceFiles
	"Shut down the source files if appropriate.  1/29/96 sw: changed so that the closing and nilification only take place if the entry was a FileStream, thus allowing stringified sources to remain in the saved image file"

	1 to: 2 do: [:i |
		((SourceFiles at: i) isKindOf: FileStream)
			ifTrue:
				[(SourceFiles at: i) close.
				SourceFiles at: i put: nil]]! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'em 3/31/2005 11:48'!
currentChangeSetString
	"SmalltalkImage current currentChangeSetString"
	^ 'Current Change Set: ' translated, ChangeSet current name! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'nk 7/29/2004 10:03'!
datedVersion
	"Answer the version of this release."

	^SystemVersion current datedVersion! !

!SmalltalkImage methodsFor: 'endian' stamp: 'yo 2/18/2004 18:24'!
endianness

	EndianCache ifNil: [EndianCache := self calcEndianness].
	^ EndianCache.
! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'NS 1/27/2004 15:55'!
event: anEvent
	"Hook for SystemChangeNotifier"

	(anEvent isRemoved and: [anEvent itemKind = SystemChangeNotifier classKind]) ifTrue: [
		anEvent item acceptsLoggingOfCompilation 
			ifTrue: [self logChange: 'Smalltalk removeClassNamed: #' , anEvent item name].
	].
	anEvent isDoIt 
		ifTrue: [self logChange: anEvent item].
	(anEvent isRemoved and: [anEvent itemKind = SystemChangeNotifier methodKind]) ifTrue: [
		anEvent itemClass acceptsLoggingOfCompilation 
			ifTrue: [self logChange: anEvent itemClass name , ' removeSelector: #' , anEvent itemSelector]].! !

!SmalltalkImage methodsFor: 'external' stamp: 'sd 6/28/2003 18:23'!
exitToDebugger
	"Primitive. Enter the machine language debugger, if one exists. Essential.
	See Object documentation whatIsAPrimitive. This primitive is to access the
	debugger when debugging the vm or a plugging in C"

	<primitive: 114>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'vm parameters' stamp: 'sd 6/27/2003 23:27'!
extraVMMemory
	"Answer the current setting of the 'extraVMMemory' VM parameter. See the comment in extraVMMemory: for details."

	^ self vmParameterAt: 23
! !

!SmalltalkImage methodsFor: 'vm parameters' stamp: 'sd 6/27/2003 23:27'!
extraVMMemory: extraBytesToReserve
	"Request that the given amount of extra memory be reserved for use by the virtual machine to leave extra C heap space available for things like plugins, network and file buffers, and so on. This request is stored when the image is saved and honored when the image is next started up. Answer the previous value of this parameter."

	extraBytesToReserve < 0
		ifTrue: [self error: 'VM memory reservation must be non-negative'].
	^ self vmParameterAt: 23 put: extraBytesToReserve
! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'sd 9/24/2003 11:40'!
extractParameters

	| pName value index globals |
	globals := Dictionary new.
	index := 3. "Muss bei 3 starten, da 2 documentName ist"
	[pName := self  getSystemAttribute: index.
	pName isEmptyOrNil] whileFalse:[
		index := index + 1.
		value := self getSystemAttribute: index.
		value ifNil: [value := ''].
 		globals at: pName asUppercase put: value.
		index := index + 1].
	^globals! !

!SmalltalkImage methodsFor: 'image cleanup' stamp: 'oscar.nierstrasz 10/18/2009 12:15'!
fixObsoleteReferences
	"SmalltalkImage current fixObsoleteReferences.
	SystemNavigation default obsoleteBehaviors size > 0
		ifTrue: [ SystemNavigation default obsoleteBehaviors inspect.
			self error:'Still have obsolete behaviors. See inspector']"

	| informee obsoleteBindings obsName realName realClass |

	Smalltalk garbageCollect; garbageCollect.

	Preference allInstances do: [:each | 
		informee := each instVarNamed: #changeInformee.
		((informee isKindOf: Behavior)
			and: [informee isObsolete])
			ifTrue: [
				Transcript show: 'Preference: '; show: each name; cr.
				each instVarNamed: #changeInformee put: (Smalltalk at: (informee name copyReplaceAll: 'AnObsolete' with: '') asSymbol)]].
 
	CompiledMethod allInstances do: [:method |
		obsoleteBindings := method literals select: [:literal |
			literal isVariableBinding
				and: [literal value isBehavior]
				and: [literal value isObsolete]].
		obsoleteBindings do: [:binding |
			obsName := binding value name.
			Transcript show: 'Binding: '; show: obsName; cr.
			realName := obsName copyReplaceAll: 'AnObsolete' with: ''.
			realClass := Smalltalk at: realName asSymbol ifAbsent: [UndefinedObject].
			binding key: binding key value: realClass]].


	Behavior flushObsoleteSubclasses.
	Smalltalk garbageCollect; garbageCollect.



! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'NS 1/16/2004 15:35'!
forceChangesToDisk
	"Ensure that the changes file has been fully written to disk by closing and re-opening it. This makes the system more robust in the face of a power failure or hard-reboot."

	| changesFile |
	changesFile := SourceFiles at: 2.
	(changesFile isKindOf: FileStream) ifTrue: [
		changesFile flush.
		SecurityManager default hasFileAccess ifTrue:[
			changesFile close.
			changesFile open: changesFile name forWrite: true].
		changesFile setToEnd.
	].
! !

!SmalltalkImage methodsFor: 'modules' stamp: 'dvf 6/30/2005 12:11'!
forgetModule: aString
	"Primitive. If the module named aString is loaded, unloaded. If not, and it is marked an unloadable, unmark it so the VM will try to load it again next time. See comment for #unloadModule:."
	<primitive: 571>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'nice 5/11/2009 23:30'!
fullNameForChangesNamed: aName
	| imgName |
	imgName := self fullNameForImageNamed: aName.
	^FileDirectory fileName: (FileDirectory baseNameFor: imgName) extension: FileDirectory changeSuffix.! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'nice 5/11/2009 23:33'!
fullNameForImageNamed: aName
	| imgDir |
	imgDir := FileDirectory on: self imagePath.
	^FileDirectory fileName: (imgDir fullNameFor: aName) extension: FileDirectory imageSuffix.! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'DamienCassou 9/29/2009 13:11'!
getFileNameFromUser

	| newName |
	newName := UIManager default
		request: 'New File Name?' translated
		initialAnswer: (FileDirectory localNameFor: self imageName).
	newName isEmptyOrNil ifTrue: [^nil].
	((FileDirectory default fileOrDirectoryExists: (self fullNameForImageNamed: newName)) or:
	 [FileDirectory default fileOrDirectoryExists: (self fullNameForChangesNamed: newName)]) ifTrue: [
		(self confirm: ('{1} already exists. Overwrite?' translated format: {newName})) ifFalse: [^nil]].
	^newName
! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'md 10/26/2003 13:08'!
getSystemAttribute: attributeID 
	"Optional. Answer the string for the system attribute with the given 
	integer ID. Answer nil if the given attribute is not defined on this 
	platform. On platforms that support invoking programs from command 
	lines (e.g., Unix), this mechanism can be used to pass command line 
	arguments to programs written in Squeak.

	By convention, the first command line argument that is not a VM
	configuration option is considered a 'document' to be filed in. Such a
	document can add methods and classes, can contain a serialized object,
	can include code to be executed, or any combination of these.

	Currently defined attributes include: 
	-1000...-1 - command line arguments that specify VM options 
	0 - the full path name for currently executing VM 
	(or, on some platforms, just the path name of the VM's directory) 
	1 - full path name of this image 
	2 - a Squeak document to open, if any 
	3...1000 - command line arguments for Squeak programs 
	1001 - this platform's operating system 
	1002 - operating system version 
	1003 - this platform's processor type
	1004 - vm version"

	<primitive: 149>
	^ nil! !

!SmalltalkImage methodsFor: 'vm parameters' stamp: 'sd 6/27/2003 23:47'!
getVMParameters	
	"Answer an Array containing the current values of the VM's internal
	parameter/metric registers.  Each value is stored in the array at the
	index corresponding to its VM register.  (See #vmParameterAt: and
	#vmParameterAt:put:.)"
	"SmalltalkImage current getVMParameters"
	
	<primitive: 254>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'testing' stamp: 'Gwenael.Casaccio 2/2/2009 10:50'!
hasDisplay

	| arg i |
	i := -1.
	[(arg := SmalltalkImage current getSystemAttribute: i) isNil]
		whileFalse: [(#('-nodisplay' '-headless') includes: arg) ifTrue: [^ false].
			i := i - 1].
	^ true! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'yo 7/2/2004 13:24'!
imageName
	"Answer the full path name for the current image."
	"SmalltalkImage current imageName"

	| str |
	str := self primImageName.
	^ (FilePath pathName: str isEncoded: true) asSqueakPathName.
! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'yo 2/24/2005 18:34'!
imageName: newName
	"Set the the full path name for the current image.  All further snapshots will use this."

	| encoded |
	encoded := (FilePath pathName: newName isEncoded: false) asVmPathName.
	self primImageName: encoded.
! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'sd 11/16/2003 13:57'!
imagePath
	"Answer the path for the directory containing the image file."
	"SmalltalkImage current imagePath"

	^ FileDirectory dirPathFor: self imageName
! !

!SmalltalkImage methodsFor: 'endian' stamp: 'sd 6/27/2003 23:25'!
isBigEndian
	^self endianness == #big! !

!SmalltalkImage methodsFor: 'endian' stamp: 'sd 6/27/2003 23:25'!
isLittleEndian
	^self endianness == #little! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'sd 11/16/2003 14:02'!
lastQuitLogPosition
	^ LastQuitLogPosition! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'sd 11/16/2003 14:03'!
lastQuitLogPosition: aNumber
	"should be only use to ensure the transition from SystemDictionary to SmalltalkImage, then  	be removed"
		
	LastQuitLogPosition := aNumber! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'al 10/12/2008 21:04'!
lastUpdateString
	"SmalltalkImage current lastUpdateString"
	^'Latest update: #' translated, SystemVersion current highestUpdate printString! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'al 10/12/2008 21:03'!
licenseString
	^ 'LICENSE
	
Licensed under the MIT License with parts under the Apache License.

Copyright (c) Pharo Project, and Contributors Copyright (c) 1996-2008 Viewpoints
Research Institute, and Contributors Copyright (c) 1996 Apple Computer, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions: The
above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

You may obtain a copy of the Apache License at
http://www.apache.org/licenses/LICENSE-2.0'! !

!SmalltalkImage methodsFor: 'modules' stamp: 'sd 6/27/2003 23:47'!
listBuiltinModule: index
	"Return the name of the n-th builtin module.
	This list is not sorted!!"
	<primitive: 572>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'modules' stamp: 'PeterHugossonMiller 9/3/2009 11:22'!
listBuiltinModules
	"SmalltalkImage current listBuiltinModules"
	"Return a list of all builtin modules (e.g., plugins). Builtin plugins are those that are 	compiled with the VM directly, as opposed to plugins residing in an external shared library. 	The list will include all builtin plugins regardless of whether they are currently loaded 
	or not. Note that the list returned is not sorted!!"

	| modules index name |
	modules := Array new writeStream.
	index := 1.
	[true] whileTrue:[
		name := self listBuiltinModule: index.
		name ifNil:[^modules contents].
		modules nextPut: name.
		index := index + 1.
	].! !

!SmalltalkImage methodsFor: 'modules' stamp: 'sd 6/27/2003 23:48'!
listLoadedModule: index
	"Return the name of the n-th loaded module.
	This list is not sorted!!"
	<primitive: 573>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'modules' stamp: 'PeterHugossonMiller 9/3/2009 11:22'!
listLoadedModules
	"SmalltalkImage current listLoadedModules"
	"Return a list of all currently loaded modules (e.g., plugins). Loaded modules are those that currently in use (e.g., active). The list returned will contain all currently active modules regardless of whether they're builtin (that is compiled with the VM) or external (e.g., residing in some external shared library). Note that the returned list is not sorted!!"
	| modules index name |
	modules := Array new writeStream.
	index := 1.
	[true] whileTrue:[
		name := self listLoadedModule: index.
		name ifNil:[^modules contents].
		modules nextPut: name.
		index := index + 1.
	].! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'ar 4/10/2005 18:02'!
logChange: aStringOrText 
	"Write the argument, aString, onto the changes file."
	| aString changesFile |
	(SourceFiles isNil or: [(SourceFiles at: 2) == nil]) ifTrue: [^ self].
	self assureStartupStampLogged.

	aString := aStringOrText asString.
	(aString findFirst: [:char | char isSeparator not]) = 0
		ifTrue: [^ self].  "null doits confuse replay"
	(changesFile := SourceFiles at: 2).
	changesFile isReadOnly ifTrue:[^self].
	changesFile setToEnd; cr; cr.
	changesFile nextChunkPut: aString.
		"If want style changes in DoIt, use nextChunkPutWithStyle:, and allow Texts to get here"
	self forceChangesToDisk.! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'MiguelCoba 7/25/2009 02:13'!
openSourceFiles

	self imageName = LastImageName ifFalse:
		["Reset the author full name to blank when the image gets moved"
		LastImageName := self imageName.
		Author fullName: ''].
	FileDirectory
		openSources: self sourcesName
		andChanges: self changesName
		forImage: LastImageName.
	StandardSourceFileArray install! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'dtl 8/5/2006 09:48'!
osVersion
	"Return the version number string of the platform we're running on"
	"SmalltalkImage current osVersion"

	^(self getSystemAttribute: 1002) asString! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'sd 6/27/2003 23:38'!
platformName
	"Return the name of the platform we're running on"

	^self getSystemAttribute: 1001! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'sd 6/27/2003 23:43'!
platformSubtype
	"Return the subType of the platform we're running on"

	^self getSystemAttribute: 1003! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'yo 7/2/2004 13:23'!
primImageName
	"Answer the full path name for the current image."
	"SmalltalkImage current imageName"

	<primitive: 121>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'yo 7/2/2004 13:30'!
primImageName: newName
	"Set the the full path name for the current image.  All further snapshots will use this."

	<primitive: 121>
	^ self primitiveFailed! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'yo 7/2/2004 13:35'!
primVmPath
	"Answer the path for the directory containing the Smalltalk virtual machine. Return the 	empty string if this primitive is not implemented."
	"SmalltalkImage current vmPath"

	<primitive: 142>
	^ ''! !

!SmalltalkImage methodsFor: 'quit' stamp: 'sd 6/28/2003 17:32'!
quitPrimitive
	"Primitive. Exit to another operating system on the host machine, if one
	exists. All state changes in the object space since the last snapshot are lost.
	Essential. See Object documentation whatIsAPrimitive."

	<primitive: 113>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: 'yo 7/25/2003 17:50'!
reconstructChanges2
	"Move all the changes and its histories onto another sources file."
	"SmalltalkImage reconstructChanges2"

	| f oldChanges classCount |
	f := FileStream fileNamed: 'ST80.temp'.
	f header; timeStamp.
	(SourceFiles at: 2) converter: MacRomanTextConverter new.
'Recoding Changes File...'
	displayProgressAt: Sensor cursorPoint
	from: 0 to: Smalltalk classNames size
	during:
		[:bar | classCount := 0.
		Smalltalk allClassesDo:
			[:class | bar value: (classCount := classCount + 1).
			class moveChangesWithVersionsTo: f.
			class putClassCommentToCondensedChangesFile: f.
			class class moveChangesWithVersionsTo: f]].
	self lastQuitLogPosition: f position.
	f trailer; close.
	oldChanges := SourceFiles at: 2.
	oldChanges close.
	FileDirectory default 
		deleteFileNamed: oldChanges name , '.old';
		rename: oldChanges name toBe: oldChanges name , '.old';
		rename: f name toBe: oldChanges name.
	Smalltalk setMacFileInfoOn: oldChanges name.
	SourceFiles at: 2
			put: (FileStream oldFileNamed: oldChanges name)! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'bf 1/12/2006 19:20'!
recordStartupStamp
	
	StartupStamp := '----STARTUP----', Time dateAndTimeNow printString, ' as ', self imageName.
! !

!SmalltalkImage methodsFor: 'vm statistics' stamp: 'stephane.ducasse 8/5/2009 17:44'!
reportCPUandRAM
	"Write several text files with useful analysis for profiling purposes.
	Overwrites any existing report.
	SmalltalkImage current reportCPUandRAM
	"	

	| stream tally |
	"VM statistics (Memory use and GC, mainly)"
	stream := FileStream forceNewFileNamed: 'Log-MemoryStats.txt'.
	[ stream nextPutAll: self vmStatisticsReportString ] 
		ensure: [ stream close ].
	
	"Process list"
	stream := FileStream forceNewFileNamed: 'Log-ProcessList.txt'.
	[
		ProcessBrowser new processNameList 
			do: [ :each | 
				stream nextPutAll: each; cr ]
	] ensure: [ stream close ].

"Fork all these, so they run in sequence, as the system is back running"
[
	"Process taking most CPU"
	stream := FileStream forceNewFileNamed: 'Log-ThePig.txt'.
	ProcessBrowser dumpPigStackOn: stream andClose: true.
	
	"Tally of all processes"
	stream := FileStream forceNewFileNamed: 'Log-FullTally.txt'.
	[
		tally := MessageTally new.
		tally spyAllEvery: 1 on: [ (Delay forMilliseconds: 1000) wait ].
		tally report: stream ] ensure: [ stream close ].

	"Memory Analysis"
	stream := FileStream forceNewFileNamed: 'Log-MemoryAnalysis.txt'.
	[ SpaceTally new printSpaceAnalysis: 1 on: stream ]
		ensure: [ stream close ]

] forkNamed: 'CPU usage analysis'! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'kph 1/29/2007 00:52'!
saveAs

	"Put up the 'saveAs' prompt, obtain a name, and save the image  under that new name."

	self saveAs: self getFileNameFromUser! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'nice 5/12/2009 00:13'!
saveAs: newName
	newName ifNil: [ ^ self ].
	(SourceFiles at: 2) ifNotNil:
		[self closeSourceFiles; "so copying the changes file will always work"
			 saveChangesInFileNamed: (self fullNameForChangesNamed: newName)].
	^self saveImageInFileNamed: (self fullNameForImageNamed: newName)! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'DamienCassou 9/29/2009 13:11'!
saveAsEmbeddedImage
	"Save the current state of the system as an embedded image"

	| dir newName newImageName newImageSegDir oldImageSegDir haveSegs |
	dir := FileDirectory default.
	newName := UIManager default request: 'Select existing VM file'
				initialAnswer: (FileDirectory localNameFor: '').
	newName isEmptyOrNil ifTrue: [^Smalltalk].
	newName := FileDirectory baseNameFor: newName asFileName.
	newImageName := newName.
	(dir includesKey: newImageName) 
		ifFalse: 
			[^self 
				inform: 'Unable to find name ' , newName , ' Please choose another name.'].
	haveSegs := false.
	Smalltalk at: #ImageSegment
		ifPresent: 
			[:theClass | 
			(haveSegs := theClass instanceCount ~= 0) 
				ifTrue: [oldImageSegDir := theClass segmentDirectory]].
	self logChange: '----SAVEAS (EMBEDDED) ' , newName , '----' 
				, Date dateAndTimeNow printString.
	self imageName: (dir fullNameFor: newImageName) asSqueakPathName.
	LastImageName := self imageName.
	self closeSourceFiles.
	haveSegs 
		ifTrue: 
			[Smalltalk at: #ImageSegment
				ifPresent: 
					[:theClass | 
					newImageSegDir := theClass segmentDirectory.	"create the folder"
					oldImageSegDir fileNames do: 
							[:theName | 
							"copy all segment files"

							newImageSegDir 
								copyFileNamed: oldImageSegDir pathName , FileDirectory slash , theName
								toFileNamed: theName]]].
	self 
		snapshot: true
		andQuit: true
		embedded: true! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'stephane.ducasse 3/1/2009 08:48'!
saveAsNewVersion
	"Save the image/changes using the next available version number."
	"SmalltalkImage current saveAsNewVersion"
	
	| newName changesName aName anIndex |
	aName := FileDirectory baseNameFor: (FileDirectory default localNameFor: self imageName).
	anIndex := aName lastIndexOf: FileDirectory extensionDelimiter ifAbsent: [nil].
	(anIndex notNil and: [(aName copyFrom: anIndex + 1 to: aName size) isAllDigits])
		ifTrue:
			[aName := aName copyFrom: 1 to: anIndex - 1].

	newName := FileDirectory default nextNameFor: aName extension: FileDirectory imageSuffix.
	changesName := self fullNameForChangesNamed: newName.

	"Check to see if there is a .changes file that would cause a problem if we saved a new .image file with the new version number"
	(FileDirectory default fileOrDirectoryExists: changesName)
		ifTrue:
			[^ self inform:
'There is already .changes file of the desired name,
', newName, '
curiously already present, even though there is
no corresponding .image file.   Please remedy
manually and then repeat your request.'].

	^self saveAs: newName


! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'tpr 12/15/2003 16:01'!
saveChangesInFileNamed: aString
	| fullChangesName |
	fullChangesName := (FileDirectory default fullNameFor: aString).
	(FileDirectory default directoryNamed:(FileDirectory dirPathFor: fullChangesName )) assureExistence.
	FileDirectory default 
		copyFileWithoutOverwriteConfirmationNamed: SmalltalkImage current changesName 
		toFileNamed: fullChangesName.
	Smalltalk setMacFileInfoOn: fullChangesName.! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'mtf 6/9/2008 21:29'!
saveImageInFileNamed: aString
	| fullImageName |
	fullImageName := (FileDirectory default fullNameFor: aString).
	(FileDirectory default directoryNamed:(FileDirectory dirPathFor: fullImageName )) assureExistence.
	^self
		changeImageNameTo: fullImageName;
		closeSourceFiles;
		openSourceFiles;  "so SNAPSHOT appears in new changes file"
		saveImageSegments;
		snapshot: true andQuit: false! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'sd 11/16/2003 13:29'!
saveImageSegments

	| haveSegs oldImageSegDir newImageSegDir |
	haveSegs := false.
	Smalltalk at: #ImageSegment ifPresent: [:theClass | 
		(haveSegs := theClass instanceCount ~= 0) ifTrue: [
			oldImageSegDir := theClass segmentDirectory]].
	haveSegs ifTrue: [
		Smalltalk at: #ImageSegment ifPresent: [:theClass |
			newImageSegDir := theClass segmentDirectory.	"create the folder"
			oldImageSegDir fileNames do: [:theName | "copy all segment files"
				| imageSegmentName |
				imageSegmentName := oldImageSegDir pathName, FileDirectory slash, theName.
				newImageSegDir 
					copyFileWithoutOverwriteConfirmationNamed: imageSegmentName
					toFileNamed: theName]]].
! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'sd 11/16/2003 14:12'!
saveSession
	self snapshot: true andQuit: false! !

!SmalltalkImage methodsFor: 'preferences' stamp: 'laza 12/6/2004 13:55'!
setPlatformPreferences
	"Set some platform specific preferences on system startup"
	| platform specs |
	Preferences automaticPlatformSettings ifFalse:[^self].
	platform := self platformName.
	specs := 	#(	
					(soundStopWhenDone false)
					(soundQuickStart false)
			).
	platform = 'Win32' ifTrue:[
		specs := #(	
					(soundStopWhenDone true)
					(soundQuickStart false)
				)].
	platform = 'Mac OS' ifTrue:[
		specs := #(	
					(soundStopWhenDone false)
					(soundQuickStart true)
				)].
	specs do:[:tuple|
		Preferences setPreference: tuple first toValue: (tuple last == true).
	].
! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'sd 11/16/2003 14:20'!
shutDown

	^ self closeSourceFiles! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'sd 11/16/2003 14:12'!
snapshot: save andQuit: quit
	^self snapshot: save andQuit: quit embedded: false! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'pavel.krivanek 11/21/2008 16:38'!
snapshot: save andQuit: quit embedded: embeddedFlag
	"Mark the changes file and close all files as part of #processShutdownList.
	If save is true, save the current state of this Smalltalk in the image file.
	If quit is true, then exit to the outer OS shell.
	The latter part of this method runs when resuming a previously saved image. This resume logic checks for a document file to process when starting up."
	| resuming msg |
	Object flushDependents.
	Object flushEvents.

	(SourceFiles at: 2) ifNotNil:[
		msg := String streamContents: [ :s |
			s nextPutAll: '----';
			nextPutAll: (save ifTrue: [ quit ifTrue: [ 'QUIT' ] ifFalse: [ 'SNAPSHOT' ] ]
							ifFalse: [quit ifTrue: [ 'QUIT/NOSAVE' ] ifFalse: [ 'NOP' ]]);
			nextPutAll: '----';
			print: Date dateAndTimeNow; space;
			nextPutAll: (FileDirectory default localNameFor: self imageName);
			nextPutAll: ' priorSource: ';
			print: LastQuitLogPosition ].
		self assureStartupStampLogged.
		save ifTrue: [ LastQuitLogPosition := (SourceFiles at: 2) setToEnd; position ].
		self logChange: msg.
		Transcript cr; show: msg
	].

	Smalltalk processShutDownList: quit.
	Cursor write show.
	save ifTrue: [resuming := embeddedFlag 
					ifTrue: [self snapshotEmbeddedPrimitive] 
					ifFalse: [self snapshotPrimitive].  "<-- PC frozen here on image file"
				resuming == false "guard against failure" ifTrue:
					["Time to reclaim segment files is immediately after a save"
					Smalltalk at: #ImageSegment
						ifPresent: [:theClass | theClass reclaimObsoleteSegmentFiles]]]
		ifFalse: [resuming := false].
	quit & (resuming == false) ifTrue: [self quitPrimitive].
	Cursor normal show.
	Smalltalk setGCParameters.
	resuming == true ifTrue: [Smalltalk clearExternalObjects].
	Smalltalk processStartUpList: resuming == true.
	resuming == true ifTrue:[
		self setPlatformPreferences.
		self recordStartupStamp].
	
	UIManager default onSnapshot.
	
	"Now it's time to raise an error"
	resuming == nil ifTrue: [self error:'Failed to write image file (disk full?)'].
	^ resuming! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'sd 11/16/2003 13:58'!
snapshotEmbeddedPrimitive
	<primitive: 247>
	^nil "indicates error writing embedded image file"! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'sd 11/16/2003 13:59'!
snapshotPrimitive
	"Primitive. Write the current state of the object memory on a file in the
	same format as the Smalltalk-80 release. The file can later be resumed,
	returning you to this exact state. Return normally after writing the file.
	Essential. See Object documentation whatIsAPrimitive."

	<primitive: 97>
	^nil "indicates error writing image file"! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'sd 9/24/2003 12:43'!
sourceFileVersionString

	^ SourceFileVersionString! !

!SmalltalkImage methodsFor: 'private source file' stamp: 'sd 9/24/2003 12:42'!
sourceFileVersionString: aString

	SourceFileVersionString := aString! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'sd 9/24/2003 12:44'!
sourcesName
	"Answer the full path to the version-stable source code"
	^ self vmPath , SourceFileVersionString , FileDirectory dot , 'sources'! !

!SmalltalkImage methodsFor: 'utilities' stamp: 'sd 1/16/2004 20:54'!
stripMethods: tripletList messageCode: messageString
	"Used to 'cap' methods that need to be protected for proprietary reasons, etc.; call this with a list of triplets of symbols of the form  (<class name>  <#instance or #class> <selector name>), and with a string to be produced as part of the error msg if any of the methods affected is reached"

	| aClass sel keywords codeString |
	tripletList do:
		[:triplet |  
			(aClass := (Smalltalk at: triplet first ifAbsent: [nil])) notNil ifTrue:
				[triplet second == #class ifTrue:
					[aClass := aClass class].
				sel := triplet third.
				keywords := sel keywords.
				(keywords size == 1 and: [keywords first asSymbol isKeyword not])
					ifTrue:
						[codeString := keywords first asString]
					ifFalse:
						[codeString := ''.
						keywords withIndexDo:
							[:kwd :index |
								codeString := codeString, ' ', (keywords at: index), ' ',
									'arg', index printString]].
				codeString := codeString, '
	self codeStrippedOut: ', (messageString surroundedBySingleQuotes).

				aClass compile: codeString classified: 'stripped']]! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'al 10/12/2008 21:05'!
systemInformationString
	^ String cr, SystemVersion current version, String cr, self lastUpdateString, String cr, String cr, String cr, self licenseString! !

!SmalltalkImage methodsFor: 'vm statistics' stamp: 'sd 7/2/2003 21:45'!
textMarkerForShortReport

	^  'Since last view	'! !

!SmalltalkImage methodsFor: 'sources, changes log' stamp: 'nk 7/29/2004 10:09'!
timeStamp: aStream 
	"Writes system version and current time on stream aStream."

	| dateTime |
	dateTime := Time dateAndTimeNow.
	aStream nextPutAll: 'From ', SmalltalkImage current datedVersion, ' [', SmalltalkImage current lastUpdateString, '] on ', (dateTime at: 1) printString,
						' at ', (dateTime at: 2) printString! !

!SmalltalkImage methodsFor: 'external' stamp: 'sd 6/28/2003 17:38'!
unbindExternalPrimitives
	"Primitive. Force all external primitives to be looked up again afterwards. Since external primitives that have not found are bound for fast failure this method will force the lookup of all primitives again so that after adding some plugin the primitives may be found."
	<primitive: 570>
	"Do nothing if the primitive fails for compatibility with older VMs"
! !

!SmalltalkImage methodsFor: 'modules' stamp: 'sd 6/27/2003 23:49'!
unloadModule: aString
	"Primitive. Unload the given module.
	This primitive is intended for development only since some
	platform do not implement unloading of DLL's accordingly.
	Also, the mechanism for unloading may not be supported
	on all platforms."
	<primitive: 571>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'vm parameters' stamp: 'tpr 4/27/2005 11:03'!
vmParameterAt: parameterIndex
	"parameterIndex is a positive integer corresponding to one of the VM's internal
	parameter/metric registers.  Answer with the current value of that register.
	Fail if parameterIndex has no corresponding register.
	VM parameters are numbered as follows:
		1	end of old-space (0-based, read-only)
		2	end of young-space (read-only)
		3	end of memory (read-only)
		4	allocationCount (read-only)
		5	allocations between GCs (read-write)
		6	survivor count tenuring threshold (read-write)
		7	full GCs since startup (read-only)
		8	total milliseconds in full GCs since startup (read-only)
		9	incremental GCs since startup (read-only)
		10	total milliseconds in incremental GCs since startup (read-only)
		11	tenures of surving objects since startup (read-only)
		12-20 specific to the translating VM
		21	root table size (read-only)
		22	root table overflows since startup (read-only)
		23	bytes of extra memory to reserve for VM buffers, plugins, etc.

		24	memory threshold above which shrinking object memory (rw)
		25	memory headroom when growing object memory (rw)
		26  interruptChecksEveryNms - force an ioProcessEvents every N milliseconds, in case the image  is not calling getNextEvent often (rw)
		27	number of times mark loop iterated for current IGC/FGC (read-only) includes ALL marking
		28	number of times sweep loop iterated  for current IGC/FGC (read-only)
		29	number of times make forward loop iterated for current IGC/FGC (read-only)
		30	number of times compact move loop iterated for current IGC/FGC (read-only)
		31	number of grow memory requests (read-only)
		32	number of shrink memory requests (read-only)
		33	number of root table entries used for current IGC/FGC (read-only)
		34	number of allocations done before current IGC/FGC (read-only)
		35	number of survivor objects after current IGC/FGC (read-only)
		36  millisecond clock when current IGC/FGC completed (read-only)
		37  number of marked objects for Roots of the world, not including Root Table entries for current IGC/FGC (read-only)
		38  milliseconds taken by current IGC  (read-only)
		39  Number of finalization signals for Weak Objects pending when current IGC/FGC completed (read-only)
		40  VM word size - 4 or 8 (read-only)"

	<primitive: 254>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'vm parameters' stamp: 'sd 6/27/2003 23:27'!
vmParameterAt: parameterIndex put: newValue
	"parameterIndex is a positive integer corresponding to one of the VM's internal
	parameter/metric registers.  Store newValue (a positive integer) into that
	register and answer with the previous value that was stored there.
	Fail if newValue is out of range, if parameterIndex has no corresponding
	register, or if the corresponding register is read-only."

	<primitive: 254>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'yo 7/2/2004 13:36'!
vmPath
	"Answer the path for the directory containing the Smalltalk virtual machine. Return the 	empty string if this primitive is not implemented."
	"SmalltalkImage current vmPath"

	^ (FilePath pathName: (self primVmPath) isEncoded: true) asSqueakPathName.
! !

!SmalltalkImage methodsFor: 'vm statistics' stamp: 'PeterHugossonMiller 9/3/2009 11:22'!
vmStatisticsReportString
	"StringHolderView open: (StringHolder new contents:
		SmalltalkImage current vmStatisticsReportString) label: 'VM Statistics'"

	| params oldSpaceEnd youngSpaceEnd memoryEnd fullGCs fullGCTime incrGCs incrGCTime tenureCount mcMisses mcHits icHits upTime sendCount upTime2 fullGCs2 fullGCTime2 incrGCs2 incrGCTime2 tenureCount2 str |
	params := self getVMParameters.
	oldSpaceEnd			:= params at: 1.
	youngSpaceEnd		:= params at: 2.
	memoryEnd			:= params at: 3.
	fullGCs				:= params at: 7.
	fullGCTime			:= params at: 8.
	incrGCs				:= params at: 9.
	incrGCTime			:= params at: 10.
	tenureCount			:= params at: 11.
	mcMisses			:= params at: 15.
	mcHits				:= params at: 16.
	icHits				:= params at: 17.
	upTime := Time millisecondClockValue.
	sendCount := mcMisses + mcHits + icHits.

	str := (String new: 1000) writeStream.
	str	nextPutAll: 'uptime			';
		print: (upTime / 1000 / 60 // 60); nextPut: $h;
		print: (upTime / 1000 / 60 \\ 60) asInteger; nextPut: $m;
		print: (upTime / 1000 \\ 60) asInteger; nextPut: $s; cr.

	str	nextPutAll: 'memory			';
		nextPutAll: memoryEnd asStringWithCommas; nextPutAll: ' bytes'; cr.
	str	nextPutAll:	'	old			';
		nextPutAll: oldSpaceEnd asStringWithCommas; nextPutAll: ' bytes (';
		print: ((oldSpaceEnd / memoryEnd * 100) roundTo: 0.1); nextPutAll: '%)'; cr.
	str	nextPutAll: '	young		';
		nextPutAll: (youngSpaceEnd - oldSpaceEnd) asStringWithCommas; nextPutAll: ' bytes (';
		print: ((youngSpaceEnd - oldSpaceEnd / memoryEnd * 100) roundTo: 0.1); nextPutAll: '%)'; cr.
	str	nextPutAll: '	used		';
		nextPutAll: youngSpaceEnd asStringWithCommas; nextPutAll: ' bytes (';
		print: ((youngSpaceEnd / memoryEnd * 100) roundTo: 0.1); nextPutAll: '%)'; cr.
	str	nextPutAll: '	free		';
		nextPutAll: (memoryEnd - youngSpaceEnd) asStringWithCommas; nextPutAll: ' bytes (';
		print: ((memoryEnd - youngSpaceEnd / memoryEnd * 100) roundTo: 0.1); nextPutAll: '%)'; cr.

	str	nextPutAll: 'GCs				';
		nextPutAll: (fullGCs + incrGCs) asStringWithCommas.
	fullGCs + incrGCs > 0 ifTrue: [
		str
			nextPutAll: ' ('; 
			print: ((upTime / (fullGCs + incrGCs)) roundTo: 1); 
			nextPutAll: 'ms between GCs)'
	].
	str cr.
	str	nextPutAll: '	full			';
		print: fullGCs; nextPutAll: ' totalling '; nextPutAll: fullGCTime asStringWithCommas; nextPutAll: 'ms (';
		print: ((fullGCTime / upTime * 100) roundTo: 1.0);
		nextPutAll: '% uptime)'.
	fullGCs = 0 ifFalse:
		[str	nextPutAll: ', avg '; print: ((fullGCTime / fullGCs) roundTo: 1.0); nextPutAll: 'ms'].
	str	cr.
	str	nextPutAll: '	incr		';
		print: incrGCs; nextPutAll: ' totalling '; nextPutAll: incrGCTime asStringWithCommas; nextPutAll: 'ms (';
		print: ((incrGCTime / upTime * 100) roundTo: 1.0);
		nextPutAll: '% uptime), avg '; print: ((incrGCTime / incrGCs) roundTo: 1.0); nextPutAll: 'ms'; cr.
	str	nextPutAll: '	tenures		';
		nextPutAll: tenureCount asStringWithCommas.
	tenureCount = 0 ifFalse:
		[str nextPutAll: ' (avg '; print: (incrGCs / tenureCount) asInteger; nextPutAll: ' GCs/tenure)'].
	str	cr.

LastStats ifNil: [LastStats := Array new: 6]
ifNotNil: [
	upTime2 := upTime - (LastStats at: 1).
	fullGCs2 := fullGCs - (LastStats at: 2).
	fullGCTime2 := fullGCTime - (LastStats at: 3).
	incrGCs2 := incrGCs - (LastStats at: 4).
	incrGCTime2 := incrGCTime - (LastStats at: 5).
	tenureCount2 := tenureCount - (LastStats at: 6).

	str	nextPutAll: self textMarkerForShortReport ;
		nextPutAll: (fullGCs2 + incrGCs2) asStringWithCommas.
	fullGCs2 + incrGCs2 > 0 ifTrue: [
		str
			nextPutAll: ' ('; 
			print: ((upTime2 / (fullGCs2 + incrGCs2)) roundTo: 1); 
			nextPutAll: 'ms between GCs)'.
	].
	str cr.
	str	nextPutAll: '	uptime		'; print: ((upTime2 / 1000.0) roundTo: 0.1); nextPutAll: 's'; cr.
	str	nextPutAll: '	full			';
		print: fullGCs2; nextPutAll: ' totalling '; nextPutAll: fullGCTime2 asStringWithCommas; nextPutAll: 'ms (';
		print: ((fullGCTime2 / upTime2 * 100) roundTo: 1.0);
		nextPutAll: '% uptime)'.
	fullGCs2 = 0 ifFalse:
		[str	nextPutAll: ', avg '; print: ((fullGCTime2 / fullGCs2) roundTo: 1.0); nextPutAll: 'ms'].
	str	cr.
	str	nextPutAll: '	incr		';
		print: incrGCs2; nextPutAll: ' totalling '; nextPutAll: incrGCTime2 asStringWithCommas; nextPutAll: 'ms (';
		print: ((incrGCTime2 / upTime2 * 100) roundTo: 1.0);
		nextPutAll: '% uptime), avg '.
	incrGCs2 > 0 ifTrue: [
		 str print: ((incrGCTime2 / incrGCs2) roundTo: 1.0); nextPutAll: 'ms'
	].
	str cr.
	str	nextPutAll: '	tenures		';
		nextPutAll: tenureCount2 asStringWithCommas.
	tenureCount2 = 0 ifFalse:
		[str nextPutAll: ' (avg '; print: (incrGCs2 / tenureCount2) asInteger; nextPutAll: ' GCs/tenure)'].
	str	cr.
].
	LastStats at: 1 put: upTime.
	LastStats at: 2 put: fullGCs.
	LastStats at: 3 put: fullGCTime.
	LastStats at: 4 put: incrGCs.
	LastStats at: 5 put: incrGCTime.
	LastStats at: 6 put: tenureCount.

	sendCount > 0 ifTrue: [
		str	nextPutAll: 'sends			';
			nextPutAll: sendCount asStringWithCommas; cr.
		str	nextPutAll: '	full			';
			nextPutAll: mcMisses asStringWithCommas;
			nextPutAll: ' ('; print: ((mcMisses / sendCount * 100) roundTo: 0.1); nextPutAll: '%)'; cr.
		str	nextPutAll: '	m-cache	';
			nextPutAll: mcHits asStringWithCommas;
			nextPutAll: ' ('; print: ((mcHits / sendCount * 100) roundTo: 0.1); nextPutAll: '%)'; cr.
		str	nextPutAll: '	i-cache		';
			nextPutAll: icHits asStringWithCommas;
			nextPutAll: ' ('; print: ((icHits / sendCount * 100) roundTo: 0.1); nextPutAll: '%)'; cr].


	^ str contents
! !

!SmalltalkImage methodsFor: 'vm statistics' stamp: 'dc 5/30/2008 10:17'!
vmStatisticsShortString
	"Convenience item for access to recent statistics only"
	"StringHolderView open: (StringHolder new contents: SmalltalkImage current vmStatisticsShortString)
		label: 'VM Recent Statistics'"
	^ self vmStatisticsReportString readStream
		upToAll: 'Since';
		upTo: Character cr;
		upToEnd! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'dtl 8/5/2006 09:47'!
vmVersion	
	"Return a string identifying the interpreter version"
	"SmalltalkImage current vmVersion"

	^self getSystemAttribute: 1004! !

Object subclass: #SystemNavigation
	instanceVariableNames: 'browserClass hierarchyBrowserClass'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'System-Support'!

!SystemNavigation commentStamp: 'sd 4/15/2003 22:30' prior: 0!
I support the navigation of the system. I act as a facade but as I could require some state
or different way of navigating the system all my behavior are on the instance side!

!SystemNavigation class methodsFor: 'accessing' stamp: 'marcus.denker 9/14/2008 21:05'!
default
	Default ifNil: [Default := self new].
	^Default! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 17:32'!
addSelectorsReferingTo: aSymbol in: class to: sortedSenders special: special byte: byte 
	{class. class class} do: [:behavior| (behavior 
		thoroughWhichSelectorsReferTo: aSymbol
		special: special
		byte: byte) do: [ :sel | 
			sortedSenders add: (MethodReference 
				class: class
				selector: sel) ]]! !

!SystemNavigation methodsFor: 'query' stamp: 'al 1/9/2006 19:13'!
allBehaviorsDo: aBlock 
	"Evaluate the argument, aBlock, for each kind of Behavior in the system 
	(that is, Object and its subclasses and Traits).
	ar 7/15/1999: The code below will not enumerate any obsolete or anonymous
	behaviors for which the following should be executed:

		Smalltalk allObjectsDo:[:obj| obj isBehavior ifTrue:[aBlock value: obj]].

	but what follows is way faster than enumerating all objects."

	aBlock value: ProtoObject.
	ProtoObject allSubclassesDoGently: aBlock.		"don't bring in ImageSegments"

	"Classes outside the ProtoObject hierarchy"
	Class subclassesDo: [:aClass |
		(aClass == ProtoObject class
			or: [aClass isInMemory not
			or: [aClass isMeta not]]) ifFalse:
			["Enumerate the non-meta class and its subclasses"
			aBlock value: aClass soleInstance.
			aClass soleInstance allSubclassesDoGently: aBlock]].
	
	Trait allInstances , ClassTrait allInstances do: [:trait |
		aBlock value: trait]! !

!SystemNavigation methodsFor: 'query' stamp: 'marcus.denker 9/29/2008 08:23'!
allCallsOn: aLiteral 
	"Answer a Collection of all the methods that call on aLiteral even deeply embedded in 
	literal array."
	"self new allCallsOn: #open:label:."
	| aCollection special thorough aList byte |
	aCollection := OrderedCollection new.
	special := Smalltalk
				hasSpecialSelector: aLiteral
				ifTrueSetByte: [:b | byte := b].
	thorough := aLiteral isSymbol.
	Cursor wait showWhile: [self allBehaviorsDo: [:class | 
					aList := thorough
								ifTrue: [class
										thoroughWhichSelectorsReferTo: aLiteral
										special: special
										byte: byte]
								ifFalse: [class
										whichSelectorsReferTo: aLiteral
										special: special
										byte: byte].
					aList do: [:sel | aCollection add: (MethodReference class: class selector: sel)]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'marcus.denker 9/29/2008 08:29'!
allCallsOn: firstLiteral and: secondLiteral
	"Answer a SortedCollection of all the methods that call on both aLiteral 
	and secondLiteral."

	| aCollection secondArray firstSpecial secondSpecial firstByte secondByte |
	self flag: #ShouldUseAllCallsOn:. "sd"
	aCollection := SortedCollection new.
	firstSpecial := Smalltalk hasSpecialSelector: firstLiteral ifTrueSetByte: [:b | firstByte := b].
	secondSpecial := Smalltalk hasSpecialSelector: secondLiteral ifTrueSetByte: [:b | secondByte := b].
	Cursor wait showWhile: [ self allBehaviorsDo: [:class |
			secondArray := class whichSelectorsReferTo: secondLiteral special: secondSpecial byte: secondByte.
			((class whichSelectorsReferTo: firstLiteral special: firstSpecial byte: firstByte) select: [:aSel | 
				(secondArray includes: aSel)]) do: [:sel | 
					aCollection add: (MethodReference class: class selector: sel)]]].
	^aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'marcus.denker 6/12/2009 15:33'!
allCallsOn: aSymbol from: aClass
	"Answer a SortedCollection of all the methods that call on aSymbol."

	| aSortedCollection special byte |
	aSortedCollection := SortedCollection new.
	special := aClass environment hasSpecialSelector: aSymbol ifTrueSetByte: [:b | byte := b ].
	aClass withAllSubclassesDo: [ :class |
		(class whichSelectorsReferTo: aSymbol special: special byte: byte) do: [:sel |
				aSortedCollection add: (MethodReference class: class selector: sel)]].
	^aSortedCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/17/2003 21:31'!
allClasses
	"currently returns all the classes defined in Smalltalk but could be customized 
	for dealing with environments and in such a case would return on really all the classes"

	^ Smalltalk allClasses

	! !

!SystemNavigation methodsFor: 'query' stamp: 'al 2/13/2006 21:52'!
allClassesAndTraits
	
	^ Smalltalk allClassesAndTraits

	! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/17/2003 21:31'!
allClassesDo: aBlock
	"currently returns all the classes defined in Smalltalk but could be customized 
	for dealing with environments and  in such a case would work on really all the classes"

	^ Smalltalk allClassesDo: aBlock

	! !

!SystemNavigation methodsFor: 'query' stamp: 'marcus.denker 9/29/2008 08:27'!
allClassesImplementing: aSelector  
	"Answer an Array of all classes that implement the message aSelector."

	| aCollection |
	aCollection := ReadWriteStream on: Array new.
	self allBehaviorsDo: [:class | (class includesSelector: aSelector) ifTrue: [aCollection nextPut: class]].
	^ aCollection contents! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:35'!
allClassesInPackageNamed: packageName
	| classes |
	classes := (SystemOrganization classesInCategory: packageName) asSet.
	(SystemOrganization categoriesMatching: packageName, '-*') do: [:category| 
		classes addAll: (SystemOrganization classesInCategory: category)].
	^classes asArray! !

!SystemNavigation methodsFor: 'query' stamp: 'oha 6/27/2008 13:37'!
allClassesWithUnimplementedCalls
	"Answer an Array of classes that have messages with calls to methods that aren't implemented
	anywhere in the system"
	| all meth dict |
	dict := Dictionary new.
	all := self systemNavigation allImplementedMessages.
	self systemNavigation allBehaviorsDo: [:cl | 
		cl selectorsDo: [:sel | 
			meth := cl compiledMethodAt: sel.
			meth primitive = 0 ifTrue: [
				meth messages do: [:m | 
					(all includes: m) ifFalse: [
						((dict at: cl ifAbsentPut: [ Dictionary new ])
							at: sel ifAbsentPut: [ OrderedCollection new])
								add: m
					]
				]
			]
		]
	].
	^ dict! !

!SystemNavigation methodsFor: 'author' stamp: 'stephane.ducasse 8/8/2009 15:03'!
allContributors
	"SystemNavigation default allContributors"
	
	| bag stamp |
	bag := Bag new.
	self allBehaviorsDo: [ :behavior |
		behavior methodsDo: [ :compiledMethod |
			stamp := compiledMethod timeStamp.
			stamp notEmpty ifTrue: [
				bag add: compiledMethod timeStamp substrings first ]]].
	^bag! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/18/2003 10:04'!
allGlobalRefs
	"Answer a set of symbols that may be refs to Global names.  In some sense we should only need the associations, but this will also catch, eg, HTML tag types."

	^ self allGlobalRefsWithout: {{}. {}}! !

!SystemNavigation methodsFor: 'query' stamp: 'stephane.ducasse 4/18/2009 11:40'!
allGlobalRefsWithout: classesAndMessagesPair 
	"Answer a set of symbols that may be refs to Global names. In some  
	sense we should only need the associations, but this will also catch, eg,  
	HTML tag types. This method computes its result in the absence of  
	specified classes and messages."
	"may be a problem if namespaces are introduced as for the moment  
	only Smalltalk is queried. sd 29/4/03"
	| globalRefs absentClasses absentSelectors |
	globalRefs := IdentitySet new: CompiledMethod instanceCount.
	absentClasses := classesAndMessagesPair first.
	absentSelectors := classesAndMessagesPair second.
	self flag: #shouldBeRewrittenUsingSmalltalkAllClassesDo:.
	"sd 29/04/03"
	Cursor execute
		showWhile: [Smalltalk classNames
				do: [:cName | ((absentClasses includes: cName)
						ifTrue: [{}]
						ifFalse: [{Smalltalk at: cName. (Smalltalk at: cName) class}])
						do: [:cl | (absentSelectors isEmpty
								ifTrue: [cl selectors]
								ifFalse: [cl selectors copyWithoutAll: absentSelectors])
								do: [:sel | "Include all capitalized symbols for good 
									measure"
									(cl compiledMethodAt: sel) literalsDo: [:m | 
											((m isSymbol)
													and: [m size > 0
															and: [m first canBeGlobalVarInitial]])
												ifTrue: [globalRefs add: m].
											(m isMemberOf: Array)
												ifTrue: [m
														do: [:x | ((x isSymbol)
																	and: [x size > 0
																			and: [x first canBeGlobalVarInitial]])
																ifTrue: [globalRefs add: x]]].
											m isVariableBinding
												ifTrue: [m key
														ifNotNil: [globalRefs add: m key]]]]]]].
	^ globalRefs! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/29/2003 13:30'!
allImplementedMessages
	"Answer a Set of all the messages that are implemented in the system."
	^ self allImplementedMessagesWithout: {{}. {}}! !

!SystemNavigation methodsFor: 'query' stamp: 'al 1/13/2006 00:24'!
allImplementedMessagesWithout: classesAndMessagesPair 
	"Answer a Set of all the messages that are implemented in the system,  
	computed in the absence of the supplied classes and messages. Note this  
	reports messages that are in the absent selectors set."
	| messages absentClasses |
	messages := IdentitySet new: CompiledMethod instanceCount.
	absentClasses := classesAndMessagesPair first.
	self flag: #shouldBeRewrittenUsingSmalltalkAllClassesDo:. "sd 29/04/03" 
	Cursor execute showWhile: [
		Smalltalk classNames , Smalltalk traitNames
			do: [:name | ((absentClasses includes: name)
				ifTrue: [{}]
				ifFalse: [{Smalltalk at: name. (Smalltalk at: name) classSide}])
					do: [:each | messages addAll: each selectors]]].
	^ messages! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/23/2003 22:31'!
allImplementorsOf: aSelector 
	"Answer a SortedCollection of all the methods that implement the message 
	aSelector."
	| aCollection |
	aCollection := SortedCollection new.
	Cursor wait
		showWhile: [self
				allBehaviorsDo: [:class | (class includesSelector: aSelector)
						ifTrue: [aCollection
								add: (MethodReference new setStandardClass: class methodSymbol: aSelector)]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'apl 4/4/2005 17:59'!
allImplementorsOf: aSelector localTo: aClass 
	"Answer a SortedCollection of all the methods that implement the message  
	aSelector in, above, or below the given class."
	| cls aCollection |
	aCollection := SortedCollection new.
	cls := aClass theNonMetaClass.
	Cursor wait
		showWhile: [cls
				withAllSuperAndSubclassesDoGently: [:class | (class includesSelector: aSelector)
						ifTrue: [aCollection
								add: (MethodReference new setStandardClass: class methodSymbol: aSelector)]].
			cls class
				withAllSuperAndSubclassesDoGently: [:class | (class includesSelector: aSelector)
						ifTrue: [aCollection
								add: (MethodReference new setStandardClass: class methodSymbol: aSelector)]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'browse' stamp: 'marcus.denker 7/24/2009 16:47'!
allMethodsInCategory: category 
	| aCollection |
	aCollection := Set new.
	Cursor wait showWhile: [
			self allBehaviorsDo: [:x | ((category = ClassOrganizer allCategory
					ifTrue: [x organization allMethodSelectors]
					ifFalse: [x organization listAtCategoryNamed: category])) do:
				[:sel | aCollection add: (MethodReference new setStandardClass: x 
methodSymbol: sel)]]].
	^aCollection.
! !

!SystemNavigation methodsFor: 'query' stamp: 'md 8/27/2005 16:42'!
allMethodsNoDoitsSelect: aBlock 
	"Like allSelect:, but strip out Doits"
	| aCollection |
	aCollection := SortedCollection new.
	Cursor execute
		showWhile: [self
				allBehaviorsDo: [:class | class
						selectorsDo: [:sel | (sel isDoIt not
									and: [aBlock
											value: (class compiledMethodAt: sel)])
								ifTrue: [aCollection
										add: (MethodReference new setStandardClass: class methodSymbol: sel)]]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/29/2003 20:41'!
allMethodsSelect: aBlock 
	"Answer a SortedCollection of each method that, when used as the block  
	argument to aBlock, gives a true result."
	| aCollection |
	aCollection := SortedCollection new.
	Cursor execute
		showWhile: [self
				allBehaviorsDo: [:class | class
						selectorsDo: [:sel | (aBlock
									value: (class compiledMethodAt: sel))
								ifTrue: [aCollection
										add: (MethodReference new setStandardClass: class methodSymbol: sel)]]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'al 1/13/2006 11:59'!
allMethodsWithSourceString: aString matchCase: caseSensitive
	"Answer a SortedCollection of all the methods that contain, in source code, aString as a substring.  Search the class comments also"

	| list count adder |
	list := Set new.
	adder := [ :mrClass :mrSel | list add: ( MethodReference new
											setStandardClass: mrClass
											methodSymbol: mrSel)].
	'Searching all source code...'
		displayProgressAt: Sensor cursorPoint
		from: 0 to: Smalltalk classNames size
		during: [:bar |
			count := 0.
			SystemNavigation default allBehaviorsDo: [:each |
				bar value: (count := count + 1).
					each selectorsDo: [:sel | 
						((each sourceCodeAt: sel) findString: aString 
							startingAt: 1 caseSensitive: caseSensitive) > 0 ifTrue: [
								sel isDoIt ifFalse: [adder value: each value: sel]]].
					(each organization classComment asString findString: aString 
							startingAt: 1 caseSensitive: caseSensitive) > 0 ifTrue: [
								adder value: each value: #Comment]	]].
			^ list asSortedCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'ar 8/18/2008 18:08'!
allObjectsDo: aBlock 
	"Evaluate the argument, aBlock, for each object in the system 
	excluding SmallIntegers."
	| object endMarker |
	object := self someObject.
	endMarker := Object new.
	[endMarker == object]
		whileFalse: [aBlock value: object.
			object := object nextObject]! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/29/2003 20:53'!
allObjectsSelect: aBlock 
	"Evaluate the argument, aBlock, for each object in the system excluding 
	SmallIntegers. Return a collection af all objects for whom the value is 
	true. "
	^ Array
		streamContents: [:s | self
				allObjectsDo: [:object | (aBlock value: object)
						ifTrue: [s nextPut: object]]]! !

!SystemNavigation methodsFor: 'query' stamp: 'nk 7/3/2003 19:51'!
allPrimitiveMethods
	"Answer an OrderedCollection of all the methods that are implemented by primitives."
	| aColl method |
	aColl := OrderedCollection new: 200.
	Cursor execute
		showWhile: [self allBehaviorsDo: [:class | class
						selectorsDo: [:sel | 
							method := class compiledMethodAt: sel.
							method primitive ~= 0
								ifTrue: [aColl addLast: class name , ' ' , sel , ' ' , method primitive printString]]]].
	^ aColl! !

!SystemNavigation methodsFor: 'query' stamp: 'nk 7/3/2003 19:49'!
allPrimitiveMethodsInCategories: aList 
	"Answer an OrderedCollection of all the methods that are implemented by 
	primitives in the given categories. 1/26/96 sw"
	"SystemNavigation new allPrimitiveMethodsInCategories:  
	#('Collections-Streams' 'Files-Streams' 'Files-Abstract' 'Files-Macintosh')"

	| aColl method |
	aColl := OrderedCollection new: 200.
	Cursor execute
		showWhile: [self
				allBehaviorsDo: [:aClass | (aList includes: (SystemOrganization categoryOfElement: aClass theNonMetaClass name asString) asString)
						ifTrue: [aClass
								selectorsDo: [:sel | 
									method := aClass compiledMethodAt: sel.
									method primitive ~= 0
										ifTrue: [aColl addLast: aClass name , ' ' , sel , ' ' , method primitive printString]]]]].
	^ aColl! !

!SystemNavigation methodsFor: 'query' stamp: 'ar 9/7/2003 17:58'!
allReferencesToPool: aPool from: aClass
	"Answer all the references to variables from aPool"
	| ref list |
	list := OrderedCollection new.
	aClass withAllSubclassesDo:[:cls|
		cls selectorsAndMethodsDo:[:sel :meth|
			ref := meth literals detect:[:lit|
				lit isVariableBinding and:[(aPool bindingOf: lit key) notNil]
			] ifNone:[nil].
			ref ifNotNil:[
				list add:(MethodReference new setStandardClass: cls methodSymbol: sel)
			].
		].
	].
	^list! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 5/5/2003 09:18'!
allSelectorsWithAnyImplementorsIn: selectorList 
	"Answer the subset of the given list which represent method selectors 
	which have at least one implementor in the system."
	| good |
	good := OrderedCollection new.
	self allBehaviorsDo: [:class | selectorList
				do: [:aSelector | (class includesSelector: aSelector)
						ifTrue: [good add: aSelector]]].
	^ good asSet asSortedArray" 
	SystemNavigation new selectorsWithAnyImplementorsIn: #( contents 
	contents: nuts)
	"! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 16:52'!
allSendersOf: selector 
	|  sortedSenders special byte |
	sortedSenders := SortedCollection new.
	special := Smalltalk 
		hasSpecialSelector: selector
		ifTrueSetByte: [ :b | byte := b ].
	self allBehaviorsDo: 
		[ :behavior | 
		self 
			addSelectorsReferingTo: selector
			in: behavior
			to: sortedSenders
			special: special
			byte: byte ].
	^ sortedSenders! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 15:54'!
allSendersOf: selector inClass: aClass 
	| sortedSenders special byte |
	sortedSenders := SortedCollection new.
	special := aClass environment hasSpecialSelector: selector ifTrueSetByte: [:b | byte := b ].
	self 
		addSelectorsReferingTo: selector
		in: aClass
		to: sortedSenders
		special: special
		byte: byte.
	^sortedSenders! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 15:58'!
allSendersOf: selector inClassCategory: category 
	| classes sortedSenders special byte |
	classes := SystemOrganization classesInCategory: category.
	sortedSenders := SortedCollection new.
	classes ifEmpty: [ ^ sortedSenders ].
	special := classes anyOne environment 
		hasSpecialSelector: selector
		ifTrueSetByte: [ :b | byte := b ].
	classes do: 
		[ :class | 
		self 
			addSelectorsReferingTo: selector
			in: class
			to: sortedSenders
			special: special
			byte: byte ].
	^ sortedSenders! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 17:12'!
allSendersOf: selector inPackageNamed: packageName
	^(self categoriesInPackageNamed: packageName) inject: SortedCollection new into: [:sortedSenders :category|
		sortedSenders, (self allSendersOf: selector inClassCategory: category)]! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/29/2003 20:12'!
allSentMessages
	"Answer the set of selectors which are sent somewhere in the system."
	^ self  allSentMessagesWithout: {{}. {}}! !

!SystemNavigation methodsFor: 'query' stamp: 'stephane.ducasse 4/18/2009 11:39'!
allSentMessagesWithout: classesAndMessagesPair 
	"Answer the set of selectors which are sent somewhere in the system,  
	computed in the absence of the supplied classes and messages."
	| sent absentClasses absentSelectors |
	sent := IdentitySet new: CompiledMethod instanceCount.
	absentClasses := classesAndMessagesPair first.
	absentSelectors := classesAndMessagesPair second.
	self flag: #shouldBeRewrittenUsingSmalltalkAllClassesDo:.
	"sd 29/04/03"
	Cursor execute showWhile: [
		Smalltalk classNames , Smalltalk traitNames do: [:name |
			((absentClasses includes: name)
				ifTrue: [{}]
				ifFalse: [{Smalltalk at: name. (Smalltalk at: name) classSide}])
					do: [:each | (absentSelectors isEmpty
						ifTrue: [each selectors]
						ifFalse: [each selectors copyWithoutAll: absentSelectors])
						do: [:sel | "Include all sels, but not if sent by self"
							(each compiledMethodAt: sel) literalsDo: [:m | 
									(m isSymbol)
										ifTrue: ["might be sent"
											m == sel
												ifFalse: [sent add: m]].
									(m isMemberOf: Array)
										ifTrue: ["might be performed"
											m
												do: [:x | (x isSymbol)
														ifTrue: [x == sel
																ifFalse: [sent add: x]]]]]]]].
			"The following may be sent without being in any literal frame"
			1
				to: Smalltalk specialSelectorSize
				do: [:index | sent
						add: (Smalltalk specialSelectorAt: index)]].
	Smalltalk presumedSentMessages
		do: [:sel | sent add: sel].
	^ sent! !

!SystemNavigation methodsFor: 'deprecated' stamp: 'AndrewBlack 9/1/2009 08:21'!
allUnSentMessages
	
	self deprecated: 'Use ''allUnsentMessages'' instead.'.
	^ self allUnsentMessages. ! !

!SystemNavigation methodsFor: 'deprecated' stamp: 'AndrewBlack 9/1/2009 08:21'!
allUnSentMessagesIn: selectorSet 
	"Answer the subset of selectorSet which are not sent anywhere in the 
	system. "
	self deprecated: 'Use ''allUnsentMessagesIn:'' instead.'. 
	^ self allUnsentMessagesIn: selectorSet! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/29/2003 19:19'!
allUnSentMessagesWithout: classesAndMessagesPair 
	"Answer the set of selectors that are implemented but not sent, computed  
	in the absence of the supplied classes and messages."
	^ (self  allImplementedMessagesWithout: classesAndMessagesPair)
		copyWithoutAll: (self  allSentMessagesWithout: classesAndMessagesPair)! !

!SystemNavigation methodsFor: 'query' stamp: 'PeterHugossonMiller 9/3/2009 11:31'!
allUnimplementedCalls
	"Answer an Array of each message that is sent by an expression in a  
	method but is not implemented by any object in the system."
	| aStream secondStream all |
	all := self allImplementedMessages.
	aStream := (Array new: 50) writeStream.
	Cursor execute
		showWhile: [self
				allBehaviorsDo: [:cl | cl
						selectorsDo: [:sel | 
							secondStream := (String new: 5) writeStream.
							(cl compiledMethodAt: sel) messages
								do: [:m | (all includes: m)
										ifFalse: [secondStream nextPutAll: m;
												 space]].
							secondStream position = 0
								ifFalse: [aStream nextPut: cl name , ' ' , sel , ' calls: ' , secondStream contents]]]].
	^ aStream contents! !

!SystemNavigation methodsFor: 'query' stamp: 'PeterHugossonMiller 9/3/2009 11:31'!
allUnimplementedNonPrimitiveCalls
	"Answer an Array of each message that is sent by an expression in a  
	method but is not implemented by any object in the system."
	| aStream secondStream all meth |
	all := self systemNavigation allImplementedMessages.
	aStream := (Array new: 50) writeStream.
	Cursor execute
		showWhile: [self systemNavigation
				allBehaviorsDo: [:cl | cl
						selectorsDo: [:sel | 
							secondStream := (String new: 5) writeStream.
							meth := cl compiledMethodAt: sel.
							meth primitive = 0 ifTrue: [
								meth messages
									do: [:m | (all includes: m)
											ifFalse: [secondStream nextPutAll: m;
													 space]].
								secondStream position = 0
									ifFalse: [aStream nextPut: cl name , ' ' , sel , ' calls: ' , secondStream contents]]]]].
	^ aStream contents! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/29/2003 13:07'!
allUnreferencedClassVariablesOf: aClass
	"Answer a list of the names of all the receiver's unreferenced class  
	vars, including those defined in superclasses"
	| aList |
	aList := OrderedCollection new.
	aClass withAllSuperclasses
		reverseDo: [:aSuperClass | aSuperClass classVarNames
				do: [:var | (self allCallsOn: (aSuperClass classPool associationAt: var)) isEmpty
						ifTrue: [aList add: var]]].
	^ aList! !

!SystemNavigation methodsFor: 'query' stamp: 'marcus.denker 10/22/2008 14:38'!
allUnsentMessages
	"SystemNavigation new allUnSentMessages"
	"Answer the set of selectors that are implemented by some object in the  
	system but not sent by any."
	^ self allUnSentMessagesWithout: {{}. {}}! !

!SystemNavigation methodsFor: 'query' stamp: 'stephane.ducasse 10/12/2008 21:01'!
allUnsentMessagesIn: selectorSet 
	"Answer the subset of selectorSet which are not sent anywhere in the 
	system. "
	^ selectorSet copyWithoutAll: self allSentMessages! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:58'!
allUnsentMessagesWithProgressBar
	| unsentMessages |
	unsentMessages := Set new.
	self
		doWithProgressBarForAllUnsentMessages: [:class :selector| 
			unsentMessages add: (MethodReference class: class selector: selector)].
	^unsentMessages! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/29/2003 19:06'!
allUnusedClassesWithout: classesAndMessagesPair 
	"Enumerates all classes in the system and returns a list of those that are 
	apparently unused. A class is considered in use if it (a) has subclasses  
	or (b) is referred to by some method or (c) has its name in use as a  
	literal."
	"SystemNavigation new unusedClasses"

	| unused cl |
	unused := Smalltalk classNames asIdentitySet
				copyWithoutAll: (self allGlobalRefsWithout: classesAndMessagesPair).
	^ unused
		reject: [:cName | 
			cl := Smalltalk at: cName.
			cl subclasses isEmpty not
				or: [cl inheritsFrom: FileDirectory]]! !

!SystemNavigation methodsFor: 'browse' stamp: 'md 8/27/2005 16:42'!
browseAllAccessesTo: instVarName from: aClass
	"Create and schedule a Message Set browser for all the receiver's methods 
	or any methods of a subclass/superclass that refer to the instance variable name."

	"self new browseAllAccessesTo: 'contents' from: Collection."
	
	| coll |
	coll := OrderedCollection new.
	Cursor wait showWhile: [
		aClass withAllSubAndSuperclassesDo: [:class | 
			(class whichSelectorsAccess: instVarName) do: [:sel |
				sel isDoIt ifFalse: [
					coll add: (
						MethodReference new
							setStandardClass: class 
							methodSymbol: sel
					)
				]
			]
		].
	].
	^ self 
		browseMessageList: coll 
		name: 'Accesses to ' , instVarName 
		autoSelect: instVarName! !

!SystemNavigation methodsFor: 'browse' stamp: 'davidroethlisberger 2/11/2009 12:07'!
browseAllCallsOn: aLiteral 
	"Create and schedule a message browser on each method that refers to 
	aLiteral. For example, SystemNavigation new browseAllCallsOn: #open:label:."
	(aLiteral isKindOf: LookupKey)
		ifTrue: [^ self
				browseMessageList: (self allCallsOn: aLiteral) asSortedCollection
				name: 'Users of ' , aLiteral key
				autoSelect: aLiteral key].
	self
		browseSendersOf: aLiteral
		name: 'Senders of ' , aLiteral
		autoSelect: aLiteral keywords first! !

!SystemNavigation methodsFor: 'browse' stamp: 'nk 6/26/2003 22:32'!
browseAllCallsOn: literal1 and: literal2 
	"Create and schedule a message browser on each method that calls on the 
	two Symbols, literal1 and literal2. For example, SystemNavigation new 
	browseAllCallsOn: #at: and: #at:put:."

	^self 
		browseMessageList: (self allCallsOn: literal1 and: literal2)
		name: literal1 printString , ' -and- ' , literal2 printString! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/16/2003 08:53'!
browseAllCallsOn: aSymbol from: aClass
	"Create and schedule a Message Set browser for all the methods that call 
	on aSymbol."

	"self new browseAllCallsOn: #/. from: Number"

	| key label |
	label := (aSymbol isKindOf: LookupKey)
			ifTrue: ['Users of ' , (key := aSymbol key)]
			ifFalse: ['Senders of ' , (key := aSymbol)].
	^ self 
		browseMessageList: (self  allCallsOn: aSymbol from: aClass)
		name: label
		autoSelect: key

	! !

!SystemNavigation methodsFor: 'browse' stamp: 'Alexandre Bergel 11/21/2009 10:34'!
browseAllCallsOn: aLiteral localTo: aClass
	"Create and schedule a message browser on each method in or below the given class that refers to
	aLiteral. For example:
		SystemNavigation default browseAllCallsOn: #open:label:.
		"

	aClass ifNil: [ ^self inform: 'no selected class' ].
	(aLiteral isKindOf: LookupKey)
		ifTrue: [self browseMessageList: (aClass allLocalCallsOn: aLiteral) asSortedCollection
					name: 'Users of ' , aLiteral key, ' local to ', aClass name
					autoSelect: aLiteral key]
		ifFalse: [self browseMessageList: (aClass allLocalCallsOn: aLiteral) asSortedCollection
					name: 'Senders of ' , aLiteral, ' local to ', aClass name
					autoSelect: aLiteral keywords first]! !

!SystemNavigation methodsFor: 'browse' stamp: 'tpr 12/17/2003 16:01'!
browseAllCallsOnClass: aClass
	"Create and schedule a message browser on each method that refers to 
	aClass. For example, SystemNavigation new browseAllCallsOnClass: Object."
	self
		browseMessageList: aClass allCallsOn asSortedCollection
		name: 'Users of class ' , aClass theNonMetaClass name
		autoSelect: aClass theNonMetaClass name! !

!SystemNavigation methodsFor: 'browse' stamp: 'davidroethlisberger 2/11/2009 12:08'!
browseAllImplementorsOf: selector 
	"Create and schedule a message browser on each method that implements 
	the message whose selector is the argument, selector. For example,  
	Smalltalk browseAllImplementorsOf: #at:put:."
	^self browseImplementorsOf: selector name: 'Implementors of ' , selector autoSelect: nil! !

!SystemNavigation methodsFor: 'browse' stamp: 'apl 4/4/2005 18:23'!
browseAllImplementorsOf: selector localTo: aClass 
	"Create and schedule a message browser on each method in or below the  
	given class  
	that implements the message whose selector is the argument, selector.  
	For example,  
	SystemNavigation new browseAllImplementorsOf: #at:put: localTo:  
	Dictionary."
	aClass
		ifNil: [^ self inform: 'no class selected'].
	^ self
		browseMessageList: ((self allImplementorsOf: selector localTo: aClass)
				collect: [:methRef | methRef actualClass name , ' ' , methRef methodSymbol])
		name: 'Implementors of ' , selector , ' local to ' , aClass name! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/15/2003 21:43'!
browseAllImplementorsOfList: selectorList
	"Create and schedule a message browser on each method that implements 
	the message whose selector is in the argument selectorList. For example, 
	Smalltalk browseAllImplementorsOf: #(at:put: size).
	1/16/96 sw: defer to the titled version"

	self browseAllImplementorsOfList: selectorList title: 'Implementors of all'! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/19/2003 12:15'!
browseAllImplementorsOfList: selectorList title: aTitle 
	"Create and schedule a message browser on each method that implements 
	the message whose selector is in the argument selectorList. For 
	example,  
	self new browseAllImplementorsOf: #(at:put: size).  
	1/16/96 sw: this variant adds the title argument.  
	1/24/96 sw: use a SortedCollection  
	2/1/96 sw: show normal cursor"
	| implementorLists flattenedList |
	implementorLists := selectorList
				collect: [:each | self allImplementorsOf: each].
	flattenedList := SortedCollection new.
	implementorLists
		do: [:each | flattenedList addAll: each].
	Cursor normal show.
	^ self browseMessageList: flattenedList name: aTitle! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/16/2003 09:17'!
browseAllMethodsInCategory: category 
	^self browseMessageList: (self allMethodsInCategory: category)
		name: category! !

!SystemNavigation methodsFor: 'browse' stamp: 'Alexandre.Bergel 7/4/2009 11:10'!
browseAllObjectReferencesTo: anObject except: objectsToExclude ifNone: aBlock 
	"Bring up a list inspector on the objects that point to anObject.
	If there are none, then evaluate aBlock on anObject.  "

	| aList shortName |
	aList := PointerFinder pointersTo: anObject except: objectsToExclude.
	aList size > 0 ifFalse: [^aBlock value: anObject].
	shortName := (anObject name ifNil: [anObject printString]) contractTo: 20.
	aList inspectWithLabel: 'Objects pointing to ' , shortName! !

!SystemNavigation methodsFor: 'browse' stamp: 'ar 9/7/2003 18:05'!
browseAllReferencesToPool: poolOrName from: aClass
	"Open a message list on all messages referencing the given pool"
	| pool list |
	(poolOrName isString)
		ifTrue:[pool := Smalltalk at: poolOrName asSymbol]
		ifFalse:[pool := poolOrName].
	list := self allReferencesToPool: pool from: aClass.
	self
		browseMessageList: list
		name: 'users of ', poolOrName name.
	^list! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/29/2003 20:43'!
browseAllSelect: aBlock 
	"Create and schedule a message browser on each method that, when used 
	as the block argument to aBlock gives a true result. For example,  
	SystemNavigation new browseAllSelect: [:method | method numLiterals >  
	10]."
	^ self
		browseMessageList: (self allMethodsSelect: aBlock)
		name: 'selected messages'! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/29/2003 20:44'!
browseAllSelect: aBlock name: aName autoSelect: autoSelectString 
	"Create and schedule a message browser on each method that, when used 
	as the block argument to aBlock gives a true result. Do not return an  
	#DoIt traces."
	"self new browseAllSelect: [:method | method numLiterals > 10] name:  
	'Methods with more than 10 literals' autoSelect: 'isDigit'"
	^ self
		browseMessageList: (self allMethodsNoDoitsSelect: aBlock)
		name: aName
		autoSelect: autoSelectString! !

!SystemNavigation methodsFor: 'browse' stamp: 'md 8/27/2005 17:17'!
browseAllStoresInto: instVarName from: aClass
	"Create and schedule a Message Set browser for all the receiver's methods 
	or any methods of a subclass/superclass that refer to the instance variable name."
	
	"self new browseAllStoresInto: 'contents' from: Collection."

	| coll |
	coll := OrderedCollection new.
	Cursor wait showWhile: [
		aClass withAllSubAndSuperclassesDo: [:class | 
			(class whichSelectorsStoreInto: instVarName) do: [:sel |
				sel isDoIt ifFalse: [
					coll add: (
						MethodReference new
							setStandardClass: class 
							methodSymbol: sel
					)
				]
			]
		].
	].
	^ self
		browseMessageList: coll 
		name: 'Stores into ' , instVarName 
		autoSelect: instVarName! !

!SystemNavigation methodsFor: 'browse' stamp: 'marcus.denker 9/14/2008 18:53'!
browseAllUnimplementedCalls
	"Create and schedule a message browser on each method that includes a 
	message that is not implemented in any object in the system."

	"self new browseAllUnimplementedCalls"

	^self browseMessageList: self allUnimplementedCalls name: 'Unimplemented calls'! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:57'!
browseAllUnsentMessages
	"SystemNavigation default browseUnsentMessages"
	| unsentMessages |
	unsentMessages := self allUnsentMessagesWithProgressBar.
	^self 
		browseMessageList: unsentMessages asSortedCollection
		name: 'All unsent messages' 
 ! !

!SystemNavigation methodsFor: 'browse' stamp: 'ar 9/27/2005 20:34'!
browseClass: aBehavior
	| targetClass |
	targetClass := aBehavior isMeta
				ifTrue: [aBehavior theNonMetaClass]
				ifFalse: [aBehavior ].
	ToolSet browse: targetClass selector: nil! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/16/2003 08:53'!
browseClassCommentsWithString: aString
	"Smalltalk browseClassCommentsWithString: 'my instances' "
	"Launch a message list browser on all class comments containing aString as a substring."

	| caseSensitive suffix list |

	suffix := (caseSensitive := Sensor shiftPressed)
		ifTrue: [' (case-sensitive)']
		ifFalse: [' (use shift for case-sensitive)'].
	list := Set new.
	Cursor wait showWhile: [
		Smalltalk allClassesDo: [:class | 
			(class organization classComment asString findString: aString 
							startingAt: 1 caseSensitive: caseSensitive) > 0 ifTrue: [
								list add: (
									MethodReference new
										setStandardClass: class
										methodSymbol: #Comment
								)
							]
		]
	].
	^ self 
		browseMessageList: list asSortedCollection
		name: 'Class comments containing ' , aString printString , suffix
		autoSelect: aString! !

!SystemNavigation methodsFor: 'browse' stamp: 'PeterHugossonMiller 9/3/2009 11:31'!
browseClassVarRefs: aClass
	"Put up a menu offering all class variable names; if the user chooses one, open up a message-list browser on all methods 
	that refer to the selected class variable"

	| lines labelStream vars allVars index owningClasses |
	lines := OrderedCollection new.
	allVars := OrderedCollection new.
	owningClasses := OrderedCollection new.
	labelStream := (String new: 200) writeStream.
	aClass withAllSuperclasses reverseDo:
		[:class |
		vars := class classVarNames asSortedCollection.
		vars do:
			[:var |
			labelStream nextPutAll: var; cr.
			allVars add: var.
			owningClasses add: class].
		vars isEmpty ifFalse: [lines add: allVars size]].
	labelStream contents isEmpty ifTrue: [^Beeper beep]. "handle nil superclass better"
	labelStream skip: -1 "cut last CR".
	index := (UIManager default chooseFrom: (labelStream contents substrings) lines: lines).
	index = 0 ifTrue: [^ self].
	self browseAllCallsOn:
		((owningClasses at: index) classPool associationAt: (allVars at: index))! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 3/28/2003 18:49'!
browseClassVariables: aClass
	aClass classPool inspectWithLabel: 'Class Variables in ' , aClass name! !

!SystemNavigation methodsFor: 'browse' stamp: 'ar 9/27/2005 20:33'!
browseClassesWithNamesContaining: aString caseSensitive: caseSensitive 
	"Smalltalk browseClassesWithNamesContaining: 'eMorph' caseSensitive: true "
	"Launch a class-list list browser on all classes whose names containg aString as a substring."

	| suffix aList |
	suffix := caseSensitive
				ifTrue: [' (case-sensitive)']
				ifFalse: [' (use shift for case-sensitive)'].
	aList := OrderedCollection new.
	Cursor wait
		showWhile: [Smalltalk
				allClassesDo: [:class | (class name includesSubstring: aString caseSensitive: caseSensitive)
						ifTrue: [aList add: class name]]].
	aList size > 0
		ifTrue: [ToolSet openClassListBrowser: aList asSet asSortedArray title: 'Classes whose names contain ' , aString , suffix]! !

!SystemNavigation methodsFor: 'browse' stamp: 'ar 9/27/2005 20:34'!
browseHierarchy: aBehavior
	| targetClass |
	targetClass := aBehavior isMeta
				ifTrue: [aBehavior theNonMetaClass]
				ifFalse: [aBehavior ].
	ToolSet browseHierarchy: targetClass selector: nil.! !

!SystemNavigation methodsFor: 'browse' stamp: 'davidroethlisberger 2/11/2009 12:09'!
browseImplementorsOf: aSelector name: labelString autoSelect: autoSelectString
	"Create and schedule a senders browser for aSelector."	
	| implementors title size |
	
	implementors := self allImplementorsOf: aSelector.
	implementors size = 0 ifTrue: 
		[^ self inform: 'There are no ' , labelString].
	
	title := (size := implementors size) > 1
		ifFalse:	[labelString]
		ifTrue:	[ labelString, ' [', size printString, ']'].
	
	ToolSet 
		browseImplementorsOf: aSelector
		name: title
		autoSelect: autoSelectString! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/15/2003 16:08'!
browseInstVarDefs: aClass
	"Copied from browseInstVarRefs.  Should be consolidated some day. 7/29/96 di
	7/30/96 sw: did the consolidation"
	"Change to use SystemNavigation  27 March 2003 sd"

	aClass chooseInstVarThenDo:	
		[:aVar | self browseAllStoresInto: aVar from: aClass]! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/15/2003 16:08'!
browseInstVarRefs: aClass
	"1/16/96 sw: moved here from Browser so that it could be used from a variety of places.
	 7/30/96 sw: call chooseInstVarThenDo: to get the inst var choice"

	aClass chooseInstVarThenDo: 
		[:aVar | self browseAllAccessesTo: aVar from: aClass]! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/16/2003 09:18'!
browseMessageList: messageList name: label 
	"Create and schedule a MessageSet browser on messageList."
	^ self   
		browseMessageList: messageList 
		name: label 
		autoSelect: nil! !

!SystemNavigation methodsFor: 'browse' stamp: 'ar 9/27/2005 20:34'!
browseMessageList: messageList name: labelString autoSelect: autoSelectString
	| title aSize |
	"Create and schedule a MessageSet browser on the message list."

	messageList size = 0 ifTrue: 
		[^ self inform: 'There are no
' , labelString].

	title := (aSize := messageList size) > 1
		ifFalse:	[labelString]
		ifTrue:	[ labelString, ' [', aSize printString, ']'].

	ToolSet 
		browseMessageSet: messageList 
		name: title 
		autoSelect: autoSelectString! !

!SystemNavigation methodsFor: 'browse' stamp: 'stephane.ducasse 5/23/2009 14:01'!
browseMethodsWhoseNamesContain: aString
	"Launch a tool which shows all methods whose names contain the given 	string; case-insensitive."
	ToolSet browseMessageNames: aString! !

!SystemNavigation methodsFor: 'browse' stamp: 'yo 7/31/2004 18:40'!
browseMethodsWithLiteral: aString
	"Launch a browser on all methods that contain string literals with aString as a substring. Make the search case-sensitive or insensitive as dictated by the caseSensitive boolean parameter"

	self browseAllSelect:
			[:method |
				method hasLiteralSuchThat: [:lit |
					(lit isString and: [lit isSymbol not]) and:
					[lit = aString]]]
		name:  'Methods with string ', aString printString
		autoSelect: aString.
! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/20/2003 14:11'!
browseMethodsWithSourceString: aString 
	"SystemNavigation new browseMethodsWithSourceString: 'SourceString'"
	"Launch a browser on all methods whose source code contains aString as 
	a substring."
	| caseSensitive suffix |
	suffix := (caseSensitive := Sensor shiftPressed)
				ifTrue: [' (case-sensitive)']
				ifFalse: [' (use shift for case-sensitive)'].
	^ self
		browseMessageList: (self allMethodsWithSourceString: aString matchCase: caseSensitive)
		name: 'Methods containing ' , aString printString , suffix
		autoSelect: aString! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/15/2003 22:28'!
browseMethodsWithString: aString
	"Launch a browser on all methods that contain string literals with aString as a substring. The search is case-insensitive, unless the shift key is pressed, in which case the search is case-sensitive."

	'string for testing'.
	^ self browseMethodsWithString: aString matchCase: Sensor shiftPressed

	"SystemNavigation new browseMethodsWithString: 'Testing' matchCase: false"
	"SystemNavigation new browseMethodsWithString: 'Testing' matchCase: true"! !

!SystemNavigation methodsFor: 'browse' stamp: 'bf 4/26/2005 11:44'!
browseMethodsWithString: aString matchCase: caseSensitive
	"Launch a browser on all methods that contain string literals with aString as a substring. Make the search case-sensitive or insensitive as dictated by the caseSensitive boolean parameter"

	self browseAllSelect:
			[:method |
				method  hasLiteralSuchThat: [:lit |
					(lit isString and: [lit isSymbol not]) and:
					[lit includesSubstring: aString caseSensitive: caseSensitive]]]
		name:  'Methods with string ', aString printString, (caseSensitive ifTrue: [' (case-sensitive)'] ifFalse: [' (case-insensitive)'])
		autoSelect: aString.
! !

!SystemNavigation methodsFor: 'browse' stamp: 'ar 9/27/2005 20:34'!
browseObsoleteMethodReferences
	"Open a browser on all referenced behaviors that are obsolete"

	"SystemNavigation default browseObsoleteMethodReferences"

	| list |
	list := self obsoleteMethodReferences.
	self 
		browseMessageList: list
		name: 'Method referencing obsoletes'
		autoSelect: nil! !

!SystemNavigation methodsFor: 'browse' stamp: 'sd 4/15/2003 20:32'!
browseObsoleteReferences  
	"self new browseObsoleteReferences"

	| references |
	references := OrderedCollection new.
	(LookupKey allSubInstances select:
		[:x | ((x value isKindOf: Behavior) and: ['AnOb*' match: x value name]) or:
		['AnOb*' match: x value class name]]) 
		do: [:x | references addAll: (self allCallsOn: x)].
	self  
		browseMessageList: references 
		name: 'References to Obsolete Classes'! !

!SystemNavigation methodsFor: 'browse' stamp: 'davidroethlisberger 2/11/2009 12:08'!
browseSendersOf: aSelector name: labelString autoSelect: autoSelectString
	| title size senders |
	"Create and schedule a senders browser for aSelector."
	
	senders := self allCallsOn: aSelector.
	senders size = 0 ifTrue: 
		[^ self inform: 'There are no ' , labelString].

	title := (size := senders size) > 1
		ifFalse:	[labelString]
		ifTrue:	[labelString, ' [', size printString, ']'].

	ToolSet 
		browseSendersOf: aSelector
		name: title 
		autoSelect: autoSelectString! !

!SystemNavigation methodsFor: 'browse' stamp: 'ar 9/27/2005 20:34'!
browseUncommentedMethodsWithInitials: targetInitials
	"Browse uncommented methods whose initials (in the time-stamp, as logged to disk) match the given initials.  Present them in chronological order.  CAUTION: It will take several minutes for this to complete."
	"Time millisecondsToRun: [SystemNavigation default browseUncommentedMethodsWithInitials: 'jm']"

	| initials timeStamp methodReferences cm |
	methodReferences := OrderedCollection new.
	self  allBehaviorsDo:
		[:aClass | aClass selectors do: [:sel |
			cm := aClass compiledMethodAt: sel.
			timeStamp := Utilities timeStampForMethod: cm.
			timeStamp isEmpty ifFalse:
				[initials := timeStamp substrings first.
				initials first isDigit ifFalse:
					[((initials = targetInitials) and: [(aClass firstPrecodeCommentFor: sel) isNil])
						ifTrue:
							[methodReferences add: (MethodReference new
								setStandardClass: aClass 
								methodSymbol: sel)]]]]].

	ToolSet
		browseMessageSet: methodReferences 
		name: 'Uncommented methods with initials ', targetInitials
		autoSelect: nil! !

!SystemNavigation methodsFor: 'browse' stamp: 'stephane.ducasse 8/8/2009 12:51'!
browseUndeclaredReferences
	"from Cuis: 0058-browseUndeclared.1"
	"SystemNavigation default"
	
	Undeclared removeUnreferencedKeys.
	Undeclared keys do: [ :k |
		self
			browseMessageList: (self allCallsOn: (Undeclared associationAt: k))
			name: 'References to Undeclared: ', k printString ]! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:59'!
browseUnsentMessagesInClass: aClass
	"SystemNavigation default browseUnsentMessagesWithProgressBarInClass: BlockContext"
	| unsentMessages |
	unsentMessages := self unsentMessagesWithProgressBarInClass: aClass.
	^self 
		browseMessageList: unsentMessages asSortedCollection
		name: 'Unsent messages in class ', aClass name 
 ! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:59'!
browseUnsentMessagesInPackageNamed: packageName
	"SystemNavigation default browseUnsentMessagesWithProgressBarInPackageNamed: 'Kernel-Contexts'"
	| unsentMessages |
	unsentMessages := self unsentMessagesWithProgressBarInPackageNamed: packageName.
	^self 
		browseMessageList: unsentMessages asSortedCollection
		name: 'Unsent messages in package ', packageName 
 ! !

!SystemNavigation methodsFor: 'accessing' stamp: 'Noury 10/26/2008 17:12'!
categoriesInPackageNamed: packageName
	^(SystemOrganization categoriesMatching: packageName), (SystemOrganization categoriesMatching: packageName, '*')! !

!SystemNavigation methodsFor: 'ui' stamp: 'AndrewBlack 9/1/2009 15:45'!
classFromPattern: pattern withCaption: aCaption
	"If there is a class whose name exactly given by pattern, return it.
	If there is only one class in the system whose name matches pattern, return it.
	Otherwise, put up a menu offering the names of all classes that match pattern, and return the class chosen, else nil if nothing chosen.
	This method ignores tab, space, & cr characters in the pattern"

	| toMatch potentialClassNames classNames exactMatch index |
	(toMatch :=  pattern copyWithoutAll:
			{Character space.  Character cr.  Character tab})
		isEmpty ifTrue: [^ nil].
	Symbol hasInterned: toMatch ifTrue:
		[:patternSymbol | Smalltalk at: patternSymbol ifPresent:
			[:maybeClass | ((maybeClass isKindOf: Class) or: [maybeClass isKindOf: Trait]) 
					ifTrue: [^ maybeClass]]].

	toMatch := (toMatch copyWithout: $.) asLowercase.
	potentialClassNames := (Smalltalk classNames , Smalltalk traitNames) asOrderedCollection.
	classNames := pattern last = $. 
		ifTrue: [potentialClassNames select:
					[:nm |  nm asLowercase = toMatch]]
		ifFalse: [potentialClassNames select: 
					[:n | n includesSubstring: toMatch caseSensitive: false]].
	classNames isEmpty ifTrue: [^ nil].
	exactMatch := classNames detect: [:each | each asLowercase = toMatch] ifNone: [nil].

	index := classNames size = 1
		ifTrue:	[1]
		ifFalse:	[exactMatch
			ifNil: [UIManager default chooseFrom: classNames lines: #() title: aCaption]
			ifNotNil: [classNames addFirst: exactMatch.
				UIManager default chooseFrom: classNames lines: #(1) title: aCaption]].
	index = 0 ifTrue: [^ nil].
	^ Smalltalk at: (classNames at: index) asSymbol

"
	self classFromPattern: 'znak' withCaption: ''
	self classFromPattern: 'orph' withCaption: ''
	self classFromPattern: 'TCompil' withCaption: ''
"
! !

!SystemNavigation methodsFor: 'ui' stamp: 'rbb 2/18/2005 14:48'!
confirmRemovalOf: aSelector on: aClass 
	"Determine if it is okay to remove the given selector. Answer 1 if it  
	should be removed, 2 if it should be removed followed by a senders  
	browse, and 3 if it should not be removed."
	| count answer caption allCalls |
	allCalls := self allCallsOn: aSelector.
	(count := allCalls size) == 0
		ifTrue: [^ 1].
	"no senders -- let the removal happen without warning"
	count == 1
		ifTrue: [(allCalls first actualClass == aClass
					and: [allCalls first methodSymbol == aSelector])
				ifTrue: [^ 1]].
	"only sender is itself"
	caption := 'This message has ' , count printString , ' sender'.
	count > 1
		ifTrue: [caption := caption copyWith: $s].
	answer := UIManager default 
		chooseFrom: #('Remove it'
				'Remove, then browse senders'
				'Don''t remove, but show me those senders'
				'Forget it -- do nothing -- sorry I asked') title: caption.
	answer == 3
		ifTrue: [self
				browseMessageList: allCalls
				name: 'Senders of ' , aSelector
				autoSelect: aSelector keywords first].
	answer == 0
		ifTrue: [answer := 3].
	"If user didn't answer, treat it as cancel"
	^ answer min: 3! !

!SystemNavigation methodsFor: 'author' stamp: 'stephane.ducasse 8/8/2009 15:14'!
contributionsOf: aString
	"SystemNavigation default contributionsOf: 'alain.plantec'"
	
	| stamp initials answer |
	answer := OrderedCollection new.
	self allBehaviorsDo: [ :behavior |
		behavior methodsDo: [ :compiledMethod |
			stamp := compiledMethod timeStamp.
			stamp notEmpty ifTrue: [
				initials := compiledMethod timeStamp substrings first.
				aString = initials ifTrue: [
					answer add: (compiledMethod selector -> compiledMethod methodClass)]]]].
	^answer! !

!SystemNavigation methodsFor: 'author' stamp: 'stephane.ducasse 8/8/2009 15:16'!
contributorsNotSignatories
	"SystemNavigation default contributorsNotSignatories"
	
	| stamp signatories initials answer |
	answer := Dictionary new.
	signatories := self signatories.
	self allBehaviorsDo: [ :behavior |
		behavior methodsDo: [ :compiledMethod |
			stamp := compiledMethod timeStamp.
			stamp notEmpty ifTrue: [
				initials :=  compiledMethod timeStamp substrings first.
				(signatories includes: initials) ifFalse: [
					(answer at: initials ifAbsentPut: [OrderedCollection new])
						add: (compiledMethod selector -> compiledMethod methodClass)]]]].
	^answer! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:37'!
doWithProgressBar: aBlock forUnsentMessagesInClass: class
	| progressMessages selectors |
	progressMessages := 'Unsent messages in class ', class name.
	selectors := class selectors asArray.
	progressMessages
		displayProgressAt: Display center
		from: 0 to: selectors size
		during: [:bar | 
			selectors with: (0 to: selectors size - 1) do: [:selector :index|
				bar value: index.
				(self isUnsentMessage: selector) ifTrue: [aBlock value: class value: selector] ]]
! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:37'!
doWithProgressBar: aBlock forUnsentMessagesInPackageNamed: packageName
	|progressMessages classes |
	Cursor wait showWhile: [classes := self allClassesInPackageNamed: packageName].
	progressMessages := 'Unsent messages in package ', packageName.
	progressMessages
		displayProgressAt: Display center
		from: 0 to: classes size
		during: [:bar | 
			classes with: (0 to: classes size - 1) do: [:class :index|
				bar value: index.
				self doWithProgressBar: aBlock forUnsentMessagesInClass: class]]! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:55'!
doWithProgressBarForAllUnsentMessages: aBlock
	|progressMessages classes |
	Cursor wait showWhile: [classes := self allClasses].
	progressMessages := 'Unsent messages in all classes'.
	progressMessages
		displayProgressAt: Display center
		from: 0 to: classes size
		during: [:bar | 
			classes with: (0 to: classes size - 1) do: [:class :index|
				bar value: index.
				self doWithProgressBar: aBlock forUnsentMessagesInClass: class]]! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 1/16/2004 21:01'!
hierarchyOfClassesSurrounding: aClass
	"Answer a list of classes in the hierarchy both above and below the given class"
	"SystemNavigation default hierarchyOfClassesSurrounding: StringHolder"
	
	| list aClassNonMeta isMeta theClassOrMeta |
	aClass ifNil: [^ OrderedCollection new].
	aClass ifNil: [^ self].
	aClassNonMeta := aClass theNonMetaClass.
	isMeta := aClassNonMeta ~~ aClass.
	list := OrderedCollection new.
	aClass allSuperclasses reverseDo:
		[:cl | list addLast: cl].
	aClassNonMeta allSubclassesWithLevelDo:
		[:cl :level |
		theClassOrMeta := isMeta ifTrue: [cl class] ifFalse: [cl].
		list addLast: theClassOrMeta]
	 	startingLevel: 0.
	^ list

! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 1/16/2004 21:03'!
hierarchyOfImplementorsOf: aSelector forClass: aClass
	"Answer a list of classes in the hierarchy both above and below the given class which implement the given selector."
	"SystemNavigation default hierarchyOfImplementorsOf: #contents forClass: StringHolder"

	^ (self hierarchyOfClassesSurrounding: aClass) select:
		[:cl | cl includesSelector: aSelector]
! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 15:30'!
isMessage: selector sentInClass: aClass 
	^(self allSendersOf: selector inClass: aClass) notEmpty ! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 15:57'!
isMessage: selector sentInClassCategory: category 
	^(self allSendersOf: selector inClassCategory: category) notEmpty ! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 16:00'!
isMessage: selector sentInPackageNamed: packageName 
	^(self allSendersOf: selector inPackageNamed: packageName) notEmpty ! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/18/2003 10:44'!
isThereAnImplementorOf: aSelector 
	"Answer true if there is at least one implementor of the selector found  
	in the system, false if there are no implementors"
	"self new isThereAnImplementorOf: #contents.  
	self new isThereAnImplementorOf: #nobodyImplementsThis."
	self 
		allBehaviorsDo: [:class | (class includesSelector: aSelector)
				ifTrue: [^ true]].
	^ false! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 16:57'!
isUnsentMessage: selector
	^(self allSendersOf: selector) isEmpty ! !

!SystemNavigation methodsFor: 'browse' stamp: 'al 12/7/2005 21:11'!
methodHierarchyBrowserForClass: aClass selector: sel
	"Create and schedule a message set browser on all implementors of the 
	currently selected message selector. Do nothing if no message is selected."
	"SystemNavigation default 
		methodHierarchyBrowserForClass: ParagraphEditor 
		selector: #isControlActive"
	
	| list tab stab aClassNonMeta isMeta theClassOrMeta |
	aClass ifNil: [^ self].
	aClass isTrait ifTrue: [^ self].
	sel ifNil: [^ self].
	aClassNonMeta := aClass theNonMetaClass.
	isMeta := aClassNonMeta ~~ aClass.
	list := OrderedCollection new.
	tab := ''.
	aClass allSuperclasses reverseDo:
		[:cl |
		(cl includesSelector: sel) ifTrue:
			[list addLast: tab , cl name, ' ', sel].
		tab := tab , '  '].
	aClassNonMeta allSubclassesWithLevelDo:
		[:cl :level |
		theClassOrMeta := isMeta ifTrue: [cl class] ifFalse: [cl].
		(theClassOrMeta includesSelector: sel) ifTrue:
			[stab := ''.  1 to: level do: [:i | stab := stab , '  '].
			list addLast: tab , stab , theClassOrMeta name, ' ', sel]]
	 	startingLevel: 0.
	self browseMessageList: list name: 'Inheritance of ' , sel

! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/20/2003 14:27'!
numberOfImplementorsOf: aSelector 
	"Answer a count of the implementors of the given selector found in the  
	system"
	"self new numberOfImplementorsOf: #contents.  
	self new numberOfImplementorsOf: #nobodyImplementsThis.  
	self new numberOfimplementorsOf: #numberOfImplementorsOf:."
	| aCount |
	aCount := 0.
	self
		allBehaviorsDo: [:class | (class includesSelector: aSelector)
				ifTrue: [aCount := aCount + 1]].
	^ aCount! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 9/23/2004 22:03'!
obsoleteBehaviors
	"SystemNavigation default obsoleteBehaviors inspect"
	"Find all obsolete behaviors including meta classes"

	| obs |
	obs := OrderedCollection new.
	Smalltalk garbageCollect.
	self 
		allObjectsDo: [:cl | (cl isBehavior
					and: [cl isObsolete])
				ifTrue: [obs add: cl]].
	^ obs asArray! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 9/23/2004 22:06'!
obsoleteClasses   

	"SystemNavigation default obsoleteClasses inspect"
	"NOTE:  Also try inspecting comments below"
	| obs c |
	obs := OrderedCollection new.  Smalltalk garbageCollect.
	Metaclass allInstancesDo:
		[:m | c := m soleInstance.
		(c ~~ nil and: ['AnOb*' match: c name asString])
			ifTrue: [obs add: c]].
	^ obs asArray

"Likely in a ClassDict or Pool...
(Association allInstances select: [:a | (a value isKindOf: Class) and: ['AnOb*' match: a value name]]) asArray
"
"Obsolete class refs or super pointer in last lit of a method...
| n l found |
Smalltalk browseAllSelect:
	[:m | found := false.
	1 to: m numLiterals do:
		[:i | (((l := m literalAt: i) isMemberOf: Association)
				and: [(l value isKindOf: Behavior)
				and: ['AnOb*' match: l value name]])
			ifTrue: [found := true]].
	found]
"! !

!SystemNavigation methodsFor: 'query' stamp: 'PeterHugossonMiller 9/3/2009 11:32'!
obsoleteMethodReferences
	"SystemNavigation default obsoleteMethodReferences"

	"Open a browser on all referenced behaviors that are obsolete"

	| obsClasses obsRefs references |
	references := Array new writeStream.
	obsClasses := self obsoleteBehaviors.
	'Scanning for methods referencing obsolete classes' 
		displayProgressAt: Sensor cursorPoint
		from: 1
		to: obsClasses size
		during: 
			[:bar | 
			obsClasses keysAndValuesDo: 
					[:index :each | 
					bar value: index.
					obsRefs := PointerFinder pointersTo: each except: obsClasses.
					obsRefs do: 
							[:ref | 
							"Figure out if it may be a global"

							(ref isVariableBinding and: [ref key isString	"or Symbol"]) 
								ifTrue: 
									[(Utilities pointersTo: ref) do: 
											[:meth | 
											(meth isKindOf: CompiledMethod) 
												ifTrue: [meth methodReference ifNotNil: [:mref | references nextPut: mref]]]]]]].
	^references contents! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:49'!
removeUnsentMessagesWithProgressBarInClass: aClass
	self
		doWithProgressBar: [:class :selector| 
			class remove: selector] 
		forUnsentMessagesInClass: aClass! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:48'!
removeUnsentMessagesWithProgressBarInPackageNamed: packageName
	self
		doWithProgressBar: [:class :selector| 
			class removeSelector: selector] 
		forUnsentMessagesInPackageNamed: packageName.
! !

!SystemNavigation methodsFor: 'query' stamp: 'stephane.ducasse 7/10/2009 16:07'!
reportSenderCountsFor: selectorList 
	"Produce a report on the number of senders of each of the selectors in 
	the list. 1/27/96 sw"
	"SystemNavigation default reportSenderCountsFor: #(foo: printString)"
	| total report thisSize |
	total := 0.
	report := '
'.
	selectorList
		do: [:selector | 
			thisSize := (self allCallsOn: selector) size.
			report := report , thisSize printString , String tab , selector printString , String cr.
			total := total + thisSize].
	report := report , '--- ------------------
'.
	report := report , total printString , String tab , 'TOTAL
'.
	^ report! !

!SystemNavigation methodsFor: 'query' stamp: 'sd 4/29/2003 15:17'!
selectAllMethods: aBlock 
	"Answer a SortedCollection of each method that, when used as the block  
	argument to aBlock, gives a true result."
	| aCollection |
	aCollection := SortedCollection new.
	Cursor execute
		showWhile: [self
				allBehaviorsDo: [:class | class
						selectorsDo: [:sel | (aBlock
									value: (class compiledMethodAt: sel))
								ifTrue: [aCollection
										add: (MethodReference new setStandardClass: class methodSymbol: sel)]]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'md 8/27/2005 17:17'!
selectAllMethodsNoDoits: aBlock 
	"Like allSelect:, but strip out Doits"
	| aCollection |
	aCollection := SortedCollection new.
	Cursor execute
		showWhile: [self
				allBehaviorsDo: [:class | class
						selectorsDo: [:sel | (sel isDoIt not
									and: [aBlock
											value: (class compiledMethodAt: sel)])
								ifTrue: [aCollection
										add: (MethodReference new setStandardClass: class methodSymbol: sel)]]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'ui' stamp: 'sd 4/15/2003 15:34'!
showMenuOf: selectorCollection withFirstItem: firstItem ifChosenDo: choiceBlock
	"Show a sorted menu of the given selectors, preceded by firstItem, and all
	abbreviated to 40 characters.  Evaluate choiceBlock if a message is chosen."

	^ self showMenuOf: selectorCollection withFirstItem: firstItem ifChosenDo: choiceBlock withCaption: nil! !

!SystemNavigation methodsFor: 'ui' stamp: 'edc 11/8/2005 10:58'!
showMenuOf: selectorCollection withFirstItem: firstItem ifChosenDo: choiceBlock withCaption: aCaption
	"Show a sorted menu of the given selectors, preceded by firstItem, and all abbreviated to 40 characters.  Use aCaption as the menu title, if it is not nil.  Evaluate choiceBlock if a message is chosen."

	| index menuLabels sortedList |
	sortedList := selectorCollection asSortedCollection.
	menuLabels := String streamContents: 
		[:strm | strm nextPutAll: (firstItem contractTo: 40).
		sortedList do: [:sel | strm cr; nextPutAll: (sel contractTo: 40)]].
	index := UIManager default chooseFrom: (menuLabels substrings) lines: #(1).
	index = 1 ifTrue: [choiceBlock value: firstItem].
	index > 1 ifTrue: [choiceBlock value: (sortedList at: index - 1)]! !

!SystemNavigation methodsFor: 'author' stamp: 'stephane.ducasse 8/8/2009 15:08'!
signatories
	^(self signatoriesString subStrings: {Character cr})
		collect: [ :each | each substrings first]! !

!SystemNavigation methodsFor: 'author' stamp: 'stephane.ducasse 8/8/2009 15:08'!
signatoriesString
	^'AB               Alexandre Bergel
ack              Alan Kay
ads              Adam Spitz
ajh              Anthony Hannan
aka              Mark Guzdial
al               Adrian Lienhard
aoy              Andres Otaduy
apb              Andrew P. Black
apl              Alain Plantec
ar               Andreas Raab
asm              Alejandro Magistrello
avi              Avi Bryant
bf               Bert Freudenberg
BG               Boris Gaertner
bh               Bob Hartwig
BJP              Bijan Parsia
bkv              Brent Vukmer
bolot            Bolot Kerimbaev
bootstrap        Pavel Krivanek
BP               Bijan Parsia
brp              Brent Pinkney
btr              Brian Rice
cbc              Chris Cunningham
ccn              Chris Norton
CdG              Cees de Groot
cds              C. David Shaffer
cmm              Chris Muller
crl              Craig Latta
cwp              Colin Putney
daf              Dave Faught
dao              danil osipchuk
DAS              David A Smith
dc               Damien Cassou
dd               Dominique Dutoit
dew              Doug Way
DF               Diego Fernandez
dgd              Diego Gomez Deck
dhhi             Dan Ingalls
di               Dan Ingalls
DM               Duane Maxwell
DSM              Duane Maxwell
dtl              Dave Lewis
dvf              Daniel Vainsencher
edc              Edgar DeCleene
efc              Eddie Cottongim
efo              Emilio Oca
em               Ernest Micklei?
emm              Ernest Micklei
es               Enrico Spinielli
FBS              Frank Shearar
fc               Frank Caggiano
fcs              Frank Sergeant
fm               Florin Mateoc
gh               Goran Krampe (nee Hultgren)
gk               Goran Krampe (nee Hultgren)
gm               German Morales
go               Georg Gollmann
gsa              German Arduino
HEG              Henrik Ekenberg
HilaireFernandes Hilaire Fernandes
HK               Herbert Konig
hmm              Hans-Martin Mosner
hpt              Hernan Tylim
huma             Lyndon Tremblay
ich.             Yuji Ichikawa
ikp              Ian Piumarta
jaf              Jan Fietz
jam              Javier Musa
jcg              Joshua Gargus
jdf              David Farber
jdr              Javier Diaz-Reinoso
je               Jorn Eyrich
je77             Jochen Rick
JF               Julian Fitzell
jhm              John Maloney
jlb              Jim Benson
jmb              Hans Baveco
JMM              John McIntosh
JMV              Juan Manuel Vuletich
jon              Jon Hylands
jrm              John-Reed Maffeo
jrp              John Pierce
jsp              Jeff Pierce
ka               Kazuhiro Abe
kfr              Karl Ramberg
KLC              Ken Causey
KR               korakurider
KTT              Kurt Thams
kwl              Klaus D. Witzel
ky               Koji Yokokawa
laza             Alexander Lazarevic
LB               Leo Burd
LC               Leandro Caniglia
LEG              Gerald Leeb
len              Luciano Esteban Notarfrancesco
lr               Lukas Renggli
lrs              Lorenzo Schiavina
ls               Lex Spoon
m3r              Maurice Rabb
mas              Mark Schwenk
MD               Markus Denker
mdr              Mike Rutenberg
mga              Markus Galli
miki             Mikael Kindborg
mikki            Mikael Kindborg
mir              Michael Rueger
mist             Michal Starke
mjg              Mark Guzdial
mjr              Mike Roberts
mjt              Mike Thomas
mk               Matej Kosik
MPW              Marcel Weiher
mrm              Martin McClure
ms               math
MU               Masashi Umezawa
mw               Martin Wirblat
nb               Naala Brewer
nice             Nicolas Cellier
nk               Ned Konz
Noury            Noury Bouraqadi
NS               Nathanael Schaerli
PHK              Peter Keeler
pk               Pavel Krivanek
pm               Patrick Mauritz
pnm              Paul McDonough
RAA              Bob Arning
raok             Richard A. O"Keefe
rbb              Brian Brown
rca              Russell Allen
reThink          Paul McDonough
rew              Roger Whitney
rhi              Robert Hirschfeld
Rik              Rik Fischer SmOOdy
RJT              Ron Teitelbaum
rr               Romain Robbes
rw               Roel Wuyts
rww              Robert Withers
sbw              Stephan B. Wessels
SD               Stephane Ducasse
sge              Steve Elkins
shrink           Pavel Krivanek
slg              Steve Gilbert
sm               Simon Michael
sma              Stefan Matthias Aust
sn               Suslov Nikolay
spfa             Stephane Rollandin
sps              Steven Swerling
SqR              Andres Valloud
SqR!!!!            Andres Valloud
SqR!!!!!!!!          Andres Valloud
sr               Stephan Rudlof
ssa              Sam S. Adams
st               Samuel Tardieu
stephaneducasse  Stephane Ducasse
stp              Stephen Travis Pope
sumim            Masato Sumi
svp              Stephen Vincent Pair
sw               Scott Wallace
T2               Toshiyuki Takeda
tak              Takashi Yamamiya
tao              Tim Olson
tb               Todd Blanchard
TBn              Torsten Bergmann
tetha            Tetsuya Hayashi
tfei             The Fourth Estate, Inc.
th               Torge Husfeldt
ti               Tobias Isenberg
TJ               TJ Leone
tk               Ted Kaehler
tk               Thomas Kowark
tlk              Tom Koenig
TN               korakurider
TPR              Tim Rowledge
TRee             Trygve Reenskaug
Tsutomu          Tsutomu Hiroshima
tween            Andy Tween
vb               Vassili Bykov
vj               Vladimir Janousek
wiz              Jerome Peace
wod              Bill Dargel
ykoubo           Koji Yokokawa
yo               Yoshiki Ohshima
zz               Serge Stinckwich
jmv
jm
jmm
md
sd
tpr'! !

!SystemNavigation methodsFor: 'browse' stamp: 'ar 9/27/2005 20:34'!
spawnHierarchyForClass: aClass selector: aSelector
	"Create and schedule a new class hierarchy browser on the requested class/selector."
	"SystemNavigation default spawnHierarchyForClass: SmallInteger selector: #hash"

	(aClass == nil)  ifTrue: [^ self].
	ToolSet browseHierarchy: aClass selector: aSelector
! !

!SystemNavigation methodsFor: 'query' stamp: 'md 7/19/2004 16:03'!
unimplemented
	"Answer an Array of each message that is sent by an expression in a method but is not implemented by any object in the system."

	| all unimplemented entry |
	all := IdentitySet new: Symbol instanceCount * 2.
	Cursor wait showWhile: 
		[self allBehaviorsDo: [:cl | cl selectorsDo: [:aSelector | all add: aSelector]]].

	unimplemented := IdentityDictionary new.
	Cursor execute showWhile: [
		self allBehaviorsDo: [:cl |
			 cl selectorsDo: [:sel |
				(cl compiledMethodAt: sel) messages do: [:m |
					(all includes: m) ifFalse: [
						entry := unimplemented at: m ifAbsent: [Array new].
						entry := entry copyWith: (cl name, '>', sel).
						unimplemented at: m put: entry]]]]].
	^ unimplemented
! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 17:22'!
unsentMessagesInCategory: category
	^(SystemOrganization classesInCategory: category) inject: Set new into: [:unsentMessages :class|
		unsentMessages, (self unsentMessagesInClass: class)]
	! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 21:04'!
unsentMessagesInClass: aClass 
	|methReferences|
	methReferences := Set new.
	aClass selectors do: [:selector|
		(self isUnsentMessage: selector) ifTrue: [
			methReferences add: (MethodReference class: aClass selector: selector)]].
	^methReferences! !

!SystemNavigation methodsFor: 'message sends' stamp: 'Noury 10/26/2008 17:26'!
unsentMessagesInPackageNamed: packageName
	| unsentMessages |
	unsentMessages := self unsentMessagesInCategory: packageName.	
	(SystemOrganization categoriesMatching: packageName, '-*') do: [:category|
		unsentMessages addAll: (self unsentMessagesInCategory: category)].
	^unsentMessages 
	! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:46'!
unsentMessagesWithProgressBarInClass: aClass
	| unsentMessages |
	unsentMessages := Set new.
	self
		doWithProgressBar: [:class :selector| 
			unsentMessages add: (MethodReference class: class selector: selector)] 
		forUnsentMessagesInClass: aClass.
	^unsentMessages! !

!SystemNavigation methodsFor: 'message sends UI' stamp: 'Noury 10/26/2008 22:30'!
unsentMessagesWithProgressBarInPackageNamed: packageName
	| unsentMessages |
	unsentMessages := Set new.
	self
		doWithProgressBar: [:class :selector| 
			unsentMessages add: (MethodReference class: class selector: selector)] 
		forUnsentMessagesInPackageNamed: packageName.
	^unsentMessages! !

Object subclass: #SystemVersion
	instanceVariableNames: 'version date highestUpdate updates'
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'System-Support'!

!SystemVersion commentStamp: 'tlk 11/14/2004 10:44' prior: 0!
I am responsible for maintaining what version of Squeak and the VM is running.  I also track all of the update items that have been included in the image.

I'm invoked at auto start to get the latest plugins, etc.!

!SystemVersion class methodsFor: 'updating' stamp: 'sd 9/30/2003 13:58'!
check: pluginVersion andRequestPluginUpdate: updateURL
	"SystemVersion check: 'zzz' andRequestPluginUpdate: 'http://www.squeakland.org/installers/update.html' "

	"We don't have a decent versioning scheme yet, so we are basically checking for a nil VM version on the mac."
	(self pluginVersion: pluginVersion newerThan: self currentPluginVersion)
		ifFalse: [^true].
	(self confirm: 'There is a newer plugin version available. Do you want to install it now?')
		ifFalse: [^false].
	HTTPClient
		requestURL: updateURL , (SmalltalkImage current platformName copyWithout: Character space) asLowercase , '.html'
		target: '_top'.
	^false! !

!SystemVersion class methodsFor: 'updating' stamp: 'sd 11/16/2003 14:18'!
checkAndApplyUpdates: availableUpdate
	"SystemVersion checkAndApplyUpdates: nil"

	^(availableUpdate isNil
		or: [availableUpdate > SystemVersion current highestUpdate])
		ifTrue: [
			(self confirm: 'There are updates available. Do you want to install them now?')
				ifFalse: [^false].
			Utilities
				readServerUpdatesThrough: availableUpdate
				saveLocally: false
				updateImage: true.
			SmalltalkImage current snapshot: true andQuit: false.
			true]
		ifFalse: [false]! !

!SystemVersion class methodsFor: 'accessing' stamp: ''!
current
	Current ifNil: [Current := SystemVersion new].
	^Current! !

!SystemVersion class methodsFor: 'updating' stamp: 'sd 9/30/2003 13:58'!
currentPluginVersion
	^SmalltalkImage current vmVersion! !

!SystemVersion class methodsFor: 'instance creation' stamp: 'mir 3/29/2001 18:06'!
newVersion: versionName
	| newVersion |
	newVersion := self new version: versionName.
	newVersion
		highestUpdate: self current highestUpdate.
	Current := newVersion
! !

!SystemVersion class methodsFor: 'accessing' stamp: 'dc 5/30/2008 10:17'!
parseVersionString: versionString 
	"Answer the version of this release as version, date, update."
	"SystemVersion parseVersionString: 'Squeak3.1alpha of 28 February 2001 [latest update: #3966]' "
	| stream version date update |
	
	[ stream := versionString readStream.
	version := stream upToAll: ' of '.
	date := Date readFrom: stream.
	stream upToAll: ' #'.
	update := Number readFrom: stream ] 
		on: Error
		do: [ ^ nil ].
	^ {  version. date. update  }! !

!SystemVersion class methodsFor: 'accessing' stamp: 'mir 8/10/2001 11:53'!
pluginVersion: availableVersionString newerThan: currentVersionString
	| currentVersion availableVersion |
	(currentVersionString isEmptyOrNil
		or: [availableVersionString isEmptyOrNil])
		ifTrue: [^true].
	currentVersion := self parseVersionString: currentVersionString.
	availableVersion := self parseVersionString: availableVersionString.
	(currentVersion isNil
		or: [availableVersion isNil])
		ifTrue: [^false].
	^(currentVersion at: 2) < (availableVersion at: 2)! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
date
	^date! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
date: newDate
	date := newDate! !

!SystemVersion methodsFor: 'accessing' stamp: 'mir 5/1/2001 18:19'!
datedVersion
	"Answer the version of this release."

	^ self version asString , ' of ' , self date printString! !

!SystemVersion methodsFor: 'accessing' stamp: 'mir 3/29/2001 18:03'!
highestUpdate
	| sortedUpdates |
	highestUpdate ifNil: [
		sortedUpdates := self updates asSortedCollection.
		highestUpdate := (sortedUpdates isEmpty
			ifTrue: [0]
			ifFalse: [sortedUpdates last])].
	^highestUpdate! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
highestUpdate: anInteger
	highestUpdate := anInteger! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
includesUpdate: anUpdate
	^self updates includes: anUpdate! !

!SystemVersion methodsFor: 'initialize' stamp: 'alain.plantec 5/28/2009 11:09'!
initialize
	super initialize.
	version := 'No version set'.
	date := Date today.
	updates := Set new.
! !

!SystemVersion methodsFor: 'accessing' stamp: 'PeterHugossonMiller 9/2/2009 15:50'!
majorMinorVersion
	"Return the major/minor version number of the form X.Y, without any 'alpha' or 'beta' or other suffix."
	"(SystemVersion new version: 'Squeak3.7alpha') majorMinorVersion" "  -->  'Squeak3.7' "
	"SystemVersion current majorMinorVersion"
	
	| char stream |
	(stream := (self version, 'x') readStream) upTo: $..
	char := stream next.
	char ifNil: [^ version].	"eg: 'Jasmine-rc1' has no $. in it."
	[char isDigit]
		whileTrue: [char := stream next].
	^ version copyFrom: 1 to: stream position - 1
! !

!SystemVersion methodsFor: 'printing' stamp: 'mir 5/1/2001 18:20'!
printOn: stream
	stream
		nextPutAll: self datedVersion;
		nextPutAll: ' update ' , self highestUpdate printString! !

!SystemVersion methodsFor: 'accessing' stamp: 'mir 3/29/2001 18:01'!
registerUpdate: update
	self updates add: update.
	self resetHighestUpdate! !

!SystemVersion methodsFor: 'accessing' stamp: 'mir 3/29/2001 18:01'!
resetHighestUpdate
	highestUpdate := nil! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
unregisterUpdate: update
	self updates remove: update ifAbsent: []! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
updates
	^updates! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
version
	^version! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
version: newVersion
	version := newVersion! !

Object subclass: #Utilities
	instanceVariableNames: ''
	classVariableNames: 'LastStats RecentSubmissions ScrapsBook UpdateDownloader UpdateUrlLists'
	poolDictionaries: ''
	category: 'System-Support'!

!Utilities commentStamp: '<historical>' prior: 0!
A repository for general and miscellaneous utilities; much of what is here are in effect global methods that don't naturally attach to anything else.  1/96 sw!

!Utilities class methodsFor: 'pharo' stamp: 'MarcusDenker 10/2/2009 10:29'!
addUpdateServer: aSpec
	"Utilities addUpdateServer: #('Pharo Updates' #('pharo.gforge.inria.fr/'))"

	UpdateUrlLists addFirst: aSpec! !

!Utilities class methodsFor: 'fetching updates' stamp: 'sr 2/12/2001 03:38'!
applyUpdatesFromDisk
	"Utilities applyUpdatesFromDisk"
	"compute highest update number"
	| updateDirectory updateNumbers |
	updateDirectory := self getUpdateDirectoryOrNil.
	updateDirectory
		ifNil: [^ self].
	updateNumbers := updateDirectory fileNames
				collect: [:fn | fn initialIntegerOrNil]
				thenSelect: [:fn | fn notNil].
	self
		applyUpdatesFromDiskToUpdateNumber: (updateNumbers
				inject: 0
				into: [:max :num | max max: num])
		stopIfGap: false! !

!Utilities class methodsFor: 'fetching updates' stamp: 'ar 9/27/2005 20:15'!
applyUpdatesFromDiskToUpdateNumber: lastUpdateNumber stopIfGap: stopIfGapFlag 
	"To use this mechanism, be sure all updates you want to have considered 
	are in a folder named 'updates' which resides in the same directory as  
	your image. Having done that, simply evaluate:  
	 
	Utilities applyUpdatesFromDiskToUpdateNumber: 1234 stopIfGap: false  
	 
	and all numbered updates <= lastUpdateNumber not yet in the image will 
	be loaded in numerical order."
	| previousHighest currentUpdateNumber done fileNames aMessage updateDirectory loaded |
	updateDirectory := self getUpdateDirectoryOrNil.
	updateDirectory ifNil: [^ self].
	previousHighest := SystemVersion current highestUpdate.
	currentUpdateNumber := previousHighest.
	done := false.
	loaded := 0.
	[done]
		whileFalse: [currentUpdateNumber := currentUpdateNumber + 1.
			currentUpdateNumber > lastUpdateNumber
				ifTrue: [done := true]
				ifFalse: [fileNames := updateDirectory fileNamesMatching: currentUpdateNumber printString , '*'.
					fileNames size > 1
						ifTrue: [^ self inform: 'ambiguity -- two files both start with ' , currentUpdateNumber printString , '
(at this point it is probably best to remedy
the situation on disk, then try again.)'].
					fileNames size == 0
						ifTrue: [Transcript cr; show: 'gap in updates from disk for update number '; print: currentUpdateNumber; show: ' found...'.
							done := stopIfGapFlag]
						ifFalse: [ChangeSet
								newChangesFromStream: (updateDirectory readOnlyFileNamed: fileNames first)
								named: fileNames first.
							SystemVersion current registerUpdate: currentUpdateNumber.
							loaded := loaded + 1]]].
	aMessage := loaded = 0
				ifTrue: ['No new updates found.']
				ifFalse: [loaded printString , ' update(s) loaded.'].
	self inform: aMessage , '
Highest numbered update is now ' , (currentUpdateNumber - 1) printString , '.'! !

!Utilities class methodsFor: 'fetching updates' stamp: 'md 9/11/2004 12:06'!
assureAbsenceOfUnstableUpdateStream
	"Check to see if the unstable Updates stream is in the list; if it is, *remove* it.  This is the *opposite* of #assureAvailabilityOfUnstableUpdateStream"

	UpdateUrlLists ifNil: [UpdateUrlLists := OrderedCollection new].
	UpdateUrlLists := UpdateUrlLists select:
		[:pair | pair first ~= 'Unstable Updates*']


"Utilities assureAbsenceOfUnstableUpdateStream"! !

!Utilities class methodsFor: 'fetching updates' stamp: 'md 9/11/2004 12:07'!
assureAvailabilityOfUnstableUpdateStream
	"Check to see if the unstable Updates stream is in the list; if not, add it"

	UpdateUrlLists ifNil: [UpdateUrlLists := OrderedCollection new].
	UpdateUrlLists do:
		[:pair | (pair first =  'Unstable Updates*') ifTrue: [^ self]].

	UpdateUrlLists addFirst: #('Unstable Updates*' #('squeak.cs.uiuc.edu/Squeak2.0/' 'update.squeakfoundation.org/external/'))

"Utilities assureAvailabilityOfUnstableUpdateStream"! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'sw 7/29/2002 02:18'!
assureMostRecentSubmissionExists
	"Make certain that the most recent submission exists"

	[RecentSubmissions size > 0 and:
		[RecentSubmissions last isValid not]] whileTrue:
			[RecentSubmissions removeLast].! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'MiguelCoba 7/25/2009 02:05'!
authorInitials
	self deprecated: 'use instead: Author fullName'.
	^ Author fullName
! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'MiguelCoba 7/25/2009 02:19'!
authorInitialsPerSe
	self deprecated: 'use instead: Author fullNamePerSe'.
	^ Author fullNamePerSe
! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'on 5/10/2008 14:53'!
authorName
	self deprecated: 'use instead: Author fullName'.
	^ Author fullName
! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'on 5/10/2008 14:54'!
authorName: aString
	self deprecated: 'use instead: Author fullName:'.
	^ Author fullName: aString! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'MiguelCoba 7/25/2009 02:28'!
authorNamePerSe
	self deprecated: 'use instead: Author fullNamePerSe'.
	^ Author fullNamePerSe
! !

!Utilities class methodsFor: 'miscellaneous' stamp: ''!
awaitMouseUpIn: box repeating: doBlock ifSucceed: succBlock
	"The mouse has gone down in box; track the mouse, inverting the box while it's within, and if, on mouse up, the cursor was still within the box, execute succBlock.  While waiting for the mouse to come up, repeatedly execute doBlock. 5/11/96 sw
	6/10/96 sw: call new method that adds extra feature"

	^ self awaitMouseUpIn: box whileMouseDownDo: doBlock whileMouseDownInsideDo: [] ifSucceed: succBlock! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'JMM 3/31/2000 20:41'!
awaitMouseUpIn: box whileMouseDownDo: doBlock1 whileMouseDownInsideDo: doBlock2 ifSucceed: succBlock
	"The mouse has gone down in box; track the mouse, inverting the box while it's within, and if, on mouse up, the cursor was still within the box, execute succBlock.  While waiting for the mouse to come up, repeatedly execute doBlock1, and also, if the cursor is within the box, execute doBlock2.  6/10/96 sw
3/31/00 JMM added logic to stop multiple redraws"

	| p inside lightForm darkForm isLight |

	p := Sensor cursorPoint.
	inside := box insetBy: 1.
	isLight := true.
	lightForm := Form fromDisplay: inside.
	darkForm := lightForm deepCopy reverse.
	[Sensor anyButtonPressed] whileTrue:
		[doBlock1 value.
		(box containsPoint: (p := Sensor cursorPoint))
			ifTrue: [doBlock2 value.
					isLight ifTrue: 
						[isLight := false.
						darkForm displayAt: inside origin]]
			ifFalse: [isLight ifFalse:
						[isLight := true.
						lightForm displayAt: inside origin]]].
	(box containsPoint: p)
		ifTrue: [lightForm displayAt: inside origin.
				^ succBlock value]
! !

!Utilities class methodsFor: 'fetching updates' stamp: 'alain.plantec 2/6/2009 17:40'!
broadcastUpdatesFrom: n1 to: n2 except: skipList
"
	Note:  This method takes its list of files from the directory named 'updates',
	which will have been created and filled by, eg,
		Utilities readServerUpdatesSaveLocally: true updateImage: true.
	These can then be rebroadcast to any server using, eg,
		Utilities broadcastUpdatesFrom: 1 to: 9999 except: #(223 224).
	If the files are already on the server, and it is only a matter
	of copying them to the index for a different version, then use...
		(ServerDirectory serverInGroupNamed: 'SqC Internal Updates*')
			exportUpdatesExcept: #().
"
	| fileNames fileNamesInOrder names choice file updateDirectory |
	updateDirectory := FileDirectory default directoryNamed: 'updates'.
	fileNames := updateDirectory fileNames select:
		[:n | n first isDigit
			and: [(n initialIntegerOrNil between: n1 and: n2)
			and: [(skipList includes: n initialIntegerOrNil) not]]].
	(file := fileNames select: [:n | (n occurrencesOf: $.) > 1]) size > 0
		ifTrue: [self halt: file first , ' has multiple periods'].
	fileNamesInOrder := fileNames asSortedCollection:
		[:a :b | a initialIntegerOrNil < b initialIntegerOrNil].

	names := ServerDirectory groupNames asSortedArray.
	choice := UIManager default chooseFrom: names values: names.
	choice ifNil: [^ self].
	(ServerDirectory serverInGroupNamed: choice)
		putUpdateMulti: fileNamesInOrder fromDirectory: updateDirectory
! !

!Utilities class methodsFor: 'identification' stamp: 'MiguelCoba 7/25/2009 02:08'!
changeStamp 
	"Answer a string to be pasted into source code to mark who changed it and when."
	^ Author fullName , ' ' , Date today mmddyyyy, ' ',
		((String streamContents: [:s | Time now print24: true on: s]) copyFrom: 1 to: 5)! !

!Utilities class methodsFor: 'identification' stamp: 'MiguelCoba 7/25/2009 02:19'!
changeStampPerSe
	"Answer a string to be pasted into source code to mark who changed it and when."

	^ (Author fullNamePerSe ifNil: ['.']) , ' ' , Date today mmddyyyy, ' ',
		((String streamContents: [:s | Time now print24: true on: s]) 
			copyFrom: 1 to: 5)! !

!Utilities class methodsFor: 'summer97 additions' stamp: 'alain.plantec 2/6/2009 17:42'!
chooseFileWithSuffix: aSuffix
	"Utilities chooseFileWithSuffix: '.gif'"
	| aList |
	aList := FileDirectory default fileNamesMatching: '*', aSuffix.
	aList size > 0
		ifTrue:
			[^ UIManager default chooseFrom: aList values: aList title: 'Choose a file' translated]
		ifFalse:
			[self inform: 'Sorry, there are no files
whose names end with' translated, ' "', aSuffix, '".'.
			^ nil]! !

!Utilities class methodsFor: 'summer97 additions' stamp: 'alain.plantec 2/6/2009 17:43'!
chooseFileWithSuffixFromList: aSuffixList withCaption: aCaption
	"Pop up a list of all files in the default directory which have a suffix in the list.  Return #none if there are none; return nil if the user backs out of the menu without making a choice."
	"Utilities chooseFileWithSuffixFromList: #('.gif' '.jpg')"
	| aList |
	aList := OrderedCollection new.
	aSuffixList do:
		[:aSuffix | aList addAll: (FileDirectory default fileNamesMatching: '*', aSuffix)].
	^ aList size > 0
		ifTrue:
			[ UIManager default chooseFrom: aList values: aList title:aCaption]
		ifFalse:
			[#none]! !

!Utilities class methodsFor: 'fetching updates' stamp: 'CdG 10/17/2005 19:36'!
chooseUpdateList
	"When there is more than one set of update servers, let the user choose which we will update from.  Put it at the front of the list. Return false if the user aborted.  If the preference #promptForUpdateServer is false, then suppress that prompt, in effect using the same server choice that was used the previous time (a convenience for those of us who always answer the same thing to the prompt.)"

	| index him |
	((UpdateUrlLists size > 1) and: [Preferences promptForUpdateServer])
		ifTrue:
			[index := UIManager default 
				chooseFrom: (UpdateUrlLists collect: [:each | each first]) 
				lines: #()
				title: 'Choose a group of servers\from which to fetch updates.' translated withCRs.
			index > 0 ifTrue:
				[him := UpdateUrlLists at: index.
				UpdateUrlLists removeAt: index.
				UpdateUrlLists addFirst: him].
			^ index > 0].
	^ true! !

!Utilities class methodsFor: 'summer97 additions' stamp: 'sw 9/13/97 20:44'!
classCategoriesStartingWith: aPrefix
	"Answer a list of system class categories beginning with the given prefix"

	"Utilities classCategoriesStartingWith: 'Files'"

	^ SystemOrganization categories select:
		[:aCat | (aCat asString findString:  aPrefix startingAt: 1) = 1]! !

!Utilities class methodsFor: 'summer97 additions' stamp: 'stephane.ducasse 1/23/2009 21:37'!
classFromPattern: pattern withCaption: aCaption
	"If there is a class whose name exactly given by pattern, return it.
	If there is only one class in the system whose name matches pattern, return it.
	Otherwise, put up a menu offering the names of all classes that match pattern, and return the class chosen, else nil if nothing chosen.
	This method ignores tab, space, & cr characters in the pattern"

	self deprecated: 'Use SystemNavigation>>classFromPattern: pattern withCaption: aCaption'.
	^ SystemNavigation default classFromPattern: pattern withCaption: aCaption.! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'alain.plantec 6/1/2008 23:13'!
cleanseOtherworldlySteppers
	"If the current project is a morphic one, then remove from its steplist
	those morphs that are not really in the world"
	"Utilities cleanseOtherworldlySteppers"
	| old delta |
	old := self currentWorld stepListSize.
	self currentWorld steppingMorphsNotInWorld
		do: [:m | m delete].
	self currentWorld cleanseStepList.
	(delta := old - self currentWorld stepListSize) > 0
		ifTrue: [Transcript cr; show: delta asString , ' morphs removed from steplist']! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 01:30'!
commandKeyMappings
	
	self deprecated: 'Use ''TheWorldMenu new commandKeyMappings'' instead.' on: '10 July 2009' in: #Pharo1.0 .
	^ TheWorldMenu commandKeyMappings 
	! !

!Utilities class methodsFor: 'closure support' stamp: 'MikeRoberts 9/20/2009 12:45'!
compileUsingClosures	"Utilities compileUsingClosures"
	"Recompile the system and do some minimal clean-ups"
	| classes compilationErrors |
	Preferences setPreference: #allowBlockArgumentAssignment toValue: true.
	compilationErrors := Set new.
	classes := Smalltalk forgetDoIts allClasses reject: [:c| c name == #GeniePlugin].

	'Recompiling The System' displayProgressAt: Sensor cursorPoint
		from: 0 to: classes size during:[:bar |
			classes withIndexDo:[:c :i|
				bar value: i.
				{ c. c class } do:[:b|
					"Transcript cr; print: b; endEntry."
					b selectors "asSortedCollection" do:[:s| 
						"Transcript cr; show: b asString, '>>', s."
						[b recompile: s from: b] on: Error do:[:ex|
							Transcript
								cr; nextPutAll: 'COMPILATION ERROR: ';
								print: b; nextPutAll: '>>'; nextPutAll: s; flush.
							compilationErrors add: (MethodReference class: b selector: s)]]]]].

	(Smalltalk respondsTo: #allTraits) ifTrue:[
		'Recompiling Traits' displayProgressAt: Sensor cursorPoint
		from: 0 to: Smalltalk allTraits size during:[:bar |
			Smalltalk allTraits do:[:t|
				t selectors do:[:s|
					[t recompile: s] on: Error do:[:ex|
							Transcript
								cr; nextPutAll: 'COMPILATION ERROR: ';
								print: t; nextPutAll: '>>'; nextPutAll: s; flush.
							compilationErrors add: (MethodReference class: t selector: s)]]]]].

	compilationErrors notEmpty ifTrue:[
		SystemNavigation default
			browseMessageList: compilationErrors asSortedCollection
			name: 'Compilation Errors'].! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 01:21'!
convertCRtoLF: fileName
	"Convert the given file to LF line endings. Put the result in a file with the extention '.lf'"

	self deprecated: 'Use ''FileStream convertCRtoLF: fileName'' instead.' 
		on: '10 July 2009' in: #Pharo1.0 .
	FileStream convertCRtoLF: fileName! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 01:21'!
createPageTestWorkspace
	"Used to generate a workspace window for testing page up and page down stuff."
	"Utilities createPageTestWorkspace"

	self deprecated: 'Use ''UIManager createPageTestWorkspace'' instead.' on: '10 July 2009' in: #Pharo1.0.
	UIManager createPageTestWorkspace
	! !

!Utilities class methodsFor: 'identification' stamp: 'tk 4/10/1998 07:16'!
dateStamp
	"Utilities dateStamp"
	^ Date today mmddyyyy, ' ',
		((String streamContents: [:s | Time now print24: true on: s]) copyFrom: 1 to: 5)! !

!Utilities class methodsFor: 'identification' stamp: 'marcus.denker 11/18/2008 11:18'!
dateTimeSuffix
	"Answer a string which indicates the date and time, intended for use in building fileout filenames, etc."

	^self monthDayTime24StringFrom: Time primSecondsClock! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'nk 5/18/2003 13:03'!
decimalPlacesForFloatPrecision: precision
	"Answer the number of decimal places that correspond to the given floatPrecision"

	^ (#(1 0.1 0.01 0.001 0.0001 0.00001 0.000001 0.0000001 0.00000001 0.000000001) indexOf: precision ifAbsent: [ ^precision log negated floor ]) - 1

"
#(1 0.1 0.01 0.001 0.0001 0.00001 0.000001 0.0000001 0.00000001 0.000000001) collect: [:prec | Utilities decimalPlacesForFloatPrecision: prec]
"! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 01:23'!
decommissionTheAllCategory
	"Utilities decommissionTheAllCategory"
	"Moves all methods that are in a category named 'all' into the default 'as yet unclassified' category"

	| org aCategory methodCount classCount any |
	self deprecated: 'No alternative is suggested.' on: '10 July 2009' in: #Pharo1.0.
	
	methodCount := 0.
	classCount := 0.
	self systemNavigation allBehaviorsDo:
		[:aClass | org := aClass organization.
			any := false.
			aClass selectorsDo:
				[:aSelector |
					aCategory := org categoryOfElement: aSelector.
					aCategory = #all ifTrue:
						[org classify: aSelector under: ClassOrganizer default suppressIfDefault: false.
						methodCount := methodCount + 1.
						any := true]].
			any ifTrue: [classCount := classCount + 1].
			org removeEmptyCategories].
	Transcript cr; show: methodCount printString, ' methods in ', classCount printString, ' classes moved
from "all" to "as yet unclassified"'
! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'sw 11/21/2001 10:58'!
doesMethod: aSelector forClass: aClass bearInitials: initials
	"Answer whether a method bears the given initials at the head of its time stamp"

	| aTimeStamp implementingClass aMethod |
	implementingClass := aClass whichClassIncludesSelector: aSelector.
	implementingClass ifNil: [^ false].
	(aMethod := implementingClass compiledMethodAt: aSelector)
		ifNil: [^ false].
	^ (aTimeStamp := self timeStampForMethod: aMethod) notNil and:
		[aTimeStamp beginsWith: initials]! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'nk 8/30/2004 08:02'!
dumpAnyOldStyleRecentSubmissions

	"simplify conversion by purging those recent submissions which are still Strings"

	RecentSubmissions := self recentMethodSubmissions reject: [ :each |
		each isString
	].! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'alain.plantec 6/1/2008 23:15'!
emergencyCollapse
	^ self
! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'NS 4/12/2004 22:47'!
event: anEvent
	"Hook for SystemChangeNotifier"

	(anEvent isCommented and: [anEvent itemKind = SystemChangeNotifier classKind])
		ifTrue: [self noteMethodSubmission: #Comment forClass: anEvent item].
	((anEvent isAdded or: [anEvent isModified]) and: [anEvent itemKind = SystemChangeNotifier methodKind])
		ifTrue: [anEvent itemRequestor ifNotNil: [self noteMethodSubmission: anEvent itemSelector forClass: anEvent itemClass]].
	((anEvent isAdded or: [anEvent isModified]) and: [anEvent itemKind = SystemChangeNotifier methodKind]) ifTrue:[
		InMidstOfFileinNotification signal
			ifFalse: [Utilities changed: #recentMethodSubmissions].
	].! !

!Utilities class methodsFor: 'fetching updates' stamp: 'di 4/29/2001 22:26'!
extractThisVersion: list
	"Pull out the part of the list that applies to this version."

	| listContents version versIndex |
	listContents := self parseListContents: list.
	version := SystemVersion current version.
	versIndex := (listContents collect: [:pair | pair first]) indexOf: version.
	versIndex = 0 ifTrue: [^ Array new].		"abort"
	^ (listContents at: versIndex) last! !

!Utilities class methodsFor: 'fetching updates' stamp: 'sw 2/26/2002 23:19'!
fileInFromUpdatesFolder: numberList
	"File in a series of updates with the given updates numbers, from the updates folder in the default directory.  The file-ins are done in numeric order, even if numberList was not sorted upon entry.
	This is useful for test-driving the retrofitting of a possibly discontinguous list of updates from an alpha version back to a stable release.

	Utilities fileInFromUpdatesFolder: #(4745 4746 4747 4748 4749 4750 4751 4752 4754 4755 4761 4762 4767 4769).
"
	| fileNames fileNamesInOrder file updateDirectory |
	updateDirectory := FileDirectory default directoryNamed: 'updates'.
	fileNames := updateDirectory fileNames select:
		[:n | n first isDigit
			and: [numberList includes: n initialIntegerOrNil]].
	(file := fileNames select: [:n | (n occurrencesOf: $.) > 1]) size > 0
		ifTrue: [self error: file first , ' has multiple periods'].
	fileNamesInOrder := fileNames asSortedCollection:
		[:a :b | a initialIntegerOrNil < b initialIntegerOrNil].

	fileNamesInOrder do:
		[:aFileName | (updateDirectory readOnlyFileNamed: aFileName) fileIntoNewChangeSet]! !

!Utilities class methodsFor: 'identification' stamp: 'di 6/13/97 13:52'!
fixStamp: changeStamp 
	| parts |
	parts := changeStamp findTokens: ' '.
	(parts size > 0 and: [parts last first isLetter]) ifTrue:
		["Put initials first in all time stamps..."
		^ String streamContents:
				[:s | s nextPutAll: parts last.
				parts allButLast do: [:p | s space; nextPutAll: p]]].
	^ changeStamp! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'sd 4/17/2003 21:04'!
fixUpProblemsWithAllCategory
	"Moves all methods that are in formally classified a category named '-- all --' into the default 'as yet unclassified' category"

	"Utilities fixUpProblemsWithAllCategory"

	| org aCategory methodCount classCount any |
	self flag: #ShouldBeMovedInClassOrganization.
	methodCount := 0.
	classCount := 0.
	self systemNavigation allBehaviorsDo:
		[:aClass | org := aClass organization.
			(org categories includes: #'-- all --') ifTrue:
				[any := false.
				aClass selectorsDo:
					[:aSelector |
						aCategory := org categoryOfElement: aSelector.
						aCategory = #'-- all --' ifTrue:
							[org classify: aSelector under: ClassOrganizer default suppressIfDefault: false.
							Transcript cr; show: aClass name, ' >> ', aSelector.
							methodCount := methodCount + 1.
							any := true]].
			any ifTrue: [classCount := classCount + 1].
			org removeEmptyCategories]].
	Transcript cr; show: methodCount printString, ' methods in ', classCount printString, ' classes moved from "-- all --" to "as yet unclassified"'
! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'dtl 11/25/2004 22:10'!
floatPrecisionForDecimalPlaces: places
	"Answer the floatPrecision that corresponds to the given number of decimal places"

	^ places caseOf:
			{[0]->[1] .
			[1]->[0.1] . 
			[2]->[0.01] .
			[3]->[0.001] .
			[4]->[0.0001] .
			[5]->[0.00001] .
			[6]->[0.000001] .
			[7]->[0.0000001] .
			[8]->[0.00000001] .
			[9]->[0.000000001]}
		otherwise:
			[(10.0 raisedTo: places negated) asFloat]

"
(0 to: 6) collect: [:i | Utilities floatPrecisionForDecimalPlaces: i]
(-10 to: 20) collect: [:i | Utilities floatPrecisionForDecimalPlaces: i]
"! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'nk 2/15/2004 09:36'!
garbageCollectAndReport
	"Do a garbage collection, and report results to the user."

	| cc reportString |
	reportString := String streamContents:
		[:aStream | 
			aStream nextPutAll: Smalltalk bytesLeftString.
			Smalltalk at: #Command ifPresent:
				[:cmdClass |
				(cc := cmdClass instanceCount) > 0 ifTrue:
					[aStream cr; nextPutAll:
		('(note: there are ', cc printString,
		                         ' undo record(s) present in your
system; purging them may free up more space.)')]]].
			
	self inform: reportString
! !

!Utilities class methodsFor: 'fetching updates' stamp: 'sr 2/12/2001 03:36'!
getUpdateDirectoryOrNil
	^ (FileDirectory default directoryNames includes: 'updates')
		ifTrue: [FileDirectory default directoryNamed: 'updates']
		ifFalse: [self inform: 'Error: cannot find "updates" folder'.
			nil]! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'sw 11/16/2001 14:43'!
getterSelectorFor: identifier
	"Answer the corresponding getter.  Two idiosyncratic vectorings herein... " 

	"Utilities getterSelectorFor: #elvis"

	| aSymbol |
	(aSymbol := identifier asSymbol) == #isOverColor: ifTrue: [^ #seesColor:].
	aSymbol == #copy ifTrue: [^ #getNewClone].

	^ ('get', (identifier asString capitalized)) asSymbol! !

!Utilities class methodsFor: 'summer97 additions' stamp: 'sw 10/6/1998 14:09'!
graphicsFileSuffixes
	"Answer a list of filename suffixes which signal file content which we are able to internalize"

	^#('.gif' '.bmp' '.jpg' '.jpeg' '.jpe', '.form')! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 01:23'!
informUser: aString during: aBlock
	"Display the message as progress during execution of the given block."
	"UIManager default informUser: 'Just a sec!!' during: [(Delay forSeconds: 1) wait]"

	self deprecated: 'Use ''UIManager default informUser: during:'' instead.' on: '10 July 2009' in: #Pharo1.0.
	^ UIManager default informUser: aString during: aBlock! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 01:24'!
informUserDuring: aBlock
	"Display a message as progress during execution of the given block."
	
	self deprecated: 'Use ''UIManager default informUserDuring:'' instead' on: '10 July 2009' in: #Pharo1.0.
	^ UIManager default informUserDuring: aBlock! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'sw 5/19/2000 16:04'!
inherentSelectorForGetter: aGetterSelector
	"Given a selector of the form #getAbc, return the inherent slotname selector that corresponds, which is to say, getterSelector with the leading 'get' removed and with the next character forced to lower case; this is the inverse of #getterSelectorFor:"

	"Utilities inherentSelectorForGetter: #getWidth"
	((aGetterSelector size < 4) or: [(aGetterSelector beginsWith: 'get') not])
			ifTrue: [ ^ aGetterSelector].
	^ ((aGetterSelector at: 4) asLowercase asString, (aGetterSelector copyFrom: 5 to: aGetterSelector size)) asSymbol! !

!Utilities class methodsFor: 'initialization' stamp: 'AlainPlantec 10/17/2009 18:03'!
initialize
	RecentSubmissions := OrderedCollection new.
	(TheWorldMenu respondsTo: #registerOpenCommand:) ifTrue: [
		TheWorldMenu registerOpenCommand: {'Recent Submissions'. {self. #browseRecentSubmissions}}]! !

!Utilities class methodsFor: 'closure support' stamp: 'MikeRoberts 9/20/2009 12:45'!
initializeClosures	"Utilities initializeClosures"
	"Eliminate the prototype BlockContext from the specialObjectsArray.  The VM doesn't use it. This paves the way for removing BlockCOntext altogether and merging ContextPart and MethodContext into e.g. Context."
	(Smalltalk specialObjectsArray at: 38) class == BlockContext 
		ifTrue:[Smalltalk specialObjectsArray at: 38 put: nil].
	"Remove unused class vars from CompiledMethod since we can't redefine its class definition directly. Add the new BlockClosure to the specialObjectsArray"
	(#(	BlockNodeCache MethodProperties SpecialConstants) 
			intersection: CompiledMethod classPool keys) 
				do:[:classVarName| CompiledMethod removeClassVarName: classVarName].
	Smalltalk recreateSpecialObjectsArray.
	"Recompile methods in ContextPart, superclasses and subclasses that access inst vars"
	ContextPart withAllSuperclasses, ContextPart allSubclasses asArray do:[:class|
		class instSize > 0 ifTrue:[
			class allInstVarNames do:[:ivn|
				(class whichSelectorsAccess: ivn) do:[:sel| class recompile: sel]]]]! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 01:25'!
inspectCollection: aCollection notifying: aView

	self deprecated: 'No replacement is suggested.' on: '10 July 2009' in: #Pharo1.0.
	
	aCollection size = 0 
		ifTrue: [aView notNil 
			ifTrue: [^ aView flash]
			ifFalse: [^ self]].
	aCollection size = 1
		ifTrue: [aCollection first inspect]
		ifFalse: [aCollection asArray inspect]! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'mdr 9/4/2000 11:07'!
instanceComparisonsBetween: fileName1 and: fileName2
	"For differential results, run printSpaceAnalysis twice with different fileNames,
	then run this method...
		Smalltalk printSpaceAnalysis: 0 on: 'STspace.text1'.
			--- do something that uses space here ---
		Smalltalk printSpaceAnalysis: 0 on: 'STspace.text2'.
		Smalltalk instanceComparisonsBetween: 'STspace.text1' and 'STspace.text2'"

	| instCountDict report f aString items className newInstCount oldInstCount newSpace oldPair oldSpace |
	instCountDict := Dictionary new.
	report := ReadWriteStream on: ''.
	f := FileStream readOnlyFileNamed: fileName1.
	[f atEnd] whileFalse:
		[aString := f upTo: Character cr.
		items := aString findTokens: ' '.
		(items size == 4 or: [items size == 5]) ifTrue:
			[instCountDict at: items first put: (Array with: items third asNumber with: items fourth asNumber)]].
	f close.

	f := FileStream readOnlyFileNamed: fileName2.
	[f atEnd] whileFalse:
		[aString := f upTo: Character cr.
		items := aString findTokens: ' '.
		(items size == 4 or: [items size == 5]) ifTrue:
			[className := items first.
			newInstCount := items third asNumber.
			newSpace := items fourth asNumber.
			oldPair := instCountDict at: className ifAbsent: [nil].
			oldInstCount := oldPair ifNil: [0] ifNotNil: [oldPair first].
			oldSpace := oldPair ifNil: [0] ifNotNil: [oldPair second].
			oldInstCount ~= newInstCount ifTrue:
				[report nextPutAll: (newInstCount - oldInstCount) printString; tab; nextPutAll: (newSpace - oldSpace) printString; tab; nextPutAll: className asString; cr]]].
	f close.

	(StringHolder new contents: report contents)
		openLabel: 'Instance count differentials between ', fileName1, ' and ', fileName2! !

!Utilities class methodsFor: 'miscellaneous' stamp: ''!
keyLike: aString satisfying: aBlock
	"Return a key like aString that satisfies aBlock.  The block should provide a test for acceptability -- typically the test is about whether the key is already in use.  aBlock should return a boolean.  8/11/96 sw"

	| stemAndSuffix suffix stem newKey |
	(aBlock value: aString) ifTrue: [^ aString].
	stemAndSuffix := aString stemAndNumericSuffix.
	suffix := stemAndSuffix last + 1.
	stem := stemAndSuffix first.
	[aBlock value: (newKey := stem, suffix printString)]
		whileFalse:
			[suffix := suffix + 1].
	^ newKey
! !

!Utilities class methodsFor: 'miscellaneous' stamp: ''!
keyLike: aString withTrailing: trailerString satisfying: aBlock
	"Return a key like (aString, trailerString) that satisfies aBlock.  The block should provide a test for acceptability -- typically the test is about whether the key is already in use.  aBlock should return a boolean.  8/11/96 sw"

	| stemAndSuffix suffix stem composite |
	composite := aString, trailerString.
	(aBlock value: composite) ifTrue: [^ composite].
	stemAndSuffix := aString stemAndNumericSuffix.
	suffix := stemAndSuffix last + 1.
	stem := stemAndSuffix first.
	[aBlock value: (composite := stem, suffix printString, trailerString)]
		whileFalse:
			[suffix := suffix + 1].
	^ composite
! !

!Utilities class methodsFor: 'fetching updates' stamp: 'tk 2/16/98 16:16'!
lastUpdateNum: updatesFileStrm
	"Look in the Updates file and see what the last sequence number is.  Warn the user if the version it is under is not this image's version."

	| verIndex seqIndex char ver seqNum |
	verIndex := seqIndex := 0.	 "last # starting a line and last digit starting a line"
	seqNum := 0.
	updatesFileStrm reset; ascii.
	[char := updatesFileStrm next.
	 updatesFileStrm atEnd] whileFalse: [
		char == Character cr ifTrue: [
			updatesFileStrm peek == $# ifTrue: [verIndex := updatesFileStrm position +1.
				seqIndex = 0 ifFalse: ["See if last num of old version if biggest so far"
					updatesFileStrm position: seqIndex.
					ver := SmallInteger readFrom: updatesFileStrm.
					seqNum := seqNum max: ver.
					updatesFileStrm position: verIndex-1]].
			updatesFileStrm peek isDigit ifTrue: [seqIndex := updatesFileStrm position]]].

	seqIndex = 0 ifFalse: ["See if last num of old version if biggest so far"
		updatesFileStrm position: seqIndex.
		ver := SmallInteger readFrom: updatesFileStrm.
		seqNum := seqNum max: ver.
		updatesFileStrm setToEnd].
	^ seqNum! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'md 11/14/2003 18:02'!
methodDiffFor: aString class: aClass selector: aSelector prettyDiffs: prettyDiffBoolean
	"Return a string comprising a source-code diff between an existing method and the source-code in aString.  DO prettyDiff if prettyDiffBoolean is true."

	^ (aClass notNil and: [aClass includesSelector: aSelector])
		ifTrue:
			[TextDiffBuilder
				buildDisplayPatchFrom: (aClass sourceCodeAt: aSelector)
				to: aString
				inClass: aClass
				prettyDiffs: prettyDiffBoolean]
		ifFalse:
			[aString copy]! !

!Utilities class methodsFor: 'identification' stamp: 'stephane.ducasse 5/23/2009 14:04'!
methodsWithInitials: targetInitials
	"Based on a do-it contributed to the Squeak mailing list by Goran Hultgen:
 Browse methods whose initials (in the time-stamp, as logged to disk) match the given initials.
 Print out the complete time-stamp table to the Transcript.
 Answer a list of (initials -> count) associations.

CAUTION: It may take several minutes for this to complete."

	"Time millisecondsToRun: [Utilities methodsWithInitials: 'bf']"

	| initials timeStamp  allSubmitters |
	initials := ''.
	timeStamp := ''.
	allSubmitters := Bag new.
	self systemNavigation
		browseAllSelect:
			[:cm |
				timeStamp := Utilities timeStampForMethod: cm.
				initials := timeStamp isEmpty
					ifTrue:
						['']
					ifFalse:
						[timeStamp substrings first].
				initials := initials isEmpty
					ifTrue:
						['<no time stamp>']
					ifFalse:
						[initials first isDigit
							ifTrue:
								['<date>']
							ifFalse:
								[initials]].
				allSubmitters add: initials.
				(initials = targetInitials)]
		name: ('Methods with initials ', targetInitials)
		autoSelect: nil.

	allSubmitters sortedCounts do: [:elem | Transcript cr; show: elem asString].
	^ allSubmitters
! !

!Utilities class methodsFor: 'identification' stamp: 'sw 11/13/1999 23:03'!
monthDayTime24StringFrom: aSecondCount
	| aDate aTime |
	"From the date/time represented by aSecondCount, produce a string which indicates the date and time in the compact form
             ddMMMhhmm		where dd is a two-digit day-of-month, MMM is the alpha month abbreviation and hhmm is the time on a 24-hr clock.

          Utilities monthDayTime24StringFrom: Time primSecondsClock
"

	aDate := Date fromSeconds: aSecondCount.
	aTime := Time fromSeconds: aSecondCount \\ 86400.

	^ (aDate dayOfMonth asTwoCharacterString), 
		(aDate monthName copyFrom: 1 to: 3), 
		(aTime hhmm24)! !

!Utilities class methodsFor: 'identification' stamp: 'sw 11/13/1999 23:03'!
monthDayTimeStringFrom: aSecondCount
	| aDate aTime |
	"From the date/time represented by aSecondCount, produce a string which indicates the date and time in the form:
		ddMMMhhmmPP	  where:
							dd is a two-digit day-of-month,
							MMM is the alpha month abbreviation,
							hhmm is the time,
							PP is either am or pm

          Utilities monthDayTimeStringFrom: Time primSecondsClock
"

	aDate := Date fromSeconds: aSecondCount.
	aTime := Time fromSeconds: aSecondCount \\ 86400.

	^ (aDate dayOfMonth asTwoCharacterString), 
		(aDate monthName copyFrom: 1 to: 3), 
		((aTime hours \\ 12) asTwoCharacterString), 
		(aTime minutes asTwoCharacterString),
		(aTime hours > 12 ifTrue: ['pm'] ifFalse: ['am'])! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'sw 7/29/2002 02:12'!
mostRecentlySubmittedMessage
	"Answer a string indicating the most recently submitted method that is still extant"

	self flag: #mref.	"fix for faster references to methods"

	self assureMostRecentSubmissionExists.
	^ RecentSubmissions last asStringOrText asString! !

!Utilities class methodsFor: 'fetching updates' stamp: 'marcus.denker 9/14/2008 19:03'!
newUpdatesOn: serverList special: indexPrefix throughNumber: aNumber
	"Return a list of fully formed URLs of update files we do not yet have.  Go to the listed servers and look at the file 'updates.list' for the names of the last N update files.  We look backwards for the first one we have, and make the list from there.  tk 9/10/97
	No updates numbered higher than aNumber (if it is not nil) are returned " 

	| existing doc list out ff raw char maxNumber itsNumber |
	maxNumber := aNumber ifNil: [99999].
	out := OrderedCollection new.
	existing := SystemVersion current updates.
	serverList do: [:server |
		doc := HTTPClient httpGet: 'http://' , server,indexPrefix,'updates.list'.
		"test here for server being up"
		doc class == RWBinaryOrTextStream ifTrue:
			[raw := doc reset; contents.	"one file name per line"
			list := self extractThisVersion: raw.
			list reverseDo: [:fileName |
				ff := (fileName findTokens: '/') last.	"allow subdirectories"
				itsNumber := ff initialIntegerOrNil. 
				(existing includes: itsNumber)
					ifFalse:
						[
						(itsNumber isNil or: [itsNumber <= maxNumber])
							ifTrue:
								[out addFirst: 'http://' , server, fileName]]
					ifTrue: [^ out]].
			((out size > 0) or: [char := doc reset; skipSeparators; next.
				(char == $*) | (char == $#)]) ifTrue:
					[^ out "we have our list"]].	"else got error msg instead of file"
		"Server was down, try next one"].
	self inform: 'All code update servers seem to be unavailable'.
	^ out! !

!Utilities class methodsFor: 'miscellaneous' stamp: ''!
nextClockwiseSideAfter: aSide
 	aSide == #left ifTrue:
		[^ #top].
	aSide == #right ifTrue:
		[^ #bottom].
	aSide == #top ifTrue:
		[^ #right].
	^ #left! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'RAA 5/28/2001 10:53'!
noteMethodSubmission: selectorName forClass: class

	| submission |

	self flag: #mref.	"fix for faster references to methods"

	self recentMethodSubmissions.	"ensure it is valid"
	class wantsChangeSetLogging ifFalse: [^ self].
	self purgeRecentSubmissionsOfMissingMethods.
	submission := class name asString, ' ', selectorName.
	RecentSubmissions removeAllSuchThat: [ :each |
		each asStringOrText = submission
	].
	RecentSubmissions size >= self numberOfRecentSubmissionsToStore ifTrue: [
		RecentSubmissions removeFirst
	].
	RecentSubmissions addLast: (
		MethodReference new
			setClass: class 
			methodSymbol: selectorName 
			stringVersion: submission
	) 
! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'sw 7/28/2002 23:20'!
numberOfRecentSubmissionsToStore
	"Answer how many methods back the 'recent method submissions' history should store"

	^ Preferences parameterAt: #numberOfRecentSubmissionsToStore ifAbsentPut: [30]! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'sw 7/28/2002 23:52'!
numberOfRecentSubmissionsToStore: aNumber
	"Set the number of Recent Submissions to store"

	Preferences setParameter: #numberOfRecentSubmissionsToStore to: aNumber! !

!Utilities class methodsFor: 'fetching updates' stamp: 'StephaneDucasse 10/18/2009 17:53'!
objectStrmFromUpdates: fileName
	"Go to the known servers and look for this file in the updates folder.  It is an auxillery file, like .morph or a .gif.  Return a RWBinaryOrTextStream on it.    Meant to be called from during the getting of updates from the server.  That assures that (Utilities serverUrls) returns the right group of servers."

	| urls doc |
	Cursor wait showWhile:
		[urls := Utilities serverUrls collect: [:url | url, 'updates/','pharo', ScriptLoader currentMajorVersionNumber asString,'/', fileName].
		urls do: [:aUrl |
			doc := HTTPSocket httpGet: aUrl accept: 'application/octet-stream'.
			"test here for server being up"
			doc class == RWBinaryOrTextStream ifTrue: [^ doc reset]]].

	self inform: 'All update servers are unavailable, or bad file name'.
	^ nil! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'stephane.ducasse 7/10/2009 15:53'!
openCommandKeyHelp
	"Open a window giving command key help."
	
	self deprecated: 'Use TheWorldMenu version instead' on: '10 July 2009' in: #Pharo1.0.! !

!Utilities class methodsFor: 'miscellaneous' stamp: ''!
oppositeCornerFrom: aCorner
	"Answer the corner diagonally opposite to aCorner.  6/27/96 sw"

	aCorner == #topLeft
		ifTrue:
			[^ #bottomRight].
	aCorner == #topRight
		ifTrue:
			[^ #bottomLeft].
	aCorner == #bottomLeft
		ifTrue:
			[^ #topRight].
	^ #topLeft! !

!Utilities class methodsFor: 'miscellaneous' stamp: ''!
oppositeModeTo: aMode
 	aMode == #readOnly ifTrue: [^ #writeOnly].
	aMode == #writeOnly ifTrue: [^ #readOnly].
	^ aMode! !

!Utilities class methodsFor: 'miscellaneous' stamp: ''!
oppositeSideTo: aSide
 	aSide == #left ifTrue:
		[^ #right].
	aSide == #right ifTrue:
		[^ #left].
	aSide == #top ifTrue:
		[^ #bottom].
	^ #top! !

!Utilities class methodsFor: 'fetching updates' stamp: 'dc 5/30/2008 10:17'!
parseListContents: listContents 
	"Parse the contents of updates.list into {{vers. {fileNames*}}*}, and return it."
	| sections vers strm line fileNames |
	sections := OrderedCollection new.
	fileNames := OrderedCollection new: 1000.
	vers := nil.
	strm := listContents readStream.
	[ strm atEnd ] whileFalse: 
		[ line := strm upTo: Character cr.
		line size > 0 ifTrue: 
			[ line first = $# 
				ifTrue: 
					[ vers ifNotNil: [ sections addLast: {  vers. (fileNames asArray)  } ].
					"Start a new section"
					vers := line allButFirst.
					fileNames resetTo: 1 ]
				ifFalse: [ line first = $* ifFalse: [ fileNames addLast: line ] ] ] ].
	vers ifNotNil: [ sections addLast: {  vers. (fileNames asArray)  } ].
	^ sections asArray
	" TEST:
 | list |
list := Utilities parseListContents: (FileStream oldFileNamed: 'updates.list') contentsOfEntireFile.
list = (Utilities parseListContents: (String streamContents: [:s | Utilities writeList: list toStream: s]))
	ifFalse: [self error: 'test failed']
	ifTrue: [self inform: 'test OK']
"! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 01:32'!
pointersTo: anObject
	"Find all occurrences in the system of pointers to the argument anObject."
	"(Utilities pointersTo: Browser) inspect."
	
	self deprecated: 'Use ''PointerFinder pointersTo:'' instead.' on: '10 July 2009' in: #Pharo1.0.
	^ PointerFinder pointersTo: anObject except: #()
! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 01:32'!
pointersTo: anObject except: objectsToExclude 
	"Find all occurrences in the system of pointers to the argument
	anObject. Remove objects in the exclusion list from the
	results. "
	self deprecated: 'Use ''PointerFinder pointersTo:except:'' instead.' on: '10 July 2009' in: #Pharo1.0.
		
	^ PointerFinder pointersTo: anObject except: objectsToExclude ! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'stephane.ducasse 7/10/2009 16:01'!
pointersToItem: index of: anArray
	"Find all occurrences in the system of pointers to the given element of the given array. This is useful for tracing up a pointer chain from an inspector on the results of a previous call of pointersTo:. To find out who points to the second element of the results, one would evaluate:

	Utilities pointersToItem: 2 of: self

in the inspector."

	self deprecated: 'Please, use PointerFinder class>>pointersToItem:of: instead' on: '10 July 2009' in: #Pharo1.0.
	^ PointerFinder pointersTo: (anArray at: index) except: (Array with: anArray)! !

!Utilities class methodsFor: 'fetching updates' stamp: 'mir 4/2/2001 16:34'!
position: updateStrm atVersion: version
	"Set the stream to the end of the last line of updates names for this version.  Usually the end of the file.  We will add a new update name.   Return the contents of the rest of the file."

	| char foundIt where data |
	updateStrm reset; ascii.
	foundIt := false.
	[char := updateStrm next.
	 updateStrm atEnd] whileFalse: [
		(char == Character cr or: [char == Character lf]) ifTrue: [
			updateStrm peek == $# ifTrue: [
				foundIt ifTrue: ["Next section"
					where := updateStrm position.
					data := updateStrm upTo: (255 asCharacter).
					updateStrm position: where.
					^ data].	"won't be found -- copy all the way to the end"
				updateStrm next.
				(updateStrm nextMatchAll: version) ifTrue: [
					(updateStrm atEnd or: [(updateStrm peek = Character cr) | 
						(updateStrm peek = Character lf)]) ifTrue: [
							foundIt := true
					]]]]].
	foundIt ifTrue: [
		updateStrm setToEnd.
		^ ''].
	self error: 'The current version does not have a section in the Updates file'.
! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'sw 9/26/2002 19:16'!
purgeFromRecentSubmissions: aMethodReference
	"Purge any reference found in RecentSubmissions to the method supplied"

	RecentSubmissions := RecentSubmissions select:
		[:aSubmission |
			Utilities setClassAndSelectorFrom: aSubmission in:
				[:aClass :aSelector | (aClass ~~ aMethodReference actualClass) or: [aSelector ~~ aMethodReference methodSymbol]]]! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'marcus.denker 9/14/2008 19:02'!
purgeRecentSubmissionsOfMissingMethods
	"Utilities purgeRecentSubmissionsOfMissingMethods"

	| keep |
	RecentSubmissions := RecentSubmissions select:
		[:aSubmission | 
			Utilities setClassAndSelectorFrom: aSubmission in:
				[:aClass :aSelector |
					keep := aClass notNil
						and: [aClass isInMemory
						and: [aSelector == #Comment or: [(aClass compiledMethodAt: aSelector ifAbsent: [nil]) notNil]]]].
			keep]! !

!Utilities class methodsFor: 'fetching updates' stamp: 'ar 9/27/2005 20:15'!
readNextUpdateFromServer
	"Utilities readNextUpdateFromServer"
	self updateFromServerThroughUpdateNumber: (ChangeSet highestNumberedChangeSet + 1)! !

!Utilities class methodsFor: 'fetching updates' stamp: 'ar 9/27/2005 20:15'!
readNextUpdatesFromDisk: n
	"Read the updates up through the current highest-update-number plus n.  Thus, 
	Utilities readNextUpdatesFromDisk: 7
will read the next seven updates from disk"

	self applyUpdatesFromDiskToUpdateNumber: ChangeSet highestNumberedChangeSet + n
		stopIfGap: false! !

!Utilities class methodsFor: 'fetching updates' stamp: 'MarcusDenker 11/28/2009 14:03'!
readServer: serverList special: indexPrefix updatesThrough: maxNumber saveLocally: saveLocally updateImage: updateImage
	"Scan the update server(s) for unassimilated updates. If maxNumber is not nil, it represents the highest-numbered update to load.  This makes it possible to update only up to a particular point.   If saveLocally is true, then save local copies of the update files on disc.  If updateImage is true, then absorb the updates into the current image."

"Utilities readServer: Utilities serverUrls updatesThrough: 828 saveLocally: true updateImage: true"

	| urls failed loaded docQueue this nextDoc docQueueSema str updateName |
	Cursor wait showWhile: [

	urls := self newUpdatesOn: (serverList collect: [:url | url, 'updates/','pharo', ScriptLoader currentMajorVersionNumber asString, '/']) 
				special: indexPrefix
				throughNumber: maxNumber.
	loaded := 0.
	failed := nil.

	"send downloaded documents throuh this queue"
	docQueue := SharedQueue2 new.

	"this semaphore keeps too many documents from beeing queueed up at a time"
	docQueueSema := Semaphore new.
	5 timesRepeat: [ docQueueSema signal ].

	"fork a process to download the updates"
	self retrieveUrls: urls ontoQueue: docQueue withWaitSema: docQueueSema.

	"process downloaded updates in the foreground"
	'Processing updates' displayProgressAt: Sensor cursorPoint from: 0 to: urls size during: [:bar |
	[ this := docQueue next.
	  nextDoc := docQueue next.  
	  nextDoc = #failed ifTrue: [ failed := this ].
	  (failed isNil and: [ nextDoc ~= #finished ])
	] whileTrue: [
		failed ifNil: [
			nextDoc reset; text.
			nextDoc size = 0 ifTrue: [ failed := this ]. ].
		failed ifNil: [
			nextDoc peek asciiValue = 4	"pure object file"
				ifTrue: [failed := this]].	"Must be fileIn, not pure object file"
		failed ifNil: [
			"(this endsWith: '.html') ifTrue: [doc := doc asHtml]."
				"HTML source code not supported here yet"
			updateImage
				ifTrue: [
					updateName := (this findTokens: '/') last.
					ChangeSet newChangesFromStream: nextDoc named: updateName.
					SystemVersion current registerUpdate: updateName initialIntegerOrNil].
			saveLocally ifTrue:
				[self saveUpdate: nextDoc onFile: (this findTokens: '/') last].	"if wanted"
			loaded := loaded + 1.
			bar value: loaded].

		docQueueSema signal].
	]].

	failed ~~ nil & (urls size - loaded > 0) ifTrue: [
		str := loaded printString ,' new update file(s) processed.'.
		str := str, '\Could not load ' withCRs, 
			(urls size - loaded) printString ,' update file(s).',
			'\Starting with "' withCRs, failed, '".'.
		self inform: str].
	^ Array with: failed with: loaded
! !

!Utilities class methodsFor: 'fetching updates' stamp: 'sw 1/10/1999 02:02'!
readServerUpdatesSaveLocally: saveLocally updateImage: updateImage
	^ self readServerUpdatesThrough: nil saveLocally: saveLocally updateImage: updateImage! !

!Utilities class methodsFor: 'fetching updates' stamp: 'rbb 2/18/2005 13:16'!
readServerUpdatesThrough: maxNumber saveLocally: saveLocally updateImage: updateImage
	"Scan the update server(s) for unassimilated updates. If maxNumber is not nil, it represents the highest-numbered update to load.  This makes it possible to update only up to a particular point.   If saveLocally is true, then save local copies of the update files on disc.  If updateImage is true, then absorb the updates into the current image.

A file on the server called updates.list has the names of the last N update files.  We look backwards for the first one we do not have, and start there"
"* To add a new update:  Name it starting with a new two-digit code.  
* Do not use %, /, *, space, or more than one period in the name of an update file.
* The update name does not need to have any relation to the version name.
* Figure out which versions of the system the update makes sense for.
* Add the name of the file to each version's category below.
* Put this file and the update file on all of the servers.
*
* To make a new version of the system:  Pick a name for it (no restrictions)
* Put # and exactly that name on a new line at the end of this file.
* During the release process, fill in exactly that name in the dialog box.
* Put this file on the server."
"When two sets of updates need to use the same directory, one of them has a * in its 
serverUrls description.  When that is true, the first word of the description is put on
the front of 'updates.list', and that is the index file used."

"Utilities readServerUpdatesThrough: 3922 saveLocally: true updateImage: true"

	| failed loaded str res servers triple tryAgain indexPrefix |
	Utilities chooseUpdateList ifFalse: [^ self].	"ask the user which kind of updates"

	servers := Utilities serverUrls copy.
	indexPrefix := (Utilities updateUrlLists first first includes: $*) 
		ifTrue: [(Utilities updateUrlLists first first findTokens: ' ') first]
						"special for internal updates"
		ifFalse: ['']. 	"normal"
	[servers isEmpty] whileFalse: [
		triple := self readServer: servers special: indexPrefix 
					updatesThrough: maxNumber 
					saveLocally: saveLocally updateImage: updateImage.

		"report to user"
		failed := triple first.
		loaded := triple second.
		tryAgain := false.
		failed ifNil: ["is OK"
			loaded = 0 ifTrue: ["found no updates"
				servers size > 1 ifTrue: ["not the last server"
					res := UIManager default 
							chooseFrom: #('Stop looking' 'Try next server')
							title: 
'No new updates on the server
', servers first, '
Would you like to try the next server?
(Normally, all servers are identical, but sometimes a
server won''t let us store new files, and gets out of date.)' 
						.
					res = 2 ifFalse: [^ self]
						 ifTrue: [servers := servers allButFirst.	"try the next server"
							tryAgain := true]]]].
		tryAgain ifFalse: [
			str := loaded printString ,' new update file(s) processed.'.
			^ self inform: str].
	].! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'marcus.denker 9/14/2008 19:02'!
recentMethodSubmissions
	"Answer the list of recent method submissions, in order.  5/16/96 sw"

	RecentSubmissions ifNil: [RecentSubmissions := OrderedCollection new].
	^ RecentSubmissions! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'jmv 5/10/2009 09:22'!
reconstructTextWindowsFromFileNamed: aName
	"Utilities reconstructTextWindowsFromFileNamed: 'TextWindows'"
	| aReferenceStream aDict |
	aReferenceStream :=  ReferenceStream fileNamed: aName.
	aDict :=  aReferenceStream next.
	aReferenceStream close.
	aDict associationsDo:
		[:assoc |
			(StringHolder new contents: assoc value) openAsMorphLabel: assoc key ]! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 02:18'!
reportSenderCountsFor: selectorList 
	"Produce a report on the number of senders of each of the selectors in 
	the list. 1/27/96 sw"
	
	self deprecated: 'Use ''SystemNavigation default reportSenderCountsFor:'' instead.' on: '10 July 2009' in: #Pharo1.0.
	^ SystemNavigation default reportSenderCountsFor: selectorList! !

!Utilities class methodsFor: 'pharo' stamp: 'sd 3/11/2008 21:33'!
resetServers

	UpdateUrlLists := OrderedCollection new.! !

!Utilities class methodsFor: 'fetching updates' stamp: 'laza 8/28/2005 04:55'!
retrieveUrls: urls ontoQueue: queue withWaitSema: waitSema 
	"download the given list of URLs. The queue will be loaded alternately  
	with url's and with the retrieved contents. If a download fails, the  
	contents will be #failed. If all goes well, a special pair with an empty  
	URL and the contents #finished will be put on the queue. waitSema is  
	waited on every time before a new document is downloaded; this keeps 
	the downloader from getting too far  ahead of the main process"
	"kill the existing downloader if there is one"
	| doc canPeek front updateCounter |
	UpdateDownloader
		ifNotNil: [UpdateDownloader terminate].
	updateCounter := 0.
	"fork a new downloading process"
	UpdateDownloader := [
		'Downloading updates' displayProgressAt: Sensor cursorPoint from: 0 to: urls size during: [:bar |
			urls
				do: [:url | 
					waitSema wait.
					queue nextPut: url.
					doc := HTTPClient httpGet: url.
					doc isString
						ifTrue: [queue nextPut: #failed.
							UpdateDownloader := nil.
							Processor activeProcess terminate]
						ifFalse: [canPeek := 120 min: doc size.
							front := doc next: canPeek.  doc skip: -1 * canPeek.
							(front beginsWith: '<!!DOCTYPE') ifTrue: [
								(front includesSubString: 'Not Found') ifTrue: [
									queue nextPut: #failed.
									UpdateDownloader := nil.
									Processor activeProcess terminate]]].
						UpdateDownloader ifNotNil: [queue nextPut: doc. updateCounter := updateCounter + 1. bar value: updateCounter]]].
			queue nextPut: ''.
			queue nextPut: #finished.
			UpdateDownloader := nil] newProcess.
	UpdateDownloader priority: Processor userInterruptPriority.
	"start the process running"
	UpdateDownloader resume! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'nb 6/17/2003 12:25'!
revertLastMethodSubmission
	| changeRecords lastSubmission theClass theSelector |
	"If the most recent method submission was a method change, revert
	that change, and if it was a submission of a brand-new method, 
	remove that method."

	RecentSubmissions isEmptyOrNil ifTrue: [^ Beeper beep].
	lastSubmission := RecentSubmissions last.
	theClass := lastSubmission actualClass ifNil: [^ Beeper beep].
	theSelector := lastSubmission methodSymbol.
	changeRecords := theClass changeRecordsAt: theSelector.
	changeRecords isEmptyOrNil ifTrue: [^ Beeper beep].
	changeRecords size == 1
		ifTrue:
			["method has no prior version, so reverting in this case means removing"
			theClass removeSelector: theSelector]
		ifFalse:
			[changeRecords second fileIn].

"Utilities revertLastMethodSubmission"! !

!Utilities class methodsFor: 'fetching updates' stamp: 'th 4/25/2000 12:59'!
saveUpdate: doc onFile: fileName
	"Save the update on a local file.  With or without the update number on the front, depending on the preference #updateRemoveSequenceNum"

	| file fName pos updateDirectory |

	(FileDirectory default directoryNames includes: 'updates') ifFalse:
		[FileDirectory default createDirectory: 'updates'].
	updateDirectory := FileDirectory default directoryNamed: 'updates'.

	fName := fileName.
	(Preferences valueOfFlag: #updateRemoveSequenceNum) ifTrue:
		[pos := fName findFirst: [:c | c isDigit not].
		fName := fName copyFrom: pos to: fName size].
	doc reset; ascii.
	(updateDirectory fileExists: fName) ifFalse:
		[file := updateDirectory newFileNamed: fName.
		file nextPutAll: doc contents.
		file close].
! !

!Utilities class methodsFor: 'fetching updates' stamp: 'nk 6/26/2003 21:12'!
serverUrls 
	"Return the current list of server URLs.  For code updates.  Format of UpdateUrlLists is 
#( ('squeak updates' ('url1' 'url2'))
    ('some other updates' ('url3' 'url4')))"

	| list |
	list := UpdateUrlLists first last.

	"If there is a dead server, return a copy with that server last" 
	Socket deadServer ifNotNil: [
		list clone withIndexDo: [:aName :ind |
		(aName beginsWith: Socket deadServer) ifTrue: [
			list := list asOrderedCollection.	"and it's a copy"
			list removeAt: ind.
			list addLast: aName]]
	].

	^ list asArray! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'MiguelCoba 7/25/2009 02:22'!
setAuthorInitials
	self deprecated: 'use instead: Author requestFullName'.
	^ Author requestFullName! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'MiguelCoba 7/25/2009 02:13'!
setAuthorInitials: aString
	self deprecated: 'use instead: Author fullName:'.
	^ Author fullName: aString.! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'MiguelCoba 7/25/2009 02:23'!
setAuthorName
	self deprecated: 'use instead: Author requestFullName'.
	^ Author requestFullName
! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'dc 5/30/2008 10:17'!
setClassAndSelectorFrom: messageIDString in: csBlock 
	"Decode strings of the form <className> [class] <selectorName>.   If <className> does not exist as a class, use nil for the class in the block"
	| aStream aClass maybeClass sel |
	(messageIDString isKindOf: MethodReference) ifTrue: [ ^ messageIDString setClassAndSelectorIn: csBlock ].
	aStream := messageIDString readStream.
	aClass := Smalltalk 
		at: (aStream upTo: $ ) asSymbol
		ifAbsent: [ nil ].
	maybeClass := aStream upTo: $ .
	sel := aStream upTo: $ .
	maybeClass = 'class' & (sel size ~= 0) 
		ifTrue: 
			[ aClass 
				ifNil: 
					[ csBlock 
						value: nil
						value: sel asSymbol ]
				ifNotNil: 
					[ csBlock 
						value: aClass class
						value: sel asSymbol ] ]
		ifFalse: 
			[ csBlock 
				value: aClass
				value: maybeClass asSymbol


			"
Utilities setClassAndSelectorFrom: 'Utilities class oppositeModeTo:' in: [:aClass :aSelector | Transcript cr; show: 'Class = ', aClass name printString, ' selector = ', aSelector printString].

Utilities setClassAndSelectorFrom: 'MessageSet setClassAndSelectorIn:' in: [:aClass :aSelector | Transcript cr; show: 'Class = ', aClass name printString, ' selector = ', aSelector printString].
" ]! !

!Utilities class methodsFor: 'fetching updates' stamp: 'mir 8/10/2001 12:30'!
setUpdateServer: groupName
	"Utilities setUpdateServer: 'Squeakland' "
	| entry index |


	entry := UpdateUrlLists detect: [:each | each first = groupName] ifNone: [^self].
	index := UpdateUrlLists indexOf: entry.
	UpdateUrlLists removeAt: index.
	UpdateUrlLists addFirst: entry! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'sw 1/30/98 15:12'!
setterSelectorFor: aName
	"Utilities setterSelectorFor: #elvis"
	^ (('set', (aName asString capitalized)), ':') asSymbol! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 02:18'!
showFormsAcrossTopOfScreen: aFormList
	"Display the given array of forms across the top of the screen, wrapping to subsequent lines if needed.    Useful for example for looking at sets of rotations and animations.  6/10/96 sw"
	"self showFormsAcrossTopOfScreen: {Cursor currentCursor asCursorForm}"
	
	self deprecated: 'Use ''Form showFormsAcrossTopOfScreen:'' instead.' on: '10 July 2009' in: #Pharo1.0.
	^ Form showFormsAcrossTopOfScreen: aFormList! !

!Utilities class methodsFor: 'deprecated-Pharo-1.0' stamp: 'AndrewBlack 9/3/2009 02:19'!
showFormsDictAcrossTopOfScreen: formDict
	"Display the given Dictionary of forms across the top of the screen, wrapping to subsequent lines if needed.  Beneath each, put the name of the associated key."

	"
	| dict methods |
	dict := Dictionary new. 
	methods := MenuIcons class selectors select: [:each | '*Icon' match: each asString].
	methods do: [:each | dict at: each put: (MenuIcons perform: each)].
	Form showFormsDictAcrossTopOfScreen: dict"

	self deprecated: 'Use ''Form showFormsDictAcrossTopOfScreen:'' instead.' on: '10 July 2009' in: #Pharo1.0.
	^ Form showFormsDictAcrossTopOfScreen: formDict

! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'sw 2/16/1999 18:07'!
simpleSetterFor: aSymbol
	"Utilities simpleSetterFor: #right"
	^ (aSymbol, ':') asSymbol! !

!Utilities class methodsFor: 'initialization' stamp: 'NS 1/26/2004 09:52'!
startUp
	SystemChangeNotifier uniqueInstance notify: self ofAllSystemChangesUsing: #event:.! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'sw 9/8/2000 10:02'!
steplistToolsWorkspace
	^ ((StringHolder new contents:  'self currentWorld listOfSteppingMorphs asArray inspectWithLabel: ''stepping morphs''.
Utilities cleanseOtherworldlySteppers.
self currentWorld steppingMorphsNotInWorld do: [:m | m delete].
self currentWorld stepListSummary.
self currentWorld stepListSize.
self currentHand attachMorph: FrameRateMorph new') embeddedInMorphicWindowLabeled: 'Steplist workspace')

setWindowColor: (Color r: 0.9 g: 0.7 b: 0.5);
			openInWorld: self currentWorld extent: (550 @ 140)

"Utilities steplistToolsWorkspace"! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'alain.plantec 6/1/2008 23:21'!
storeTextWindowContentsToFileNamed: aName 
	"Utilities storeTextWindowContentsToFileNamed: 'TextWindows'"
	"there is a reference to World, but this method seems to be unused"
	| windows aDict assoc aRefStream |
	aDict := Dictionary new.
	windows := World submorphs
				select: [:m | m isSystemWindow].
	windows
		do: [:w | 
			assoc := w titleAndPaneText.
			assoc
				ifNotNil: [w holdsTranscript
						ifFalse: [aDict add: assoc]]].
	aDict size = 0
		ifTrue: [^ self inform: 'no windows found to export.'].
	aRefStream := ReferenceStream fileNamed: aName.
	aRefStream nextPut: aDict.
	aRefStream close.
	self inform: 'Done!!  ' , aDict size printString , ' window(s) exported.'! !

!Utilities class methodsFor: 'fetching updates' stamp: 'ar 9/27/2005 20:15'!
summariesForUpdates: startNumber through: stopNumber
	"Answer the concatenation of summary strings for updates numbered in the given range"

	^ String streamContents: [:aStream |
		((ChangeSet changeSetsNamedSuchThat:
			[:aName | aName first isDigit and:
						[aName initialIntegerOrNil >= startNumber] and:
						[aName initialIntegerOrNil <= stopNumber]]) asSortedCollection:
				[:a :b | a name < b name]) do:
					[:aChangeSet | aStream cr; nextPutAll: aChangeSet summaryString]]

"Utilities summariesForUpdates: 4899 through: 4903"

! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'sw 7/29/2002 02:23'!
timeStampForMethod: method
	"Answer the authoring time-stamp for the given method, retrieved from the sources or changes file. Answer the empty string if no time stamp is available."
	"Utilities timeStampForMethod: (Utilities class compiledMethodAt: #timeStampForMethod:)"

	^ method timeStamp! !

!Utilities class methodsFor: 'fetching updates' stamp: 'di 4/30/2001 11:28'!
updateComment
"The following used to be at the beginning of the update file.
	Now it is here to simplify parsing the file...

* To add a new update:  Name it starting with a new four-digit code.  
* Do not use %, /, *, space, or more than one period in the name of an update file.
* The update name does not need to have any relation to the version name.
* Figure out which versions of the system the update makes sense for.
* Add the name of the file to each version's category below.
* Put this file and the update file on all of the servers.
*
* To make a new version of the system:  Pick a name for it (no restrictions)
* Put # and exactly that name on a new line at the end of this file.
* During the release process, fill in exactly that name in the dialog box.
* Put a copy of updates.list on the server.
*
* Special file with a different name for Disney Internal Updates.  
* No need to move or rename files to release them to external updates.
"! !

!Utilities class methodsFor: 'fetching updates' stamp: 'sw 10/13/1998 16:03'!
updateFromServer
	"Update the image by loading all pending updates from the server.  Also save local copies of the update files if the #updateSavesFile preference is set to true"

	self readServerUpdatesSaveLocally: Preferences updateSavesFile updateImage: true! !

!Utilities class methodsFor: 'fetching updates' stamp: 'sw 1/10/1999 01:59'!
updateFromServerThroughUpdateNumber: aNumber
	"Update the image by loading all pending updates from the server.  Also save local copies of the update files if the #updateSavesFile preference is set to true"

	self readServerUpdatesThrough: aNumber saveLocally: Preferences updateSavesFile updateImage: true! !

!Utilities class methodsFor: 'fetching updates' stamp: 'tk 5/7/1998 17:03'!
updateUrlLists

	UpdateUrlLists ifNil: [UpdateUrlLists := OrderedCollection new].
	^ UpdateUrlLists! !

!Utilities class methodsFor: 'fetching updates' stamp: 'di 4/29/2001 14:04'!
writeList: listContents toStream: strm
	"Write a parsed updates.list out as text.
	This is the inverse of parseListContents:"

	| fileNames version |
	strm reset.
	listContents do:
		[:pair | version := pair first.  fileNames := pair last.
		strm nextPut: $#; nextPutAll: version; cr.
		fileNames do: [:fileName | strm nextPutAll: fileName; cr]].
	strm close! !

!Utilities class methodsFor: 'fetching updates' stamp: 'RAA 12/17/2000 16:19'!
zapUpdateDownloader

	UpdateDownloader ifNotNil: [UpdateDownloader terminate].
	UpdateDownloader := nil.! !

!Utilities methodsFor: 'look in class' stamp: 'sw 10/13/1998 13:14'!
seeClassSide
	"All the code for Utilitites is on the class side"! !

Object subclass: #WindowColorRegistry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

WindowColorRegistry class
	instanceVariableNames: 'registry'!

!WindowColorRegistry commentStamp: 'hpt 10/9/2005 22:54' prior: 0!
I provide to the applications developer a place where they can register their WindowColorSpecification for their application's windows.
!

WindowColorRegistry class
	instanceVariableNames: 'registry'!

!WindowColorRegistry classSide methodsFor: 'registry' stamp: 'hpt 10/9/2005 23:11'!
initialize
	self refresh.! !

!WindowColorRegistry classSide methodsFor: 'registry' stamp: 'sd 9/14/2006 20:07'!
refresh
	"This is a one-time only method for bootstraping the new registry. Here we will scan all classes for #windowColorSpecification methods and register those to the registry"

	registry := nil.
	((self systemNavigation allClassesImplementing: #windowColorSpecification)
		collect: [:aClass | aClass theNonMetaClass windowColorSpecification])
		do: [:spec | self registerColorSpecification: spec toClassNamed: spec classSymbol ].! !

!WindowColorRegistry classSide methodsFor: 'registry' stamp: 'hpt 10/9/2005 23:08'!
registerColorSpecification: aColorSpec toClassNamed: aClassName
	self registry at: aClassName asSymbol put: aColorSpec.! !

!WindowColorRegistry classSide methodsFor: 'registry' stamp: 'hpt 10/9/2005 23:23'!
registeredWindowColorSpecFor: aClassName
	"Return the Window Color Spec for the given class. "
	^self registry at: aClassName asSymbol ifAbsent: [].
! !

!WindowColorRegistry classSide methodsFor: 'registry' stamp: 'hpt 10/9/2005 23:17'!
registeredWindowColorSpecs
	^self registry values! !

!WindowColorRegistry classSide methodsFor: 'registry' stamp: 'sd 9/14/2006 20:11'!
registry
	^registry ifNil: [registry := Dictionary new].! !

!WindowColorRegistry classSide methodsFor: 'registry' stamp: 'sd 9/14/2006 20:11'!
unregisterColorSpecificationForClassNamed: aClassName
	
	self registry removeKey: aClassName asSymbol ! !

!WindowColorRegistry methodsFor: 'notes' stamp: 'hpt 10/9/2005 22:54'!
seeClassSide
	"All the code for WindowColorRegistry is on the class side."! !

!ClassDescription methodsFor: '*system-support' stamp: 'dvf 8/23/2003 12:45'!
allUnreferencedClassVariables
	"Answer a list of the names of all the receiver's unreferenced class vars, including those defined in superclasses"

	^ self systemNavigation allUnreferencedClassVariablesOf: self! !
SecurityManager initialize!
SARInstaller initialize!
ExternalDropHandler initialize!
AutoStart initialize!
DummySoundSystem initialize!
Utilities initialize!
ExternalSettings initialize!
MczInstaller initialize!
SmalltalkImage initialize!
ExternalSemaphoreTable initialize!
RealEstateAgent initialize!
DefaultExternalDropHandler initialize!
SystemDictionary initialize!
BooleanPreferenceView initialize!
HaloThemePreferenceView initialize!
WindowColorRegistry initialize!
